/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: Action.java
*/

package edu.uofminnesota.DEMAIS.core.annotation;

//Import needed packages
import java.util.*;
import edu.uofminnesota.DEMAIS.core.parser.*;



public class Action {

    // String representation of the action
    private String actionString;
   
    // Stack representation of the action (after parsing)
    private Stack actionStack;



    // Constructors
    public Action() {
	this("", null);
    }

    public Action(String aString) {
	this(aString, null);
    }
    
    public Action(String aString, Stack aStack) {
	actionString = aString;
	actionStack = aStack;
    }


    public Action(Action copy) {
	actionString = null;
	if (copy.actionString != null) {
	    actionString = new String(copy.actionString);
	}

	// Do not copy the stack (should be empty anyway)
	actionStack = null;
    }



    // Public methods
    public String getActionString() {
	return actionString;
    }

    public void setActionString(String aString) {
	actionString = aString;
    }


    public Action reverse() {
	Tokenizer tokenizer = new Tokenizer(actionString);
	String reverseString = "";
	
	// Get the first token
	Token token = tokenizer.getNextToken();
	// Loop through the tokens of the action string
	while (! (token.equals(Terminal.EOF))) {
	    
	    // Reverse the actions
	    if (token.equals(Terminal.BEGIN)) {
		reverseString += "end";

	    } else if (token.equals(Terminal.END)) {
		reverseString += "begin";

	    } else if (token.equals(Terminal.DISPLAY)) {
		reverseString = "undisplay";

	    } else if (token.equals(Terminal.UNDISPLAY)) {
		reverseString += "display";
	
	    } else if (token.equals(Terminal.HIGHLIGHT)) {
		reverseString += "unhighlight";

	    } else if (token.equals(Terminal.UNHIGHLIGHT)) {
		reverseString += "highlight";

	    } else {
		reverseString += token.getLexeme();
	    }
	    
	    token = tokenizer.getNextToken();
	    if (! (token.equals(Terminal.EOF))) {
		reverseString += " ";
	    }
	}
	
	return (new Action(reverseString));
    }


    public Object clone() {
	return (new Action(this));
    }


    public String toString() {
	return actionString;
    }


    // Protected methods
    protected Stack getActionStack() {
	return actionStack;
    }

    protected void setActionStack(Stack aStack) {
	actionStack = aStack;
    }

}
