/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/



/*
  File: Condition.java
*/

package edu.uofminnesota.DEMAIS.core.annotation;

//Import needed packages
import java.lang.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.meal.NamedModel;



public class Condition {

    // String representation of the condition
    private String conditionString;

    // Stack representation of the condition (after parsing)
    private Stack conditionStack;

    // Traceables found during parsing of this condition
    private Vector traceables;



    // Constructors
    public Condition() {
	this("", null);
    }

    public Condition(String cString) {
	this(cString, null);
    }
    
    public Condition(String cString, Stack cStack) {
	conditionString = cString;
	conditionStack = cStack;

	traceables = new Vector();
    }

    public Condition(Condition copy) {
	conditionString = null;
	if (copy.conditionString != null) {
	    conditionString = new String(copy.conditionString);
	}

	conditionStack = null;
	traceables = new Vector();
    }




    // Public methods
    public String getConditionString() {
	return conditionString;
    }

    public void setConditionString(String cString) {
	conditionString = cString;
    }

    public Vector getTraceables() {
	return traceables;
    }

    public void addTraceable(NamedModel nm) {
	if (! traceables.contains(nm)) {
	    traceables.add(nm);
	}
    }

    public void setTraceables(Vector t) {
	traceables = t;
    }


    public Condition reverse() {
	String reverseString = "";
	int index = -1;
	int skip = 0;
	
	if ((index = conditionString.indexOf("When")) >= 0) {
	    reverseString = "When ";
	    skip = index + 4;
	} else if ((index = conditionString.indexOf("After")) >= 0) {
	    reverseString = "After ";
	    skip = index + 5;
	} else if ((index = conditionString.indexOf("Once")) >= 0) {
	    reverseString = "Once ";
	    skip = index + 4;
	} else {
	    return null;
	}

	reverseString += "!(";
	reverseString += conditionString.substring(skip + 1);
	reverseString += ")";
	
	return (new Condition(reverseString));
    }
    

    public String toString() {
	return conditionString;
    }


    // Create a deep copy of this condition
    public Object clone() {
	return (new Condition(this));
    }


    // Protected methods
    protected Stack getConditionStack() {
	return conditionStack;
    }

    protected void setConditionStack(Stack cStack) {
	conditionStack = cStack;
    }




}
