/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: Nonterminal.java
*/

// Current package
package edu.uofminnesota.DEMAIS.core.parser;

// Import needed packages
import java.lang.*;


/*
  For the Nonterminal definitions, see the file Grammar.doc in this directory
*/
public class Nonterminal {

    // The nonterminals
    public static final Nonterminal DEMAISCondition;
    public static final Nonterminal Condition;
    public static final Nonterminal AfterExpression;
    public static final Nonterminal WhenExpression;
    public static final Nonterminal OnceExpression;
    public static final Nonterminal AfterEventExpression;
    public static final Nonterminal AfterEventExpressionPrime;
    public static final Nonterminal AfterARTerm;
    public static final Nonterminal AfterARTermPrime;
    public static final Nonterminal AfterARFactor;
    public static final Nonterminal SimpleTimeExpression;
    public static final Nonterminal TimeUnits;
    public static final Nonterminal MediaObjectEventExpression;
    public static final Nonterminal MediaObjectExpression;
    public static final Nonterminal MediaObject;
    public static final Nonterminal IndirectMediaObjectExpression;
    public static final Nonterminal DefiniteArticle;
    public static final Nonterminal IndirectMediaObject;
    public static final Nonterminal MediaObjectEvent;
    public static final Nonterminal MediaObjectEventPresentTense;
    public static final Nonterminal MediaObjectEventBegins;
    public static final Nonterminal MediaObjectEventEnds;
    public static final Nonterminal MediaObjectEventReaches;
    public static final Nonterminal PointExpression;
    public static final Nonterminal MediaObjectEventPastTense;
    public static final Nonterminal MediaObjectEventHas;
    public static final Nonterminal MediaObjectEventHasPrime;
    public static final Nonterminal MediaObjectEventHasTenseVerb;
    public static final Nonterminal EventExpression;
    public static final Nonterminal EventExpressionPrime;
    public static final Nonterminal AExpression;
    public static final Nonterminal AExpressionPrime;
    public static final Nonterminal BExpression;
    public static final Nonterminal BExpressionPrime;
    public static final Nonterminal CExpression;
    public static final Nonterminal CExpressionPrime;
    public static final Nonterminal Term;
    public static final Nonterminal TermPrime;
    public static final Nonterminal Factor;
    public static final Nonterminal UserObjectOrIndirectMediaObjectExpression;
    public static final Nonterminal AUserObjectExpression;
    public static final Nonterminal AUserExpression;
    public static final Nonterminal IndefiniteArticle;
    public static final Nonterminal UserEventExpression;
    public static final Nonterminal ButtonEventExpression;
    public static final Nonterminal ButtonSelectsExpression;
    public static final Nonterminal ButtonClicksExpression;
    public static final Nonterminal ButtonDoubleClicksExpression;
    public static final Nonterminal OnExpression;
    public static final Nonterminal ButtonPressesExpression;
    public static final Nonterminal TextEntryEventExpression;
    public static final Nonterminal DragDropEventExpression;
    public static final Nonterminal RolloverEventExpression;
    public static final Nonterminal IndirectMediaObjectExpression1;
    public static final Nonterminal MediaObjectEvent1;
    public static final Nonterminal DEMAISAction;
    public static final Nonterminal ActionSequence;
    public static final Nonterminal ActionExpressionPrime;
    public static final Nonterminal ActionExpression;
    public static final Nonterminal DisplayExpression;
    public static final Nonterminal UndisplayExpression;
    public static final Nonterminal HighlightExpression;
    public static final Nonterminal UnhighlightExpression;
    public static final Nonterminal BeginExpression;
    public static final Nonterminal EndExpression;
    public static final Nonterminal PauseExpression;
    public static final Nonterminal NarrateExpression;
    public static final Nonterminal SpeakExpression;
    public static final Nonterminal NavigateExpression;
    public static final Nonterminal TransitionExpression;
    public static final Nonterminal TransitionParamExpression;
    public static final Nonterminal TransitionType;
    public static final Nonterminal TransitionSpeed;
    public static final Nonterminal SetExpression;
    public static final Nonterminal MediaObjectOrVariable;
    public static final Nonterminal AssignExpression;
    public static final Nonterminal WaitExpression;
    public static final Nonterminal WaitEventExpression;
    public static final Nonterminal Storyboard;

    // Number of nonterminals
    private static final int numNonterminals;



    // Static initializer
    static {
	
	DEMAISCondition = new Nonterminal("DEMAISCondition", 0);
	Condition = new Nonterminal("Condition", 1);
	AfterExpression = new Nonterminal("AfterExpression", 2);
	WhenExpression = new Nonterminal("WhenExpression", 3);
	OnceExpression = new Nonterminal("OnceExpression", 4);
	AfterEventExpression = new Nonterminal("AfterEventExpression", 5);
	AfterEventExpressionPrime = new Nonterminal("AfterEventExpressionPrime", 6);
	AfterARTerm = new Nonterminal("AfterARTerm", 7);
	AfterARTermPrime = new Nonterminal("AfterARTermPrime", 8);
	AfterARFactor = new Nonterminal("AfterARFactor", 9);
	SimpleTimeExpression = new Nonterminal("SimpleTimeExpression", 10);
	TimeUnits = new Nonterminal("TimeUnits", 11);
	MediaObjectEventExpression = new Nonterminal("MediaObjectEventExpression", 12);
	MediaObjectExpression = new Nonterminal("MediaObjectExpression", 13);
	MediaObject = new Nonterminal("MediaObject", 14);
	IndirectMediaObjectExpression = new Nonterminal("IndirectMediaObjectExpression", 15);
	DefiniteArticle = new Nonterminal("DefiniteArticle", 16);
	IndirectMediaObject = new Nonterminal("IndirectMediaObject", 17);
	MediaObjectEvent = new Nonterminal("MediaObjectEvent", 18);
	MediaObjectEventPresentTense = new Nonterminal("MediaObjectEventPresentTense", 19);
	MediaObjectEventBegins = new Nonterminal("MediaObjectEventBegins", 20);
	MediaObjectEventEnds = new Nonterminal("MediaObjectEventEnds", 21);
	MediaObjectEventReaches = new Nonterminal("MediaObjectEventReaches", 22);
	PointExpression = new Nonterminal("PointExpression", 23);
	MediaObjectEventPastTense = new Nonterminal("MediaObjectEventPastTense", 24);
	MediaObjectEventHas = new Nonterminal("MediaObjectEventHas", 25);
	MediaObjectEventHasPrime = new Nonterminal("MediaObjectEventHasPrime", 26);
	MediaObjectEventHasTenseVerb = new Nonterminal("MediaObjectEventHasTenseVerb", 27);
	EventExpression = new Nonterminal("EventExpression", 28);
	EventExpressionPrime = new Nonterminal("EventExpressionPrime", 29);
	AExpression = new Nonterminal("AExpression", 30);
	AExpressionPrime = new Nonterminal("AExpressionPrime", 31);
	BExpression = new Nonterminal("BExpression", 32);
	BExpressionPrime = new Nonterminal("BExpressionPrime", 33);
	CExpression = new Nonterminal("CExpression", 34);
	CExpressionPrime = new Nonterminal("CExpressionPrime", 35);
	Term = new Nonterminal("Term", 36);
	TermPrime = new Nonterminal("TermPrime", 37);
	Factor = new Nonterminal("Factor", 38);
	UserObjectOrIndirectMediaObjectExpression = new Nonterminal("UserObjectOrIndirectMediaObjectExpression", 39);
	AUserObjectExpression = new Nonterminal("AUserObjectExpression", 40);
	AUserExpression = new Nonterminal("AUserExpression", 41);
	IndefiniteArticle = new Nonterminal("IndefiniteArticle", 42);
	UserEventExpression = new Nonterminal("UserEventExpression", 43);
	ButtonEventExpression = new Nonterminal("ButtonEventExpression", 44);
	ButtonSelectsExpression = new Nonterminal("ButtonSelectsExpression", 45);
	ButtonClicksExpression = new Nonterminal("ButtonClicksExpression", 46);
	ButtonDoubleClicksExpression = new Nonterminal("ButtonDoubleClicksExpression", 47);
	OnExpression = new Nonterminal("OnExpression", 48);
	ButtonPressesExpression = new Nonterminal("ButtonPressesExpression", 49);
	TextEntryEventExpression = new Nonterminal("TextEntryEventExpression", 50);
	DragDropEventExpression = new Nonterminal("DragDropEventExpression", 51);
	RolloverEventExpression = new Nonterminal("RolloverEventExpression", 52);
	IndirectMediaObjectExpression1 = new Nonterminal("IndirectMediaObjectExpression1", 53);
	MediaObjectEvent1 = new Nonterminal("MediaObjectEvent1", 54);
	DEMAISAction = new Nonterminal("DEMAISAction", 55);
	ActionSequence = new Nonterminal("ActionSequence", 56);
	ActionExpressionPrime = new Nonterminal("ActionExpressionPrime", 57);
	ActionExpression = new Nonterminal("ActionExpression", 58);
	DisplayExpression = new Nonterminal("DisplayExpression", 59);
	UndisplayExpression = new Nonterminal("UndisplayExpression", 60);
	HighlightExpression = new Nonterminal("HighlightExpression", 61);
	UnhighlightExpression = new Nonterminal("UnhighlightExpression", 62);
	BeginExpression = new Nonterminal("BeginExpression", 63);
	EndExpression = new Nonterminal("EndExpression", 64);
	PauseExpression = new Nonterminal("PauseExpression", 65);
	NarrateExpression = new Nonterminal("NarrateExpression", 66);
	SpeakExpression = new Nonterminal("SpeakExpression", 67);
	NavigateExpression = new Nonterminal("NavigateExpression", 68);
	TransitionExpression = new Nonterminal("TransitionExpression", 69);
	TransitionParamExpression = new Nonterminal("TransitionParamExpression", 70);
	TransitionType = new Nonterminal("TransitionType", 71);
	TransitionSpeed = new Nonterminal("TransitionSpeed", 72);
	SetExpression = new Nonterminal("SetExpression", 73);
	MediaObjectOrVariable = new Nonterminal("MediaObjectOrVariable", 74);
	AssignExpression = new Nonterminal("AssignExpression", 75);
	WaitExpression = new Nonterminal("WaitExpression", 76);
	WaitEventExpression = new Nonterminal("WaitEventExpression", 77);
	Storyboard = new Nonterminal("Storyboard", 78);

	numNonterminals = 79;
    }


    public static final int numNonterminals() {
	return numNonterminals;
    }
    

    
    // Instance variables

    // The name of the nonterminal
    private String name;

    // The integer value of the nonterminal
    private Integer value;



    // Constructors
    private Nonterminal() {
	this(null, 0);
    }

    public Nonterminal(String n, int v) {
	this(n, new Integer(v));
    }

    public Nonterminal(String n, Integer v) {
	name = n;
	value = v;
    }


    public int intValue() {
	return value.intValue();
    }

    public Integer toInteger() {
	return value;
    }

    public String getName() {
	return name;
    }


    public int hashCode() {
	return name.hashCode();
    }


    public boolean equals(Object obj) {
	if (obj instanceof Nonterminal) {
	    return (equals( (Nonterminal) obj));

	} else if (obj instanceof Integer) {
	    return (equals( (Integer) obj));
	}
	return (this == obj);
    }

    public boolean equals(Nonterminal obj) {
	return (value.equals(obj.value));
    }

    public boolean equals(Integer obj) {
	return (value.equals(obj));
    }
}



