/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: Opcode.java
*/

// Current package
package edu.uofminnesota.DEMAIS.core.parser;

// Import needed packages
import java.lang.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.meal.*;




public class Opcode {

    // The Opcode values
    public static final Opcode AFTER;
    public static final Opcode AND;
    public static final Opcode ANIMATION;
    public static final Opcode AUDIO;
    public static final Opcode BEGIN;
    public static final Opcode BEGINS;
    public static final Opcode BEGUN;
    public static final Opcode CLICKS;
    public static final Opcode DELAY;
    public static final Opcode DISPLAY;
    public static final Opcode DISSOLVE;
    public static final Opcode DIVIDE;
    public static final Opcode DOUBLECLICKS;
    public static final Opcode DRAG_AND_DROP;
    public static final Opcode END;
    public static final Opcode ENDED;
    public static final Opcode ENDS;
    public static final Opcode ENTERS_TEXT;
    public static final Opcode EQUAL;
    public static final Opcode FAST;
    public static final Opcode GOTO;
    public static final Opcode GREATERTHAN;
    public static final Opcode GREATERTHAN_EQUAL;
    public static final Opcode HIGHLIGHT;
    public static final Opcode HOLD;
    public static final Opcode LESSTHAN;
    public static final Opcode LESSTHAN_EQUAL;
    public static final Opcode MEDIA_OBJECT;
    public static final Opcode MEDIUM;
    public static final Opcode MINUS;
    public static final Opcode MULTIPLY;
    public static final Opcode NARRATE;
    public static final Opcode NARRATION;
    public static final Opcode NOT;
    public static final Opcode ONCE;
    public static final Opcode OR;
    public static final Opcode PAUSE;
    public static final Opcode PLUS;
    public static final Opcode PRESSES;
    public static final Opcode REAL_NUMBER;
    public static final Opcode REACHES;
    public static final Opcode ROLLOVER;
    public static final Opcode SELECTS;
    public static final Opcode SET;
    public static final Opcode SET_SCOPE_TO_LOCAL;
    public static final Opcode SET_SCOPE_TO_APPLICATION;
    public static final Opcode SLOW;
    public static final Opcode SPEAK;
    public static final Opcode STORYBOARD;
    public static final Opcode TRANSITION;
    public static final Opcode UNDISPLAY;
    public static final Opcode UNHIGHLIGHT;
    public static final Opcode USER;
    public static final Opcode VARIABLE;
    public static final Opcode VIDEO;
    public static final Opcode WAIT;
    public static final Opcode WHEN;
    public static final Opcode WIPE;


    // Instance variables

    // The value of the opcode
    private Integer code;

    // Additional information for some opcodes
    private double realNumber;
    private NamedModel object;
    private DataModel scope;

 

    // Static initializer
    static {

	AFTER = new Opcode(0);
	AND = new Opcode(1);
	ANIMATION = new Opcode(2);
	AUDIO = new Opcode(3);
	BEGIN = new Opcode(4);
	BEGINS = new Opcode(5);
	BEGUN = new Opcode(6);
	CLICKS = new Opcode(7);
	DELAY = new Opcode(8);
	DISPLAY = new Opcode(9);
	DISSOLVE = new Opcode(10);
	DIVIDE = new Opcode(11);
	DOUBLECLICKS = new Opcode(12);
	DRAG_AND_DROP = new Opcode(13);
	END = new Opcode(14);
	ENDED = new Opcode(15);
	ENDS = new Opcode(16);
	ENTERS_TEXT = new Opcode(17);
	EQUAL = new Opcode(18);
	FAST = new Opcode(19);
	GOTO = new Opcode(20);
	GREATERTHAN = new Opcode(21);
	GREATERTHAN_EQUAL = new Opcode(22);
	HIGHLIGHT = new Opcode(23);
	HOLD = new Opcode(24);
	LESSTHAN = new Opcode(25);
	LESSTHAN_EQUAL = new Opcode(26);
	MEDIA_OBJECT = new Opcode(27);
	MEDIUM = new Opcode(28);
	MINUS = new Opcode(29);
	MULTIPLY = new Opcode(30);
	NARRATE = new Opcode(31);
	NARRATION = new Opcode(32);
	NOT = new Opcode(33);
	ONCE = new Opcode(34);
	OR = new Opcode(35);
	PAUSE = new Opcode(36);
	PLUS = new Opcode(37);
	PRESSES = new Opcode(38);
	REAL_NUMBER = new Opcode(39);
	REACHES = new Opcode(40);
	ROLLOVER = new Opcode(41);
	SELECTS = new Opcode(42);
	SET = new Opcode(43);
	SET_SCOPE_TO_LOCAL = new Opcode(44);
	SET_SCOPE_TO_APPLICATION = new Opcode(45);
	SLOW = new Opcode(46);
	SPEAK = new Opcode(47);
	STORYBOARD = new Opcode(48);
	TRANSITION = new Opcode(49);
	UNDISPLAY = new Opcode(50);
	UNHIGHLIGHT = new Opcode(51);
	USER = new Opcode(52);
	VARIABLE = new Opcode(53);
	VIDEO = new Opcode(54);
	WAIT = new Opcode(55);
	WHEN = new Opcode(56);
	WIPE = new Opcode(57);
    }



    // Constructors
    private Opcode() {
	this(-1);
    }

    public Opcode(int i) {
	this(new Integer(i));
    }

    public Opcode(Integer code) {
	this(code, 0.0, null, null);
    }

    public Opcode(Integer code, double rn, NamedModel obj, DataModel model) {
	this.code = code;
	realNumber = rn;
	object = obj;
	scope = model;
    }

    public Opcode(Opcode copy) {
	this(new Integer(copy.code.intValue()), copy.realNumber, copy.object, copy.scope);
    }


    public Object clone() {
	return (new Opcode(this));
    }


    public int intValue() {
	return code.intValue();
    }

    public double getRealNumber() {
	return realNumber;
    }

    public void setRealNumber(double r) {
	realNumber = r;
    }

    public NamedModel getObject() {
	return object;
    }

    public void setObject(NamedModel no) {
	object = no;
    }

    public DataModel getScope() {
	return scope;
    }

    public void setScope(DataModel model) {
	scope = model;
    }


    public boolean equals(Object obj) {
	if (obj instanceof Opcode) {
	    return (equals((Opcode) obj));
	
	} else if (obj instanceof Integer) {
	    return (equals((Integer) obj));
	}
	return (this == obj);
    }

    public boolean equals(Opcode c) {
	return (code.equals(c.code));
    }

    public boolean equals(Integer i) {
	return (code.equals(i));
    }


    public String toString() {
	String s = "[";
	s += code.toString();

	if (this.equals(Opcode.REAL_NUMBER)) {
	    s += " ";
	    s += Double.toString(realNumber);

	} else if (this.equals(Opcode.VARIABLE) ||
		   this.equals(Opcode.MEDIA_OBJECT) ||
		   this.equals(Opcode.STORYBOARD) ) {
	    s += " ";
	    if (object != null) {
		s += object.getName();
	    } else {
		s += "null";
	    }
	}

	s += "]";
	return s;
    }

}
