/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: ParseException.java

  This exception stores information regarding any error that occurs during the parsing of an annotation's condition or action.
*/

// Current package
package edu.uofminnesota.DEMAIS.core.parser;

// Import needed packages
import java.lang.*;
import edu.uofminnesota.DEMAIS.core.annotation.*;



public class ParseException extends Exception {

    protected Annotation annotation;
    protected int wordCount;
    protected String component;
    protected String errorMsg;


    // Constructors
    public ParseException() {
	this(null, 0, null, null);
    }

    public ParseException(String msg) {
	this(null, 0, null, msg);
    }

    public ParseException(String c, String msg) {
	this(null, 0, c, msg);
    }

    public ParseException(int wc, String c, String msg) {
	this(null, wc, c, msg);
    }

    public ParseException(Annotation a, int wc, String c, String msg) {
	annotation = a;
	wordCount = wc;
	component = c;
	errorMsg = msg;
    }

    
    // Public methods
    public void setAnnotation(Annotation a) {
	annotation = a;
    }

    public Annotation getAnnotation() {
	return annotation;
    }

    public void setWordCount(int wc) {
	wordCount = wc;
    }

    public int getWordCount() {
	return wordCount;
    }

    public String toString() {
	return("Annotation: " + annotation.toString() + "\n" +
	       "Annotation Error:" + "\n" +
	       "Wordcount: " + Integer.toString(wordCount) + "\n" +
	       "Component: " + component + "\n" +
	       "ErrorMsg: " + errorMsg);
    }


}
