/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: Parsetable.java
*/

// Current package
package edu.uofminnesota.DEMAIS.core.parser;

// Import needed packages
import java.lang.*;
import java.util.*;

/*
  The parseTable was constructed based on the algorithms given in the book 'Compilers: Principles, Techniques, and Tools' written by Aho, Sethi, and Ulman.
 */


public class Parsetable {

    // A hash Table mapping a terminal string to its corresponding Integer
    public static int Table[][];


    // Static initializer
    static {
	int numNT = Nonterminal.numNonterminals();
	int numT = Terminal.numTerminals();

	// Build the Table
	Table = new int[numNT][];
	for (int i=0; i<numNT; ++i) {
	    Table[i] = new int[numT];
	    for (int j=0; j<numT; ++j) {
		// Initialize all entries to an ERROR value
		Table[i][j] = -1;
	    }
	}

	/*
	  Now fill in the parse Table according to the FIRST and FOLLOW sets.
	*/

	Table[Nonterminal.DEMAISCondition.intValue()][Terminal.AFTER.intValue()] = 1;
	Table[Nonterminal.DEMAISCondition.intValue()][Terminal.WHEN.intValue()] = 1;
	Table[Nonterminal.DEMAISCondition.intValue()][Terminal.ONCE.intValue()] = 1;

	Table[Nonterminal.Condition.intValue()][Terminal.AFTER.intValue()] = 2;

	Table[Nonterminal.Condition.intValue()][Terminal.WHEN.intValue()] = 3;

	Table[Nonterminal.Condition.intValue()][Terminal.ONCE.intValue()] = 4;

	Table[Nonterminal.AfterExpression.intValue()][Terminal.AFTER.intValue()] = 5;
	
	Table[Nonterminal.WhenExpression.intValue()][Terminal.WHEN.intValue()] = 6;
	
	Table[Nonterminal.OnceExpression.intValue()][Terminal.ONCE.intValue()] = 7;
	
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.REAL_NUMBER.intValue()] = 8;
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.VARIABLE.intValue()] = 8;
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 8;
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.THE.intValue()] = 8;
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.ANIMATION.intValue()] = 8;
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.AUDIO.intValue()] = 8;
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.NARRATION.intValue()] = 8;
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.VIDEO.intValue()] = 8;
	Table[Nonterminal.AfterEventExpression.intValue()][Terminal.LEFT_PAREN.intValue()] = 8;
	
	Table[Nonterminal.AfterEventExpressionPrime.intValue()][Terminal.PLUS.intValue()] = 9;

	Table[Nonterminal.AfterEventExpressionPrime.intValue()][Terminal.MINUS.intValue()] = 10;

	Table[Nonterminal.AfterEventExpressionPrime.intValue()][Terminal.EOF.intValue()] = 11;
	Table[Nonterminal.AfterEventExpressionPrime.intValue()][Terminal.RIGHT_PAREN.intValue()] = 11;
	
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.REAL_NUMBER.intValue()] = 12;
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.VARIABLE.intValue()] = 12;
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 12;
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.THE.intValue()] = 12;
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.ANIMATION.intValue()] = 12;
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.AUDIO.intValue()] = 12;
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.NARRATION.intValue()] = 12;
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.VIDEO.intValue()] = 12;
	Table[Nonterminal.AfterARTerm.intValue()][Terminal.LEFT_PAREN.intValue()] = 12;

	Table[Nonterminal.AfterARTermPrime.intValue()][Terminal.MULTIPLY.intValue()] = 13;

	Table[Nonterminal.AfterARTermPrime.intValue()][Terminal.DIVIDE.intValue()] = 14;
	
	Table[Nonterminal.AfterARTermPrime.intValue()][Terminal.EOF.intValue()] = 15;
	Table[Nonterminal.AfterARTermPrime.intValue()][Terminal.RIGHT_PAREN.intValue()] = 15;
	Table[Nonterminal.AfterARTermPrime.intValue()][Terminal.PLUS.intValue()] = 15;
	Table[Nonterminal.AfterARTermPrime.intValue()][Terminal.MINUS.intValue()] = 15;
	
	Table[Nonterminal.AfterARFactor.intValue()][Terminal.REAL_NUMBER.intValue()] = 16;

	Table[Nonterminal.AfterARFactor.intValue()][Terminal.VARIABLE.intValue()] = 17;

	Table[Nonterminal.AfterARFactor.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 18;
	Table[Nonterminal.AfterARFactor.intValue()][Terminal.THE.intValue()] = 18;
	Table[Nonterminal.AfterARFactor.intValue()][Terminal.ANIMATION.intValue()] = 18;
	Table[Nonterminal.AfterARFactor.intValue()][Terminal.AUDIO.intValue()] = 18;
	Table[Nonterminal.AfterARFactor.intValue()][Terminal.NARRATION.intValue()] = 18;
	Table[Nonterminal.AfterARFactor.intValue()][Terminal.VIDEO.intValue()] = 18;

	Table[Nonterminal.AfterARFactor.intValue()][Terminal.LEFT_PAREN.intValue()] = 19;
	
	Table[Nonterminal.SimpleTimeExpression.intValue()][Terminal.REAL_NUMBER.intValue()] = 20;

	Table[Nonterminal.TimeUnits.intValue()][Terminal.SECONDS.intValue()] = 21;

	Table[Nonterminal.TimeUnits.intValue()][Terminal.SECOND.intValue()] = 22;
	
	Table[Nonterminal.TimeUnits.intValue()][Terminal.EOF.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.RIGHT_PAREN.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.OR.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.AND.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.LESSTHAN.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.LESSTHAN_EQUAL.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.GREATERTHAN.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.GREATERTHAN_EQUAL.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.EQUAL.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.NOT_EQUAL.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.PLUS.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.MINUS.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.MULTIPLY.intValue()] = 23;
	Table[Nonterminal.TimeUnits.intValue()][Terminal.DIVIDE.intValue()] = 23;

	Table[Nonterminal.MediaObjectEventExpression.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 24;
	Table[Nonterminal.MediaObjectEventExpression.intValue()][Terminal.THE.intValue()] = 24;
	Table[Nonterminal.MediaObjectEventExpression.intValue()][Terminal.ANIMATION.intValue()] = 24;
	Table[Nonterminal.MediaObjectEventExpression.intValue()][Terminal.AUDIO.intValue()] = 24;
	Table[Nonterminal.MediaObjectEventExpression.intValue()][Terminal.NARRATION.intValue()] = 24;
	Table[Nonterminal.MediaObjectEventExpression.intValue()][Terminal.VIDEO.intValue()] = 24;

	Table[Nonterminal.MediaObjectExpression.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 25;

	Table[Nonterminal.MediaObjectExpression.intValue()][Terminal.THE.intValue()] = 26;
	Table[Nonterminal.MediaObjectExpression.intValue()][Terminal.ANIMATION.intValue()] = 26;
	Table[Nonterminal.MediaObjectExpression.intValue()][Terminal.AUDIO.intValue()] = 26;
	Table[Nonterminal.MediaObjectExpression.intValue()][Terminal.NARRATION.intValue()] = 26;
	Table[Nonterminal.MediaObjectExpression.intValue()][Terminal.VIDEO.intValue()] = 26;

	Table[Nonterminal.MediaObject.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 27;

	Table[Nonterminal.IndirectMediaObjectExpression.intValue()][Terminal.THE.intValue()] = 28;

	Table[Nonterminal.IndirectMediaObjectExpression.intValue()][Terminal.ANIMATION.intValue()] = 29;
	Table[Nonterminal.IndirectMediaObjectExpression.intValue()][Terminal.AUDIO.intValue()] = 29;
	Table[Nonterminal.IndirectMediaObjectExpression.intValue()][Terminal.NARRATION.intValue()] = 29;
	Table[Nonterminal.IndirectMediaObjectExpression.intValue()][Terminal.VIDEO.intValue()] = 29;

	Table[Nonterminal.DefiniteArticle.intValue()][Terminal.THE.intValue()] = 30;

	Table[Nonterminal.IndirectMediaObject.intValue()][Terminal.ANIMATION.intValue()] = 31;

	Table[Nonterminal.IndirectMediaObject.intValue()][Terminal.AUDIO.intValue()] = 32;

	Table[Nonterminal.IndirectMediaObject.intValue()][Terminal.NARRATION.intValue()] = 33;

	Table[Nonterminal.IndirectMediaObject.intValue()][Terminal.VIDEO.intValue()] = 34;

	Table[Nonterminal.MediaObjectEvent.intValue()][Terminal.BEGINS.intValue()] = 35;
	Table[Nonterminal.MediaObjectEvent.intValue()][Terminal.ENDS.intValue()] = 35;
	Table[Nonterminal.MediaObjectEvent.intValue()][Terminal.REACHES.intValue()] = 35;

	Table[Nonterminal.MediaObjectEvent.intValue()][Terminal.HAS.intValue()] = 36;

	Table[Nonterminal.MediaObjectEventPresentTense.intValue()][Terminal.BEGINS.intValue()] = 37;

	Table[Nonterminal.MediaObjectEventPresentTense.intValue()][Terminal.ENDS.intValue()] = 38;

	Table[Nonterminal.MediaObjectEventPresentTense.intValue()][Terminal.REACHES.intValue()] = 39;

	Table[Nonterminal.MediaObjectEventBegins.intValue()][Terminal.BEGINS.intValue()] = 40;

	Table[Nonterminal.MediaObjectEventEnds.intValue()][Terminal.ENDS.intValue()] = 41;

	Table[Nonterminal.MediaObjectEventReaches.intValue()][Terminal.REACHES.intValue()] = 42;

	Table[Nonterminal.PointExpression.intValue()][Terminal.LEFT_BRACKET.intValue()] = 43;

	Table[Nonterminal.MediaObjectEventPastTense.intValue()][Terminal.HAS.intValue()] = 44;

	Table[Nonterminal.MediaObjectEventHas.intValue()][Terminal.HAS.intValue()] = 45;

	Table[Nonterminal.MediaObjectEventHasPrime.intValue()][Terminal.NOT.intValue()] = 46;

	Table[Nonterminal.MediaObjectEventHasPrime.intValue()][Terminal.BEGUN.intValue()] = 47;
	Table[Nonterminal.MediaObjectEventHasPrime.intValue()][Terminal.ENDED.intValue()] = 47;

	Table[Nonterminal.MediaObjectEventHasTenseVerb.intValue()][Terminal.BEGUN.intValue()] = 48;

	Table[Nonterminal.MediaObjectEventHasTenseVerb.intValue()][Terminal.ENDED.intValue()] = 49;

	Table[Nonterminal.EventExpression.intValue()][Terminal.REAL_NUMBER.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.VARIABLE.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.THE.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.A.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.USER.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.ANIMATION.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.AUDIO.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.NARRATION.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.VIDEO.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.NOT.intValue()] = 50;
	Table[Nonterminal.EventExpression.intValue()][Terminal.LEFT_PAREN.intValue()] = 50;

	Table[Nonterminal.EventExpressionPrime.intValue()][Terminal.OR.intValue()] = 51;

	Table[Nonterminal.EventExpressionPrime.intValue()][Terminal.EOF.intValue()] = 52;
	Table[Nonterminal.EventExpressionPrime.intValue()][Terminal.RIGHT_PAREN.intValue()] = 52;

	Table[Nonterminal.AExpression.intValue()][Terminal.REAL_NUMBER.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.VARIABLE.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.THE.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.A.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.USER.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.ANIMATION.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.AUDIO.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.NARRATION.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.VIDEO.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.NOT.intValue()] = 53;
	Table[Nonterminal.AExpression.intValue()][Terminal.LEFT_PAREN.intValue()] = 53;

	Table[Nonterminal.AExpressionPrime.intValue()][Terminal.AND.intValue()] = 54;

	Table[Nonterminal.AExpressionPrime.intValue()][Terminal.EOF.intValue()] = 55;
	Table[Nonterminal.AExpressionPrime.intValue()][Terminal.RIGHT_PAREN.intValue()] = 55;
	Table[Nonterminal.AExpressionPrime.intValue()][Terminal.OR.intValue()] = 55;

	Table[Nonterminal.BExpression.intValue()][Terminal.REAL_NUMBER.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.VARIABLE.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.THE.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.A.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.USER.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.ANIMATION.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.AUDIO.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.NARRATION.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.VIDEO.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.NOT.intValue()] = 56;
	Table[Nonterminal.BExpression.intValue()][Terminal.LEFT_PAREN.intValue()] = 56;

	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.LESSTHAN.intValue()] = 57;

	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.LESSTHAN_EQUAL.intValue()] = 58;

	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.GREATERTHAN.intValue()] = 59;

	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.GREATERTHAN_EQUAL.intValue()] = 60;

	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.EQUAL.intValue()] = 61;

	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.NOT_EQUAL.intValue()] = 62;

	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.EOF.intValue()] = 63;
	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.RIGHT_PAREN.intValue()] = 63;
	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.OR.intValue()] = 63;
	Table[Nonterminal.BExpressionPrime.intValue()][Terminal.AND.intValue()] = 63;

	Table[Nonterminal.CExpression.intValue()][Terminal.REAL_NUMBER.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.VARIABLE.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.THE.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.A.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.USER.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.ANIMATION.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.AUDIO.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.NARRATION.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.VIDEO.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.NOT.intValue()] = 64;
	Table[Nonterminal.CExpression.intValue()][Terminal.LEFT_PAREN.intValue()] = 64;

	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.PLUS.intValue()] = 65;

	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.MINUS.intValue()] = 66;

	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.EOF.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.RIGHT_PAREN.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.OR.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.AND.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.LESSTHAN.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.LESSTHAN_EQUAL.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.GREATERTHAN.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.GREATERTHAN_EQUAL.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.EQUAL.intValue()] = 67;
	Table[Nonterminal.CExpressionPrime.intValue()][Terminal.NOT_EQUAL.intValue()] = 67;

	Table[Nonterminal.Term.intValue()][Terminal.REAL_NUMBER.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.VARIABLE.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.THE.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.A.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.USER.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.ANIMATION.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.AUDIO.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.NARRATION.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.VIDEO.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.NOT.intValue()] = 68;
	Table[Nonterminal.Term.intValue()][Terminal.LEFT_PAREN.intValue()] = 68;

	Table[Nonterminal.TermPrime.intValue()][Terminal.MULTIPLY.intValue()] = 69;

	Table[Nonterminal.TermPrime.intValue()][Terminal.DIVIDE.intValue()] = 70;

	Table[Nonterminal.TermPrime.intValue()][Terminal.EOF.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.RIGHT_PAREN.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.OR.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.AND.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.LESSTHAN.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.LESSTHAN_EQUAL.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.GREATERTHAN.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.GREATERTHAN_EQUAL.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.EQUAL.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.NOT_EQUAL.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.PLUS.intValue()] = 71;
	Table[Nonterminal.TermPrime.intValue()][Terminal.MINUS.intValue()] = 71;

	Table[Nonterminal.Factor.intValue()][Terminal.REAL_NUMBER.intValue()] = 72;

	Table[Nonterminal.Factor.intValue()][Terminal.VARIABLE.intValue()] = 73;

	Table[Nonterminal.Factor.intValue()][Terminal.THE.intValue()] = 74;

	Table[Nonterminal.Factor.intValue()][Terminal.A.intValue()] = 75;
	Table[Nonterminal.Factor.intValue()][Terminal.USER.intValue()] = 75;

	Table[Nonterminal.Factor.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 76;
	Table[Nonterminal.Factor.intValue()][Terminal.ANIMATION.intValue()] = 76;
	Table[Nonterminal.Factor.intValue()][Terminal.AUDIO.intValue()] = 76;
	Table[Nonterminal.Factor.intValue()][Terminal.NARRATION.intValue()] = 76;
	Table[Nonterminal.Factor.intValue()][Terminal.VIDEO.intValue()] = 76;

	Table[Nonterminal.Factor.intValue()][Terminal.NOT.intValue()] = 77;

	Table[Nonterminal.Factor.intValue()][Terminal.LEFT_PAREN.intValue()] = 78;

	Table[Nonterminal.UserObjectOrIndirectMediaObjectExpression.intValue()][Terminal.USER.intValue()] = 79;

	Table[Nonterminal.UserObjectOrIndirectMediaObjectExpression.intValue()][Terminal.ANIMATION.intValue()] = 80;
	Table[Nonterminal.UserObjectOrIndirectMediaObjectExpression.intValue()][Terminal.AUDIO.intValue()] = 80;
	Table[Nonterminal.UserObjectOrIndirectMediaObjectExpression.intValue()][Terminal.NARRATION.intValue()] = 80;
	Table[Nonterminal.UserObjectOrIndirectMediaObjectExpression.intValue()][Terminal.VIDEO.intValue()] = 80;

	Table[Nonterminal.AUserObjectExpression.intValue()][Terminal.A.intValue()] = 81;
	Table[Nonterminal.AUserObjectExpression.intValue()][Terminal.USER.intValue()] = 81;

	Table[Nonterminal.AUserExpression.intValue()][Terminal.A.intValue()] = 82;

	Table[Nonterminal.AUserExpression.intValue()][Terminal.USER.intValue()] = 83;

	Table[Nonterminal.IndefiniteArticle.intValue()][Terminal.A.intValue()] = 84;

	Table[Nonterminal.UserEventExpression.intValue()][Terminal.SELECTS.intValue()] = 85;
	Table[Nonterminal.UserEventExpression.intValue()][Terminal.CLICKS.intValue()] = 85;
	Table[Nonterminal.UserEventExpression.intValue()][Terminal.DOUBLECLICKS.intValue()] = 85;
	Table[Nonterminal.UserEventExpression.intValue()][Terminal.PRESSES.intValue()] = 85;

	Table[Nonterminal.UserEventExpression.intValue()][Terminal.ENTERS.intValue()] = 86;

	Table[Nonterminal.UserEventExpression.intValue()][Terminal.DRAGS.intValue()] = 87;

	Table[Nonterminal.UserEventExpression.intValue()][Terminal.ROLLS.intValue()] = 88;

	Table[Nonterminal.ButtonEventExpression.intValue()][Terminal.SELECTS.intValue()] = 89;

	Table[Nonterminal.ButtonEventExpression.intValue()][Terminal.CLICKS.intValue()] = 90;

	Table[Nonterminal.ButtonEventExpression.intValue()][Terminal.DOUBLECLICKS.intValue()] = 91;

	Table[Nonterminal.ButtonEventExpression.intValue()][Terminal.PRESSES.intValue()] = 92;

	Table[Nonterminal.ButtonSelectsExpression.intValue()][Terminal.SELECTS.intValue()] = 93;

	Table[Nonterminal.ButtonClicksExpression.intValue()][Terminal.CLICKS.intValue()] = 94;

	Table[Nonterminal.OnExpression.intValue()][Terminal.ON.intValue()] = 95;

	Table[Nonterminal.OnExpression.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 96;

	Table[Nonterminal.ButtonDoubleClicksExpression.intValue()][Terminal.DOUBLECLICKS.intValue()] = 97;

	Table[Nonterminal.ButtonPressesExpression.intValue()][Terminal.PRESSES.intValue()] = 98;

	Table[Nonterminal.TextEntryEventExpression.intValue()][Terminal.ENTERS.intValue()] = 99;

	Table[Nonterminal.DragDropEventExpression.intValue()][Terminal.DRAGS.intValue()] = 100;

	Table[Nonterminal.RolloverEventExpression.intValue()][Terminal.ROLLS.intValue()] = 101;
	
	Table[Nonterminal.IndirectMediaObjectExpression1.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 102;

	Table[Nonterminal.IndirectMediaObjectExpression1.intValue()][Terminal.ANIMATION.intValue()] = 103;
	Table[Nonterminal.IndirectMediaObjectExpression1.intValue()][Terminal.AUDIO.intValue()] = 103;
	Table[Nonterminal.IndirectMediaObjectExpression1.intValue()][Terminal.NARRATION.intValue()] = 103;
	Table[Nonterminal.IndirectMediaObjectExpression1.intValue()][Terminal.VIDEO.intValue()] = 103;

	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.BEGINS.intValue()] = 104;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.ENDS.intValue()] = 104;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.REACHES.intValue()] = 104;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.HAS.intValue()] = 104;

	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.EOF.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.RIGHT_PAREN.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.OR.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.AND.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.LESSTHAN.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.LESSTHAN_EQUAL.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.GREATERTHAN.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.GREATERTHAN_EQUAL.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.EQUAL.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.NOT_EQUAL.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.PLUS.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.MINUS.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.MULTIPLY.intValue()] = 105;
	Table[Nonterminal.MediaObjectEvent1.intValue()][Terminal.DIVIDE.intValue()] = 105;


	Table[Nonterminal.DEMAISAction.intValue()][Terminal.DISPLAY.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.UNDISPLAY.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.HIGHLIGHT.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.UNHIGHLIGHT.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.BEGIN.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.END.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.PAUSE.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.NAVIGATE.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.GOTO.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.GO.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.JUMP.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.TRANSITION.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.SET.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.WAIT.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.DELAY.intValue()] = 106;
	Table[Nonterminal.DEMAISAction.intValue()][Terminal.HOLD.intValue()] = 106;

	Table[Nonterminal.ActionSequence.intValue()][Terminal.DISPLAY.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.UNDISPLAY.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.HIGHLIGHT.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.UNHIGHLIGHT.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.BEGIN.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.END.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.PAUSE.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.NAVIGATE.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.GOTO.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.GO.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.JUMP.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.TRANSITION.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.SET.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.WAIT.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.DELAY.intValue()] = 107;
	Table[Nonterminal.ActionSequence.intValue()][Terminal.HOLD.intValue()] = 107;

	Table[Nonterminal.ActionExpressionPrime.intValue()][Terminal.COMMA.intValue()] = 108;

	Table[Nonterminal.ActionExpressionPrime.intValue()][Terminal.EOF.intValue()] = 109;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.DISPLAY.intValue()] = 110;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.UNDISPLAY.intValue()] = 111;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.HIGHLIGHT.intValue()] = 112;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.UNHIGHLIGHT.intValue()] = 113;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.BEGIN.intValue()] = 114;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.END.intValue()] = 115;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.PAUSE.intValue()] = 116;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.NARRATE.intValue()] = 117;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.SPEAK.intValue()] = 118;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.NAVIGATE.intValue()] = 119;
	Table[Nonterminal.ActionExpression.intValue()][Terminal.GO.intValue()] = 119;
	Table[Nonterminal.ActionExpression.intValue()][Terminal.GOTO.intValue()] = 119;
	Table[Nonterminal.ActionExpression.intValue()][Terminal.JUMP.intValue()] = 119;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.TRANSITION.intValue()] = 120;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.SET.intValue()] = 121;

	Table[Nonterminal.ActionExpression.intValue()][Terminal.WAIT.intValue()] = 122;
	Table[Nonterminal.ActionExpression.intValue()][Terminal.DELAY.intValue()] = 122;
	Table[Nonterminal.ActionExpression.intValue()][Terminal.HOLD.intValue()] = 122;

	Table[Nonterminal.DisplayExpression.intValue()][Terminal.DISPLAY.intValue()] = 123;

	Table[Nonterminal.UndisplayExpression.intValue()][Terminal.UNDISPLAY.intValue()] = 124;

	Table[Nonterminal.HighlightExpression.intValue()][Terminal.HIGHLIGHT.intValue()] = 125;

	Table[Nonterminal.UnhighlightExpression.intValue()][Terminal.UNHIGHLIGHT.intValue()] = 126;

	Table[Nonterminal.BeginExpression.intValue()][Terminal.BEGIN.intValue()] = 127;

	Table[Nonterminal.EndExpression.intValue()][Terminal.END.intValue()] = 128;

	Table[Nonterminal.PauseExpression.intValue()][Terminal.PAUSE.intValue()] = 129;

	Table[Nonterminal.NarrateExpression.intValue()][Terminal.NARRATE.intValue()] = 130;

	Table[Nonterminal.SpeakExpression.intValue()][Terminal.SPEAK.intValue()] = 131;

	Table[Nonterminal.NavigateExpression.intValue()][Terminal.GOTO.intValue()] = 132;

	Table[Nonterminal.NavigateExpression.intValue()][Terminal.GO.intValue()] = 133;

	Table[Nonterminal.NavigateExpression.intValue()][Terminal.JUMP.intValue()] = 134;

	Table[Nonterminal.NavigateExpression.intValue()][Terminal.NAVIGATE.intValue()] = 135;

	Table[Nonterminal.TransitionExpression.intValue()][Terminal.TRANSITION.intValue()] = 136;

	Table[Nonterminal.TransitionParamExpression.intValue()][Terminal.LEFT_BRACKET.intValue()] = 137;

	Table[Nonterminal.TransitionType.intValue()][Terminal.DISSOLVE.intValue()] = 138;

	Table[Nonterminal.TransitionType.intValue()][Terminal.WIPE.intValue()] = 139;

	Table[Nonterminal.TransitionSpeed.intValue()][Terminal.SLOW.intValue()] = 140;

	Table[Nonterminal.TransitionSpeed.intValue()][Terminal.MEDIUM.intValue()] = 141;

	Table[Nonterminal.TransitionSpeed.intValue()][Terminal.FAST.intValue()] = 142;

	Table[Nonterminal.SetExpression.intValue()][Terminal.SET.intValue()] = 143;

	Table[Nonterminal.MediaObjectOrVariable.intValue()][Terminal.MEDIA_OBJECT.intValue()] = 144;

	Table[Nonterminal.MediaObjectOrVariable.intValue()][Terminal.VARIABLE.intValue()] = 145;

	Table[Nonterminal.AssignExpression.intValue()][Terminal.TO.intValue()] = 146;
	
	Table[Nonterminal.AssignExpression.intValue()][Terminal.ASSIGN.intValue()] = 147;

	Table[Nonterminal.WaitExpression.intValue()][Terminal.WAIT.intValue()] = 148;
	Table[Nonterminal.WaitExpression.intValue()][Terminal.DELAY.intValue()] = 148;
	Table[Nonterminal.WaitExpression.intValue()][Terminal.HOLD.intValue()] = 148;

	Table[Nonterminal.WaitEventExpression.intValue()][Terminal.WAIT.intValue()] = 149;

	Table[Nonterminal.WaitEventExpression.intValue()][Terminal.DELAY.intValue()] = 150;

	Table[Nonterminal.WaitEventExpression.intValue()][Terminal.HOLD.intValue()] = 151;

	Table[Nonterminal.Storyboard.intValue()][Terminal.STORYBOARD.intValue()] = 152;

    }
}
