/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: Production.java
*/

// Current package
package edu.uofminnesota.DEMAIS.core.parser;

// Import needed packages
import java.lang.*;
import java.util.*;

/*
  For the production definitions, see the file Grammar.doc in this directory

  Note: When the terminals and nonterminals of a production are added to a vector below, they are added in reverse order; i.e., add(0, ..). When the production elements are subsequently pushed onto the parseStack, they will be pushed on in the correct order.
*/



public class Production {
    // Represents a production (a sequence of terminals and nonterminals)
    private static Vector p;

    // A hash table mapping an integer to its corresponding production
    private static Hashtable map;


    // Static initializer
    static {

	map = new Hashtable(175);
	
	p = new Vector(1);
	p.add(0, Nonterminal.Condition);
	map.put(new Integer(1), p);

	p = new Vector(1);
	p.add(0, Nonterminal.AfterExpression);
	map.put(new Integer(2), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.WhenExpression);
	map.put(new Integer(3), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.OnceExpression);
	map.put(new Integer(4), p);
	
	p = new Vector(3);
	p.add(0, Terminal.AFTER);
	p.add(0, Nonterminal.AfterEventExpression);
	p.add(0, Opcode.AFTER);
	map.put(new Integer(5), p);
	
	p = new Vector(3);
	p.add(0, Terminal.WHEN);
	p.add(0, Nonterminal.EventExpression);
	p.add(0, Opcode.WHEN);
	map.put(new Integer(6), p);
	
	p = new Vector(3);
	p.add(0, Terminal.ONCE);
	p.add(0, Nonterminal.EventExpression);
	p.add(0, Opcode.ONCE);
	map.put(new Integer(7), p);

	p = new Vector(2);
	p.add(0, Nonterminal.AfterARTerm);
	p.add(0, Nonterminal.AfterEventExpressionPrime);
	map.put(new Integer(8), p);

	p = new Vector(4);
	p.add(0, Terminal.PLUS);
	p.add(0, Nonterminal.AfterARTerm);
	p.add(0, Opcode.PLUS);
	p.add(0, Nonterminal.AfterEventExpressionPrime);
	map.put(new Integer(9), p);	
	
	p = new Vector(4);
	p.add(0, Terminal.MINUS);
	p.add(0, Nonterminal.AfterARTerm);
	p.add(0, Opcode.MINUS);
	p.add(0, Nonterminal.AfterEventExpressionPrime);
	map.put(new Integer(10), p);	
	
	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(11), p);

	p = new Vector(2);
	p.add(0, Nonterminal.AfterARFactor);
	p.add(0, Nonterminal.AfterARTermPrime);
	map.put(new Integer(12), p);
	
	p = new Vector(4);
	p.add(0, Terminal.MULTIPLY);
	p.add(0, Nonterminal.AfterARFactor);
	p.add(0, Opcode.MULTIPLY);
	p.add(0, Nonterminal.AfterARTermPrime);
	map.put(new Integer(13), p);

	p = new Vector(4);
	p.add(0, Terminal.DIVIDE);
	p.add(0, Nonterminal.AfterARFactor);
	p.add(0, Opcode.DIVIDE);
	p.add(0, Nonterminal.AfterARTermPrime);
	map.put(new Integer(14), p);

	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(15), p);

	p = new Vector(1);
	p.add(0, Nonterminal.SimpleTimeExpression);
	map.put(new Integer(16), p);

	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Terminal.VARIABLE);
	p.add(0, Opcode.VARIABLE);
	map.put(new Integer(17), p);

	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEventExpression);
	map.put(new Integer(18), p);

	p = new Vector(3);
	p.add(0, Terminal.LEFT_PAREN);
	p.add(0, Nonterminal.AfterEventExpression);
	p.add(0, Terminal.RIGHT_PAREN);
	map.put(new Integer(19), p);

	p = new Vector(3);
	p.add(0, Terminal.REAL_NUMBER);
	p.add(0, Opcode.REAL_NUMBER);
	p.add(0, Nonterminal.TimeUnits);
	map.put(new Integer(20), p);

	p = new Vector(1);
	p.add(0, Terminal.SECONDS);
	map.put(new Integer(21), p);
	
	p = new Vector(1);
	p.add(0, Terminal.SECOND);
	map.put(new Integer(22), p);
	
	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(23), p);

	p = new Vector(2);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Nonterminal.MediaObjectEvent);
	map.put(new Integer(24), p);

	p = new Vector(1);
	p.add(0, Nonterminal.MediaObject);
	map.put(new Integer(25), p);

	p = new Vector(1);
	p.add(0, Nonterminal.IndirectMediaObjectExpression);
	map.put(new Integer(26), p);

	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Terminal.MEDIA_OBJECT);
	p.add(0, Opcode.MEDIA_OBJECT);
	map.put(new Integer(27), p);
	
	p = new Vector(2);
	p.add(0, Nonterminal.DefiniteArticle);
	p.add(0, Nonterminal.IndirectMediaObject);
	map.put(new Integer(28), p);

	p = new Vector(1);
	p.add(0, Nonterminal.IndirectMediaObject);
	map.put(new Integer(29), p);

	p = new Vector(1);
	p.add(0, Terminal.THE);
	map.put(new Integer(30), p);
	
	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Terminal.ANIMATION);
	p.add(0, Opcode.ANIMATION);
	map.put(new Integer(31), p);
	
	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Terminal.AUDIO);
	p.add(0, Opcode.AUDIO);
	map.put(new Integer(32), p);
	
	p = new Vector(3);
	p.add(0, Terminal.NARRATION);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Opcode.NARRATION);
	map.put(new Integer(33), p);

	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Terminal.VIDEO);
	p.add(0, Opcode.VIDEO);
	map.put(new Integer(34), p);

	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEventPresentTense);
	map.put(new Integer(35), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEventPastTense);
	map.put(new Integer(36), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEventBegins);
	map.put(new Integer(37), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEventEnds);
	map.put(new Integer(38), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEventReaches);
	map.put(new Integer(39), p);

	p = new Vector(2);
	p.add(0, Terminal.BEGINS);
	p.add(0, Opcode.BEGINS);
	map.put(new Integer(40), p);

	p = new Vector(2);
	p.add(0, Terminal.ENDS);
	p.add(0, Opcode.ENDS);
	map.put(new Integer(41), p);

	p = new Vector(3);
	p.add(0, Terminal.REACHES);
	p.add(0, Nonterminal.PointExpression);
	p.add(0, Opcode.REACHES);
	map.put(new Integer(42), p);
	
	p = new Vector(7);
	p.add(0, Terminal.LEFT_BRACKET);
	p.add(0, Terminal.REAL_NUMBER);
	p.add(0, Opcode.REAL_NUMBER);
	p.add(0, Terminal.COMMA);
	p.add(0, Terminal.REAL_NUMBER);
	p.add(0, Opcode.REAL_NUMBER);
	p.add(0, Terminal.RIGHT_BRACKET);
	map.put(new Integer(43), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEventHas);
	map.put(new Integer(44), p);

	p = new Vector(2);
	p.add(0, Terminal.HAS);
	p.add(0, Nonterminal.MediaObjectEventHasPrime);
	map.put(new Integer(45), p);

	p = new Vector(3);
	p.add(0, Terminal.NOT);
	p.add(0, Nonterminal.MediaObjectEventHasTenseVerb);
	p.add(0, Opcode.NOT);
	map.put(new Integer(46), p);

	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEventHasTenseVerb);
	map.put(new Integer(47), p);

	p = new Vector(2);
	p.add(0, Terminal.BEGUN);
	p.add(0, Opcode.BEGUN);
	map.put(new Integer(48), p);
	
	p = new Vector(2);
	p.add(0, Terminal.ENDED);
	p.add(0, Opcode.ENDED);
	map.put(new Integer(49), p);

	p = new Vector(2);
	p.add(0, Nonterminal.AExpression);
	p.add(0, Nonterminal.EventExpressionPrime);
	map.put(new Integer(50), p);

	p = new Vector(4);
	p.add(0, Terminal.OR);
	p.add(0, Nonterminal.AExpression);
	p.add(0, Opcode.OR);
	p.add(0, Nonterminal.EventExpressionPrime);
	map.put(new Integer(51), p);
	
	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(52), p);
	
	p = new Vector(2);
	p.add(0, Nonterminal.BExpression);
	p.add(0, Nonterminal.AExpressionPrime);
	map.put(new Integer(53), p);
	
	p = new Vector(4);
	p.add(0, Terminal.AND);
	p.add(0, Nonterminal.BExpression);
	p.add(0, Opcode.AND);
	p.add(0, Nonterminal.AExpressionPrime);
	map.put(new Integer(54), p);

	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(55), p);

	p = new Vector(2);
	p.add(0, Nonterminal.CExpression);
	p.add(0, Nonterminal.BExpressionPrime);
	map.put(new Integer(56), p);

	p = new Vector(3);
	p.add(0, Terminal.LESSTHAN);
	p.add(0, Nonterminal.CExpression);
	p.add(0, Opcode.LESSTHAN);
	map.put(new Integer(57), p);

	p = new Vector(3);
	p.add(0, Terminal.LESSTHAN_EQUAL);
	p.add(0, Nonterminal.CExpression);
	p.add(0, Opcode.LESSTHAN_EQUAL);
	map.put(new Integer(58), p);
	
	p = new Vector(3);
	p.add(0, Terminal.GREATERTHAN);
	p.add(0, Nonterminal.CExpression);
	p.add(0, Opcode.GREATERTHAN);
	map.put(new Integer(59), p);
	
	p = new Vector(3);
	p.add(0, Terminal.GREATERTHAN_EQUAL);
	p.add(0, Nonterminal.CExpression);
	p.add(0, Opcode.GREATERTHAN_EQUAL);
	map.put(new Integer(60), p);
	
	p = new Vector(3);
	p.add(0, Terminal.EQUAL);
	p.add(0, Nonterminal.CExpression);
	p.add(0, Opcode.EQUAL);
	map.put(new Integer(61), p);
	
	p = new Vector(4);
	p.add(0, Terminal.NOT_EQUAL);
	p.add(0, Nonterminal.CExpression);
	p.add(0, Opcode.EQUAL);
	p.add(0, Opcode.NOT);
	map.put(new Integer(62), p);
	
	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(63), p);

	p = new Vector(2);
	p.add(0, Nonterminal.Term);
	p.add(0, Nonterminal.CExpressionPrime);
	map.put(new Integer(64), p);

	p = new Vector(4);
	p.add(0, Terminal.PLUS);
	p.add(0, Nonterminal.Term);
	p.add(0, Opcode.PLUS);
	p.add(0, Nonterminal.CExpressionPrime);
	map.put(new Integer(65), p);
	
	p = new Vector(4);
	p.add(0, Terminal.MINUS);
	p.add(0, Nonterminal.Term);
	p.add(0, Opcode.MINUS);
	p.add(0, Nonterminal.CExpressionPrime);
	map.put(new Integer(66), p);
	
	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(67), p);
	
	p = new Vector(2);
	p.add(0, Nonterminal.Factor);
	p.add(0, Nonterminal.TermPrime);
	map.put(new Integer(68), p);

	p = new Vector(4);
	p.add(0, Terminal.MULTIPLY);
	p.add(0, Nonterminal.Factor);
	p.add(0, Opcode.MULTIPLY);
	p.add(0, Nonterminal.TermPrime);
	map.put(new Integer(69), p);

	p = new Vector(4);
	p.add(0, Terminal.DIVIDE);
	p.add(0, Nonterminal.Factor);
	p.add(0, Opcode.DIVIDE);
	p.add(0, Nonterminal.TermPrime);
	map.put(new Integer(70), p);

	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(71), p);

	p = new Vector(1);
	p.add(0, Nonterminal.SimpleTimeExpression);
	map.put(new Integer(72), p);

	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Terminal.VARIABLE);
	p.add(0, Opcode.VARIABLE);
	map.put(new Integer(73), p);

	p = new Vector(2);
	p.add(0, Terminal.THE);
	p.add(0, Nonterminal.UserObjectOrIndirectMediaObjectExpression);
	map.put(new Integer(74), p);

	p = new Vector(1);
	p.add(0, Nonterminal.AUserObjectExpression);
	map.put(new Integer(75), p);

	p = new Vector(1);
	p.add(0, Nonterminal.IndirectMediaObjectExpression1);
	map.put(new Integer(76), p);

	
	p = new Vector(3);
	p.add(0, Terminal.NOT);
	p.add(0, Nonterminal.EventExpression);
	p.add(0, Opcode.NOT);
	map.put(new Integer(77), p);

	p = new Vector(3);
	p.add(0, Terminal.LEFT_PAREN);
	p.add(0, Nonterminal.EventExpression);
	p.add(0, Terminal.RIGHT_PAREN);
	map.put(new Integer(78), p);

	p = new Vector(3);
	p.add(0, Terminal.USER);
	p.add(0, Opcode.USER);
	p.add(0, Nonterminal.UserEventExpression);
	map.put(new Integer(79), p);

	p = new Vector(2);
	p.add(0, Nonterminal.IndirectMediaObject);
	p.add(0, Nonterminal.MediaObjectEvent1);
	map.put(new Integer(80), p);

	p = new Vector(2);
	p.add(0, Nonterminal.AUserExpression);
	p.add(0, Nonterminal.UserEventExpression);
	map.put(new Integer(81), p);

	p = new Vector(3);
	p.add(0, Nonterminal.IndefiniteArticle);
	p.add(0, Terminal.USER);
	p.add(0, Opcode.USER);
	map.put(new Integer(82), p);

	p = new Vector(2);
	p.add(0, Terminal.USER);
	p.add(0, Opcode.USER);
	map.put(new Integer(83), p);

	p = new Vector(1);
	p.add(0, Terminal.A);
	map.put(new Integer(84), p);

	p = new Vector(1);
	p.add(0, Nonterminal.ButtonEventExpression);
	map.put(new Integer(85), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.TextEntryEventExpression);
	map.put(new Integer(86), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.DragDropEventExpression);
	map.put(new Integer(87), p);

	p = new Vector(1);
	p.add(0, Nonterminal.RolloverEventExpression);
	map.put(new Integer(88), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.ButtonSelectsExpression);
	map.put(new Integer(89), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.ButtonClicksExpression);
	map.put(new Integer(90), p);

	p = new Vector(1);
	p.add(0, Nonterminal.ButtonDoubleClicksExpression);
	map.put(new Integer(91), p);
	
	p = new Vector(1);
	p.add(0, Nonterminal.ButtonPressesExpression);
	map.put(new Integer(92), p);
	
	p = new Vector(3);
	p.add(0, Terminal.SELECTS);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Opcode.SELECTS);
	map.put(new Integer(93), p);

	p = new Vector(4);
	p.add(0, Terminal.CLICKS);
	p.add(0, Nonterminal.OnExpression);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Opcode.CLICKS);
	map.put(new Integer(94), p);

	p = new Vector(1);
	p.add(0, Terminal.ON);
	map.put(new Integer(95), p);
	
	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(96), p);
	
	p = new Vector(4);
	p.add(0, Terminal.DOUBLECLICKS);
	p.add(0, Nonterminal.OnExpression);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Opcode.DOUBLECLICKS);
	map.put(new Integer(97), p);

	p = new Vector(3);
	p.add(0, Terminal.PRESSES);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Opcode.PRESSES);
	map.put(new Integer(98), p);

	p = new Vector(5);
	p.add(0, Terminal.ENTERS);
	p.add(0, Terminal.TEXT);
	p.add(0, Terminal.INTO);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Opcode.ENTERS_TEXT);
	map.put(new Integer(99), p);
	
	p = new Vector(5);
	p.add(0, Terminal.DRAGS);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Terminal.TO);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Opcode.DRAG_AND_DROP);
	map.put(new Integer(100), p);

	p = new Vector(4);
	p.add(0, Terminal.ROLLS);
	p.add(0, Terminal.OVER);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Opcode.ROLLOVER);
	map.put(new Integer(101), p);

	p = new Vector(2);
	p.add(0, Nonterminal.MediaObject);
	p.add(0, Nonterminal.MediaObjectEvent1);
	map.put(new Integer(102), p);

	p = new Vector(2);
	p.add(0, Nonterminal.IndirectMediaObject);
	p.add(0, Nonterminal.MediaObjectEvent1);
	map.put(new Integer(103), p);

	p = new Vector(1);
	p.add(0, Nonterminal.MediaObjectEvent);
	map.put(new Integer(104), p);
	
	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(105), p);

	p = new Vector(1);
	p.add(0, Nonterminal.ActionSequence);
	map.put(new Integer(106), p);

	p = new Vector(2);
	p.add(0, Nonterminal.ActionExpression);
	p.add(0, Nonterminal.ActionExpressionPrime);
	map.put(new Integer(107), p);

	p = new Vector(3);
	p.add(0, Terminal.COMMA);
	p.add(0, Nonterminal.ActionExpression);
	p.add(0, Nonterminal.ActionExpressionPrime);
	map.put(new Integer(108), p);

	p = new Vector(1);
	//p.add(0, Terminal.EPSILON);
	map.put(new Integer(109), p);

	p = new Vector(1);
	p.add(0, Nonterminal.DisplayExpression);
	map.put(new Integer(110), p);

	p = new Vector(1);
	p.add(0, Nonterminal.UndisplayExpression);
	map.put(new Integer(111), p);

	p = new Vector(1);
	p.add(0, Nonterminal.HighlightExpression);
	map.put(new Integer(112), p);

	p = new Vector(1);
	p.add(0, Nonterminal.UnhighlightExpression);
	map.put(new Integer(113), p);

	p = new Vector(1);
	p.add(0, Nonterminal.BeginExpression);
	map.put(new Integer(114), p);

	p = new Vector(1);
	p.add(0, Nonterminal.EndExpression);
	map.put(new Integer(115), p);

	p = new Vector(1);
	p.add(0, Nonterminal.PauseExpression);
	map.put(new Integer(116), p);

	p = new Vector(1);
	p.add(0, Nonterminal.NarrateExpression);
	map.put(new Integer(117), p);

	p = new Vector(1);
	p.add(0, Nonterminal.SpeakExpression);
	map.put(new Integer(118), p);

	p = new Vector(1);
	p.add(0, Nonterminal.NavigateExpression);
	map.put(new Integer(119), p);

	p = new Vector(1);
	p.add(0, Nonterminal.TransitionExpression);
	map.put(new Integer(120), p);

	p = new Vector(1);
	p.add(0, Nonterminal.SetExpression);
	map.put(new Integer(121), p);

	p = new Vector(1);
	p.add(0, Nonterminal.WaitExpression);
	map.put(new Integer(122), p);

	p = new Vector(3);
	p.add(0, Terminal.DISPLAY);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.DISPLAY);
	map.put(new Integer(123), p);

	p = new Vector(3);
	p.add(0, Terminal.UNDISPLAY);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.UNDISPLAY);
	map.put(new Integer(124), p);

	p = new Vector(3);
	p.add(0, Terminal.HIGHLIGHT);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.HIGHLIGHT);
	map.put(new Integer(125), p);

	p = new Vector(3);
	p.add(0, Terminal.UNHIGHLIGHT);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.UNHIGHLIGHT);
	map.put(new Integer(126), p);	

	p = new Vector(3);
	p.add(0, Terminal.BEGIN);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.BEGIN);
	map.put(new Integer(127), p);

	p = new Vector(3);
	p.add(0, Terminal.END);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.END);
	map.put(new Integer(128), p);	

	p = new Vector(3);
	p.add(0, Terminal.PAUSE);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.PAUSE);
	map.put(new Integer(129), p);	

	p = new Vector(3);
	p.add(0, Terminal.NARRATE);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.NARRATE);
	map.put(new Integer(130), p);	

	p = new Vector(3);
	p.add(0, Terminal.SPEAK);
	p.add(0, Nonterminal.MediaObjectExpression);
	p.add(0, Opcode.SPEAK);
	map.put(new Integer(131), p);	

	p = new Vector(3);
	p.add(0, Terminal.GOTO);
	p.add(0, Nonterminal.Storyboard);
	p.add(0, Opcode.GOTO);
	map.put(new Integer(132), p);	
	
	p = new Vector(4);
	p.add(0, Terminal.GO);
	p.add(0, Terminal.TO);
	p.add(0, Nonterminal.Storyboard);
	p.add(0, Opcode.GOTO);
	map.put(new Integer(133), p);	

	p = new Vector(4);
	p.add(0, Terminal.JUMP);
	p.add(0, Terminal.TO);
	p.add(0, Nonterminal.Storyboard);
	p.add(0, Opcode.GOTO);
	map.put(new Integer(134), p);	
	
	p = new Vector(4);
	p.add(0, Terminal.NAVIGATE);
	p.add(0, Terminal.TO);
	p.add(0, Nonterminal.Storyboard);
	p.add(0, Opcode.GOTO);
	map.put(new Integer(135), p);

	p = new Vector(5);
	p.add(0, Terminal.TRANSITION);
	p.add(0, Nonterminal.TransitionParamExpression);
	p.add(0, Terminal.TO);
	p.add(0, Nonterminal.Storyboard);
	p.add(0, Opcode.TRANSITION);
	map.put(new Integer(136), p);

	p = new Vector(5);
	p.add(0, Terminal.LEFT_BRACKET);
	p.add(0, Nonterminal.TransitionType);
	p.add(0, Terminal.COMMA);
	p.add(0, Nonterminal.TransitionSpeed);
	p.add(0, Terminal.RIGHT_BRACKET);
	map.put(new Integer(137), p);
	
	p = new Vector(2);
	p.add(0, Terminal.DISSOLVE);
	p.add(0, Opcode.DISSOLVE);
	map.put(new Integer(138), p);

	p = new Vector(2);
	p.add(0, Terminal.WIPE);
	p.add(0, Opcode.WIPE);
	map.put(new Integer(139), p);
	
	p = new Vector(2);
	p.add(0, Terminal.SLOW);
	p.add(0, Opcode.SLOW);
	map.put(new Integer(140), p);
	
	p = new Vector(2);
	p.add(0, Terminal.MEDIUM);
	p.add(0, Opcode.MEDIUM);
	map.put(new Integer(141), p);
	
	p = new Vector(2);
	p.add(0, Terminal.FAST);
	p.add(0, Opcode.FAST);
	map.put(new Integer(142), p);

	p = new Vector(4);
	p.add(0, Terminal.SET);
	p.add(0, Nonterminal.MediaObjectOrVariable);
	p.add(0, Nonterminal.AssignExpression);
	p.add(0, Opcode.SET);
	map.put(new Integer(143), p);

	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Terminal.MEDIA_OBJECT);
	p.add(0, Opcode.MEDIA_OBJECT);
	map.put(new Integer(144), p);

	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_LOCAL);
	p.add(0, Terminal.VARIABLE);
	p.add(0, Opcode.VARIABLE);
	map.put(new Integer(145), p);

	p = new Vector(3);
	p.add(0, Terminal.TO);
	p.add(0, Terminal.REAL_NUMBER);
	p.add(0, Opcode.REAL_NUMBER);
	map.put(new Integer(146), p);
	
	p = new Vector(3);
	p.add(0, Terminal.ASSIGN);
	p.add(0, Terminal.REAL_NUMBER);
	p.add(0, Opcode.REAL_NUMBER);
	map.put(new Integer(147), p);

	p = new Vector(1);
	p.add(0, Nonterminal.WaitEventExpression);
	map.put(new Integer(148), p);

	p = new Vector(3);
	p.add(0, Terminal.WAIT);
	p.add(0, Nonterminal.SimpleTimeExpression);
	p.add(0, Opcode.WAIT);
	map.put(new Integer(149), p);	
	
	p = new Vector(3);
	p.add(0, Terminal.DELAY);
	p.add(0, Nonterminal.SimpleTimeExpression);
	p.add(0, Opcode.DELAY);
	map.put(new Integer(150), p);	
	
	p = new Vector(3);
	p.add(0, Terminal.HOLD);
	p.add(0, Nonterminal.SimpleTimeExpression);
	p.add(0, Opcode.HOLD);
	map.put(new Integer(151), p);
	
	p = new Vector(3);
	p.add(0, Opcode.SET_SCOPE_TO_APPLICATION);
	p.add(0, Terminal.STORYBOARD);
	p.add(0, Opcode.STORYBOARD);
	map.put(new Integer(152), p);

    }


    public static final Collection getProduction(Integer i) {
	return ( (Collection) map.get(i));
    }

    public static final Collection getProduction(int i) {
	return getProduction(new Integer(i));
    }


    public static final String toString(Integer i) {
	Collection c = getProduction(i);
	String production = "";

	Iterator it = c.iterator();
	while (it.hasNext()) {
	    Object obj = it.next();
	    production += obj.toString();
	    production += " ";
	}
	
	return production;
    }
    
    public static final String toString(int i) {
	return toString(new Integer(i));
    }

}
