/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: SymbolTableManager.java
*/

// Current package
package edu.uofminnesota.DEMAIS.core.parser;

// Import needed packages
import java.lang.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;




public class SymbolTableManager {

    // Scoped symbol tables
    private static SymbolTable applicationScope;
    private static SymbolTable globalScope;

    // A hash table of symbol tables 
    private static Hashtable dataModelScope;



    static {
	applicationScope = new SymbolTable(50);
	globalScope = new SymbolTable(10);
	dataModelScope = new Hashtable(50);
    }


    // Constructor
    public SymbolTableManager() {
	// empty
    }


    // Public interface methods
    public static NamedModel getFromApplicationScope(NamedModel key) {
	return applicationScope.get(key);
    }

    public static NamedModel getFromApplicationScope(String key) {
	return applicationScope.get(key);
    }

    public static NamedModel getFromApplicationScope(Integer key) {
	return applicationScope.get(key);
    }


    public static NamedModel getFromGlobalScope(NamedModel key) {
	return globalScope.get(key);
    }

    public static NamedModel getFromGlobalScope(String key) {
	return globalScope.get(key);
    }

    public static NamedModel getFromGlobalScope(Integer key) {
	return globalScope.get(key);
    }


    public static NamedModel getFromLocalScope(DataModel localScope, NamedModel key) {
	if (localScope != null) {
	    SymbolTable table = (SymbolTable) dataModelScope.get(localScope);
	    if (table != null) {
		return table.get(key);
	    }
	}
	return null;
    }

    public static NamedModel getFromLocalScope(DataModel localScope, String key) {
	if (localScope != null) {
	    SymbolTable table = (SymbolTable) dataModelScope.get(localScope);
	    if (table != null) {
		return table.get(key);
	    }
	}
	return null;
    }

    public static NamedModel getFromLocalScope(DataModel localScope, Integer key) {
	if (localScope != null) {
	    SymbolTable table = (SymbolTable) dataModelScope.get(localScope);
	    if (table != null) {
		return table.get(key);
	    }
	}
	return null;
    }
 

    public static NamedModel get(int scope, DataModel localScope, NamedModel key) {
	NamedModel obj = null;
	switch (scope) {
	case PropertyManager.APPLICATION_SCOPE:
	    obj = getFromApplicationScope(key);
	    break;
	case PropertyManager.GLOBAL_SCOPE:
	    obj = getFromGlobalScope(key);
	    if (obj == null) {
		obj = getFromApplicationScope(key);
	    }
	    break;
	case PropertyManager.LOCAL_SCOPE:
	    obj = getFromLocalScope(localScope, key);
	    if (obj == null) {
		obj = getFromGlobalScope(key);
		if (obj == null) {
		    obj = getFromApplicationScope(key);
		}
	    }
	    break;
	default:
	    break;
	}

	return obj;
    }


    public static NamedModel get(int scope, DataModel localScope, String key) {
	NamedModel obj = null;

	switch (scope) {
	case PropertyManager.APPLICATION_SCOPE:
	    obj = getFromApplicationScope(key);
	    break;
	case PropertyManager.GLOBAL_SCOPE:
	    obj = getFromGlobalScope(key);
	    if (obj == null) {
		obj = getFromApplicationScope(key);
	    }
	    break;
	case PropertyManager.LOCAL_SCOPE:
	    obj = getFromLocalScope(localScope, key);
	    if (obj == null) {
		obj = getFromGlobalScope(key);
		if (obj == null) {
		    obj = getFromApplicationScope(key);
		}
	    }
	    break;
	default:
	    break;
	}

	return obj;
    }


    public static NamedModel get(int scope, DataModel localScope, Integer key) {
	NamedModel obj = null;
	switch (scope) {
	case PropertyManager.APPLICATION_SCOPE:
	    obj = getFromApplicationScope(key);
	    break;
	case PropertyManager.GLOBAL_SCOPE:
	    obj = getFromGlobalScope(key);
	    if (obj == null) {
		obj = getFromApplicationScope(key);
	    }
	    break;
	case PropertyManager.LOCAL_SCOPE:
	    obj = getFromLocalScope(localScope, key);
	    if (obj == null) {
		obj = getFromGlobalScope(key);
		if (obj == null) {
		    obj = getFromApplicationScope(key);
		}
	    }
	    break;
	default:
	    break;
	}

	return obj;
    }



    public static void putApplicationScope(NamedModel object) {
	applicationScope.put(object);
    }

    public static void putGlobalScope(NamedModel object) {
	globalScope.put(object);
    }

    public static void putLocalScope(DataModel localScope, NamedModel object) {
	SymbolTable table = null;

	// Does the hashtable contain this local scope?
	if (dataModelScope.containsKey(localScope)) {
	    table = (SymbolTable) dataModelScope.get(localScope);

	} else {
	    // Otherwise, create a new scope
	    table = new SymbolTable(25);
	    dataModelScope.put(localScope, table);
	}

	// Now add the object
	table.put(object);
    }


    public static void put(int scope, DataModel localScope, NamedModel object) {

	switch (scope) {
	case PropertyManager.APPLICATION_SCOPE:
	    putApplicationScope(object);
	    break;
	case PropertyManager.GLOBAL_SCOPE:
	    putGlobalScope(object);
	    break;
	case PropertyManager.LOCAL_SCOPE:
	    putLocalScope(localScope, object);
	    break;
	default:
	    DEMAISError.showErrorMessage("Put(): Unable to put object: " + Integer.toString(scope), "SymbolTableManager", false);
	    break;
	}
    }



    public static void remove(int scope, DataModel localScope, NamedModel key) {
	NamedModel no = null;
	switch(scope) {
	case PropertyManager.APPLICATION_SCOPE: 
	    applicationScope.remove(key);
	    break;
	case PropertyManager.GLOBAL_SCOPE:
	    if (globalScope.remove(key) == null) {
		applicationScope.remove(key);
	    }
	    break;
	case PropertyManager.LOCAL_SCOPE:
	    SymbolTable local = (SymbolTable) dataModelScope.get(localScope);
	    if (local != null) {
		if (local.remove(key) == null) {
		    if (globalScope.remove(key) == null) {
			applicationScope.remove(key);
		    }
		}

	    } else {
		// Local == null
		if (globalScope.remove(key) == null) {
		    applicationScope.remove(key);
		}
	    }
	    break;
	default:
	    break;
	}
    }


    public static NamedModel resolveIndirectReference(int scope, DataModel localScope, Opcode code) {
	NamedModel object = null;
	
	switch (scope) {
	case PropertyManager.APPLICATION_SCOPE:
	    // Search the application scope
	    object = applicationScope.resolveIndirectReference(code);
	    break;

	case PropertyManager.GLOBAL_SCOPE:
	    // Search the global scope
	    object = globalScope.resolveIndirectReference(code);
	    if (object == null) {
		// Search the application scope
		object = applicationScope.resolveIndirectReference(code);
	    }
	    break;

	case PropertyManager.LOCAL_SCOPE:
	    SymbolTable table = (SymbolTable) dataModelScope.get(localScope);
	    if (table != null) {
		object = (NamedModel) table.resolveIndirectReference(code);
		if (object == null) {
		    // Search the global scope
		    object = globalScope.resolveIndirectReference(code);
		    if (object == null) {
			// Search the application scope
			object = applicationScope.resolveIndirectReference(code);
		    }
		}
	    } else {
		// table == null
		// Search the global scope
		object = globalScope.resolveIndirectReference(code);
		if (object == null) {
		    // Search the application scope
		    object = applicationScope.resolveIndirectReference(code);
		}
	    }
	    break;

	default:
	    break;
	}

	return object;
    }
}
