/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: Terminal.java
*/

// Current package
package edu.uofminnesota.DEMAIS.core.parser;

// Import needed packages
import java.lang.*;
import java.util.*;

/*
  For the terminal definitions, see the file Grammar.doc in this directory
*/


public class Terminal {

    // The terminal symbols of the annotation language
    public static final Terminal A;
    public static final Terminal AFTER;
    public static final Terminal AND;
    public static final Terminal ANIMATION;
    public static final Terminal ASSIGN;
    public static final Terminal AUDIO;
    public static final Terminal BEGIN;
    public static final Terminal BEGINS;
    public static final Terminal BEGUN;
    public static final Terminal CLICKS;
    public static final Terminal COMMA;
    public static final Terminal DELAY;
    public static final Terminal DISPLAY;
    public static final Terminal DISSOLVE;
    public static final Terminal DIVIDE;
    public static final Terminal DOUBLECLICKS;
    public static final Terminal DRAGS;
    public static final Terminal END;
    public static final Terminal ENDED;
    public static final Terminal ENDS;
    public static final Terminal ENTERS;
    public static final Terminal EQUAL;
    public static final Terminal FAST;
    public static final Terminal GO;
    public static final Terminal GOTO;
    public static final Terminal GREATERTHAN;
    public static final Terminal GREATERTHAN_EQUAL;
    public static final Terminal HAS;
    public static final Terminal HIGHLIGHT;
    public static final Terminal HOLD;
    public static final Terminal INTO;
    public static final Terminal JUMP;
    public static final Terminal LEFT_BRACKET;
    public static final Terminal LEFT_PAREN;
    public static final Terminal LESSTHAN;
    public static final Terminal LESSTHAN_EQUAL;
    public static final Terminal MEDIA_OBJECT;
    public static final Terminal MEDIUM;
    public static final Terminal MINUS;
    public static final Terminal MULTIPLY;
    public static final Terminal NARRATE;
    public static final Terminal NARRATION;
    public static final Terminal NAVIGATE;
    public static final Terminal NOT;
    public static final Terminal NOT_EQUAL;
    public static final Terminal ON;
    public static final Terminal ONCE;
    public static final Terminal OR;
    public static final Terminal OVER;
    public static final Terminal PAUSE;
    public static final Terminal PLAY;
    public static final Terminal PLUS;
    public static final Terminal PRESSES;
    public static final Terminal RESTART;
    public static final Terminal RIGHT_PAREN;
    public static final Terminal REAL_NUMBER;
    public static final Terminal REACHES;
    public static final Terminal RIGHT_BRACKET;
    public static final Terminal ROLLS;
    public static final Terminal SECOND;
    public static final Terminal SECONDS;
    public static final Terminal SELECTS;
    public static final Terminal SLOW;
    public static final Terminal SET;
    public static final Terminal SPEAK;
    public static final Terminal STORYBOARD;
    public static final Terminal TEXT;
    public static final Terminal THE;
    public static final Terminal TO;
    public static final Terminal TRANSITION;
    public static final Terminal UNDISPLAY;
    public static final Terminal UNHIGHLIGHT;
    public static final Terminal USER;
    public static final Terminal VARIABLE;
    public static final Terminal VIDEO;
    public static final Terminal WAIT;
    public static final Terminal WHEN;
    public static final Terminal WIPE;
    public static final Terminal EPSILON;
    public static final Terminal EOF;
    public static final Terminal UNKNOWN;

    // Number of terminals in the language
    private static final int numTerminals;

    // A hash table mapping a terminal string to its corresponding Terminal object
    private static Hashtable map;



    // Static initializer
    static {

	A = new Terminal("a", 0);
	AFTER = new Terminal("after", 1);
	AND = new Terminal("and", 2);
	ANIMATION = new Terminal("animation", 3);
	ASSIGN = new Terminal("assign", 4);
	AUDIO = new Terminal("audio", 5);
	BEGIN = new Terminal("begin", 6);
	BEGINS = new Terminal("begins", 7);
	BEGUN = new Terminal("begun", 8);
	CLICKS = new Terminal("clicks", 9);
	COMMA = new Terminal(",", 10);
	DELAY = new Terminal("delay", 11);
	DISPLAY = new Terminal("display", 12);
	DISSOLVE = new Terminal("dissolve", 13);
	DIVIDE = new Terminal("divide", 14);
	DOUBLECLICKS = new Terminal("doubleclicks", 15);
	DRAGS = new Terminal("drags", 16);
	END = new Terminal("end", 17);
	ENDED = new Terminal("ended", 17);
	ENDS = new Terminal("ends", 19);
	ENTERS = new Terminal("enters", 20);
	EQUAL = new Terminal("=", 21);
	FAST = new Terminal("fast", 22);
	GO = new Terminal("go", 23);
	GOTO = new Terminal("goto", 24);
	GREATERTHAN = new Terminal(">", 25);
	GREATERTHAN_EQUAL = new Terminal(">=", 26);
	HAS = new Terminal("has", 27);
	HIGHLIGHT = new Terminal("highlight", 28);
	HOLD = new Terminal("hold", 29);
	INTO = new Terminal("into", 30);
	JUMP = new Terminal("jump", 31);
	LEFT_BRACKET = new Terminal("[", 32);
	LEFT_PAREN = new Terminal("(", 33);
	LESSTHAN = new Terminal("<", 34);
	LESSTHAN_EQUAL = new Terminal("<=", 35);
	MEDIA_OBJECT = new Terminal("media_object", 36);
	MEDIUM = new Terminal("medium", 37);
	MINUS = new Terminal("-", 38);
	MULTIPLY = new Terminal("*", 39);
	NARRATE = new Terminal("narrate", 40);
	NARRATION = new Terminal("narration", 41);
	NAVIGATE = new Terminal("navigate", 42);
	NOT = new Terminal("!", 43);
	NOT_EQUAL = new Terminal("!=", 44);
	ON = new Terminal("on", 45);
	ONCE = new Terminal("once", 46);
	OR = new Terminal("or", 47);
	OVER = new Terminal("over", 48);
	PAUSE = new Terminal("pause", 49);
	PLAY = new Terminal("play", 50);
	PLUS = new Terminal("+", 51);
	PRESSES = new Terminal("presses", 52);
	RIGHT_PAREN = new Terminal(")", 53);
	REAL_NUMBER = new Terminal("real_number", 54);
	REACHES = new Terminal("reaches", 55);
	RESTART = new Terminal("restart", 56);
	RIGHT_BRACKET = new Terminal("]", 57);
	ROLLS = new Terminal("rolls", 58);
	SECOND = new Terminal("second", 59);
	SECONDS = new Terminal("seconds", 60);
	SELECTS = new Terminal("selects", 61);
	SET = new Terminal("set", 62);
	SLOW = new Terminal("slow", 63);
	SPEAK = new Terminal("speak", 64);
	STORYBOARD = new Terminal("storyboard", 65);
	TEXT = new Terminal("text", 66);
	THE = new Terminal("the", 67);
	TO = new Terminal("to", 68);
	TRANSITION = new Terminal("transition", 69);
	UNDISPLAY = new Terminal("undisplay", 70);
	UNHIGHLIGHT = new Terminal("unhighlight", 71);
	USER = new Terminal("user", 72);
	VARIABLE = new Terminal("variable", 73);
	VIDEO = new Terminal("video", 74);
	WAIT = new Terminal("wait", 75);
	WHEN = new Terminal("when", 76);
	WIPE = new Terminal("wipe", 77);
	EPSILON = new Terminal("epsilon", 78);
	EOF = new Terminal("eof", 79);
	UNKNOWN = new Terminal("unknown", 80);

	// Set total number of terminals
	numTerminals = 81;

	// build the hash table
	map = new Hashtable(100);

	map.put("A", A);
	map.put("AFTER", AFTER);
	map.put("AND", AND);
	map.put("&&", AND);
	map.put("ANIMATION", ANIMATION);
	map.put("=", ASSIGN);
	map.put("AUDIO", AUDIO);
	map.put("MUSIC", AUDIO);
	map.put("BEGIN", BEGIN);
	map.put("START", BEGIN);
	map.put("PLAY", BEGIN);
	map.put("RESTART", BEGIN);
	map.put("BEGINS", BEGINS);
	map.put("STARTS", BEGINS);
	map.put("BEGUN", BEGUN);
	map.put("STARTED", BEGUN);
	map.put("CLICKS", CLICKS);
	map.put(",", COMMA);
	map.put("DELAY", DELAY);
	map.put("DISPLAY", DISPLAY);
	map.put("SHOW", DISPLAY);
	map.put("DISSOLVE", DISSOLVE);
	map.put("/", DIVIDE);
	map.put("DOUBLECLICKS", DOUBLECLICKS);
	map.put("DOUBLE_CLICKS", DOUBLECLICKS);
	map.put("DRAGS", DRAGS);
	map.put("END", END);
	map.put("STOP", END);
	map.put("FINISH", END);
	map.put("ENDED", ENDED);
	map.put("FINISHED", ENDED);
	map.put("STOPPED", ENDED);
	map.put("ENDS", ENDS);
	map.put("STOPS", ENDS);
	map.put("FINISHES", ENDS);
	map.put("ENTERS", ENTERS);
	map.put("==", EQUAL);
	map.put("FAST", FAST);
	map.put("GO", GO);
	map.put("GOTO", GOTO);
	map.put(">", GREATERTHAN);
	map.put(">=", GREATERTHAN_EQUAL);
	map.put("HAS", HAS);
	map.put("HIGHLIGHT", HIGHLIGHT);
	map.put("HOLD", HOLD);
	map.put("INTO", INTO);
	map.put("JUMP", JUMP);
	map.put("[", LEFT_BRACKET);
	map.put("(", LEFT_PAREN);
	map.put("<", LESSTHAN);
	map.put("<=", LESSTHAN_EQUAL);
	map.put("-", MINUS);
	map.put("*", MULTIPLY);
	map.put("MEDIUM", MEDIUM);
	map.put("NARRATE", NARRATE);
	map.put("NARRATION", NARRATION);
	map.put("NAVIGATE", NAVIGATE);
	map.put("NOT", NOT);
	map.put("!", NOT);
	map.put("!=", NOT_EQUAL);
	map.put("<>", NOT_EQUAL);
	map.put("ON", ON);
	map.put("ONCE", ONCE);
	map.put("OR", OR);
	map.put("||", OR);
	map.put("OVER", OVER);
	map.put("PAUSE", PAUSE);
	map.put("+", PLUS);
	map.put("PRESSES", PRESSES);
	map.put(")", RIGHT_PAREN);
	map.put("REACHES", REACHES);
	map.put("]", RIGHT_BRACKET);
	map.put("ROLLS", ROLLS);
	map.put("SECOND", SECOND);
	map.put("SECONDS", SECONDS);
	map.put("SELECTS", SELECTS);
	map.put("SET", SET);
	map.put("SLOW", SLOW);
	map.put("SPEAK", SPEAK);
	map.put("TEXT", TEXT);
	map.put("THE", THE);
	map.put("TO", TO);
	map.put("TRANSITION", TRANSITION);
	map.put("UNDISPLAY", UNDISPLAY);
	map.put("UNSHOW", UNDISPLAY);
	map.put("UNHIGHLIGHT", UNHIGHLIGHT);
	map.put("USER", USER);
	map.put("VIDEO", VIDEO);
	map.put("WAIT", WAIT);
	map.put("WHEN", WHEN);
	map.put("WIPE", WIPE);
	map.put("EPSILON", EPSILON);
	map.put("EOF", EOF);
	map.put("UNKNOWN", UNKNOWN);

    }


    // Returns null if terminal string is not in the lookup table
    // However, the string may still represent a REAL_NUMBER, VARIABLE,
    // MEDIA_OBJECT, or STORYBOARD token
    public static Terminal toTerminal(String terminalS) {
	String upperS = terminalS.toUpperCase();
	return ( (Terminal) map.get(upperS));
    }

    
    public static int numTerminals() {
	return numTerminals;
    }




    // Instance variables

    // The string name of the terminal symbol
    private String name;
    
    // The integer value of the terminal symbol
    private Integer value;



    // Contructors
    private Terminal() {
	this(null, 0);
    }

    public Terminal(String n, int v) {
	this(n, new Integer(v));
    }

    public Terminal(String n, Integer v) {
	name = n;
	value = v;
    }



    // Public methods
    public int intValue() {
	return value.intValue();
    }

    public Integer toInteger() {
	return value;
    }
    
    public String toString() {
	return name;
    }

    public int hashCode() {
	return name.hashCode();
    }

    public boolean equals(Object obj) {
	if (obj instanceof Terminal) {
	    return (equals( (Terminal) obj));
	
	} else if (obj instanceof Integer) {
	    return (equals( (Integer) obj));
	}
	
	return (this == obj);
    }


    public boolean equals(Terminal t) {
	return (value.equals(t.value));
    }

    public boolean equals(Integer i) {
	return (value.equals(i));
    }

}











