/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: Token.java
*/

// Current package
package edu.uofminnesota.DEMAIS.core.parser;

// Import needed packages
import java.lang.*;
import edu.uofminnesota.DEMAIS.meal.*;



public class Token {

    // The terminal value
    private Terminal terminal;

    // The string-based lexeme of the token
    private String lexeme;

    // Used for REAL_NUMBER
    private double value;

    // USED for VARIABLE, MEDIA_OBJECT, and STORYBOARD
    private NamedModel object;

    // The data model (scope) to which this object belongs
    private DataModel scope;

    // Word count (from the left) within the annotation
    private int wordCount;



    // Constructors
    public Token() {
	this(Terminal.UNKNOWN, null, 0.0, null, null, 0);
    }

    public Token(Terminal t) {
	this(t, null, 0.0, null, null, 0);
    }

    public Token(Terminal t, String l) {
	this(t, l, 0.0, null, null, 0);
    }

    public Token(Terminal t, String l, double v) {
	this(t, l, v, null, null, 0);
    }

    public Token(Terminal t, String l, double v, NamedModel no) {
	this(t, l, v, no, null, 0);
    }

    public Token(Terminal t, String l, double v, NamedModel no, DataModel model, int wc) {
	terminal = t;
	lexeme = l;
	value = v;
	object = no;
	scope = model;
	wordCount = wc;
    }


    // Public methods
    public void setTerminal(Terminal t) {
	terminal = t;
    }

    public Terminal getTerminal() {
	return terminal;
    }

    public void setLexeme(String l) {
	lexeme = l;
    }

    public String getLexeme() {
	return lexeme;
    }

    public void setValue(int i) {
	setValue((double) i);
    }

    public void setValue(double d) {
	value = d;
    }

    public double getValue() {
	return value;
    }

    public void setObject(NamedModel no) {
	object = no;
    }

    public NamedModel getObject() {
	return object;
    }

    public DataModel getScope() {
	return scope;
    }

    public void setScope(DataModel s) {
	scope = s;
    }

    public void setWordCount(int wc) {
	wordCount = wc;
    }

    public int getWordCount() {
	return wordCount;
    }

    public boolean equals(Token token) {
	return (terminal.equals(token.terminal));
    }

    public boolean equals(Terminal t) {
	return (terminal.equals(t));
    }

    public String toString() {
	return terminal.toString();
    }

}
