/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: DEMAISReader.java
*/

package edu.uofminnesota.DEMAIS.fileio;


// Import necessary packages
import java.lang.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;





public class DEMAISReader {

    private DEMAISReader() {
    }



    // Must be given full path name to the file
    public static void readDEMAISFile(String filePathName) throws IOException {
	File file = new File(filePathName);
	readDEMAISFile(file);
    }


    public static void readDEMAISFile(File demaisFile) throws IOException {
	// Check that the file exists and that it can be read from
	if ( (! demaisFile.exists()) || (! demaisFile.canRead()) ) {
	    throw (new IOException("Cannot open selected file for input"));
	}

	// Read from the file
	DataInputStream stream = null;
	try {
	    stream = new DataInputStream(new FileInputStream(demaisFile));
	    doRead(stream);
	    stream.close();
	
	} catch (IOException xcptn) {
	    throw xcptn;
	}
    }



    private static void doRead(DataInputStream stream) throws IOException {
	// Read cookie header
	int header = stream.readInt();
	if (header != 1061971) {
	    // Not a DEMAIS file
	    throw (new IOException("File is not a DEMAIS design file"));
	}

	// Read the total number of models to follow
	int numModels = stream.readInt();
	//System.out.println("Total models: " + Integer.toString(numModels));

	// Read the total number of Storyboard models to follow
	int numSModels = stream.readInt();
	//System.out.println("Total storyboard models: " + Integer.toString(numSModels));
	for (int s=0; s<numSModels; ++s) {
	    // Read a Storyboard
	    StoryboardModel sModel = new StoryboardModel(stream);
	}


	// Read the total number of Narration models to follow
	int numNModels = stream.readInt();
	for (int n=0; n<numNModels; ++n) {
	    // Read a NarrationModel
	    NarrationModel nModel = new NarrationModel(stream);
	}

	// Read the total number of MultiView models to follow
	int numMVModels = stream.readInt();
	for (int mv=0; mv<numMVModels; ++mv) {
	    // Read a MultiviewModel
	    MultiViewModel mvModel = new MultiViewModel(stream);
	}


	// finish the IO
	Collection collection = StoryboardModel.getStoryboardModels();
	Iterator it = collection.iterator();
	while (it.hasNext()) {
	    StoryboardModel sModel = (StoryboardModel) it.next();
	    sModel.completeIO();
	}

	// finish the IO
	collection = MultiViewModel.getMultiViewModels();
	it = collection.iterator();
	while (it.hasNext()) {
	    MultiViewModel mvModel = (MultiViewModel) it.next();
	    mvModel.completeIO();
	}


	// Check that all the models were read
	if (numModels != (numSModels + numNModels + numMVModels)) {
	    throw (new IOException("Unexpected number of models"));
	}

	// Set the current model
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, 
				    StoryboardModel.getFirstStoryboardModel());
	
    }
    
}



