/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
  File: DEMAISWriter.java
*/
package edu.uofminnesota.DEMAIS.fileio;


// Import necessary packages
import java.lang.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;




public class DEMAISWriter {

    private DEMAISWriter() {
    }
   

    // Must be given full path name to the file
    public static void writeDEMAISFile(String filePathName) throws IOException {
	File file = new File(filePathName);
	writeDEMAISFile(file);
    }


    public static void writeDEMAISFile(File demaisFile) throws IOException {
	// Create the data output stream
	DataOutputStream stream = null;
	try {
	    if (! demaisFile.exists()) {
		demaisFile.createNewFile();
	    }
	    stream = new DataOutputStream(new FileOutputStream(demaisFile));
	    doWrite(stream);
	    stream.flush();
	    stream.close();
	    
	} catch (IOException xcptn) {
	    try {
		// Try and close the file
		if (stream != null) {
		    stream.flush();
		    stream.close();
		}
	    } catch(IOException xcptn2) {
	    }
	    
	    // Re-throw the exception
	    throw xcptn;
	}

    }



    private static void doWrite(DataOutputStream stream) throws IOException {
	// Write cookie header
	stream.writeInt(1061971);

	// Obtain collections of the three primary models
	Collection sModels = StoryboardModel.getStoryboardModels();
	Collection nModels = NarrationModel.getNarrationModels();
	Collection mvModels = MultiViewModel.getMultiViewModels();

	// Write the total number of models to follow
	stream.writeInt(sModels.size() + nModels.size() + mvModels.size());

	// Write the total number of Storyboard models to follow
	stream.writeInt(sModels.size());
	// Write the storyboards
	Iterator it = sModels.iterator();
	while (it.hasNext()) {
	    StoryboardModel sModel = (StoryboardModel) it.next();
	    sModel.write(stream);
	}

	// Write the total number of Narration models to follow
	stream.writeInt(nModels.size());
	// Write the narrations
	it = nModels.iterator();
	while (it.hasNext()) {
	    NarrationModel nModel = (NarrationModel) it.next();
	    nModel.write(stream);
	}

	// Write the total number of MultiViewModels
	stream.writeInt(mvModels.size());
	// Write the multiviews
	it = mvModels.iterator();
	while (it.hasNext()) {
	    MultiViewModel mvModel = (MultiViewModel) it.next();
	    mvModel.write(stream);
	}

	// Done
    }


}

