/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: AnimationModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.util.*;
import java.io.*;




public class AnimationModel extends ContinuousMediaModel {
    
    /*
      A list of objects to notify when the animation reaches specified locations 
    */
    private Vector locationListeners;



    // Constructors
    private AnimationModel() {
	this(0, null, null, null);
    }
    
    public AnimationModel(int objScope, DataModel localScope, String modelName, String mediaFileName) {
	super(objScope, localScope, modelName, mediaFileName);
	locationListeners = new Vector();
    }
    
    
    public AnimationModel(AnimationModel copy) {
	super(copy);
	locationListeners = new Vector();
    }


    public AnimationModel(DataInputStream stream, DataModel localScope) throws IOException {
	super(stream, localScope);
    }




    // Public methods
    public void addLocationListener(AnimationModelListenerInterface obj, int x, int y) {
	AnimLocationListener l = new AnimLocationListener(this, obj, x, y);
	if (! locationListeners.contains(l)) {
	    locationListeners.add(l);
	}
    }


    public void removeLocationListener(AnimationModelListenerInterface obj, int x, int y) {
	Object object = null;
	Vector toRemove = new Vector();

	for (Enumeration e = locationListeners.elements(); e.hasMoreElements(); ) {
	    AnimLocationListener all = (AnimLocationListener) e.nextElement();
	    if ( (obj == all.getListener()) &&
		  (x == all.getX()) &&
		  (y == all.getY())) {
		toRemove.add(all);
		break;
		
	    }
	}
	if (toRemove.size() > 0) {
	    locationListeners.removeAll(toRemove);
	}
    }

    public void removeLocationListeners() {
	locationListeners.clear();
    }


    public void removeAnimationModelListeners() {
	removeLocationListeners();
    }

    public void removeListeners() {
	super.removeListeners();
	removeAnimationModelListeners();
    }

    public void unOperationalize() {
	super.unOperationalize();
	removeAnimationModelListeners();
    }

    public void deleteModel() {
	super.deleteModel();
	removeAnimationModelListeners();
    }


}


