/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: AudioModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;





public class AudioModel extends JMFContinuousMediaModel {

    // Maps a base file name to an already existing AudioModel
    private static Hashtable nameToAudioModel;


    // The storyboard in which the audio controller will be displayed
    private AudioRectangleInkStroke aris;



    // Static initializer
    static {
	nameToAudioModel = new Hashtable(20);
	
	// Load the audio files at startup
	try {
	    File audioDirectory = new File(PropertyManager.APPLICATION_AUDIO_DIRECTORY);
	    // Load audio files from the directory, if it exists
	    if ( (audioDirectory.exists()) && (audioDirectory.isDirectory()) ) {
		loadAudioModelsFromDirectory(audioDirectory);
	    }
	    
	} catch (Exception xcptn) {
	    DEMAISError.showErrorMessage(xcptn.toString(), "AudioModel", false);
	}
    }
    
    
    private static void loadAudioModelsFromDirectory(File directory) { 
	try {
	    // Open the audio directory
	    File fileList[] = directory.listFiles();
	    for (int i=0; i<fileList.length; ++i) {
		String filePathName = fileList[i].getAbsolutePath();
		String objName = filePathName;
		int index = filePathName.lastIndexOf(PropertyManager.FILE_SEPARATOR);
		if (index >= 0) {
		    objName = filePathName.substring(index + 1);
		}
		
		index = objName.lastIndexOf(".");
		if (index >= 0) {
		    objName = objName.substring(0, index);
		}
		
		objName = objName.toUpperCase();
		AudioModel audioModel = new AudioModel(PropertyManager.GLOBAL_SCOPE, null, objName, filePathName, null);
		nameToAudioModel.put(objName, audioModel);
		if (objName.equals("EXCLAIM")) {
		    // Initializes the JMF!
		    audioModel.realizePlayer();
		}
	    }
	    
	} catch (Exception xcptn) {
	    DEMAISError.showErrorMessage(xcptn.toString(), "AudioModel", false);
	}
    }


    public static AudioModel getAudioModel(String name) {
	// Strip off unwanted parts of the name
	int index = name.lastIndexOf(PropertyManager.FILE_SEPARATOR);
	if (index >= 0) {
	    name = name.substring(index + 1);
	}
	
	index = name.lastIndexOf(".");
	if (index >= 0) {
	    name = name.substring(0, index);
	}
	
	return (AudioModel) nameToAudioModel.get(name.toUpperCase());
    }


    public static Vector getAudioModelNames() {
	Vector names = new Vector();
	for (Enumeration e = nameToAudioModel.keys(); e.hasMoreElements();) {
	    names.add(e.nextElement());
	}
	return names;
    }


    // Reset all audio models
    public static void resetModels() {
	for (Enumeration e = nameToAudioModel.elements(); e.hasMoreElements();) {
	    AudioModel model = (AudioModel) e.nextElement();
	    model.reset();
	}
    }
    

    // Used to reference this object so that it gets loaded!
    public static void touch() {
    }





    // Constructors
    private AudioModel() {
	this(0, null, null, null, null);
    }
    
    public AudioModel(int objScope, DataModel localScope, String modelName, String mediaFileName, AudioRectangleInkStroke aris) {
	super(objScope, localScope, modelName, mediaFileName);
	this.aris = aris;
    }

    
    public AudioModel(AudioModel copy, AudioRectangleInkStroke aris) {
	this(copy, aris, aris.getStoryboard().getModel());
    }
    
    public AudioModel(AudioModel copy, AudioRectangleInkStroke aris, DataModel toModel) {
	super(copy, toModel);
	this.aris = aris;
    }


    public AudioModel(DataInputStream stream, AudioRectangleInkStroke aris, DataModel localScope) throws IOException {
	super(stream, localScope);
	this.aris = aris;
    }


    // Public methods
    public void begin() {
	super.begin();
	if (aris != null) {
	    aris.begin();
	}
    }

    public void display() {
	if (aris != null) {
	    aris.display();
	}
    }

    public void unDisplay() {
	if (aris != null) {
	    aris.unDisplay();
	}
    }

    public void highlight() {
	if (aris != null) {
	    aris.highlight();
	}
    }

    public void unHighlight() {
	if (aris != null) {
	    aris.unHighlight();
	}
    }


    public Object clone(AudioRectangleInkStroke aris) {
	return (new AudioModel(this, aris));
    }

}

