/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContinuousMediaModelInterface.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;



public abstract interface ContinuousMediaModelInterface {
   
    // The different states for a continuous media model
    public static final int RESET = 0;
    public static final int INITIAL = 1;
    public static final int BEGINNING = 2;
    public static final int BEGUN = 3;
    public static final int PAUSING = 4;
    public static final int PAUSED = 5;
    public static final int ENDING = 6;
    public static final int ENDED = 7;
    public static final float GAIN = (float) 0.8;


    // Used by the runtime system
    public abstract void reset();
    public abstract void begin();
    public abstract void end();
    public abstract void pause();
    public abstract void resume();
    public abstract void setMediaTime(double value);
    public abstract double getMediaTime();
    public abstract void setRate(double value);
    public abstract double getRate();
    public abstract void mediaBeginEvent();
    public abstract boolean wasBeginEvent();
    public abstract void setWasBeginEvent(boolean b, boolean notify);
    public abstract void mediaEndEvent();
    public abstract boolean wasEndEvent();
    public abstract void setWasEndEvent(boolean b, boolean notify);

    public abstract void addBeginListener(ContinuousMediaModelListenerInterface cli);
    public abstract void addEndListener(ContinuousMediaModelListenerInterface cli);


}
