/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: DataModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.util.*;
import java.io.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.layer.*;
import edu.uofminnesota.DEMAIS.core.annotation.*;
import edu.uofminnesota.DEMAIS.core.parser.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;




public abstract class DataModel extends NamedModel {

    // The set of annotations associated with this data model
    private Vector annotation_list;

    // A list of open view on this model
    protected Vector openModelViews;



    // Constructors
    private DataModel() {
	this(0, null, null);
    }

    public DataModel(int scope, DataModel localScope, String modelName) {
	super(scope, localScope, modelName);
	annotation_list = new Vector();
	openModelViews = new Vector();
    }


    public DataModel(DataModel copy) {
	this(copy, true);
    }

    public DataModel(DataModel copy, boolean copyOfModels) {
	super(copy);

	// Copy the annotation list
	annotation_list = null;
	if (copy.annotation_list != null) {
	    annotation_list = new Vector(copy.annotation_list.size()); 
	    Iterator it = copy.annotation_list.iterator();
	    while (it.hasNext()) {
		Annotation annotation = (Annotation) it.next();
		annotation_list.add(annotation.clone());
	    }
	} else {
	    annotation_list = new Vector();
	}

	// Do not copy the views vector, but do create it
	openModelViews = new Vector();
    }


    public DataModel(DataInputStream stream) throws IOException {
	super(stream, null);

	annotation_list = new Vector();
	openModelViews = new Vector();
    }

    public void completeIO() {
    }



    public void addAnnotation(Annotation a) {
	if (! annotation_list.contains(a)) {
	    annotation_list.add(a);
	}
    }

    public void removeAnnotation(Annotation a) {
	annotation_list.remove(a);
    }

    public void removeAnnotations() {
	annotation_list.clear();
    }

    public Vector getAnnotations() {
	return annotation_list;
    }

    public int numAnnotations() {
	return annotation_list.size();
    }


    public void operationalize() {
    }


    public void parseAnnotations() {
	// Perform an initial parse of all the annotations
	Vector toRemove = new Vector();
	
	Iterator it = annotation_list.iterator();
	while (it.hasNext()) {
	    Annotation annotation = (Annotation) it.next();
	    try {
		// Parse the annotation - which sets the condition and action stacks
		Parser.parse(annotation);
		
	    } catch (ParseException xcptn) {
		// Annotation could not be parsed
		System.out.println(xcptn.toString());
		toRemove.add(annotation);
	    }
	}
	// Keep only those annotations that parsed correctly
	annotation_list.removeAll(toRemove);
    }


    public void unOperationalize() {
	annotation_list.removeAllElements();
    }

 
    public void addView(DataModelViewInterface dmvi) {
	if (! openModelViews.contains(dmvi)) {
	    openModelViews.add(dmvi);
	}
    }

    public void removeView(DataModelViewInterface dmvi) {
	openModelViews.remove(dmvi);
    }


    // Delete the model from the system - close out all views on this model
    public void deleteModel() {
	// Destroy the parent model
	super.deleteModel();

	// Remove the annotations
	annotation_list.clear();

	// Remove and close out the views
	Iterator it = openModelViews.iterator();
	while (it.hasNext()) {
	    DataModelViewInterface dmvi = (DataModelViewInterface) it.next();
	    dmvi.deleteView(this, false);
	}
	openModelViews.clear();

	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.TRUE);
    }


    public void refreshViews() {
	Iterator it = openModelViews.iterator();
	while (it.hasNext()) {
	    DataModelViewInterface dmvi = (DataModelViewInterface) it.next();
	    dmvi.refreshView(this);
	}
    }


    public void realize(boolean showControls, boolean recreate) {
    }
    public void unRealize(boolean keepVisuals) {
    }


    // Layer interface
    public boolean isLayerVisible(int layer) {
	return false;
    }
    public boolean isLayerSelected(int layer) {
	return false;
    }
    public int getSelectedLayer() {
	return 0;
    }
    public int getNumLayers() {
	return 1;
    }
    public LayerModel getSelectedLayerModel() {
	return null;
    }
    public String getLayerName(int layer) {
	return null;
    }
    public LayerModel getLayerModel(int layer) {
	return null;
    }
    public void setLayerName(int layer, String name) {
    }
    public void setLayerVisible(int layer, boolean visible) {
    }
    public void setLayerSelected(int layer, boolean selected) {
    }
    public void newLayer(LayerModel lm) {
    }
    public void copyLayer(int layerNum, String newLayerName, int newLayerNum, boolean visible, boolean selected) {
    }
    public void deleteLayer(int layerNum, int newSelectedLayer) {
    }
    public void repositionLayer(int layerNum, int toPosition) {
    }
    public void flattenLayers() {
    }
    public LayerList getLayers() {
	return null;
    }

    public abstract Storyboard getStoryboard();
    public abstract Object clone();
    public abstract Object clone(boolean copyOfModels);
    public abstract ContentManagerCellView getContentManagerCellView();


}
