/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ImageModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.io.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.core.parser.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;




public class ImageModel extends StaticMediaModel {

    // Reference to the containing ink stroke
    private ImageRectangleInkStroke iris;



    // Constructors
    private ImageModel() {
	this(0, null, null, null, null);
    }

    public ImageModel(int objScope, DataModel localScope, String modelName, String mediaFileName, ImageRectangleInkStroke iris) {
	super(objScope, localScope, modelName, mediaFileName);
	this.iris = iris;
    }


    public ImageModel(ImageModel copy, ImageRectangleInkStroke iris) {
	this(copy, iris, iris.getStoryboard().getModel());
    }

    public ImageModel(ImageModel copy, ImageRectangleInkStroke iris, DataModel toModel) {
	super(copy, toModel);
	this.iris = iris;
    }


    public ImageModel(DataInputStream stream, ImageRectangleInkStroke iris, DataModel localScope) throws IOException {
	super(stream, localScope);
	this.iris = iris;
    }



    // Public methods
    public void display() {
	iris.display();
    }

    public void unDisplay() {
	iris.unDisplay();
    }

    public void highlight() {
	iris.highlight();
    }

    public void unHighlight() {
	iris.unHighlight();
    }


    public Object clone(ImageRectangleInkStroke stroke) {
	return (new ImageModel(this, stroke));
    }

}
