/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: StaticMediaModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.util.*;
import java.io.*;
import java.awt.*;



public abstract class StaticMediaModel extends MediaModel 
    implements StaticMediaModelInterface {


    // status variables
    protected boolean wasDraggedAndDropped;
    protected MediaModel droppedOnObject;

    // Event listeners
    private Vector dragAndDropListeners;




    // Constructors
    private StaticMediaModel() {
	this(0, null, null, null);
    }

    public StaticMediaModel(int objScope, DataModel localScope, String modelName, String mediaFileName) {
	super(objScope, localScope, modelName, mediaFileName);
	dragAndDropListeners = new Vector();
	droppedOnObject = null;
    }


    public StaticMediaModel(StaticMediaModel copy) {
	this(copy, copy.localScope);
    }

    public StaticMediaModel(StaticMediaModel copy, DataModel toModel) {
	super(copy, toModel);
	dragAndDropListeners = new Vector();
	droppedOnObject = null;
    }


    public StaticMediaModel(DataInputStream stream, DataModel localScope) throws IOException {
	super(stream, localScope);
	dragAndDropListeners = new Vector();
	droppedOnObject = null;
    }




    public boolean wasDraggedAndDropped() {
	return wasDraggedAndDropped;
    }

    public boolean wasDraggedAndDroppedOn(MediaModel target) {
	return ( (wasDraggedAndDropped) && (droppedOnObject != null) &&
		 (droppedOnObject.equals(target)));
    }
    
    public void setWasDraggedAndDroppedOn(boolean b, MediaModel onObject, boolean notify) {
	wasDraggedAndDropped = b;
	droppedOnObject = onObject;
	if (notify) {
	    notifyDragAndDropListeners();
	}
    }



    public void addDragAndDropListener(StaticMediaModelListenerInterface dli) {
	if (! dragAndDropListeners.contains(dli)) {
	    dragAndDropListeners.add(dli);
	}
    }

    public void removeDragAndDropListener(StaticMediaModelListenerInterface dli) {
	dragAndDropListeners.remove(dli);
    }

    public void removeDragAndDropListeners() {
	dragAndDropListeners.clear();
    }

    public void removeStaticMediaModelListeners() {
	removeDragAndDropListeners();
    }

    public void removeListeners() {
	super.removeListeners();
	removeStaticMediaModelListeners();
    }

    public void unOperationalize() {
	super.unOperationalize();
	removeStaticMediaModelListeners();
    }

    public void deleteModel() {
	super.deleteModel();
	removeStaticMediaModelListeners();
    }

 
    // Private methods
    private void notifyDragAndDropListeners() {	
	Iterator it = dragAndDropListeners.iterator(); 
	while (it.hasNext()) {
	    StaticMediaModelListenerInterface listener = (StaticMediaModelListenerInterface) it.next();
	    listener.wasDraggedAndDropped(this, droppedOnObject);
	}
    }

}


