/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: StoryboardModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.util.*;
import java.io.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.layer.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;




public class StoryboardModel extends DataModel {


    // Used to create subsequent storyboard names
    private static int SM_IDENTIFIER;

    // A global list of storyboard models in the system
    private static Vector storyboardModels;



    // The set of layers associated with this storyboard
    private LayerList layers;

    // A Storyboard
    private Storyboard storyboard;






    // Static initializer
    static {
	SM_IDENTIFIER = 0;
	storyboardModels = new Vector();
    }


    public static int getSMIdentifier() {
	++SM_IDENTIFIER;
	return SM_IDENTIFIER;
    }

    public static void setSMIdentifier(int id) {
	SM_IDENTIFIER = id;
    }

    public static void addStoryboardModel(StoryboardModel model) {
	storyboardModels.add(model);
	PropertyManager.setProperty(PropertyManager.APPLICATION_STORYBOARD_COUNT, new Integer(storyboardModels.size()));
    }


    public static void removeStoryboardModel(StoryboardModel model) {
	storyboardModels.remove(model);
	PropertyManager.setProperty(PropertyManager.APPLICATION_STORYBOARD_COUNT, new Integer(storyboardModels.size()));
	
    }

    
    public static StoryboardModel getFirstStoryboardModel() {
	if (storyboardModels.size() > 0) {
	    return (StoryboardModel) storyboardModels.firstElement();
	}
	return null;
    }

    
    // Returns the storyboard model that comes before model in the list
    // If model is the 0th element, then the 1st element is returned
    public static StoryboardModel getPreviousStoryboardModel(StoryboardModel model) {
	return getPreviousStoryboardModel(model, true);
    }

    public static StoryboardModel getPreviousStoryboardModel(StoryboardModel model, boolean flag) {
	StoryboardModel prev = null;

	boolean first = true;
	Iterator it = storyboardModels.iterator();
	while (it.hasNext()) {
	    StoryboardModel current = (StoryboardModel) it.next();
	    if (current.equals(model)) {
		break;
	    }
	    first = false;
	    prev = current;
	}
     
	if (first && flag) {
	    // First storyboard was deleted, so return the following one
	    if (storyboardModels.size() > 1) {
		prev = (StoryboardModel) storyboardModels.elementAt(1);
	    }
	}

	return prev;
    }


    // Returns the storyboard model that comes after model in the list
    public static StoryboardModel getNextStoryboardModel(StoryboardModel model) {
	StoryboardModel next = null;
	Iterator it = storyboardModels.iterator();
	while (it.hasNext()) {
	    StoryboardModel current = (StoryboardModel) it.next();
	    if (current.equals(model)) {
		if (it.hasNext()) {
		    next = (StoryboardModel) it.next();
		}
	    }
	}
	return next;
    }


    public static StoryboardModel getStoryboardModelById(Integer id) {
	Iterator it = storyboardModels.iterator();
	while (it.hasNext()) {
	    StoryboardModel current = (StoryboardModel) it.next();
	    if (current.equals(id)) {
		return current;
	    }
	}
	return null;
    }


    public static StoryboardModel getStoryboardModelByName(String name) {
	Iterator it = storyboardModels.iterator();
	while (it.hasNext()) {
	    StoryboardModel current = (StoryboardModel) it.next();
	    String cName = current.getName();
	    if ((cName != null) && (cName.equals(name))) {
		return current;
	    }
	}
	return null;
    }



    public static Vector getStoryboardModels() {
	return storyboardModels;
    }
    
    public static Vector getStoryboardModelNames() {
	return getStoryboardModelNames(false);
    }

    public static Vector getStoryboardModelNames(boolean excludeCurrent) {
	StoryboardModel current = (StoryboardModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	if (current == null) {
	    excludeCurrent = false;
	}

	Vector v = new Vector();
	Iterator it = storyboardModels.iterator();
	while (it.hasNext()) {
	    StoryboardModel sModel = (StoryboardModel) it.next();
	    if ((! excludeCurrent) || (!current.equals(sModel))) {
		v.add(sModel.getName());
	    }
	}
	return v;
    }



    // Constructors
    private StoryboardModel() {
	this(0, null, null);
    }

    public StoryboardModel(int scope, DataModel localScope, String modelName) {
	super(scope, localScope, modelName);
	if (modelName == null) {
	    setName("S" + Integer.toString(StoryboardModel.getSMIdentifier()), false);
	}

	// Create initial set of layers
	layers = new LayerList();
	layers.addLayer(new ContentLayer("Content", 0, true, true));

	storyboard = new Storyboard(this);
	
	StoryboardModel.addStoryboardModel(this);
	ContentManagerFrame.getFrame().addDataModel(this);
    }


    public StoryboardModel(StoryboardModel copy) {
	this(copy, true);
    }

    public StoryboardModel(StoryboardModel copy, boolean copyOfModels) {
	super(copy, copyOfModels);
	setName("S" + Integer.toString(StoryboardModel.getSMIdentifier()), false);

	// Copy the layer vector
	layers = null;
	if (copy.layers != null) { 
	    // Makes a deep copy of the layer list
	    layers = (LayerList) copy.layers.clone();

	} else {
	    layers = new LayerList();
	    layers.addLayer(new ContentLayer("Content", 0, true, true));
	}

	// Clone the storyboard
	storyboard = null;
	if (copy.storyboard != null) {
	    storyboard = (Storyboard) copy.storyboard.clone(this, copyOfModels);
	
	} else {
	    storyboard = new Storyboard(this);
	}

	// Add to the global list of storyboard models
	StoryboardModel.addStoryboardModel(this);
	ContentManagerFrame.getFrame().addDataModel(this);
    }


    public StoryboardModel(DataInputStream stream) throws IOException {
	super(stream);

	// Read the static identifier value
	SM_IDENTIFIER = stream.readInt();

	// Read the layer information
	layers = new LayerList(stream);

	// Read the storyboard
	storyboard = new Storyboard(stream, this);	

	// Add to the global list of storyboard models
	StoryboardModel.addStoryboardModel(this);
	ContentManagerFrame.getFrame().addDataModel(this);
    }




    public void completeIO() {
	super.completeIO();
	storyboard.completeIO();
    }


    public void operationalize() {
	layers.operationalize();
	storyboard.operationalize();

	// Operationalize the storyboard first, which sets the annotations
	// Then operationalize the parent, which parses those annotations
	super.operationalize();
    }

    public void unOperationalize() {
	super.unOperationalize();
	layers.unOperationalize();
	storyboard.unOperationalize();
    }

    public void realize(boolean showControls, boolean recreate) {
	super.realize(showControls, recreate);
	storyboard.realize(showControls, recreate);
    }
    
    public void unRealize(boolean keepVisuals) {
	super.unRealize(keepVisuals);
	storyboard.unRealize(keepVisuals);
    }


    // Layer methods
    public boolean isLayerVisible(int layer) {
	return layers.isLayerVisible(layer);
    }

    public boolean isLayerSelected(int layer) {
	return layers.isLayerSelected(layer);
    }


    public int getSelectedLayer() {
	return layers.getSelectedLayer();
    }


    public int getNumLayers() {
	return layers.getNumLayers();
    }


    public LayerModel getSelectedLayerModel() {
	return layers.getSelectedLayerModel();
    }


    public String getLayerName(int layer) {
	return layers.getLayerName(layer);
    }


    public LayerModel getLayerModel(int layer) {
	return layers.getLayerModel(layer);
    }


    public void setLayerName(int layer, String name) {
	layers.setLayerName(layer, name);
	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.TRUE);
    }


    public void setLayerVisible(int layer, boolean visible) {
	layers.setLayerVisible(layer, visible);
	storyboard.setLayerVisible(layer, visible);
	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.TRUE);
    }


    public void setLayerSelected(int layer, boolean selected) {
	layers.setLayerSelected(layer, selected);
	storyboard.setLayerSelected(layer, selected);
    }
    

    public void newLayer(LayerModel newLayerModel) {
	layers.addLayer(newLayerModel);
	storyboard.newLayer(newLayerModel.getLayer());
	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.TRUE);
    }


    public void copyLayer(int layerNum, String newLayerName, int newLayerNum, boolean visible, boolean selected) {
	layers.copyLayer(layerNum, newLayerName, newLayerNum, visible, selected);
	storyboard.copyLayer(layerNum, newLayerNum);
	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.TRUE);
    }

    public void deleteLayer(int layerNum, int newSelectedLayer) {
	layers.deleteLayer(layerNum);
	storyboard.deleteLayer(layerNum, newSelectedLayer);
	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.TRUE);
    }

    public void repositionLayer(int layerNum, int toPosition) {
	layers.repositionLayer(layerNum, toPosition);
	storyboard.repositionLayer(layerNum, toPosition);
	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.TRUE);
    }


    public void flattenLayers() {
	layers.flattenLayers();
	storyboard.flattenLayers(1);
	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.TRUE);
    }


    public LayerList getLayers() {
	return layers;
    }



    // Delete the model from the system - close out all views on this model
    public void deleteModel() {
	// Destroy the parent model first
	super.deleteModel();

	// Allow the storyboard to clean up
	if (storyboard != null) {
	    storyboard.deleteStoryboard();
	}

	// Set the current storyboard if we are deleting the current one
	DataModel cModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	if (cModel != null) {
	    if (this.equals(cModel)) {

		DataModel nextModel = StoryboardModel.getPreviousStoryboardModel(this);
		if (nextModel == null) {
		    nextModel = MultiViewModel.getFirstMultiViewModel();
		}
		PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, nextModel);
	    }
	}

	// Remove the storyboard from the global list
	StoryboardModel.removeStoryboardModel(this);
    }


    public Storyboard getStoryboard() {
	return storyboard;
    }

    
    // View operations
    public ContentManagerCellView getContentManagerCellView() {
	StoryboardContentManagerCellView view = new StoryboardContentManagerCellView(this);
	addView(view);
	return view;
    }


    
    public Object clone() {
	return (new StoryboardModel(this));
    }

    public Object clone(boolean copyOfModels) {
	return (new StoryboardModel(this, copyOfModels));
    }



    public void write(DataOutputStream stream) throws IOException {
	super.write(stream);

	// Write the static identifier value
	stream.writeInt(SM_IDENTIFIER);

	// Write the layer vector
	layers.write(stream);

	// Write the storyboard
	storyboard.write(stream);

    }

}
