/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: TextEntryModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.util.*;
import java.io.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;



// Represents an object having a name 
public class TextEntryModel extends NamedModel 
    implements TextEntryModelInterface {

    // The referring rectangle ink stroke
    private RectangleInkStroke ris;

    // Listeners to notify when the user enters text
    private Vector textEntryListeners;
    protected boolean wasTextEntered;




    // Constructors
    private TextEntryModel() {
	this(0, null, null, null);
    }
    
    public TextEntryModel(int objScope, DataModel localScope, String modelName, RectangleInkStroke ris) {
	super(objScope, localScope, modelName);
	this.ris = ris;
	textEntryListeners = new Vector();
	wasTextEntered = false;
    }


    public TextEntryModel(TextEntryModel copy, RectangleInkStroke ris) {
	this(copy, ris, ris.getStoryboard().getModel());
    }

    public TextEntryModel(TextEntryModel copy, RectangleInkStroke ris, DataModel toModel) {
	super(copy, toModel);
	this.ris = ris;
	textEntryListeners = new Vector();
	wasTextEntered = false;
    }


    public TextEntryModel(DataInputStream stream, RectangleInkStroke ris, DataModel localScope) throws IOException {
	super(stream, localScope);
	this.ris = ris;
	textEntryListeners = new Vector();
	wasTextEntered = false;
    }



    // Public methods
    public void textEntered() {
	setWasTextEntered(true, true);
    }

    public void setWasTextEntered(boolean b) {
	setWasTextEntered(b, false);
    }

    public void setWasTextEntered(boolean b, boolean notify) {
	wasTextEntered = b;
	if (notify) {
	    notifyTextEntryListeners();
	}
    }

    public boolean wasTextEntered() {
	return wasTextEntered;
    }


    public void addTextEntryListener(TextEntryModelListenerInterface tli) {
	if (! textEntryListeners.contains(tli)) {
	    textEntryListeners.add(tli);
	}
    }

    public void removeTextEntryListener(TextEntryModelListenerInterface tli) {
	textEntryListeners.remove(tli);
    }

    public void removeTextEntryListeners() {
	textEntryListeners.clear();
    }


    public void removeTextEntryModelListeners() {
	removeTextEntryListeners();
    }
    
    
    public void removeListeners() {
	super.removeListeners();
	removeTextEntryModelListeners();
    }

    public void unOperationalize() {
	super.unOperationalize();
	removeTextEntryModelListeners();
    }

    public void deleteModel() {
	super.deleteModel();
	removeTextEntryModelListeners();
    }

    
    public Object clone(RectangleInkStroke ris) {
	return (new TextEntryModel(this, ris));
    }

    
    private void notifyTextEntryListeners() {
	Iterator it = textEntryListeners.iterator();
	while(it.hasNext()) {
	    TextEntryModelListenerInterface listener = (TextEntryModelListenerInterface) it.next();
	    listener.textEntered(this);
	}
    }
}

