/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: TextToSpeechSynthesizer.java
  
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.util.Locale;
import javax.speech.Central;
import javax.speech.Engine;
import javax.speech.EngineException;
import javax.speech.synthesis.Synthesizer;
import javax.speech.synthesis.SynthesizerModeDesc;
import javax.speech.synthesis.SynthesizerProperties;
import javax.speech.synthesis.Voice;
import javax.speech.synthesis.Speakable;
import javax.speech.synthesis.SpeakableAdapter;
import javax.speech.synthesis.SpeakableListener;
import javax.speech.synthesis.SpeakableEvent;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;

import com.cloudgarden.speech.*;
import com.cloudgarden.audio.*;
import com.cloudgarden.speech.userinterface.*;


public class TextToSpeechSynthesizer {

    // Implement a singleton pattern
    private static TextToSpeechSynthesizer tts;


    // The java-enabled synthesizer
    private Synthesizer synthesizer;



    // Static initializer
    static {
	tts = null;
    }

    public static TextToSpeechSynthesizer getTTS() {
	if (tts == null) {
	    tts = new TextToSpeechSynthesizer();
	}
	return tts;
    }


    private TextToSpeechSynthesizer() {

	/* Display the Cloud Garden chooser */
	try {
		SpeechEngineChooser chooser = SpeechEngineChooser.getSynthesizerDialog();
		chooser.show();
	    SynthesizerModeDesc desc = chooser.getSynthesizerModeDesc();
	    synthesizer = Central.createSynthesizer(desc);
	    
	    if (synthesizer != null) {
			try {
			synthesizer.allocate();
			synthesizer.waitEngineState(Engine.ALLOCATED);
			synthesizer.resume();
			SynthesizerProperties props = synthesizer.getSynthesizerProperties();
			props.setVoice(chooser.getVoice());	
		
			} catch (Exception xcptn1) {
				DEMAISError.showErrorMessage(xcptn1.toString(), "TextToSpeech", false);
				synthesizer = null;
			}
		}
	} catch (Exception xcptn2) {
	    DEMAISError.showErrorMessage(xcptn2.toString(), "TextToSpeech", false);
	    synthesizer = null;
 	}
    }


    public boolean isTextToSpeechAvailable() {
	return (synthesizer != null);
    }


    public void speakPlainText(String plainText) {
	speakPlainText(plainText, null);
    }


    public void speakPlainText(String plainText, SpeakableListener listener) {
	if (synthesizer != null) {
	    try {
		synthesizer.cancelAll();
		synthesizer.speakPlainText(plainText, listener);

	    } catch (Exception xcptn) {
		System.out.println(xcptn.toString());
	    }
	}
    }


    public void speakJSMLText(String jsmlText) {
	speakJSMLText(jsmlText, null);
    }

    public void speakJSMLText(String jsmlText, SpeakableListener listener) {
	if (synthesizer != null) {
	    try {
		synthesizer.cancelAll();
		synthesizer.speak(jsmlText, listener);

	    } catch (Exception xcptn) {
		System.out.println(xcptn.toString());
	    }
	}
    }


    public void pauseSpeaking() {
	if (synthesizer != null) {
	    try {
		synthesizer.pause();

	    } catch (Exception xcptn) {
		System.out.println(xcptn.toString());
	    }
	}
    }


    public void resumeSpeaking() {
	if (synthesizer != null) {
	    try {
		synthesizer.resume();

	    } catch (Exception xcptn) {
		System.out.println(xcptn.toString());
	    }
	}
    }


    public void stopSpeaking() {
	if (synthesizer != null) {
	    try {
		synthesizer.cancelAll();

	    } catch (Exception xcptn) {
		System.out.println(xcptn.toString());
	    }
	}
    }
    
}








