/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: Variable.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.util.*;
import java.io.*;



/*
  Defines a variable in the DEMAIS annotation language
*/
public class Variable extends NamedModel {
 
    // The value of the variable
    private double value;
 
    // A list of objects to notify when the variable changes value
    private Vector valueListeners;



    // Constructors
    private Variable() {
	this(0, null, null, 0.0);
    }
    
    public Variable(int scope, DataModel localScope, String name, double v) {
	super(scope, localScope, name);
	if (name == null) {
	    setName("Variable" + getIdentifier().toString(), false);
	}
	value = v;
	valueListeners = new Vector();
    }




    public Variable(Variable copy) {
	this(copy, copy.localScope);
    }

    public Variable(Variable copy, DataModel toModel) {
	super(copy, toModel);
	setName("Variable" + getIdentifier().toString(), false);
	value = copy.value;
	valueListeners = new Vector();
    }


    public Variable(DataInputStream stream, DataModel localScope) throws IOException {
	super(stream, localScope);
	value = stream.readDouble();
	valueListeners = new Vector();
    }



    // Public interface methods
    public void setValue(double v) {
	setValue(v, true);
    }

    public void setValue(double v, boolean notifyListeners) {
	value = v;
	if (notifyListeners) {
	    notifyValueListeners();
	}
    }

    public double getValue() {
	return value;
    }

    public void addValueListener(VariableListenerInterface vli) {
	if (! valueListeners.contains(vli)) {
	    valueListeners.add(vli);
	}
    }


    public void removeValueListener(VariableListenerInterface vli) {
	valueListeners.remove(vli);
    }

    public void removeValueListeners() {
	valueListeners.clear();
    }

    public void removeVariableListeners() {
	removeValueListeners();
    }


    public void removeListeners() {
	super.removeListeners();
	removeVariableListeners();
    }

    public void unOperationalize() {
	super.unOperationalize();
	removeVariableListeners();
    }

    public void deleteModel() {
	super.deleteModel();
	removeVariableListeners();
    }


    public Object clone() {
	return (new Variable(this));
    }

    public Object clone(DataModel toModel) {
	return (new Variable(this, toModel));
    }


    private void notifyValueListeners() {
	Iterator it = valueListeners.iterator();
	while (it.hasNext()) {
	    VariableListenerInterface listener = (VariableListenerInterface) it.next();
	    listener.valueChanged(this);
	}
    }


    public void write(DataOutputStream stream) throws IOException {
	super.write(stream);
	
	stream.writeDouble(value);
    }
}

