/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: VideoModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal;

// Import needed packages
import java.lang.*;
import java.io.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;




public class VideoModel extends JMFContinuousMediaModel {

    // The storyboard in which the video will be displayed
    private VideoRectangleInkStroke vris;




    // Constructors
    private VideoModel() {
	this(0, null, null, null, null);
    }
    
    public VideoModel(int objScope, DataModel localScope, String modelName, String mediaFileName, VideoRectangleInkStroke vris) {
	super(objScope, localScope, modelName, mediaFileName);
	this.vris = vris;
    }

    
    public VideoModel(VideoModel copy, VideoRectangleInkStroke vris) {
	this(copy, vris, vris.getStoryboard().getModel());
    }
    
    public VideoModel(VideoModel copy, VideoRectangleInkStroke vris, DataModel toModel) {
	super(copy, toModel);
	this.vris = vris;
    }


    public VideoModel(DataInputStream stream, VideoRectangleInkStroke vris, DataModel localScope) throws IOException {
	super(stream, localScope);
	this.vris = vris;
    }

   

    // Public methods
    public void begin() {
	super.begin();
	vris.begin();
    }

    public void display() {
	vris.display();
    }

    public void unDisplay() {
	vris.unDisplay();
    }

    public void highlight() {
	vris.highlight();
    }

    public void unHighlight() {
	vris.unHighlight();
    }


    public Object clone(VideoRectangleInkStroke vris) {
	return (new VideoModel(this, vris));
    }

}



