/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: LayerModel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.meal.layer;

// Import needed packages
import java.lang.*;
import java.io.*;
import edu.uofminnesota.DEMAIS.property.*;



public class LayerModel extends Object {

    // The layer name
    private String name;

    // The layer number
    private int layer;

    // If layer is visible
    private boolean visible;
    private boolean prev_v;

    // If layer selected
    private boolean selected;



    protected LayerModel() {
    }

    protected LayerModel(String name, int layer, boolean visible, boolean selected) {
	this.name = name;
	this.layer = layer;
	this.visible = visible;
	this.selected = selected;
	prev_v = false;
    }

    protected LayerModel(LayerModel copy) {
	this(copy, copy.layer);
    }

    protected LayerModel(LayerModel copy, int toLayer) {
	name = null;
	if (copy.name != null) {
	    name = new String(copy.name);
	}
	layer = toLayer;
	visible = copy.visible;
	selected = copy.selected;
	prev_v = false;
    }

    protected LayerModel(DataInputStream stream) throws IOException {	
	int length = stream.readInt();
	name = "";
	for (int i=0; i<length; ++i) {
	    name += stream.readChar();
	}
	layer = stream.readInt();
	visible = stream.readBoolean();
	selected = stream.readBoolean();
	prev_v = false;
    }



    public String getName() {
	return name;
    }

    public void setName(String n) {
	name = n;
    }

    public int getLayer() {
	return layer;
    }

    public void setLayer(int l) {
	layer = l;
    }
    

    public boolean getVisible() {
	return visible;
    }

    public boolean isVisible() {
	return visible;
    }

    public void setVisible(boolean v) {
	visible = v;
    }


    public boolean getSelected() {
	return selected;
    }

    public boolean isSelected() {
	return selected;
    }

    public void setSelected(boolean s) {
	selected = s;
    }

    
    public void operationalize() {
	prev_v = visible;
	visible = true;
    }

    public void unOperationalize() {
	visible = prev_v;
    }


    public boolean isAnnotationLayer() {
	return false;
    }


    public boolean equals(Object obj) {
	if (obj instanceof LayerModel) {
	    return (equals ( (LayerModel) obj));
	}
	return (this == obj);
    }

    public boolean equals(LayerModel lm) {
	return (layer == lm.layer);
    }


    public Object clone() {
	return (new LayerModel(this));
    }


    public void write(DataOutputStream stream) throws IOException {
	if (name != null) {   
	    stream.writeInt(name.length());
	    stream.writeChars(name);
	} else {
	    stream.writeInt(0);
	}

	stream.writeInt(layer);
	int mode = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
	if (mode == PropertyManager.APPLICATION_EDIT) {
	    stream.writeBoolean(visible);
	} else {
	    stream.writeBoolean(prev_v);
	}
	stream.writeBoolean(selected);
    }

}
