/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: PropertyFileManager.java
*/

// Current package
package edu.uofminnesota.DEMAIS.property;

// Necessary imports
import java.io.*;
import java.util.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;



public class PropertyFileManager {
    
    // Name of property file to open - assumed that the file is located 
    // in the directory from which DEMAIS was started
    protected static final String PROPERTY_FILE = "properties";
    

    // Instance data

    // stores the name/value attribute pairs
    private HashMap table;

    // Actual file reader object
    private BufferedReader bReader;

    // One line of text from the file
    private String lineOfText;

    // Position within the line of text
    private int position;




    // Constructor
    public PropertyFileManager() {
	this(PROPERTY_FILE);
    }

    public PropertyFileManager(String propertyFile) {
	table = new HashMap(20);

	// Load the values, regardless of whether the property file can be opened or not
	table.put("APPLICATION_INPUT_DEVICE", null);

	table.put("APPLICATION_AUDIO_DIRECTORY", null);
	table.put("APPLICATION_IMAGE_DIRECTORY", null);
	table.put("APPLICATION_DESIGN_DIRECTORY", null);

	table.put("APPLICATION_CURSOR_FILE", null);

	table.put("APPLICATION_TITLE_FONT_NAME", null);
	table.put("APPLICATION_TITLE_FONT_SIZE", null);
	table.put("APPLICATION_TITLE_FONT_FACE", null);

	table.put("APPLICATION_TEXT_FIELD_FONT_NAME", null);
	table.put("APPLICATION_TEXT_FIELD_FONT_SIZE", null);
	table.put("APPLICATION_TEXT_FIELD_FONT_FACE", null);

	table.put("APPLICATION_TEXT_AREA_FONT_NAME", null);
	table.put("APPLICATION_TEXT_AREA_FONT_SIZE", null);
	table.put("APPLICATION_TEXT_AREA_FONT_FACE", null);

	table.put("STORYBOARD_TEXT_FONT_NAME", null);
	table.put("STORYBOARD_TEXT_FONT_SIZE", null);
	table.put("STORYBOARD_TEXT_FONT_FACE", null);


	table.put("MULTIVIEW_TITLE_FONT_NAME", null);
	table.put("MULTIVIEW_TITLE_FONT_SIZE", null);
	table.put("MULTIVIEW_TITLE_FONT_FACE", null);

	table.put("MULTIVIEW_NARRATION_TEXT_FONT_NAME", null);
	table.put("MULTIVIEW_NARRATION_TEXT_FONT_SIZE", null);
	table.put("MULTIVIEW_NARRATION_TEXT_FONT_FACE", null);


	table.put("CONTENT_MANAGER_WIDTH", null);
	table.put("CONTENT_MANAGER_HEIGHT", null);
	table.put("CONTENT_MANAGER_ROWS", null);
	table.put("CONTENT_MANAGER_COLUMNS", null);

	table.put("APPLICATION_WIDTH", null);
	table.put("APPLICATION_HEIGHT", null);
	table.put("APPLICATION_RECTANGLE_THRESHHOLD", null);

	table.put("INK_STROKE_LINE_WIDTH", null);
	table.put("GESTURE_STROKE_LINE_WIDTH", null);

	table.put("SPEECH_SYNTHESIZER_INSTALLED", null);



	lineOfText = null;
	position = 0;

	propertyFile = System.getProperty("user.dir") + 
	    System.getProperty("file.separator") + 
	    propertyFile;
	
	bReader = null;
	try {
	    bReader = new BufferedReader(new FileReader(propertyFile));
	} catch (Exception x1) {
	    DEMAISError.showErrorMessage(x1.toString(), "PropertyFileManager", false);
	    DEMAISError.logException(x1);
	    return;
	}

	// Read the contents of the file
	readFromFile();

	// Close the file
	try {
	    bReader.close();

	} catch (Exception xcptn) {
	    DEMAISError.showErrorMessage(xcptn.toString(), "PropertyFileManager", false);
	    DEMAISError.logException(xcptn);
	}
    }


    // Return a value for the attribute name
    public Object getValue(String attrName) {
	return table.get(attrName);
    }


    private void readFromFile() {
	// Read the file line by line
	while ( (lineOfText = readLine()) != null) {
	    parseAndStoreNameValuePair();
	    // Reset the variables for the next line of text
	    lineOfText = null;
	    position = 0;
	}
    }


    private void parseAndStoreNameValuePair() {
	skipWhiteSpace();
	String name = readName();

	if (isValidName(name)) {
	    skipWhiteSpace();
	    skipEqual();
	    skipWhiteSpace();

	    String value = readValue();
	    if (value != null) {
		storeNameValuePair(name, value);
	    }
	}
    }


    private boolean isValidName(String name) {
	return (table.containsKey(name));
    }


    private void storeNameValuePair(String name, String value) {
	// List of all of the possible attribute names here!

	if (name.equals("APPLICATION_INPUT_DEVICE")) {
	    Integer device = null;
	    if (value.equals("STYLUS")) {
		device = new Integer(PropertyManager.STYLUS_INPUT_DEVICE);
	    
	    } else if (value.equals("MOUSE")) {
		device = new Integer(PropertyManager.MOUSE_INPUT_DEVICE);
	    }
	    table.put(name, device);

	} else if (name.equals("APPLICATION_AUDIO_DIRECTORY")) {
	    table.put(name, value);

	} else if (name.equals("APPLICATION_IMAGE_DIRECTORY")) {
	    table.put(name, value);
	
	} else if (name.equals("APPLICATION_DESIGN_DIRECTORY")) {
	    table.put(name, value);

	} else if (name.equals("APPLICATION_CURSOR_FILE")) {
	    table.put(name, value);


	} else if (name.equals("APPLICATION_TITLE_FONT_NAME")) {
	    table.put(name, value);

	} else if (name.equals("APPLICATION_TITLE_FONT_SIZE")) {
	    try {
		int size = Integer.parseInt(value);
		table.put(name, new Integer(size));
	    } catch (Exception xcptn) {
	    }
	    
	} else if (name.equals("APPLICATION_TITLE_FONT_FACE")) {
	    int face = Font.BOLD;
	    if (value.equals("BOLD")) {
		face = Font.BOLD;
	    } else if (value.equals("ITALIC")) {
		face = Font.ITALIC;
	    } else if (value.equals("PLAIN")) {
		face = Font.PLAIN;
	    }
	    table.put(name, new Integer(face));

	} else if (name.equals("APPLICATION_TEXT_AREA_FONT_NAME")) {
	    table.put(name, value);

	} else if (name.equals("APPLICATION_TEXT_AREA_FONT_SIZE")) {
	    try {
		int size = Integer.parseInt(value);
		table.put(name, new Integer(size));
	    } catch (Exception xcptn) {
	    }
	    
	} else if (name.equals("APPLICATION_TEXT_AREA_FONT_FACE")) {
	    int face = Font.PLAIN;
	    if (value.equals("BOLD")) {
		face = Font.BOLD;
	    } else if (value.equals("ITALIC")) {
		face = Font.ITALIC;
	    } 
	    table.put(name, new Integer(face));


	} else if (name.equals("APPLICATION_TEXT_FIELD_FONT_NAME")) {
	    table.put(name, value);

	} else if (name.equals("APPLICATION_TEXT_FIELD_FONT_SIZE")) {
	    try {
		int size = Integer.parseInt(value);
		table.put(name, new Integer(size));
	    } catch (Exception xcptn) {
	    }
	    
	} else if (name.equals("APPLICATION_TEXT_FIELD_FONT_FACE")) {
	    int face = Font.PLAIN;
	    if (value.equals("BOLD")) {
		face = Font.BOLD;
	    } else if (value.equals("ITALIC")) {
		face = Font.ITALIC;
	    } 
	    table.put(name, new Integer(face));

	} else if (name.equals("STORYBOARD_TEXT_FONT_NAME")) {
	    table.put(name, value);

	} else if (name.equals("STORYBOARD_TEXT_FONT_SIZE")) {
	    try {
		int size = Integer.parseInt(value);
		table.put(name, new Integer(size));
	    } catch (Exception xcptn) {
	    }
	    
	} else if (name.equals("STORYBOARD_TEXT_FONT_FACE")) {
	    int face = Font.PLAIN;
	    if (value.equals("BOLD")) {
		face = Font.BOLD;
	    } else if (value.equals("ITALIC")) {
		face = Font.ITALIC;
	    } 
	    table.put(name, new Integer(face));


	} else if (name.equals("MULTIVIEW_TITLE_FONT_NAME")) {
	    table.put(name, value);

	} else if (name.equals("MULTIVIEW_TITLE_FONT_SIZE")) {
	    try {
		int size = Integer.parseInt(value);
		table.put(name, new Integer(size));
	    } catch (Exception xcptn) {
	    }
	    
	} else if (name.equals("MULTIVIEW_TITLE_FONT_FACE")) {
	    int face = Font.BOLD;
	    if (value.equals("BOLD")) {
		face = Font.BOLD;
	    } else if (value.equals("ITALIC")) {
		face = Font.ITALIC;
	    } else if (value.equals("PLAIN")) {
		face = Font.PLAIN;
	    }
	    table.put(name, new Integer(face));

	    
	} else if (name.equals("MULTIVIEW_NARRATION_TEXT_FONT_NAME")) {
	    table.put(name, value);

	} else if (name.equals("MULTIVIEW_NARRATION_TEXT_FONT_SIZE")) {
	    try {
		int size = Integer.parseInt(value);
		table.put(name, new Integer(size));
	    } catch (Exception xcptn) {
	    }
	    
	} else if (name.equals("MULTIVIEW_NARRATION_TEXT_FONT_FACE")) {
	    int face = Font.PLAIN;
	    if (value.equals("BOLD")) {
		face = Font.BOLD;
	    } else if (value.equals("ITALIC")) {
		face = Font.ITALIC;
	    } else if (value.equals("PLAIN")) {
		face = Font.PLAIN;
	    }
	    table.put(name, new Integer(face));


	} else if (name.equals("CONTENT_MANAGER_WIDTH")) {
	    try {
		int val = Integer.parseInt(value);
		table.put(name, new Integer(val));
	    } catch (Exception xcptn) {
	    }

	} else if (name.equals("CONTENT_MANAGER_HEIGHT")) {
	    try {
		int val = Integer.parseInt(value);
		table.put(name, new Integer(val));
	    } catch (Exception xcptn) {
	    }

	} else if (name.equals("CONTENT_MANAGER_ROWS")) {
	    try {
		int val = Integer.parseInt(value);
		table.put(name, new Integer(val));
	    } catch (Exception xcptn) {
	    }

	} else if (name.equals("CONTENT_MANAGER_COLUMNS")) {
	    try {
		int val = Integer.parseInt(value);
		table.put(name, new Integer(val));
	    } catch (Exception xcptn) {
	    }

	} else if (name.equals("APPLICATION_WIDTH")) {
	    try {
		int val = Integer.parseInt(value);
		table.put(name, new Integer(val));
	    } catch (Exception xcptn) {
	    }

	} else if (name.equals("APPLICATION_HEIGHT")) {
	    try {
		int val = Integer.parseInt(value);
		table.put(name, new Integer(val));
	    } catch (Exception xcptn) {
	    }

	} else if (name.equals("APPLICATION_RECTANGLE_THRESHHOLD")) {
	    try {
		double threshhold = Double.parseDouble(value);
		table.put(name, new Double(threshhold));

	    } catch (Exception xcptn) {
	    }
	}

	else if (name.equals("INK_STROKE_LINE_WIDTH")) {
	    try {
		double lw = Double.parseDouble(value);
		table.put(name, new Double(lw));
		
	    } catch (Exception xcptn) {
	    }
	}
	
	else if (name.equals("GESTURE_STROKE_LINE_WIDTH")) {
	    try {
		double lw = Double.parseDouble(value);
		table.put(name, new Double(lw));

	    } catch (Exception xcptn) {
	    }
	}

	else if (name.equals("SPEECH_SYNTHESIZER_INSTALLED")) {
	    try {
		if (value.equals("TRUE")) {
		    table.put(name, new Boolean(true));
		} else {
		    table.put(name, new Boolean(false));
		}
	    } catch (Exception xcptn) {
	    }
	}   

    }



    private String readName() {
	String name = "";
	// Read until end of string, white space, or '=' sign
	char c = 'a';
	while ( (position < lineOfText.length()) && 
		( (c = lineOfText.charAt(position)) != -1 ) &&
		(!Character.isWhitespace(c)) &&
		(c != '=') ) {
	    name += c;
	    ++position;
	}	
	return name.toUpperCase();
    }


    private void skipEqual() {
	++position;
    }

    
    private String readValue() {
	// Value is just the remainder of the string
	String value = lineOfText.substring(position);
	if (value != null) {
	    value = value.trim();
	    if ((value != null) && (value.length() > 0)) {
		value = value.toUpperCase();
		return value;
	    }
	}
	return null;
    }


    private void skipWhiteSpace() {	
	while ( (position < lineOfText.length()) && 
		(Character.isWhitespace(lineOfText.charAt(position))) ) {
	    ++position;
	}
    }


    private String readLine() {
	String s = null;
	try {
	    s = bReader.readLine();

	} catch (IOException xcptn) {
	    DEMAISError.showErrorMessage(xcptn.toString(), "PropertyFileManager", false);
	    DEMAISError.logException(xcptn);
	}
	return s;
    }

}
