/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: PropertyManager.java
*/

// Current package
package edu.uofminnesota.DEMAIS.property;

// Necessary imports
import java.lang.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;



public class PropertyManager {

    // GLOBAL PROPERTY CONSTANTS


    // Application frame reference
    public static JFrame APPLICATION_FRAME_WIDGET = null;

    // Constant definitions for symbol table scope
    public static final int APPLICATION_SCOPE = 0;
    public static final int GLOBAL_SCOPE = 1;
    public static final int LOCAL_SCOPE = 2;

    // Available transition types
    public static final int DISSOLVE_TRANSITION = 0;
    public static final int WIPE_TRANSITION = 1;

    // Available transition speeds
    public static final int TRANSITION_SLOW = 0;
    public static final int TRANSITION_MEDIUM = 1;
    public static final int TRANSITION_FAST = 2;

    // Available layer types
    public static final int CONTENT_LAYER = 0;
    public static final int ANNOTATION_LAYER = 1;
    public static final int INTERACTION_LAYER = 2;

    // Application modes
    public static final int APPLICATION_EDIT = 0;
    public static final int APPLICATION_PLAY = 1;

    // Available input devices
    public static final int MOUSE_INPUT_DEVICE = 0;
    public static final int STYLUS_INPUT_DEVICE = 1;

    // Edit operations
    public static final int CUT_OPERATION = 0;
    public static final int COPY_OPERATION = 1;
    public static final int PASTE_OPERATION = 2;
    public static final int UNDO_OPERATION = 3;

    // Separator used for directory path names 
    public static final String FILE_SEPARATOR = System.getProperty("file.separator");

    // Where the application is invoked from
    public static final String APPLICATION_DIRECTORY = System.getProperty("user.dir");

    // Basic colors used in the application
    public static final Color DARK_RED = Color.red.darker();
    public static final Color DARK_GREEN = Color.green.darker();
    public static final Color DARK_BLUE = new Color(54, 110, 175);


    // The color to use for a storyboard's background
    public static final Color STORYBOARD_BACKGROUND_COLOR = new Color(254, 253, 237);

    // The color to use for text being edited on a storyboard
    public static final Color STORYBOARD_TEXT_EDIT_COLOR = new Color(0, 0, 255);


    // Selected color for a layer control panel
    public static final Color LAYER_CONTROL_SELECTED_COLOR = new Color(10, 36, 106);
    
    // Unselected color for a layer control panel
    public static Color LAYER_CONTROL_UNSELECTED_COLOR = null;
    
    // Text font to use for the layer names
    public static final Font LAYER_CONTROL_LAYER_TEXT_FONT = new Font("Arial", Font.PLAIN, 14);
    
    // Text font to use for the content (layer 0) layer name
    public static final Font LAYER_CONTROL_BACKGROUND_LAYER_TEXT_FONT = new Font("Arial", Font.ITALIC, 14);
    
    // Layer Control frame dimensions
    public static final int LAYER_CONTROL_WIDTH = 175;
    public static final int LAYER_CONTROL_HEIGHT = 275;
    
    // Layer control layer dimensions
    public static final int LAYER_CONTROL_LAYER_WIDTH = 100;
    public static final int LAYER_CONTROL_LAYER_HEIGHT = 38;
    public static final Dimension LAYER_CONTROL_LAYER_DIMENSION = new Dimension(LAYER_CONTROL_LAYER_WIDTH, LAYER_CONTROL_LAYER_HEIGHT);
    
    public static final Font CONTENT_MANAGER_TITLE_FONT = new Font("Arial", Font.PLAIN, 10);
    public static final Color CONTENT_MANAGER_SELECT_COLOR = new Color(95, 99, 250);    

    // Content Manager edit menu widget
    public static JMenu CONTENT_MANAGER_EDIT_MENU_WIDGET = null;


    // Icons for the menu items
    public static final ImageIcon UNDO_ENABLED_ICON;
    public static final ImageIcon UNDO_DISABLED_ICON;
    public static final ImageIcon UNDO_SELECTED_ICON;

    public static final ImageIcon CUT_ENABLED_ICON;
    public static final ImageIcon CUT_DISABLED_ICON;
    public static final ImageIcon CUT_SELECTED_ICON; 

    public static final ImageIcon COPY_ENABLED_ICON; 
    public static final ImageIcon COPY_DISABLED_ICON;
    public static final ImageIcon COPY_SELECTED_ICON;

    public static final ImageIcon PASTE_ENABLED_ICON;
    public static final ImageIcon PASTE_DISABLED_ICON;
    public static final ImageIcon PASTE_SELECTED_ICON;



    // NON-MODIFIABLE PROPERTIES, BUT CAN BE OVERRIDDEN IN THE 'properties' FILE


    // Threshhold to use for rectangle recognition
    public static double APPLICATION_RECTANGLE_THRESHHOLD;

    // Current input device being used
    public static int APPLICATION_INPUT_DEVICE;

    // Number of points allowable for a tap
    public static int APPLICATION_MAX_TAP_POINTS;

    // Location from which to pre-load audio
    public static String APPLICATION_AUDIO_DIRECTORY;
    
    // Image directory for application images
    public static String APPLICATION_IMAGE_DIRECTORY;

    // Directory to store design files (*.dms) by default
    public static String APPLICATION_DESIGN_DIRECTORY;

    // Default cursor for the application
    public static String APPLICATION_CURSOR_FILE;
    public static Cursor APPLICATION_CURSOR;

    // Default fonts for various widgets
    public static String APPLICATION_TITLE_FONT_NAME;
    public static int APPLICATION_TITLE_FONT_SIZE;
    public static int APPLICATION_TITLE_FONT_FACE;
    public static Font APPLICATION_TITLE_FONT;

    public static String APPLICATION_TEXT_FIELD_FONT_NAME;
    public static int APPLICATION_TEXT_FIELD_FONT_SIZE;
    public static int APPLICATION_TEXT_FIELD_FONT_FACE;
    public static Font APPLICATION_TEXT_FIELD_FONT;

    // Sets the fonts for the voice script editor
    public static String APPLICATION_TEXT_AREA_FONT_NAME;
    public static int APPLICATION_TEXT_AREA_FONT_SIZE;
    public static int APPLICATION_TEXT_AREA_FONT_FACE;
    public static Font APPLICATION_TEXT_AREA_FONT;

    // Base text font to use for a storyboard
    public static String STORYBOARD_TEXT_FONT_NAME;
    public static int STORYBOARD_TEXT_FONT_SIZE;
    public static int STORYBOARD_TEXT_FONT_FACE;
    public static Font STORYBOARD_TEXT_FONT;


    // Text font for each content's title in the multiview editor
    public static String MULTIVIEW_TITLE_FONT_NAME;
    public static int MULTIVIEW_TITLE_FONT_SIZE;
    public static int MULTIVIEW_TITLE_FONT_FACE;
    public static Font MULTIVIEW_TITLE_FONT;

    // Text font for a narration inside of a multiview
    public static String MULTIVIEW_NARRATION_TEXT_FONT_NAME;
    public static int MULTIVIEW_NARRATION_TEXT_FONT_SIZE;
    public static int MULTIVIEW_NARRATION_TEXT_FONT_FACE;
    public static Font MULTIVIEW_NARRATION_TEXT_FONT;


    // Width of the content manager window
    public static int CONTENT_MANAGER_WIDTH;

    // Height of the content manager window
    public static int CONTENT_MANAGER_HEIGHT;

    // Number of rows in the content manager window
    public static int CONTENT_MANAGER_ROWS;

    // Number of columns in the content manager window
    public static int CONTENT_MANAGER_COLUMNS;

    // The width of an edit (ink) stroke
    public static float INK_STROKE_LINE_WIDTH;

    // The width of a gesture stroke
    public static float GESTURE_STROKE_LINE_WIDTH;


    // Do we have a speech synthesizer installed?
    public static boolean SPEECH_SYNTHESIZER_INSTALLED;




    // MODIFIABLE PROPERTIES THAT CAN BE OBSERVED

    // Width of the main application window
    public static final Integer APPLICATION_WIDTH;

    // Height of the main application window
    public static final Integer APPLICATION_HEIGHT;
    
    // Whether an usaved change exists
    public static final Integer APPLICATION_SAVE_NEEDED;

    // Whether the user has requested to exit the application
    public static final Integer APPLICATION_EXITING;

    // The application is about to exit
    public static final Integer APPLICATION_SHUTTING_DOWN;
    
    // Internal system use only!
    public static final Integer APPLICATION_SHUT_DOWN_NOW;

    // An undoable action was just performed
    public static final Integer APPLICATION_UNDO_LIST_EMPTY;

    // Application file name
    public static final Integer APPLICATION_CURRENT_FILE_NAME;

    // Directory of last save/open
    public static final Integer APPLICATION_LAST_FILE_DIRECTORY;

    // The last directory from where media was loaded
    public static final Integer APPLICATION_LAST_MEDIA_DIRECTORY;

    // Mode of application in terms of editing/executing the storyboards
    public static final Integer APPLICATION_MODE;

    // Current data model in focus
    public static final Integer APPLICATION_CURRENT_DATA_MODEL;

    // Number of storyboard models in the application
    public static final Integer APPLICATION_STORYBOARD_COUNT;

    // Number of multiview models in the application
    public static final Integer APPLICATION_MULTIVIEW_COUNT;

    // The last visual language icon tapped, -1 means none
    public static final Integer APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED;


    // An object is selected in the storyboard view
    public static final Integer STORYBOARD_OBJECT_SELECTED;
 
    // An object exists in the storyboard clipboard
    public static final Integer STORYBOARD_OBJECT_IN_CLIPBOARD;

    // The last edit operation on a storyboard
    public static final Integer STORYBOARD_LAST_EDIT_OPERATION;


    // Whether text is selected in the voice script editor
    public static final Integer VS_TEXT_SELECTED;

    // Whether text is in the vs clipboard
    public static final Integer VS_TEXT_IN_CLIPBOARD;
    
    // Whether undo list is empty in the voice script editor
    public static final Integer VS_UNDO_LIST_EMPTY;


    // Whether an object is in the content manager clipboard
    public static final Integer CONTENT_MANAGER_OBJECT_IN_CLIPBOARD;
    
    // Number of objects selected in the content manager
    public static final Integer CONTENT_MANAGER_OBJECT_SELECTED_COUNT;

    // If the Content Manager is visible (not iconified)
    public static final Integer CONTENT_MANAGER_VISIBLE;


    // The currently selected layer
    public static final Integer LAYER_CONTROL_LAYER_SELECTED;

    // Number of layers displayed in the Layer Control,
    // which reflects the number of layers in the current storyboard model
    public static final Integer LAYER_CONTROL_LAYER_COUNT;

    // Whether the user (or system) hid the Layer Control
    public static final Integer LAYER_CONTROL_HIDDEN_BY_USER;

    // Whether the layer control is currently visible
    public static final Integer LAYER_CONTROL_VISIBLE;


    // Whether the object property frame is currently visible
    public static final Integer OBJECT_PROPERTY_FRAME_VISIBLE;

    // Whether the user (or system) hid the object property frame
    public static final Integer OBJECT_PROPERTY_FRAME_HIDDEN_BY_USER;




    // Maps a property to its value (Object)
    private static HashMap pvTable;

    // Maps a property to its listeners
    private static HashMap plTable;

    // A queue of changed properties to propagate
    private static PropertyManagerQueue queue;



    // Static initializer
    static {

	pvTable = new HashMap(50);
	plTable = new HashMap(50);
	queue = new PropertyManagerQueue();

	// Open the properties file
	PropertyFileManager userDefaults = new PropertyFileManager(PropertyFileManager.PROPERTY_FILE);
	
	Integer iValue = null;
	Double dValue = null;
	String sValue = null;
	Boolean bValue = null;


	// NON-MODIFIABLE PROPERTIES, BUT CAN BE OVERRIDDEN IN THE 'properties' FILE

	APPLICATION_RECTANGLE_THRESHHOLD = 70.0;
	dValue = (Double) userDefaults.getValue("APPLICATION_RECTANGLE_THRESHHOLD");
	if (dValue != null) {
	    APPLICATION_RECTANGLE_THRESHHOLD = dValue.doubleValue();
	}


	APPLICATION_INPUT_DEVICE = STYLUS_INPUT_DEVICE;
	iValue = (Integer) userDefaults.getValue("APPLICATION_INPUT_DEVICE");
	if (iValue != null) {
	    APPLICATION_INPUT_DEVICE = iValue.intValue();
	}

	APPLICATION_MAX_TAP_POINTS = 3;
	if (APPLICATION_INPUT_DEVICE == STYLUS_INPUT_DEVICE) {
	    APPLICATION_MAX_TAP_POINTS = 5;
	}

	
	APPLICATION_AUDIO_DIRECTORY = APPLICATION_DIRECTORY + FILE_SEPARATOR + 
	    "media" +  FILE_SEPARATOR + "audio";
	sValue = (String) userDefaults.getValue("APPLICATION_AUDIO_DIRECTORY");
	if (sValue != null) {
	    APPLICATION_AUDIO_DIRECTORY = sValue;
	}

	APPLICATION_IMAGE_DIRECTORY = APPLICATION_DIRECTORY + FILE_SEPARATOR + 
	    "data" + FILE_SEPARATOR + "images";
	sValue = (String) userDefaults.getValue("APPLICATION_IMAGE_DIRECTORY");
	if (sValue != null) {
	    APPLICATION_IMAGE_DIRECTORY = sValue;
	}

	APPLICATION_DESIGN_DIRECTORY = APPLICATION_DIRECTORY + FILE_SEPARATOR + "designs";
	sValue = (String) userDefaults.getValue("APPLICATION_DESIGN_DIRECTORY");
	if (sValue != null) {
	    APPLICATION_DESIGN_DIRECTORY = sValue;
	}

	APPLICATION_CURSOR_FILE = APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "pencil.gif";
	sValue = (String) userDefaults.getValue("APPLICATION_CURSOR_FILE");
	if (sValue != null) {
	    APPLICATION_CURSOR_FILE = sValue;
	}
	APPLICATION_CURSOR = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);


	APPLICATION_TITLE_FONT_NAME = "Arial";
	sValue = (String) userDefaults.getValue("APPLICATION_TITLE_FONT_NAME");
	if (sValue != null) {
	    APPLICATION_TITLE_FONT_NAME = sValue;
	}

	APPLICATION_TITLE_FONT_SIZE = 14;
	iValue = (Integer) userDefaults.getValue("APPLICATION_TITLE_FONT_SIZE");
	if (iValue != null) {
	    APPLICATION_TITLE_FONT_SIZE = iValue.intValue();
	}
	
	APPLICATION_TITLE_FONT_FACE = Font.BOLD;
	iValue = (Integer) userDefaults.getValue("APPLICATION_TITLE_FONT_FACE");
	if (iValue != null) {
	    APPLICATION_TITLE_FONT_FACE = iValue.intValue();
	}

	APPLICATION_TITLE_FONT = new Font(APPLICATION_TITLE_FONT_NAME,
					  APPLICATION_TITLE_FONT_FACE,
					  APPLICATION_TITLE_FONT_SIZE);
	if (APPLICATION_TITLE_FONT == null) {
	    APPLICATION_TITLE_FONT = new Font("Times", Font.PLAIN, 14);
	}


	APPLICATION_TEXT_FIELD_FONT_NAME = "Times";
	sValue = (String) userDefaults.getValue("APPLICATION_TEXT_FIELD_FONT_NAME");
	if (sValue != null) {
	    APPLICATION_TEXT_FIELD_FONT_NAME = sValue;
	}

	APPLICATION_TEXT_FIELD_FONT_SIZE = 12;
	iValue = (Integer) userDefaults.getValue("APPLICATION_TEXT_FIELD_FONT_SIZE");
	if (iValue != null) {
	    APPLICATION_TEXT_FIELD_FONT_SIZE = iValue.intValue();
	}
	
	APPLICATION_TEXT_FIELD_FONT_FACE = Font.PLAIN;
	iValue = (Integer) userDefaults.getValue("APPLICATION_TEXT_FIELD_FONT_FACE");
	if (iValue != null) {
	    APPLICATION_TEXT_FIELD_FONT_FACE = iValue.intValue();
	}

	APPLICATION_TEXT_FIELD_FONT = new Font(APPLICATION_TEXT_FIELD_FONT_NAME,
					       APPLICATION_TEXT_FIELD_FONT_FACE,
					       APPLICATION_TEXT_FIELD_FONT_SIZE);
	if (APPLICATION_TEXT_FIELD_FONT == null) {
	    APPLICATION_TEXT_FIELD_FONT = new Font("Times", Font.PLAIN, 12);
	}


	APPLICATION_TEXT_AREA_FONT_NAME = "Times";
	sValue = (String) userDefaults.getValue("APPLICATION_TEXT_AREA_FONT_NAME");
	if (sValue != null) {
	    APPLICATION_TEXT_AREA_FONT_NAME = sValue;
	}

	APPLICATION_TEXT_AREA_FONT_SIZE = 18;
	iValue = (Integer) userDefaults.getValue("APPLICATION_TEXT_AREA_FONT_SIZE");
	if (iValue != null) {
	    APPLICATION_TEXT_AREA_FONT_SIZE = iValue.intValue();
	}
	
	APPLICATION_TEXT_AREA_FONT_FACE = Font.PLAIN;
	iValue = (Integer) userDefaults.getValue("APPLICATION_TEXT_AREA_FONT_FACE");
	if (iValue != null) {
	    APPLICATION_TEXT_AREA_FONT_FACE = iValue.intValue();
	}

	APPLICATION_TEXT_AREA_FONT = new Font(APPLICATION_TEXT_AREA_FONT_NAME,
					      APPLICATION_TEXT_AREA_FONT_FACE,
					      APPLICATION_TEXT_AREA_FONT_SIZE);
	if (APPLICATION_TEXT_AREA_FONT == null) {
	    APPLICATION_TEXT_AREA_FONT = new Font("Times", Font.PLAIN, 18);
	}


	STORYBOARD_TEXT_FONT_NAME = "Anke Print";
	sValue = (String) userDefaults.getValue("STORYBOARD_TEXT_FONT_NAME");
	if (sValue != null) {
	    STORYBOARD_TEXT_FONT_NAME = sValue;
	}

	STORYBOARD_TEXT_FONT_SIZE = 22;
	iValue = (Integer) userDefaults.getValue("STORYBOARD_TEXT_FONT_SIZE");
	if (iValue != null) {
	    STORYBOARD_TEXT_FONT_SIZE = iValue.intValue();
	}
	
	STORYBOARD_TEXT_FONT_FACE = Font.PLAIN;
	iValue = (Integer) userDefaults.getValue("STORYBOARD_TEXT_FONT_FACE");
	if (iValue != null) {
	    STORYBOARD_TEXT_FONT_FACE = iValue.intValue();
	}
	
	STORYBOARD_TEXT_FONT = new Font(STORYBOARD_TEXT_FONT_NAME,
					STORYBOARD_TEXT_FONT_FACE,
					STORYBOARD_TEXT_FONT_SIZE);
	if (STORYBOARD_TEXT_FONT == null) {
	    STORYBOARD_TEXT_FONT = new Font("Times", Font.PLAIN, 20);
	}



	MULTIVIEW_TITLE_FONT_NAME = "Arial";
	sValue = (String) userDefaults.getValue("MULTIVIEW_TITLE_FONT_NAME");
	if (sValue != null) {
	    MULTIVIEW_TITLE_FONT_NAME = sValue;
	}

	MULTIVIEW_TITLE_FONT_SIZE = 14;
	iValue = (Integer) userDefaults.getValue("MULTIVIEW_TITLE_FONT_SIZE");
	if (iValue != null) {
	    MULTIVIEW_TITLE_FONT_SIZE = iValue.intValue();
	}
	
	MULTIVIEW_TITLE_FONT_FACE = Font.BOLD;
	iValue = (Integer) userDefaults.getValue("MULTIVIEW_TITLE_FONT_FACE");
	if (iValue != null) {
	    MULTIVIEW_TITLE_FONT_FACE = iValue.intValue();
	}
	
	MULTIVIEW_TITLE_FONT = new Font(MULTIVIEW_TITLE_FONT_NAME,
					MULTIVIEW_TITLE_FONT_FACE,
					MULTIVIEW_TITLE_FONT_SIZE);
	if (MULTIVIEW_TITLE_FONT == null) {
	    MULTIVIEW_TITLE_FONT = new Font("Arial", Font.BOLD, 14);
	}


	MULTIVIEW_NARRATION_TEXT_FONT_NAME = "Arial";
	sValue = (String) userDefaults.getValue("MULTIVIEW_NARRATION_TEXT_FONT_NAME");
	if (sValue != null) {
	    MULTIVIEW_NARRATION_TEXT_FONT_NAME = sValue;
	}

	MULTIVIEW_NARRATION_TEXT_FONT_SIZE = 14;
	iValue = (Integer) userDefaults.getValue("MULTIVIEW_NARRATION_TEXT_FONT_SIZE");
	if (iValue != null) {
	    MULTIVIEW_NARRATION_TEXT_FONT_SIZE = iValue.intValue();
	}
	
	MULTIVIEW_NARRATION_TEXT_FONT_FACE = Font.BOLD;
	iValue = (Integer) userDefaults.getValue("MULTIVIEW_NARRATION_TEXT_FONT_FACE");
	if (iValue != null) {
	    MULTIVIEW_NARRATION_TEXT_FONT_FACE = iValue.intValue();
	}
	
	MULTIVIEW_NARRATION_TEXT_FONT = new Font(MULTIVIEW_NARRATION_TEXT_FONT_NAME,
						 MULTIVIEW_NARRATION_TEXT_FONT_FACE,
						 MULTIVIEW_NARRATION_TEXT_FONT_SIZE);
	if (MULTIVIEW_NARRATION_TEXT_FONT == null) {
	    MULTIVIEW_NARRATION_TEXT_FONT = new Font("Arial", Font.PLAIN, 14);
	}


	CONTENT_MANAGER_WIDTH = 525;
	iValue = (Integer) userDefaults.getValue("CONTENT_MANAGER_WIDTH");
	if (iValue != null) {
	    CONTENT_MANAGER_WIDTH = iValue.intValue();
	}

	CONTENT_MANAGER_HEIGHT = 425;
	iValue = (Integer) userDefaults.getValue("CONTENT_MANAGER_HEIGHT");
	if (iValue != null) {
	    CONTENT_MANAGER_HEIGHT = iValue.intValue();
	}

	CONTENT_MANAGER_ROWS = 10;
	iValue = (Integer) userDefaults.getValue("CONTENT_MANAGER_ROWS");
	if (iValue != null) {
	    CONTENT_MANAGER_ROWS = iValue.intValue();
	}

	CONTENT_MANAGER_COLUMNS = 5;
	iValue = (Integer) userDefaults.getValue("CONTENT_MANAGER_COLUMNS");
	if (iValue != null) {
	    CONTENT_MANAGER_COLUMNS = iValue.intValue();
	}

	// Set the stroke widths
	INK_STROKE_LINE_WIDTH = 2.0f;
	dValue = (Double) userDefaults.getValue("INK_STROKE_LINE_WIDTH");
	if (dValue != null) {
	    INK_STROKE_LINE_WIDTH = (float) dValue.doubleValue();
	}

	GESTURE_STROKE_LINE_WIDTH = 4.0f;
	dValue = (Double) userDefaults.getValue("GESTURE_STROKE_LINE_WIDTH");
	if (dValue != null) {
	    GESTURE_STROKE_LINE_WIDTH = (float) dValue.doubleValue();
	}



	// Icons for the menu items
	UNDO_ENABLED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "undo-gesture-enabled.gif");
	UNDO_DISABLED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "undo-gesture-disabled.gif");
	UNDO_SELECTED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "undo-gesture-selected.gif");
	
	CUT_ENABLED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "cut-gesture-enabled.gif");
	CUT_DISABLED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "cut-gesture-disabled.gif");
	CUT_SELECTED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "cut-gesture-selected.gif");
	
	COPY_ENABLED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "copy-gesture-enabled.gif");
	COPY_DISABLED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "copy-gesture-disabled.gif");
	COPY_SELECTED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "copy-gesture-selected.gif");
	
	PASTE_ENABLED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "paste-gesture-enabled.gif");
	PASTE_DISABLED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "paste-gesture-disabled.gif");
	PASTE_SELECTED_ICON = new ImageIcon(APPLICATION_IMAGE_DIRECTORY + FILE_SEPARATOR + "paste-gesture-selected.gif");
	




	SPEECH_SYNTHESIZER_INSTALLED = true;
	bValue = (Boolean) userDefaults.getValue("SPEECH_SYNTHESIZER_INSTALLED");
	if (bValue != null) {
	    SPEECH_SYNTHESIZER_INSTALLED = (boolean) bValue.booleanValue();
	}


	// MODIFIABLE PROPERTIES THAT CAN BE OBSERVED

	Dimension screenDimension = Toolkit.getDefaultToolkit().getScreenSize();

	APPLICATION_WIDTH = new Integer(0);
	pvTable.put(APPLICATION_WIDTH, new Integer(screenDimension.width));
	plTable.put(APPLICATION_WIDTH, new Vector(1));
	iValue = (Integer) userDefaults.getValue("APPLICATION_WIDTH");
	if (iValue != null) {
	    setProperty(APPLICATION_WIDTH, iValue, false);
	}

	APPLICATION_HEIGHT = new Integer(1);
	pvTable.put(APPLICATION_HEIGHT, new Integer(screenDimension.height));
	plTable.put(APPLICATION_HEIGHT, new Vector(1));
	iValue = (Integer) userDefaults.getValue("APPLICATION_HEIGHT");
	if (iValue != null) {
	    setProperty(APPLICATION_HEIGHT, iValue, false);
	}

	APPLICATION_SAVE_NEEDED = new Integer(2);
	pvTable.put(APPLICATION_SAVE_NEEDED, Boolean.FALSE);
	plTable.put(APPLICATION_SAVE_NEEDED, new Vector(5));

	APPLICATION_EXITING = new Integer(3);
	pvTable.put(APPLICATION_EXITING, Boolean.FALSE);
	plTable.put(APPLICATION_EXITING, new Vector(2));

	APPLICATION_SHUTTING_DOWN = new Integer(4);
	pvTable.put(APPLICATION_SHUTTING_DOWN, Boolean.FALSE);
	plTable.put(APPLICATION_SHUTTING_DOWN, new Vector(5));

	APPLICATION_SHUT_DOWN_NOW = new Integer(5);
	pvTable.put(APPLICATION_SHUT_DOWN_NOW, Boolean.FALSE);
	plTable.put(APPLICATION_SHUT_DOWN_NOW, new Vector(1));

	APPLICATION_UNDO_LIST_EMPTY = new Integer(6);
	pvTable.put(APPLICATION_UNDO_LIST_EMPTY, Boolean.TRUE);
	plTable.put(APPLICATION_UNDO_LIST_EMPTY, new Vector(1));

	APPLICATION_CURRENT_FILE_NAME = new Integer(7);
	pvTable.put(APPLICATION_CURRENT_FILE_NAME, null);
	plTable.put(APPLICATION_CURRENT_FILE_NAME, new Vector(1));

	APPLICATION_LAST_FILE_DIRECTORY = new Integer(8);
	pvTable.put(APPLICATION_LAST_FILE_DIRECTORY, APPLICATION_DESIGN_DIRECTORY);
	plTable.put(APPLICATION_LAST_FILE_DIRECTORY, new Vector(1));

	APPLICATION_LAST_MEDIA_DIRECTORY = new Integer(9);
	pvTable.put(APPLICATION_LAST_MEDIA_DIRECTORY, APPLICATION_DIRECTORY + FILE_SEPARATOR + "media");
	plTable.put(APPLICATION_LAST_MEDIA_DIRECTORY, new Vector(1));

	APPLICATION_MODE = new Integer(10);
	pvTable.put(APPLICATION_MODE, new Integer(APPLICATION_EDIT));
	plTable.put(APPLICATION_MODE, new Vector(5));

	APPLICATION_CURRENT_DATA_MODEL = new Integer(11);
	pvTable.put(APPLICATION_CURRENT_DATA_MODEL, null);
	plTable.put(APPLICATION_CURRENT_DATA_MODEL, new Vector(5));

	APPLICATION_STORYBOARD_COUNT = new Integer(12);
	pvTable.put(APPLICATION_STORYBOARD_COUNT, new Integer(0));
	plTable.put(APPLICATION_STORYBOARD_COUNT, new Vector(5));

	APPLICATION_MULTIVIEW_COUNT = new Integer(13);
	pvTable.put(APPLICATION_MULTIVIEW_COUNT, new Integer(0));
	plTable.put(APPLICATION_MULTIVIEW_COUNT, new Vector(5));
		
	APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED = new Integer(14);
	pvTable.put(APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED, new Integer(-1));
	plTable.put(APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED, new Vector(2));
	    
	STORYBOARD_OBJECT_SELECTED = new Integer(16);
	pvTable.put(STORYBOARD_OBJECT_SELECTED, Boolean.FALSE);
	plTable.put(STORYBOARD_OBJECT_SELECTED, new Vector(1));

	STORYBOARD_OBJECT_IN_CLIPBOARD = new Integer(17);
	pvTable.put(STORYBOARD_OBJECT_IN_CLIPBOARD, Boolean.FALSE);
	plTable.put(STORYBOARD_OBJECT_IN_CLIPBOARD, new Vector(1));

	STORYBOARD_LAST_EDIT_OPERATION = new Integer(18);
	pvTable.put(STORYBOARD_LAST_EDIT_OPERATION, new Integer(-1));
	plTable.put(STORYBOARD_LAST_EDIT_OPERATION, new Vector(5));

	
	VS_TEXT_SELECTED = new Integer(19);
	pvTable.put(VS_TEXT_SELECTED, Boolean.FALSE);
	plTable.put(VS_TEXT_SELECTED, new Vector(1));
	
	VS_TEXT_IN_CLIPBOARD = new Integer(20);
	pvTable.put(VS_TEXT_IN_CLIPBOARD, Boolean.FALSE);
	plTable.put(VS_TEXT_IN_CLIPBOARD, new Vector(1));

	VS_UNDO_LIST_EMPTY = new Integer(21);
	pvTable.put(VS_UNDO_LIST_EMPTY, Boolean.TRUE);
	plTable.put(VS_UNDO_LIST_EMPTY, new Vector(1));
	

	CONTENT_MANAGER_OBJECT_IN_CLIPBOARD = new Integer(22);
	pvTable.put(CONTENT_MANAGER_OBJECT_IN_CLIPBOARD, Boolean.FALSE);
	plTable.put(CONTENT_MANAGER_OBJECT_IN_CLIPBOARD, new Vector(1));

	CONTENT_MANAGER_OBJECT_SELECTED_COUNT = new Integer(23);
	pvTable.put(CONTENT_MANAGER_OBJECT_SELECTED_COUNT, new Integer(0));
	plTable.put(CONTENT_MANAGER_OBJECT_SELECTED_COUNT, new Vector(1));

	CONTENT_MANAGER_VISIBLE = new Integer(24);
	pvTable.put(CONTENT_MANAGER_VISIBLE, Boolean.FALSE);
	plTable.put(CONTENT_MANAGER_VISIBLE, new Vector(2));


	LAYER_CONTROL_LAYER_SELECTED = new Integer(25);
	pvTable.put(LAYER_CONTROL_LAYER_SELECTED, new Integer(0));
	plTable.put(LAYER_CONTROL_LAYER_SELECTED, new Vector(2));

	LAYER_CONTROL_LAYER_COUNT = new Integer(26);
	pvTable.put(LAYER_CONTROL_LAYER_COUNT, new Integer(3));
	plTable.put(LAYER_CONTROL_LAYER_COUNT, new Vector(2));

	// If the Layer Control was hidden by the user
	LAYER_CONTROL_HIDDEN_BY_USER = new Integer(27);
	pvTable.put(LAYER_CONTROL_HIDDEN_BY_USER, Boolean.FALSE);
	plTable.put(LAYER_CONTROL_HIDDEN_BY_USER, new Vector(1));

	LAYER_CONTROL_VISIBLE = new Integer(28);
	pvTable.put(LAYER_CONTROL_VISIBLE, Boolean.TRUE);
	plTable.put(LAYER_CONTROL_VISIBLE, new Vector(2));

	OBJECT_PROPERTY_FRAME_HIDDEN_BY_USER = new Integer(29);
	pvTable.put(OBJECT_PROPERTY_FRAME_HIDDEN_BY_USER, Boolean.TRUE);
	plTable.put(OBJECT_PROPERTY_FRAME_HIDDEN_BY_USER, new Vector(1));

	OBJECT_PROPERTY_FRAME_VISIBLE = new Integer(30);
	pvTable.put(OBJECT_PROPERTY_FRAME_VISIBLE, Boolean.FALSE);
	plTable.put(OBJECT_PROPERTY_FRAME_VISIBLE, new Vector(2));

    }



    public static void signal() {
	queue.signal();
    }

    // Flush the pending properties
    public static void flushQ() {
	queue.flush();
    }

    public static void setProperty(Integer property, Object newValue) {
	setProperty(property, newValue, true);
    }
    
    public static void setProperty(Integer property, Object newValue, boolean notify) {
	Object oldValue = pvTable.get(property);
	pvTable.put(property, newValue);
	
	if (notify) {
	    queue.insert(property, oldValue, newValue);
	}
    }


    public static Object getProperty(Integer property) {
	return (pvTable.get(property));
    }


    public static void addPropertyListener(Integer property, PropertyListenerInterface listener) {
	Vector v = (Vector) plTable.get(property);
	if ((v != null) && (! v.contains(listener))) {
	    v.add(listener);
	} 
    }

    public static void removePropertyListener(Integer property, PropertyListenerInterface listener) {
	Vector v = (Vector) plTable.get(property);
	if (v != null) {
	    v.remove(listener);
	}
    }

    protected static void notifyPropertyListeners(Integer property, Object oldValue, Object newValue) {
	Vector v = (Vector) plTable.get(property);
	if (v != null) {
	    Iterator it = ((Vector) v.clone()).iterator();
	    while (it.hasNext()) {
		PropertyListenerInterface listener = (PropertyListenerInterface) it.next();
		listener.propertyChanged(property, oldValue, newValue);
	    }
	}
    }



    // Constructor
    private PropertyManager() {
	// Empty
    }


}



