/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ChangeMonitorQueue.java

  A queue of changed objects for the CM to handle. When an object changes such as a variable or CMO, the CM is notified. However, if the CM immediately processes the notification, then a DFS of the annotations is performed. In most cases, a BFS search is desirable. Thus, all changed objects are placed on a queue and Swing's invokeLater method is used to execute the change notifications within the CM. By queuing the change notifications, any notifiactions referring to the same object can be collapsed into a single notification. Also, using the invokeLater method supports interactivity while the firing of the DEMAIS rules are taking place.
*/

// Current package
package edu.uofminnesota.DEMAIS.runtime;

// Import needed packages
import java.lang.*;
import java.util.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;



public class ChangeMonitorQueue extends Vector
    implements Runnable {


    // Constructors
    public ChangeMonitorQueue() {
	this(25);
    }

    public ChangeMonitorQueue(int capacity) {
	super(capacity);
    }


    // Public methods
    public void insert(NamedModel model) {
	// Only insert the object if a notification for that object is not already present
	if (! contains(model)) {
	    add(model);
	}

	if (size() == 1) {
	    SwingUtilities.invokeLater(this);
	}
    }


    // Runnable interface
    public void run() {
	ChangeMonitor cm = ChangeMonitor.getCM();
	while (size() > 0) {
	    // Remove the head of the queue
	    NamedModel object = (NamedModel) remove(0);
	    cm.doObjectChanged(object);    
	}
    }
}
