/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: DEMAISRuntime.java
*/

// Current package
package edu.uofminnesota.DEMAIS.runtime;

// Import needed packages
import java.lang.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.core.annotation.*;
import edu.uofminnesota.DEMAIS.core.parser.*;



public class DEMAISRuntime {

    public static void loadDataModel(DataModel model) {
  
	// Load each annotation in the model
	Iterator it = model.getAnnotations().iterator();
	while (it.hasNext()) {
	    Annotation annotation = (Annotation) it.next();
	    // Annotation has already been parsed

	    // Reset the action fired flag
	    annotation.setActionFired(false);

	    try {
		// Find and monitor the traceable objects
		ChangeMonitor.getCM().monitorAnnotation(annotation);
		
		// Perform the initial evaluation of the annotation
		LogicValue lv = Evaluator.getEvaluator().evaluateAnnotation(annotation);
		//System.out.println(annotation.toString() + ": " + lv.toString());

	    } catch (EvaluationException xcptn) {
		System.out.println(xcptn.toString());
	    }
	}
    }


    public static void removeDataModel(DataModel model) {
	Iterator it = model.getAnnotations().iterator();
	while (it.hasNext()) {
	    Annotation annotation = (Annotation) it.next();
	    ChangeMonitor.getCM().removeAnnotation(annotation);
	    Evaluator.getEvaluator().removeAnnotation(annotation);
	    Scheduler.getScheduler().removeAnnotation(annotation);
	}	
    }

    
    // Completely clear out the entire runtime system!
    public static void removeAll() {
	ChangeMonitor.getCM().removeAll();
	Evaluator.getEvaluator().removeAll();
	Scheduler.getScheduler().removeAll();
    }

}
