/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: EvaluationException.java

  This exception stores information regarding any error that occurs during the evaluation of an annotation's condition or action.
*/

// Current package
package edu.uofminnesota.DEMAIS.runtime;

// Import needed packages
import edu.uofminnesota.DEMAIS.core.annotation.*;
import edu.uofminnesota.DEMAIS.core.parser.*;



public class EvaluationException extends Exception {
    
    // Store information about the exception
    private String errorMsg;
    private Opcode offendingCode;
    private Annotation annotation;


    public EvaluationException() {
	this(null, null, null);
    }

    public EvaluationException(String msg) {
	this(msg, null, null);
    }

    public EvaluationException(String msg, Opcode c) {
	this(msg, c, null);
    }

    public EvaluationException(String msg, Opcode c, Annotation a) {
	errorMsg = msg;
	offendingCode = c;
	annotation = a;
    }


    public void setAnnotation(Annotation a) {
	annotation = a;
    }

    public void setCode(Opcode c) {
	offendingCode = c;
    }

    public String toString() {
	if ( (annotation != null) && (offendingCode != null)) {
	    return ("Evaluation Error: " + errorMsg + "\n" +
		    "Annotation: " + annotation.toString() + "\n" +
		    "Error opcode: " + offendingCode.toString());
	} else if (annotation != null) {
	    return ("Evaluation Error: " + errorMsg + "\n" +
		    "Annotation: " + annotation.toString());
	} else if (offendingCode != null) {
	    return ("Evaluation Error: " + errorMsg + "\n" +
		    "Error opcode: " + offendingCode.toString());
	} else {
	    return ("Evaluation Error: " + errorMsg);
	}
    }
}
