/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: LogicCMedia.java
*/

// Current package
package edu.uofminnesota.DEMAIS.runtime;

// Import needed packages
import java.lang.*;

/*
  During the evaluation of an annotation condition, we need to transform a media object into a logic value
*/

public class LogicCMedia extends LogicValue {

    
    // Constructors
    public LogicCMedia() {
	this(0.0, 0.0, 0.0);
    }

    public LogicCMedia(double v, double r) {
	this(v, r, 0.0);
    }

    public LogicCMedia(double v, double r, double p) {
	super(v, r, p);
    }


    // Public methods
    public LogicValue and(WBT wbt) {
	LogicValue lv = null;

	if (isTrueAlways()) {
	    lv = new WBT(wbt.prediction);
	} else {
	    lv = LogicValue.FALSE;
	}
	return lv;
    }


    public LogicValue and(WBF wbf) {
	LogicValue lv = null;

	if (isTrueAlways()) {
	    lv = new WBF(wbf.prediction);
	} else {
	    lv = LogicValue.FALSE;
	}
	return lv;
    }


    public LogicValue and(LogicConstant lc) {
	LogicValue lv = null;

	if (isTrueAlways() && lc.isTrueAlways()) {
	    lv = LogicValue.TRUE;
	} else {
	    lv = LogicValue.FALSE;
	}
	return lv;
    }


    public LogicValue and(LogicCMedia cm) {
	LogicValue lv = null;

	if (isTrueAlways() && cm.isTrueAlways()) {
	    lv = LogicValue.TRUE;
	} else {
	    lv = LogicValue.FALSE;
	}
	return lv;
    }

    public LogicValue and(LogicValue lv) {
	return lv.and(this);
    }


    public LogicValue or(WBT wbt) {
	LogicValue lv = null;
	
	if (isTrueAlways()) {
	    lv = LogicValue.TRUE;
	} else {
	    lv = new WBT(wbt.prediction);
	}
	return lv;
    }


    public LogicValue or(WBF wbf) {
	LogicValue lv = null;
	
	if (isTrueAlways()) {
	    lv = LogicValue.TRUE;
	} else {
	    lv = new WBF(wbf.prediction);
	}
	return lv;
    }


    public LogicValue or(LogicConstant lc) {
	LogicValue lv = null;

	if (isTrueAlways() || lc.isTrueAlways()) {
	    lv = LogicValue.TRUE;
	} else {
	    lv = LogicValue.FALSE;
	}
	return lv;
    }


    public LogicValue or(LogicCMedia cm) {
	LogicValue lv = null;

	if (isTrueAlways() || cm.isTrueAlways()) {
	    lv = LogicValue.TRUE;
	} else {
	    lv = LogicValue.FALSE;
	}
	return lv;
    }


    public LogicValue or(LogicValue lv) {
	return lv.or(this);
    }


    // Convert to a predictive logic value
    public LogicValue not() {
	LogicValue lv = null;
	if (isTrueAlways()) {
	    lv = LogicValue.FALSE; 
	} else {
	    lv = LogicValue.TRUE;
	}

	return lv;
    }


    public boolean isFalseNow() {
	return isFalseAlways();
    }

    public boolean isFalseAlways() {
	return (value == 0.0);
    }

    public boolean isTrueNow() {
	return isTrueAlways();
    }

    public boolean isTrueAlways() {
	return (value != 0.0);
    }

    public String toString() {
	if (isFalseAlways()) {
	    return "WBT(-1)";
	} else {
	    return "WBF(-1)";
	}
    }

}


