/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: SchedulerQueue.java

  A synchronized queue for the scheduler's actions. Because of the Timer thread, we must synchronize access to the queue. The queue of actions is sorted by the time at which an action is to be invoked.

*/


// Current package
package edu.uofminnesota.DEMAIS.runtime;

// Import needed packages
import java.lang.*;
import java.util.*;



public class SchedulerQueue extends Vector { 

    // Constructors
    protected SchedulerQueue() {
	this(10);
    }

    protected SchedulerQueue(int capacity) {
	super(capacity);
    }


    // Returns the index at which the item was inserted
    protected int insertActionItem(long scheduledTime, SchedulerActionItem scheduledAction) {
	int location = size();

	int i=0;
	Iterator it = iterator();
	while (it.hasNext()) {   
	    SchedulerActionItem item = (SchedulerActionItem) it.next(); 
	    if (scheduledTime < item.getTime()) {
		location = i;
		break;
	    }
	    ++i;
	}

	// Insert the item into the queue
	insertElementAt(scheduledAction, location);

	// Return the inserted location 
	return location;
    }

}
