/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: WBT.java
*/

// Current package
package edu.uofminnesota.DEMAIS.runtime;

// Import needed packages
import java.lang.*;



public class WBT extends LogicValue {


    // Constructors
    public WBT() {
	this(0.0, 0.0, LogicValue.INFINITY);
    }

    public WBT(double p) {
	this(0.0, 0.0, p);
    }

    public WBT(double v, double r, double p) {
	super(v, r, p);
    }


    // Public methods
    public LogicValue and(WBT wbt) {
	LogicValue lv = null;
	// WBT(x) && WBT(y)
	double x = this.prediction;
	double y = wbt.prediction;

	if (isFalseAlways() || wbt.isFalseAlways()) {
	    lv = LogicValue.FALSE;
	} else if (x <= y) {
	    lv = new WBT(y);
	} else {
	    lv = new WBT(x);
	}
	return lv;
    }


    public LogicValue and(WBF wbf) {
	LogicValue lv = null;
	// WBT(x) && WBF(y)
	double x = this.prediction;
	double y = wbf.prediction;

	if (isFalseAlways()) {
	    lv = LogicValue.FALSE;
	} else if (wbf.isTrueAlways()) {
	    lv = new WBT(x);
	} else if (x < y) {
	    lv = new WBT(x);
	} else {
	    lv = LogicValue.FALSE;
	}
	return lv;
    }


    public LogicValue and(LogicConstant lc) {
	return (lc.and(this));
    }

    public LogicValue and(LogicCMedia cm) {
	return (cm.and(this));
    }

    public LogicValue and(LogicValue lv) {
	return (lv.and(this));
    }


    public LogicValue or(WBT wbt) {
	LogicValue lv = null;
	// WBT(x) || WBT(y)
	double x = this.prediction;
	double y = wbt.prediction;

	if (isFalseAlways()) {
	    lv = new WBT(y);
	} else if (wbt.isFalseAlways()) {
	    lv = new WBT(x);
	} else if (x <= y) {
	    lv = new WBT(x);
	} else {
	    lv = new WBT(y);
	}
	return lv;
    }


    public LogicValue or(WBF wbf) {
	LogicValue lv = null;
	// WBT(x) || WBF(y)
	double x = this.prediction;
	double y = wbf.prediction;

	if (isFalseAlways()) {
	    lv = new WBF(y);
	} else if (wbf.isTrueAlways()) {
	    lv = LogicValue.TRUE;
	} else if (x > y) {
	    lv = new WBF(y);
	} else {
	    lv = LogicValue.TRUE;
	}
	return lv;
    }

    
    public LogicValue or(LogicConstant lc) {
	return (lc.or(this));
    }

    public LogicValue or(LogicCMedia cm) {
	return (cm.or(this));
    }

    public LogicValue or(LogicValue lv) {
	return (lv.or(this));
    }


    public LogicValue not() {
	return (new WBF(prediction));
    }


    public boolean isFalseNow() {
	return true;
    }

    public boolean isFalseAlways() {
	return (prediction == LogicValue.INFINITY);
    }

    public boolean isTrueNow() {
	return false;
    }

    public boolean isTrueAlways() {
	return false;
    }

    public String toString() {
	return "WBT(" + Double.toString(prediction) + ")";
    }
}


