/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentEditorNamePanel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;



public class ContentEditorNamePanel extends JPanel {

    private JLabel contentLabel;
    private JTextField contentField;
    private Listener listener;
    private boolean fromMe;


    public ContentEditorNamePanel() {
	super(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	// Create the listener
	listener = new Listener();
	
	c.weightx = 1.0;
	c.gridx = 0;
	c.gridy = 0;
	c.anchor = GridBagConstraints.WEST;
	c.fill = GridBagConstraints.BOTH;
	add(createContentEntry(), c);
	fromMe = false;

	// Set the current name
	DataModel model = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	if (model != null) {
	    model.addNameListener(listener);
	    fromMe = true;
	    contentField.setText(model.getName());
	    fromMe = false;
	}
    }
    


    public String getName() {
	return contentField.getText();
    }


    private JPanel createContentEntry() {
	JPanel container = new JPanel(new GridBagLayout());	
	GridBagConstraints c = new GridBagConstraints();
		
	contentLabel = new JLabel("Storyboard");
	contentLabel.setFont(PropertyManager.APPLICATION_TITLE_FONT);
	c.gridx = 0;
	c.gridy = 0;
	container.add(contentLabel, c);
	
	contentField = new JTextField(15);
	contentField.setFont(PropertyManager.APPLICATION_TEXT_FIELD_FONT);

	contentField.getDocument().addDocumentListener(listener);

	c.gridx = 1;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.insets = new Insets(0, 4, 0, 0);
	container.add(contentField, c);
	
	return container;
    }

    
    private void handleNameChanged() {
	DataModel model = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	if (model != null) {
	    model.setName(contentField.getText(), true);
	}
    }
    
	

    class Listener implements DocumentListener,
			      PropertyListenerInterface,
			      NamedModelNameListenerInterface {

	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	}


	public void changedUpdate(DocumentEvent e) {
	}
	public void insertUpdate(DocumentEvent e) {
	    if (! fromMe) {
		handleNameChanged();
	    }
	}
	public void removeUpdate(DocumentEvent e) {
	    if (! fromMe) {
		handleNameChanged();
	    }
	}
 

	// PropertyManager Interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		DataModel fromModel = (DataModel) oldValue;
		DataModel toModel = (DataModel) newValue;
		if (fromModel != null) {
		    fromModel.removeNameListener(this);
		}

		if (toModel != null) {
		    // Causes a document change event
		    fromMe = true;
		    contentField.setEnabled(true);
		    contentField.setText(toModel.getName());
		    fromMe = false;
		    toModel.addNameListener(this);
		    if (toModel instanceof StoryboardModel) {
			contentLabel.setText("Storyboard");
		    } else if (toModel instanceof MultiViewModel) {
			contentLabel.setText("MultiView");
		    }

		} else {
		    // Causes a document change event
		    fromMe = true;
		    contentField.setEnabled(false);
		    contentField.setText("");
		    fromMe = false;
		}
	    }
	}
	
	// Name listener interface
	public void nameChanged(NamedModel model, String oldName, String newName) {
	    // Setting the text causes a document change event - careful of inifinite loops
	    if (! newName.equals(contentField.getText())) {
		fromMe = true;
		contentField.setText(newName);
		fromMe = false;
	    }
	}    
    }
}

