/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentEditorPanel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;




public class ContentEditorPanel extends JPanel {

    private ContentEditorPanel() {
    }

    public ContentEditorPanel(Storyboard storyboard) {
	super(new GridBagLayout());
	
	GridBagConstraints c = new GridBagConstraints();
	
	// Will hold the shortcut panel
	JPanel shortcutPanelC = new JPanel(new GridBagLayout());
	
	// Will hold the name panel
	JPanel namePanelC = new JPanel(new GridBagLayout());

	// Will hold the visual language panel
	JPanel visualLanguagePanelC = new JPanel(new GridBagLayout());


	// Use a JSplitPane to separate the shortcut panel from the name panel
	JSplitPane shortcutNameSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, shortcutPanelC, namePanelC);
	shortcutNameSplitPane.setOneTouchExpandable(false);
	shortcutNameSplitPane.setResizeWeight(0.0);

	// Use a JSplitPane to separate the shortcut/nae pane from the visual language panel
	JSplitPane snEventSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, shortcutNameSplitPane, visualLanguagePanelC);

  	snEventSplitPane.setOneTouchExpandable(false);
	snEventSplitPane.setResizeWeight(1.0);

	JPanel contentPanelC = new JPanel(new GridBagLayout());

	// Use a JSplitPane to separate the bottom split pane from the content sketching canvas
	JSplitPane bottomTopSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, contentPanelC, snEventSplitPane);
  	bottomTopSplitPane.setOneTouchExpandable(false);
	bottomTopSplitPane.setResizeWeight(1.0);


	ContentEditorNamePanel np = new ContentEditorNamePanel();
	c.gridx = 0;
	c.gridy = 0;       
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.insets = new Insets(0, 5, 0, 0);
	namePanelC.add(np, c);

	VisualLanguagePanel vlPanel = new VisualLanguagePanel();
	c.gridx = 0;
	c.gridy = 0;       
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.anchor = GridBagConstraints.CENTER;
	c.insets = new Insets(0, 5, 0, 0);
	//c.insets = new Insets(0, 2, 0, 0);
	visualLanguagePanelC.add(vlPanel, c);

	ShortcutPanel shortcutPanel = new ShortcutPanel();
	c.gridx = 0;
	c.gridy = 0;
	c.anchor = GridBagConstraints.WEST;
	c.fill = GridBagConstraints.NONE;
	c.weighty = 0.0;
	c.insets = new Insets(0, 0, 0, 0);
	shortcutPanelC.add(shortcutPanel, c);

	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	ContentPanel contentPanel = new ContentPanel(storyboard);
	contentPanelC.add(contentPanel, c);
	
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	add(bottomTopSplitPane, c);
    }
   
}


