/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentPanel.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.runtime.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.multiview.*;




public class ContentPanel extends JPanel {

    // A cached copy of the constraints used for the storyboard
    private GridBagConstraints constraints;

    // Property listener
    private Listener listener;


    private ContentPanel() {
    }

    public ContentPanel(Storyboard storyboard) {
	setLayout(new GridBagLayout());
	listener = new Listener();

	constraints = new GridBagConstraints();
	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = 1.0;
	constraints.weighty = 1.0;
	add(storyboard, constraints);
    }

  
  
    class Listener implements PropertyListenerInterface {

	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	}

	
	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {

	    if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		DataModel fromModel = (DataModel) oldValue;
		DataModel toModel = (DataModel) newValue;

		int mode = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		switch (mode) {
		    
		case PropertyManager.APPLICATION_EDIT: 
		    // Switching data models while in edit mode
		    if (fromModel != null) {
			// Inform the current storyboard its about to be switched out
			fromModel.getStoryboard().onUnInstall();
		    }

		    // remove it
		    removeAll();
		    
		    if (toModel != null) {
			// Inform the new storyboard its about to be switched in
			toModel.getStoryboard().onInstall();
			add(toModel.getStoryboard(), constraints);
			toModel.getStoryboard().requestFocus();
		    } 
		    
		    revalidate();
		    paintImmediately(0, 0, getWidth(), getHeight());

		    break;

		case PropertyManager.APPLICATION_PLAY: 
		    // Switching data models while in play mode
		    if (fromModel != null) {
			// Inform the current storyboard its about to be switched out
			fromModel.getStoryboard().onRuntimeUnInstall(toModel);
			DEMAISRuntime.removeDataModel(fromModel);
		    }

		    // remove it
		    removeAll();

		    if (toModel != null) {
			// Inform the new storyboard its about to be switched in
			toModel.getStoryboard().onRuntimeInstall();
			add(toModel.getStoryboard(), constraints);
			toModel.getStoryboard().requestFocus();
			// Execute the annotations on this model
			DEMAISRuntime.loadDataModel(toModel);
		    } 

		    revalidate();
		    paintImmediately(0, 0, getWidth(), getHeight());

		    break;
	     
		default:
		    break;
		}

		
	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		DataModel cModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
		int fromMode = ((Integer) oldValue).intValue();
		int toMode = ((Integer) newValue).intValue();

		switch (toMode) {
		    
		case PropertyManager.APPLICATION_EDIT: 

		    if (fromMode == PropertyManager.APPLICATION_PLAY) {
			// Switching from play mode to edit mode

			// Shut off the runtime engine right away!
			DEMAISRuntime.removeAll();
			
			// Turn off all audio
			AudioModel.resetModels();
			
			// Unoperationalize the storyboards
			Iterator it = StoryboardModel.getStoryboardModels().iterator();
			while (it.hasNext()) {
			    StoryboardModel model = (StoryboardModel) it.next();
			    model.unOperationalize();
			}

			// Unoperationalize the narrations
			it = NarrationModel.getNarrationModels().iterator(); 
			while (it.hasNext()) {
			    NarrationModel model = (NarrationModel) it.next();
			    model.unOperationalize();
			}

			
			// Unoperationalize the multiviews
			it = MultiViewModel.getMultiViewModels().iterator();
			while (it.hasNext()) {
			    MultiViewModel model = (MultiViewModel) it.next();
			    model.unOperationalize();
			}
			
			// Locate model to switch to
			StoryboardModel toModel = null;
			if (cModel instanceof StoryboardModel) {
			    toModel = (StoryboardModel) cModel;
			} else {
			    toModel = StoryboardModel.getFirstStoryboardModel();
			}

			// Set the frame cursor
			JFrame frame = PropertyManager.APPLICATION_FRAME_WIDGET;
			frame.setCursor(PropertyManager.APPLICATION_CURSOR);

			PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, toModel);
		    } 

		    break;
		    
		case PropertyManager.APPLICATION_PLAY: 

		    if (fromMode == PropertyManager.APPLICATION_EDIT) {
			// Switching from edit to play mode
			
			// Set the frame cursor
			JFrame frame = PropertyManager.APPLICATION_FRAME_WIDGET;
			frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			
			// Turn off the audio/video
			if (cModel != null) {
			    cModel.getStoryboard().onUnInstall();
			}
			
			// Operationalize the models first, then parse the resulting annotations
			Iterator it = StoryboardModel.getStoryboardModels().iterator();
			while (it.hasNext()) {
			    StoryboardModel model = (StoryboardModel) it.next();
			    model.operationalize();
			}
			
			it = NarrationModel.getNarrationModels().iterator(); 
			while (it.hasNext()) {
			    NarrationModel model = (NarrationModel) it.next();
			    model.operationalize();
			}

			it = MultiViewModel.getMultiViewModels().iterator();
			while (it.hasNext()) {
			    MultiViewModel model = (MultiViewModel) it.next();
			    model.operationalize();
			}

			// A multiview might add more annotations....so operationalize
		        // the multiviews first, then parse the annotations...

			it = StoryboardModel.getStoryboardModels().iterator();
			while (it.hasNext()) {
			    StoryboardModel model = (StoryboardModel) it.next();
			    model.parseAnnotations();
			}


			// Parse and load all of the NarrationModels	    
			it = NarrationModel.getNarrationModels().iterator(); 
			while (it.hasNext()) {
			    NarrationModel model = (NarrationModel) it.next();
			    model.parseAnnotations();
			    DEMAISRuntime.loadDataModel(model);
			}
			
			
			// Obtain the storyboard to load
			// Look at current model first
			StoryboardModel loadModel = null;
			if (cModel instanceof StoryboardModel) {
			    loadModel = (StoryboardModel) cModel;
			} else {
			    loadModel = StoryboardModel.getFirstStoryboardModel();
			}		 
			PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, loadModel);
		    }
		    break;
		
		default:
		    break;
		}
	    }   
	}
    }
}





