/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentPropertyPanel.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;



public class ContentPropertyPanel extends JPanel {

    private static final int NAME_FIELD_WIDTH = 15;

    private ContentPropertyFrame cFrame;
    private Listener listener;

    private JTextField titleField;
    private JButton closeButton;



    // Constructors
    private ContentPropertyPanel() {
	this(null);
    }

    public ContentPropertyPanel(ContentPropertyFrame frame) {
	super(new GridBagLayout());

	cFrame = frame;
	listener = new Listener();

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(10, 10, 0, 10);
	add(createTitlePanel(), c);

	++c.gridy;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.NONE;
	c.insets = new Insets(10, 0, 10, 10);
	c.anchor = GridBagConstraints.EAST;
	add(createClosePanel(), c);

    }


    public JPanel createTitlePanel() {
	JPanel titlePanel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	JLabel titleLabel = new JLabel("Name ");
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	titlePanel.add(titleLabel, c);

	DataModel model = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	String name = "";
	if (model != null) {
	    name = model.getName();
	}
	titleField = new JTextField(name, NAME_FIELD_WIDTH);
	titleField.addActionListener(listener);
	titleField.setFont(PropertyManager.APPLICATION_TEXT_FIELD_FONT);

	++c.gridx;
	c.weightx = 1.0;
	c.fill = GridBagConstraints.NONE;
	c.insets = new Insets(0, 3, 0, 0);
	titlePanel.add(titleField, c);

	return titlePanel;
    }

    
    public JPanel createClosePanel() {
	JPanel closePanel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	closeButton = new JButton("Close");
	closeButton.addActionListener(listener);
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 1.0;
	c.fill = GridBagConstraints.NONE;
	closePanel.add(closeButton, c);

	return closePanel;
    }


    public void updateProperties() {
	// Update the model - which notifies each of the open views
	DataModel model = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	if (model != null) {
	    model.setName(titleField.getText(), true);
	}
    }


    class Listener implements ActionListener {
	
	public Listener() {
	}

	public void actionPerformed(ActionEvent e) {
	    Object selection = e.getSource();
	    if (selection == closeButton) {
		cFrame.closeWindow();

	    } else if (selection == titleField) {
		cFrame.closeWindow();
	    }
	}
    }
}
