/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: DEMAISError.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.io.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.property.*;




public class DEMAISError implements PropertyListenerInterface {

    // error log file
    private static PrintWriter ERROR_LOG;



    // static initializer
    static {
	ERROR_LOG = null;
	String fileName = System.getProperty("user.dir") + System.getProperty("file.separator") + "error.log";
	try {
	    ERROR_LOG = new PrintWriter(new FileWriter(fileName, true));
	} catch (IOException xcptn) {
	    ERROR_LOG = null;
	}
	DEMAISError demaisError = new DEMAISError();
    }


    private DEMAISError() {
	PropertyManager.addPropertyListener(PropertyManager.APPLICATION_SHUTTING_DOWN, this);
    }

    public static void showErrorMessage(String message, String title, boolean exit) {
	// Ask user to overwrite
	Object[] options = {"OK"};
	JFrame frame = PropertyManager.APPLICATION_FRAME_WIDGET;
	JOptionPane.showOptionDialog(frame, message, title, 
				     JOptionPane.OK_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE,
				     null, options, options[0]);
	logErrorMessage(message);
	if (exit) {
	    PropertyManager.setProperty(PropertyManager.APPLICATION_EXITING, Boolean.TRUE);
	}
    }


    public static void logErrorMessage(String message) {
	if (ERROR_LOG != null) {
	    ERROR_LOG.println(message);
	    ERROR_LOG.println();
	    ERROR_LOG.flush();
	}
    }

    public static void logException(Exception xcptn) {
	if (ERROR_LOG != null) {
	    xcptn.printStackTrace(ERROR_LOG);
	    ERROR_LOG.println();
	    ERROR_LOG.flush();
	}
    }


    public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	if (property.equals(PropertyManager.APPLICATION_SHUTTING_DOWN)) {
	    boolean b = ((Boolean) newValue).booleanValue();
	    if ((b) && (ERROR_LOG != null)) {
		try {
		    ERROR_LOG.close();
		} catch (Exception xcptn) {
		}
	    }
	}
    }
}
