/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: DEMAISFrame.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.fileio.*;
import edu.uofminnesota.DEMAIS.usrinterface.layer.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.gesture.*;





public class DEMAISFrame extends JFrame {

    private static int AUTO_SAVE_COUNTER = 0;
    private JPanel contentPane;
    private Listener listener;
    private javax.swing.Timer autoSaveTimer;



    // Change here affects all top level windows in the application
    public static String getLookAndFeelClassName() {
	return (UIManager.getSystemLookAndFeelClassName());
    }


    public DEMAISFrame() {
	super("Untitled1");
	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

	// Initialize PropertyManager values and load specific classes now
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_FILE_NAME, 
				    PropertyManager.APPLICATION_DESIGN_DIRECTORY + 
				    PropertyManager.FILE_SEPARATOR +
				    "Untitled1.dms");
	// Set the frame widget
	PropertyManager.APPLICATION_FRAME_WIDGET = this;

	// Attempt to create the custom cursor
	ImageIcon cursorIcon = new ImageIcon(PropertyManager.APPLICATION_CURSOR_FILE);
	Image cursorImage = cursorIcon.getImage();
	if (cursorImage != null) {
	    Cursor defaultCursor = Toolkit.getDefaultToolkit().createCustomCursor(cursorImage, new Point(2, 29), "Pencil");
	    if (defaultCursor != null) {
		PropertyManager.APPLICATION_CURSOR = defaultCursor;
	    }
	}
	setCursor(PropertyManager.APPLICATION_CURSOR);

	// Create first storyboard model
	StoryboardModel firstModel = new StoryboardModel(PropertyManager.APPLICATION_SCOPE, null, null);
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, firstModel);
	Storyboard firstStoryboard = firstModel.getStoryboard();

	// Set the tooltip parameters
	ToolTipManager ttm = ToolTipManager.sharedInstance();
	ttm.setInitialDelay(100);
	ttm.setDismissDelay(1000);
	
	// Loads the gesture file
	RubineInterpreter.getRubineInterpreter();

	// Initializes the JMF
	AudioModel.touch();

	// Initializes the Text-to-speech system
	//TextToSpeechSynthesizer.getTTS();

	// Create the listener;
	listener = new Listener();
	
	// Listen for window events
	addWindowListener(listener);
	
	setLookAndFeel();
	createMenus();
	setupContentPane(firstStoryboard);

	pack();
	setRootSize();
	makeVisible(true);
	firstStoryboard.requestFocus();
	Storyboard.WIDTH = firstStoryboard.getWidth();
	Storyboard.HEIGHT = firstStoryboard.getHeight();

	ContentManagerFrame cmFrame = ContentManagerFrame.getFrame();
	cmFrame.setState(Frame.ICONIFIED);
	cmFrame.setVisible(true);
	
	// Force other classes to load - This is important as property observers are not
	// notified unless they have been loaded!
	// Speeds loading of the Narration editor (as it references the TTS for button enabling)
	PieMenuBar.getPieMenuBar();
	LayerControl.getLayerControl();

	// Finally, signal the PropertyManager so that it can begin firing notifications
	PropertyManager.signal();

	autoSaveTimer = null;
	//autoSaveTimer = new javax.swing.Timer(100000, new AutoSaveListener());
	//autoSaveTimer.setRepeats(true);
	//autoSaveTimer.setInitialDelay(1000 * 60 * 5);
	//autoSaveTimer.setDelay(1000 * 60 * 5);
	//autoSaveTimer.setCoalesce(true);
	//autoSaveTimer.start();
	
    }
    

    // Make the interface visible to the user
    public void makeVisible(boolean b) {
	setVisible(b);
    }


    private void setRootSize() {
	int width = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_WIDTH)).intValue();
	int height = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_HEIGHT)).intValue();
	setSize(new Dimension(width - 5, height - 5));
    }


    // Private methods
    private void setLookAndFeel() {
	try {
	    UIManager.setLookAndFeel(DEMAISFrame.getLookAndFeelClassName());

	} catch (Exception xcptn) {
	    DEMAISError.showErrorMessage(xcptn.toString(), "DEMAISFrame", false);
	    DEMAISError.logException(xcptn);
	}
    }
	
    
    private void setupContentPane(Storyboard storyboard) {
	contentPane = new JPanel(new GridBagLayout());
	contentPane.setBorder(BorderFactory.createLoweredBevelBorder());
	setContentPane(contentPane);

	GridBagConstraints c = new GridBagConstraints();

	ContentEditorPanel cPanel = new ContentEditorPanel(storyboard);
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridheight = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	contentPane.add(cPanel, c);
	
    }


    private void createMenus() {
	this.setJMenuBar(new FrameMenuBar());
    }


    private void setTitleName() {
	DataModel model = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	String fileName = (String) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_FILE_NAME);
	if (fileName != null) {
	    int index = fileName.lastIndexOf(PropertyManager.FILE_SEPARATOR);
	    if (index >= 0) {
		fileName = fileName.substring(index + 1);
	    }
	    
	    index = fileName.lastIndexOf(".");
	    if (index >= 0) {
		fileName = fileName.substring(0, index);
	    }
	    String title = fileName;
	    if (model != null) {
		title += " - " + model.getName();
	    }
	    setTitle(title);
	}
    }



    class AutoSaveListener implements ActionListener {
	public AutoSaveListener() {
	}

	public void actionPerformed(ActionEvent evt) {
	    // Create autoSave file name
	    String fileName = (String) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_FILE_NAME);
	    fileName += "~";
	    fileName += Integer.toString(AUTO_SAVE_COUNTER);
	    fileName += ".dms";

	    ++AUTO_SAVE_COUNTER;

	    // Write the file to the disk
	    try {
		DEMAISWriter.writeDEMAISFile(fileName);
		
	    } catch (Exception xcptn) {
		DEMAISError.showErrorMessage(xcptn.toString(), "File Write Error", false);
		DEMAISError.logException(xcptn);
	    }
	}
    }



    class Listener extends WindowAdapter 
	implements PropertyListenerInterface, 
		   NamedModelNameListenerInterface {

	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_FILE_NAME, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_EXITING, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_SHUTTING_DOWN, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_SHUT_DOWN_NOW, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	}
	

	// WindowListener Interface
	public void windowActivated(WindowEvent e) {
	    // Window has come into focus, give focus to the current storyboard
	    DataModel cModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	    if ((cModel != null) && (cModel instanceof StoryboardModel)) {
		StoryboardModel sModel = (StoryboardModel) cModel;
		sModel.getStoryboard().requestFocus();
	    }
	}
	
	public void windowDeiconified(WindowEvent e) {
	    // Window has come into focus, give focus to the current storyboard
	    DataModel cModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	    if ((cModel != null) && (cModel instanceof StoryboardModel)) {
		StoryboardModel sModel = (StoryboardModel) cModel;
		sModel.getStoryboard().requestFocus();
	    }
	}

	public void windowClosing(WindowEvent e) {
	    // Fire application exiting change event
	    PropertyManager.setProperty(PropertyManager.APPLICATION_EXITING, Boolean.TRUE);
	}
	
	
	// Data Model name changed
	public void nameChanged(NamedModel obj, String oldName, String newName) {
	    setTitleName();
	}

	
	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_CURRENT_FILE_NAME)) {
		setTitleName();
		
	    } else if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		DataModel oldModel = (DataModel) oldValue;
		DataModel newModel = (DataModel) newValue;
		
		if (oldModel != null) {
		    oldModel.removeNameListener(this);
		}
		if (newModel != null) {
		    newModel.addNameListener(this);
		    setTitleName();
		}
	    
	    } else if (property.equals(PropertyManager.APPLICATION_EXITING)) {
		boolean exiting = ((Boolean) newValue).booleanValue();
		if (exiting) {
		    boolean saveNeeded = ((Boolean) PropertyManager.getProperty(PropertyManager.APPLICATION_SAVE_NEEDED)).booleanValue();
		    int retVal = 0;
		    if (saveNeeded) {
			retVal = FileMenuCommands.getFileMenuCommands().handleUnsavedChanges();
		    }
		    if (retVal != 2) {
			// Exit the application, but allow listeners to clean up!
			PropertyManager.setProperty(PropertyManager.APPLICATION_SHUTTING_DOWN, Boolean.TRUE);
		    }
		}
	    
	    } else if (property.equals(PropertyManager.APPLICATION_SHUTTING_DOWN)) {
		boolean b = ((Boolean) newValue).booleanValue();
		if (b) {
		    PropertyManager.setProperty(PropertyManager.APPLICATION_SHUT_DOWN_NOW, Boolean.TRUE);
		}
		
	    } else if (property.equals(PropertyManager.APPLICATION_SHUT_DOWN_NOW)) {
		boolean b = ((Boolean) newValue).booleanValue();
		if (b) {
		    System.out.println("Exiting...");
		    System.exit(0);
		}
	    }
	}
    }
}
