/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: EditMenuCommands.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.awt.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;




public class EditMenuCommands {

    private static EditMenuCommands emCommands;

    // Clipboard
    private Collection clipboard;
    private Listener listener;



    static {
	emCommands = null;
    }


    public static EditMenuCommands getEditMenuCommands() {
	if (emCommands == null) {
	    emCommands = new EditMenuCommands();
	}
	return emCommands;
    }



    private EditMenuCommands() {
	listener = new Listener();
	clipboard = new Vector();
    }


    public void handleUndo() {
	int lastOp = ((Integer)PropertyManager.getProperty(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION)).intValue();
	if (lastOp == PropertyManager.CUT_OPERATION) {
	    // Paste what's in the clipboard - with no translation
	    handlePaste(0, 0);

	} else if (lastOp == PropertyManager.PASTE_OPERATION) {
	    // Cut what's in the clipboard
	    handleCut(clipboard);
	}
	PropertyManager.setProperty(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION, new Integer(PropertyManager.UNDO_OPERATION));
	
    }


    // Cut these objects from the current storyboard
    public void handleCut(Collection c) {
	clipboard = getCurrentModel().getStoryboard().cutGraphicalObjects(c);
	PropertyManager.setProperty(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION, new Integer(PropertyManager.CUT_OPERATION));
	PropertyManager.setProperty(PropertyManager.STORYBOARD_OBJECT_IN_CLIPBOARD, Boolean.TRUE);
    }


    public void handleCut() {
	clipboard = getCurrentModel().getStoryboard().cutSelectedGraphicalObjects();
	PropertyManager.setProperty(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION, new Integer(PropertyManager.CUT_OPERATION));
	PropertyManager.setProperty(PropertyManager.STORYBOARD_OBJECT_IN_CLIPBOARD, Boolean.TRUE);
	
    }


    public void handleCopy() {
	clipboard = getCurrentModel().getStoryboard().getSelectedGraphicalObjects();
	PropertyManager.setProperty(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION, new Integer(PropertyManager.COPY_OPERATION));
	PropertyManager.setProperty(PropertyManager.STORYBOARD_OBJECT_IN_CLIPBOARD, Boolean.TRUE);
    }


    // Called by a paste gesture command
    public void handlePaste(Rectangle gestureBounds) {
	Iterator it = clipboard.iterator();
	if (it.hasNext()) {
	    GraphicalObject go = (GraphicalObject) it.next();
	    Rectangle r = go.getBasicBounds();
	    int deltaX = gestureBounds.x - r.x;
	    if (gestureBounds.width > r.width) {
		deltaX += ((gestureBounds.width - r.width) / 2);
	    }

	    int deltaY = gestureBounds.y - r.y;
	    if (gestureBounds.height > r.height) {
		deltaY += ((gestureBounds.height - r.height) / 2);
	    }
	    handlePaste(deltaX, deltaY);
	}
    }
    

    public void handlePaste() {
	Iterator it = clipboard.iterator();
	if (it.hasNext()) {
	    GraphicalObject go = (GraphicalObject) it.next();
	    Storyboard fromS = go.getStoryboard();
	    Storyboard toS = getCurrentModel().getStoryboard();
	    if ((fromS.equals(toS)) && 
		(go.getLayer() == toS.getCurrentLayer())) {
		handlePaste(20, 10);
	    } else {
		handlePaste(0, 0);
	    }
	}
    }

    public void handlePaste(int deltaX, int deltaY) {
	if (clipboard.size() > 0) {
	    Storyboard storyboard = getCurrentModel().getStoryboard();
	    clipboard = storyboard.pasteGraphicalObjects(clipboard, deltaX, deltaY, storyboard);
	    if (clipboard.size() > 0) {
		PropertyManager.setProperty(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION, new Integer(PropertyManager.PASTE_OPERATION));

	    }
	}
    }


    public void handleStoryboardEdit() {
	DataModel toModel = StoryboardModel.getFirstStoryboardModel();
	if (toModel == null) {
	    toModel = new StoryboardModel(PropertyManager.APPLICATION_SCOPE, null, null);
	}
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, toModel);
    }

    public void handleMultiViewEdit() {
	DataModel toModel = MultiViewModel.getFirstMultiViewModel();
	if (toModel == null) {
	    toModel = new MultiViewModel(PropertyManager.APPLICATION_SCOPE, null, null);
	}
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, toModel);
    }


    private DataModel getCurrentModel() {
	return ((DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL));
    }



    class Listener implements PropertyListenerInterface {
	// Constructor
	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	}

	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int toMode = ((Integer) newValue).intValue();
		if (toMode == PropertyManager.APPLICATION_PLAY) {
		    clipboard.clear();
		    PropertyManager.setProperty(PropertyManager.STORYBOARD_OBJECT_IN_CLIPBOARD, Boolean.FALSE);
		}  
	    }
	}
    }
}
 
