/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: FileMenu.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;




public class FileMenu extends JMenu {

    // The listener for all menu items (an inner class below)
    private Listener listener;

    // The menu items
    private JMenuItem fileNew, fileOpen, fileSave, fileSaveas, filePrint, fileExit;



    public FileMenu() {
	super("File");
	listener = new Listener();

	fileNew = new JMenuItem("New");
	fileNew.addActionListener(listener);
	add(fileNew);

	fileOpen = new JMenuItem("Open");
	fileOpen.addActionListener(listener);
	add(fileOpen);

	fileSave = new JMenuItem("Save");
	fileSave.setEnabled(false);
	fileSave.addActionListener(listener);
	add(fileSave);

	fileSaveas = new JMenuItem("Save As");
	fileSaveas.addActionListener(listener);
	add(fileSaveas);

	filePrint = new JMenuItem("Print");
	filePrint.addActionListener(listener);
	add(filePrint);

	fileExit = new JMenuItem("Exit");
	fileExit.addActionListener(listener);
	add(fileExit);
    }


    
    // An inner class to handle all menu item events
    class Listener implements ActionListener, PropertyListenerInterface {

	// Constructor
	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_SAVE_NEEDED, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	}
	

	// Menu item callback
	public void actionPerformed(ActionEvent e) {
	    Object selection = e.getSource();
	    
	    if (selection == fileNew) {
		FileMenuCommands.getFileMenuCommands().handleFileNew();

	    } else if (selection == fileOpen) {
		FileMenuCommands.getFileMenuCommands().handleFileOpen();
		
	    } else if (selection == fileSave) {
		FileMenuCommands.getFileMenuCommands().handleFileSave();

	    } else if (selection == fileSaveas) {
		FileMenuCommands.getFileMenuCommands().handleFileSaveas();
		
	    } else if (selection == filePrint) {
		FileMenuCommands.getFileMenuCommands().handleFilePrint();

	    } else if (selection == fileExit) {
		FileMenuCommands.getFileMenuCommands().handleFileExit();

	    } else {
		System.out.println("ERROR: Unknown file menu command");
	    }
	}


	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_SAVE_NEEDED)) {
		boolean saveNeeded = ((Boolean) newValue).booleanValue();
		fileSave.setEnabled(saveNeeded);
		
	    } else if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		DataModel model = (DataModel) newValue;
		if (model != null) {
		    int mode = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		    filePrint.setEnabled(mode == PropertyManager.APPLICATION_EDIT);
		    
		} else {
		    filePrint.setEnabled(false);
		}
		
	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int mode = ((Integer) newValue).intValue();
		fileNew.setEnabled(mode == PropertyManager.APPLICATION_EDIT);
		fileOpen.setEnabled(mode == PropertyManager.APPLICATION_EDIT);
		filePrint.setEnabled(mode == PropertyManager.APPLICATION_EDIT);
	    }
	}
    }
}



