/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: FileMenuCommands.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.print.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.fileio.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;
import edu.uofminnesota.DEMAIS.usrinterface.util.*;




public class FileMenuCommands {

    // The single instance of this object
    private static FileMenuCommands fmCommands;
    
    // Static variable for naming new files
    private static int fileCounter;


    // Application frame widget
    private JFrame frame;



    static {
	fileCounter = 1;
	fmCommands = null;
    }


    public static FileMenuCommands getFileMenuCommands() {
	if (fmCommands == null) {
	    fmCommands = new FileMenuCommands();
	}
	return fmCommands;
    }

    public static int GetNextFileId() {
	++fileCounter;
	return fileCounter;
    }

    public static void SetFileId(int id) {
	fileCounter = id;
    }

	

    private FileMenuCommands() {
	frame = PropertyManager.APPLICATION_FRAME_WIDGET;
    }


    public void handleFileNew() {
	// Check if a save is needed
	boolean saveNeeded = ((Boolean) PropertyManager.getProperty(PropertyManager.APPLICATION_SAVE_NEEDED)).booleanValue();
	if (saveNeeded) {
	    // Ask the user to save the changes
	    int retVal = handleUnsavedChanges();
	    if (retVal == 2) {
		return;
	    }
	}
	
	// Clear the current design model and create a new one
	clearCurrentModel();
	buildNewModel();	
    }
    
    
    public void handleFileOpen() {
	// Get the last directory used
	String lastDir = (String) PropertyManager.getProperty(PropertyManager.APPLICATION_LAST_FILE_DIRECTORY);
	DFileChooser chooser = new DFileChooser(lastDir);
	chooser.setCursor(PropertyManager.APPLICATION_CURSOR);

	int returnVal = chooser.showOpenDialog(frame);
	
	if (returnVal == JFileChooser.APPROVE_OPTION) {		
	    // Set the properties
	    File selectedFile = chooser.getSelectedFile();
	    String selectedFilePathName = selectedFile.getAbsolutePath();
	    String selectedFileDirectory = chooser.getCurrentDirectory().getAbsolutePath();
	    PropertyManager.setProperty(PropertyManager.APPLICATION_LAST_FILE_DIRECTORY, selectedFileDirectory);
	    PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_FILE_NAME, selectedFilePathName);
	    
	    // Clear current model
	    clearCurrentModel();

	    // Read the existing design file		
	    try {
		DEMAISReader.readDEMAISFile(selectedFile);
		checkForFileId(selectedFilePathName);
		PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.FALSE);
		
	    } catch (Exception xcptn) {
		DEMAISError.showErrorMessage(xcptn.toString(), "File Read Error", false);
		DEMAISError.logException(xcptn);
	    }
	}
    }
    

    private void checkForFileId(String filePath) {
	// Check for untitled name -> c:\directory\Untitled4.dms
	int index = filePath.lastIndexOf(PropertyManager.FILE_SEPARATOR);
	if (index >= 0) {
	    filePath = filePath.substring(index+1);
	    // filePath -> Untitled4.dms
	    index = filePath.lastIndexOf('.');
	    if (index >= 0) {
		filePath = filePath.substring(0, index);
		// filePath -> Untitled4
		filePath = filePath.toUpperCase();
		// filePath -> UNTITLED4
		index = filePath.indexOf("UNTITLED");
		if ((index == 0) && (filePath.length() >= 9)) {
		    filePath = filePath.substring(8);
		    // filePath -> 4
		    int id = -1;
		    try {
			id = Integer.parseInt(filePath);
			SetFileId(id);
		    } catch (Exception xcptn) {
		    }
		}    
	    }
	}
    }
    

    private void clearCurrentModel() {
	// Remove storyboards
	Iterator it = ((Vector)StoryboardModel.getStoryboardModels().clone()).iterator();
	while (it.hasNext()) {
	    StoryboardModel sModel = (StoryboardModel) it.next();
	    sModel.deleteModel();
	}
	
	// Remove Narrations
	it = ((Vector)NarrationModel.getNarrationModels().clone()).iterator();
	while (it.hasNext()) {
	    NarrationModel nModel = (NarrationModel) it.next();
	    nModel.deleteModel();
	}
	
	// Remove MultiViews
	it = ((Vector)MultiViewModel.getMultiViewModels().clone()).iterator();
	while (it.hasNext()) {
	    MultiViewModel mvModel = (MultiViewModel) it.next();
	    mvModel.deleteModel();
	}
	
	
	// Reset Properties
	PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.FALSE);
	PropertyManager.setProperty(PropertyManager.APPLICATION_EXITING, Boolean.FALSE);
	PropertyManager.setProperty(PropertyManager.APPLICATION_UNDO_LIST_EMPTY, Boolean.TRUE);
	PropertyManager.setProperty(PropertyManager.APPLICATION_MODE, new Integer(PropertyManager.APPLICATION_EDIT));
	PropertyManager.setProperty(PropertyManager.APPLICATION_STORYBOARD_COUNT, new Integer(0));
	PropertyManager.setProperty(PropertyManager.APPLICATION_MULTIVIEW_COUNT, new Integer(0));	
	PropertyManager.setProperty(PropertyManager.APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED, new Integer(-1));

	PropertyManager.setProperty(PropertyManager.STORYBOARD_OBJECT_SELECTED, Boolean.FALSE);
	PropertyManager.setProperty(PropertyManager.STORYBOARD_OBJECT_IN_CLIPBOARD, Boolean.FALSE);
	PropertyManager.setProperty(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION, new Integer(-1));

	PropertyManager.setProperty(PropertyManager.VS_TEXT_SELECTED, Boolean.FALSE);
	PropertyManager.setProperty(PropertyManager.VS_TEXT_IN_CLIPBOARD, Boolean.FALSE);
	PropertyManager.setProperty(PropertyManager.VS_UNDO_LIST_EMPTY, Boolean.TRUE);

	PropertyManager.setProperty(PropertyManager.CONTENT_MANAGER_OBJECT_IN_CLIPBOARD, Boolean.FALSE);
	PropertyManager.setProperty(PropertyManager.CONTENT_MANAGER_OBJECT_SELECTED_COUNT, new Integer(0));
	
    }
    
    
    private void buildNewModel() {	    	   
	// Set the file property - which will reset the frame title
	String newFileName = PropertyManager.APPLICATION_DESIGN_DIRECTORY +
	    PropertyManager.FILE_SEPARATOR +
	    "Untitled" + 
	    Integer.toString(GetNextFileId()) + 
	    ".dms";
	
	StoryboardModel.setSMIdentifier(0);
	NarrationModel.setNMIdentifier(0);
	MultiViewModel.setMVIdentifier(0);
	
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_FILE_NAME, newFileName);
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, new StoryboardModel(PropertyManager.APPLICATION_SCOPE, null, null));
    }
    
    
    
    public void handleFileSave() {
	String file = (String) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_FILE_NAME);
	handleSaveToFile(file);
    }
    
    public void handleFileSaveas() {
	String lastDir = (String) PropertyManager.getProperty(PropertyManager.APPLICATION_LAST_FILE_DIRECTORY);
	DFileChooser chooser = new DFileChooser(lastDir);
	chooser.setCursor(PropertyManager.APPLICATION_CURSOR);

	int returnVal = chooser.showSaveDialog(frame);
	
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File selectedFile = chooser.getSelectedFile();
	    String selectedFilePathName = selectedFile.getAbsolutePath();
	    String selectedFileDirectory = chooser.getCurrentDirectory().getAbsolutePath();
	    
	    // Check if the file exists
	    if (checkIfFileExists(selectedFile) == 0) {
		// OK - save the file
		// Update the properties
		PropertyManager.setProperty(PropertyManager.APPLICATION_LAST_FILE_DIRECTORY, selectedFileDirectory);
		PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_FILE_NAME, selectedFilePathName);
		
		handleSaveToFile(selectedFilePathName);
	    }
	}
    }
    
    
    // Returns:
    //  0 for OK
    //  1 for Cancel
    private int checkIfFileExists(File file) {
	int returnVal = 0;
	
	if (file.exists()) {
	    // Ask user to overwrite
	    Object[] options = { "OK", "CANCEL" };
	    returnVal = JOptionPane.showOptionDialog(frame, "Overwrite the existing file?", "Warning", 
						     JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
						     null, options, options[0]);
	}
	return returnVal;
    }
    
    
    private void handleSaveToFile(String filePathName) {
	// Reset the property and menus
	
	// Save the application data to the file
	try {
	    DEMAISWriter.writeDEMAISFile(filePathName);
	    PropertyManager.setProperty(PropertyManager.APPLICATION_SAVE_NEEDED, Boolean.FALSE);
	    
	} catch (Exception xcptn) {
	    DEMAISError.showErrorMessage(xcptn.toString(), "File Write Error", false);
	    DEMAISError.logException(xcptn);
	}
    }
    

    public void handleFilePrint() {
	PrinterJob printerJob = PrinterJob.getPrinterJob();
	if (printerJob.printDialog()) {
	    try {
		Book book = new Book();
		PageFormat pageFormat1 = new PageFormat();
		pageFormat1.setOrientation(PageFormat.LANDSCAPE);
		
		Iterator it = StoryboardModel.getStoryboardModels().iterator();
		while (it.hasNext()) {
		    StoryboardModel sModel = (StoryboardModel) it.next();
		    book.append(sModel.getStoryboard(), pageFormat1);
		}
		
		it = MultiViewModel.getMultiViewModels().iterator();
		while (it.hasNext()) {
		    MultiViewModel mvModel = (MultiViewModel) it.next();
		    book.append(mvModel.getStoryboard(), pageFormat1);
		}
		
		PageFormat pageFormat2 = new PageFormat();
		pageFormat2.setOrientation(PageFormat.PORTRAIT);		    
		it = NarrationModel.getNarrationModels().iterator();
		while (it.hasNext()) {
		    NarrationModel nModel = (NarrationModel) it.next();
		    book.append(nModel, pageFormat2);
		}
		
		printerJob.setPageable(book);
		printerJob.print();
		
	    } catch (PrinterException xcptn) {
		DEMAISError.showErrorMessage(xcptn.toString(), "Printer Error", false);
		DEMAISError.logException(xcptn);
	    }
	}
    }
    
    
    
    public void handleFileExit() {
	PropertyManager.setProperty(PropertyManager.APPLICATION_EXITING, Boolean.TRUE);
    }
    
    
    // Ask the user to save the unsaved changes
    protected int handleUnsavedChanges() {
	Object[] options = { "Yes", "No", "Cancel"};
	int returnVal = JOptionPane.showOptionDialog(frame, "You have unsaved changes - Would you like to save them now?", "Unsaved Changes", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[0]);
	
	if (returnVal == 0) {
	    // User pressed OK - Save the file to current location
	    handleFileSave();
	}
	return returnVal;
    }


}
