/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: FrameMenuBar.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.lang.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;



public class FrameMenuBar extends JMenuBar {

    // The menu bar objects
    private JMenu fileMenu, editMenu, layerMenu, storyboardMenu;
    private JMenu multiViewMenu, operationMenu, windowMenu; 
 
    private Listener listener;



    public FrameMenuBar() {

	listener = new Listener();

	fileMenu = new FileMenu();
	add(fileMenu);

	editMenu = new EditMenu();
	add(editMenu);

	storyboardMenu = new StoryboardMenu();
	add(storyboardMenu);

	layerMenu = new LayerMenu();
	add(layerMenu);

	multiViewMenu = new MultiViewMenu();
	multiViewMenu.setVisible(false);
	add(multiViewMenu);

	operationMenu = new OperationMenu();
	add(operationMenu);

	windowMenu = new WindowMenu();
	add(windowMenu);

    }


    class Listener implements PropertyListenerInterface {
	public Listener() {
	    // Listen for property changes
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	}


	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		int mode = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		if (mode == PropertyManager.APPLICATION_EDIT) {
		    DataModel toModel = (DataModel) newValue;
		    if (toModel != null) {
			if (toModel instanceof StoryboardModel) {
			    storyboardMenu.setVisible(true);
			    layerMenu.setVisible(true);
			    multiViewMenu.setVisible(false);
			    
			} else if (toModel instanceof MultiViewModel) {
			    storyboardMenu.setVisible(false);
			    layerMenu.setVisible(false);
			    multiViewMenu.setVisible(true);
			}
		    }
		}

	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int fromMode = ((Integer) oldValue).intValue();
		int toMode = ((Integer) newValue).intValue();
		if ((fromMode == PropertyManager.APPLICATION_EDIT) &&
		    (toMode == PropertyManager.APPLICATION_PLAY)) {
		    storyboardMenu.setVisible(true);
		    layerMenu.setVisible(true);
		    multiViewMenu.setVisible(false);
		}
	    }
	}
    }
}


