/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File ObjectPropertyFrame.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;
    

// Necessary imports
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.meal.layer.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;




public class ObjectPropertyFrame extends JDialog {

    // Singleton reference
    private static ObjectPropertyFrame objectFrame;
    private static Font FONT;


    // Cached reference to current data model
    private DataModel model;
 
    // Event listener
    private Listener listener;

    // Property labels and values
    private JLabel xLabel, xValue;
    private JLabel yLabel, yValue;
    private JLabel wLabel, wValue;
    private JLabel hLabel, hValue;
    private JLabel lLabel, lValue;

    // Recognition checkbox
    private JCheckBox checkBox;

    // Close button
    private JButton closeButton;

    // Width of a value label
    private Dimension labelDimension;



    // Static initializer
    static {
	objectFrame = null;
	FONT = new Font("Arial", Font.PLAIN, 12);
    }

    public static ObjectPropertyFrame getObjectPropertyFrame() {
	if (objectFrame == null) {
	    objectFrame = new ObjectPropertyFrame();
	}
	return objectFrame;
    }



    private ObjectPropertyFrame() {
	super(PropertyManager.APPLICATION_FRAME_WIDGET, "Ink Stroke Properties", false);
	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	
	// Set the cursor
	setCursor(PropertyManager.APPLICATION_CURSOR);

	// Retrieve current storyboard model
	model = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);

	listener = new Listener();

	// Listen for window events
	addWindowListener(listener);

	// Create the content panel
	JPanel contentPanel = new JPanel(new GridBagLayout());
	setContentPane(contentPanel);

	labelDimension = new Dimension(computeLabelWidth(), 10);;

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 1.0;
	c.fill = GridBagConstraints.BOTH;
	c.insets = new Insets(10, 10, 0, 10);
	contentPanel.add(createXYWHLPanel(), c);

	++c.gridy;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.insets = new Insets(15, 10, 0, 10);
	contentPanel.add(createRecognitionPanel(), c);

	++c.gridy;
	c.insets = new Insets(12, 0, 10, 10);
	contentPanel.add(createClosePanel(), c);

	updateProperties();

	pack();
	setRootLocation();
	makeVisible(false, true);
    }



    // Public methods
    public void makeVisible(boolean visible, boolean byUser) {
	if (byUser) {
	    PropertyManager.setProperty(PropertyManager.OBJECT_PROPERTY_FRAME_HIDDEN_BY_USER, new Boolean(! visible));
	}

	/* 
	   If the request is to make the Layer Control visible, then it must not have been previously closed by the user
	 */
	boolean hiddenByUser = ((Boolean)PropertyManager.getProperty(PropertyManager.OBJECT_PROPERTY_FRAME_HIDDEN_BY_USER)).booleanValue();
	if ((visible) && (!hiddenByUser)) {
	    setVisible(true);
	    PropertyManager.setProperty(PropertyManager.OBJECT_PROPERTY_FRAME_VISIBLE, Boolean.TRUE);

	} else if (!visible) {
	    setVisible(false);
	    PropertyManager.setProperty(PropertyManager.OBJECT_PROPERTY_FRAME_VISIBLE, Boolean.FALSE);

	}
    }


    
    public void updateProperties() {
	if ((model != null) && (model.getStoryboard() != null)) {
	    Collection c = model.getStoryboard().getSelectedGraphicalObjects();
	    updateProperties(c);
	}
    }


    public void updateProperties(Collection c) {
	RectangleInkStroke ris = null;
	InkStroke inkStroke = null;
	String x = "0";
	String y = "0";
	String w = "0";
	String h = "0";
	String l = "0";

	// Locate first rectangle and inkStroke object, if any
	if (c != null) {
	    Iterator it = c.iterator();
	    while (it.hasNext()) {
		GraphicalObject go = (GraphicalObject) it.next();
		if (go instanceof RectangleInkStroke) {
		    ris = (RectangleInkStroke) go;

		} else if (go instanceof BehavioralInkStroke) {
		    // Do nothing

		} else if (go instanceof InkStroke) {
		    inkStroke = (InkStroke) go;
		}
	    }
	}

	// Set fields based on first rectangle or ink stroke object
	Rectangle r = null;
	int layer = 0;
	if (ris != null) {
	    r = ris.getBasicBounds();
	    layer = ris.getLayer();
	} else if (inkStroke != null) {
	    r = inkStroke.getBasicBounds();
	    layer = inkStroke.getLayer();
	}

	if (r != null) {
	    x = Integer.toString(r.x);
	    y = Integer.toString(r.y);
	    w = Integer.toString(r.width);
	    h = Integer.toString(r.height);
	    l = Integer.toString(layer);   
	}

	xValue.setText(x);
	yValue.setText(y);
	wValue.setText(w);
	hValue.setText(h);
	lValue.setText(l);
	
	xLabel.setEnabled((ris != null) || (inkStroke != null));
	xValue.setEnabled((ris != null) || (inkStroke != null));
	
	yLabel.setEnabled((ris != null) || (inkStroke != null));
	yValue.setEnabled((ris != null) || (inkStroke != null));
	
	wLabel.setEnabled((ris != null) || (inkStroke != null));
	wValue.setEnabled((ris != null) || (inkStroke != null));
	
	hLabel.setEnabled((ris != null) || (inkStroke != null));
	hValue.setEnabled((ris != null) || (inkStroke != null));
	
	lLabel.setEnabled((ris != null) || (inkStroke != null));
	lValue.setEnabled((ris != null) || (inkStroke != null));
	
	checkBox.setSelected(ris != null);
	checkBox.setEnabled((ris != null) || (inkStroke != null));
    }



    private void setRootLocation() {
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension me = getSize();

	Point p = new Point( (int)(screen.getWidth() - me.getWidth()) / 2, 
			     (int)(screen.getHeight() - me.getHeight()) / 2);
	setLocation(p);
    }

 
    private JPanel createXYWHLPanel() {
	JPanel panel = new JPanel(new GridBagLayout());
	panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Bounds & Layer", TitledBorder.LEADING, TitledBorder.TOP));

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.25;
	c.weighty = 0.25;
	c.fill = GridBagConstraints.NONE;
	c.insets = new Insets(0, 5, 0, 0);
	panel.add(createXPanel("X:", "0"), c);

	c.gridx = 1;
	c.insets = new Insets(0, 12, 0, 0);
	panel.add(createYPanel("Y:", "0"), c);

	c.gridx = 2;
	c.insets = new Insets(0, 12, 0, 5);
	panel.add(createLPanel("L:", "0"), c);

	c.gridx = 0;
	c.gridy = 1;
	c.insets = new Insets(0, 5, 0, 0);
	panel.add(createWPanel("W:", "0"), c);

	c.gridx = 1;
	c.insets = new Insets(0, 12, 0, 0);
	panel.add(createHPanel("H:", "0000"), c);

	return panel;
    }


    private JPanel createXPanel(String name, String value) {
	JPanel panel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.NONE;
	
	xLabel = new JLabel(name);
	xLabel.setFont(FONT);
	panel.add(xLabel, c);
	
	xValue = new JLabel(value);
	xValue.setFont(FONT);
	xValue.setMinimumSize(labelDimension);
	xValue.setPreferredSize(labelDimension);
	c.gridx = 1;
	c.insets = new Insets(0, 4, 0, 0);
	panel.add(xValue, c);
	
	return panel;
    }


    private JPanel createYPanel(String name, String value) {
	JPanel panel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.NONE;

	yLabel = new JLabel(name);
	yLabel.setFont(FONT);

	panel.add(yLabel, c);

	yValue = new JLabel(value);
	yValue.setFont(FONT);
	yValue.setMinimumSize(labelDimension);
	yValue.setPreferredSize(labelDimension);

	c.gridx = 1;
	c.insets = new Insets(0, 4, 0, 0);
	panel.add(yValue, c);

	return panel;
    }


    private JPanel createWPanel(String name, String value) {
	JPanel panel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.NONE;

	wLabel = new JLabel(name);
	wLabel.setFont(FONT);
	panel.add(wLabel, c);

	wValue = new JLabel(value);
	wValue.setFont(FONT);
	wValue.setMinimumSize(labelDimension);
	wValue.setPreferredSize(labelDimension);

	c.gridx = 1;
	c.insets = new Insets(0, 4, 0, 0);
	panel.add(wValue, c);

	return panel;
    }


    private JPanel createHPanel(String name, String value) {
	JPanel panel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.NONE;

	hLabel = new JLabel(name);
	hLabel.setFont(FONT);
	panel.add(hLabel, c);

	hValue = new JLabel(value);
	hValue.setFont(FONT);
	hValue.setMinimumSize(labelDimension);
	hValue.setPreferredSize(labelDimension);

	c.gridx = 1;
	c.insets = new Insets(0, 4, 0, 0);
	panel.add(hValue, c);

	return panel;
    }


    private JPanel createLPanel(String name, String value) {
	JPanel panel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.NONE;

	lLabel = new JLabel(name);
	lLabel.setFont(FONT);
	panel.add(lLabel, c);

	lValue = new JLabel(value);
	lValue.setFont(FONT);
	lValue.setMinimumSize(labelDimension);
	lValue.setPreferredSize(labelDimension);

	c.gridx = 1;
	c.insets = new Insets(0, 4, 0, 0);
	panel.add(lValue, c);

	return panel;
    }


    private int computeLabelWidth() {
	int width = 25;

	// Obtain a graphics object
	JFrame frame = PropertyManager.APPLICATION_FRAME_WIDGET;
	Graphics g = frame.getContentPane().getGraphics();
	if (g != null) {
	    FontMetrics fm = g.getFontMetrics(FONT);
	    width = fm.charWidth('0') * 5;
	}
	return width;
    }

    
    public JPanel createRecognitionPanel() {
	JPanel panel = new JPanel(new GridBagLayout());
	panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Recognition", TitledBorder.LEADING, TitledBorder.TOP));
	
	checkBox = new JCheckBox("Rectangle");
	checkBox.setActionCommand("rectangle");
	checkBox.addActionListener(listener);
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.NONE;
	panel.add(checkBox, c);
	
	return panel;
    }
    
    
    
    private JPanel createClosePanel() {
	JPanel closePanel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	
	closeButton = new JButton("Close");
	closeButton.setActionCommand("close");
	closeButton.addActionListener(listener);

	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.EAST;
	closePanel.add(closeButton, c);
	
	return closePanel;
    }

    public void handleDemote() {
	makeSelectionARectangle(false);
    }

    public void handlePromote() {
	makeSelectionARectangle(true);
    }
    
    private void makeSelectionARectangle(boolean value) {
	Collection c = model.getStoryboard().getSelectedGraphicalObjects();
	if (c != null) {
	    Iterator it = c.iterator();
	    while (it.hasNext()) {
		GraphicalObject go = (GraphicalObject) it.next();
		if (go instanceof RectangleInkStroke) {
		    model.getStoryboard().forceRectangleRecognition(go, value);
		} else if (go instanceof BehavioralInkStroke) {
		    // Do nothing
		} else if (go instanceof InkStroke) {
		    model.getStoryboard().forceRectangleRecognition(go, value);
		}
	    }
	}
    }


    
    class Listener extends WindowAdapter 
	implements 
		ActionListener,
		PropertyListenerInterface {
	
	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	}


	public void windowClosing(WindowEvent e) {
	    // Hide the window, but do not dispose of it
	    makeVisible(false, true);
	}
	

	public void actionPerformed(ActionEvent evt) {
	    String cmd = evt.getActionCommand();
	    if (cmd.equals("rectangle")) {
		makeSelectionARectangle(checkBox.isSelected());
		if (checkBox.isSelected()) {
		    ++PropertyManager.APPLICATION_RECTANGLE_THRESHHOLD;
		} else {
		    --PropertyManager.APPLICATION_RECTANGLE_THRESHHOLD;
		}

	    } else if (cmd.equals("close")) {
		makeVisible(false, true);
	    }
	}


	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		int mode = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		if (mode == PropertyManager.APPLICATION_EDIT) {
		    model = (DataModel) newValue;
		}
		
	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int fromMode = ((Integer) oldValue).intValue();
		int toMode = ((Integer) newValue).intValue();
		if ((fromMode == PropertyManager.APPLICATION_EDIT) &&
		    (toMode == PropertyManager.APPLICATION_PLAY)) {
		    makeVisible(false, true);
		}
		// Going from play to edit mode is captured by a model change
	    }
	}
    }
}
