/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: PieEditMenu.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.awt.event.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.util.*;




public class PieEditMenu extends PieMenu {

    // The listener for all menu items (an inner class below)
    private Listener listener;

    // The menu items
    private JMenuItem editUndo, editCut, editCopy;
    private JMenuItem editPaste, editMultiView, editStoryboard;



    public PieEditMenu() {
	super("Edit");
	listener = new Listener();
   
	editUndo = add("Undo");
	editUndo.setEnabled(false);
	editUndo.addActionListener(listener);

	editCut = add("Cut");
	editCut.setEnabled(false);
	editCut.addActionListener(listener);

	editCopy = add("Copy");
	editCopy.setEnabled(false);
	editCopy.addActionListener(listener);

	editPaste = add("Paste");
	editPaste.setEnabled(false);
	editPaste.addActionListener(listener);

	editMultiView = add("MultiView");
	editMultiView.setEnabled(true);
	editMultiView.addActionListener(listener);

	editStoryboard = add("Storyboard");
	editStoryboard.setEnabled(false);
	editStoryboard.addActionListener(listener);
    }


    // An inner class to handle all menu item events
    class Listener implements ActionListener, PropertyListenerInterface {
	
	// Constructor
	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	    PropertyManager.addPropertyListener(PropertyManager.STORYBOARD_OBJECT_SELECTED, this);
	    PropertyManager.addPropertyListener(PropertyManager.STORYBOARD_OBJECT_IN_CLIPBOARD, this);
	    PropertyManager.addPropertyListener(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION, this);
	}


	// Menu item callback
	public void actionPerformed(ActionEvent e) {
	    Object selection = e.getSource();
	 
	    if (selection == editUndo) {
		EditMenuCommands.getEditMenuCommands().handleUndo();

	    } else if (selection == editCut) {
		EditMenuCommands.getEditMenuCommands().handleCut();

	    } else if (selection == editCopy) {
		EditMenuCommands.getEditMenuCommands().handleCopy();
 
	    } else if (selection == editPaste) {
		EditMenuCommands.getEditMenuCommands().handlePaste();

	    } else if (selection == editStoryboard) {
		EditMenuCommands.getEditMenuCommands().handleStoryboardEdit();
		
	    } else if (selection == editMultiView) {
		EditMenuCommands.getEditMenuCommands().handleMultiViewEdit();
		
	    } else {
		System.out.println("ERROR: Unknown edit menu command");
	    }
	}


	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    DataModel cModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);

	    if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		DataModel model = (DataModel) newValue;

		int mode = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		if (mode == PropertyManager.APPLICATION_PLAY) {
		    // If we are switching data models in PLAY mode, do nothing!
		    return;
		}
		
		if (model == null) {
		    editUndo.setText("Undo");
		    editUndo.setEnabled(false);
		    editCut.setEnabled(false);
		    editCopy.setEnabled(false);
		    editPaste.setEnabled(false);
		    editStoryboard.setVisible(true);
		    editStoryboard.setEnabled(true);
		    editMultiView.setVisible(false);
		    editMultiView.setEnabled(false);

		} else {
		  
		    // Not null, not in play mode
		    if (model instanceof StoryboardModel) {
			editUndo.setText("Undo");
			editUndo.setEnabled(false);

			// Cut and copy are already set from property change

			boolean b = ((Boolean)PropertyManager.getProperty(PropertyManager.STORYBOARD_OBJECT_IN_CLIPBOARD)).booleanValue();
			editPaste.setEnabled(b);
			
			//editMultiView.setVisible(true);
			editMultiView.setEnabled(true);
			
			//editStoryboard.setVisible(false);
			editStoryboard.setEnabled(false);

		    } else if (model instanceof MultiViewModel) {
			editUndo.setText("Undo");
			editUndo.setEnabled(false);

			// Cut and copy already set

			editPaste.setEnabled(false);

			//editMultiView.setVisible(false);
			editMultiView.setEnabled(false);

			//editStoryboard.setVisible(true);
			editStoryboard.setEnabled(true);
		    }
		}


	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int toMode = ((Integer) newValue).intValue();
		switch (toMode) {
		    
		case PropertyManager.APPLICATION_PLAY:
		    // Going from edit mode to play mode
		    editUndo.setEnabled(false);
		    editUndo.setText("Undo");
		    editCut.setEnabled(false);
		    editCopy.setEnabled(false);
		    editPaste.setEnabled(false);
		    editMultiView.setEnabled(false);
		    editStoryboard.setEnabled(false);
		    break;
		    
		default:
		    break;
		}
		
	    } else if (property.equals(PropertyManager.STORYBOARD_OBJECT_SELECTED)) {
		boolean b = ((Boolean) newValue).booleanValue();
		editCut.setEnabled(b);
		if (cModel instanceof MultiViewModel) {
		    b = false;
		}
		editCopy.setEnabled(b);

	    } else if (property.equals(PropertyManager.STORYBOARD_OBJECT_IN_CLIPBOARD)) {
		boolean b = ((Boolean) newValue).booleanValue();
		if (cModel instanceof MultiViewModel) {
		    b = false;
		}
		editPaste.setEnabled(b);
	    
	    } else if (property.equals(PropertyManager.STORYBOARD_LAST_EDIT_OPERATION)) {
		if (cModel instanceof MultiViewModel) {
		    return;
		}
		
		int op = ((Integer) newValue).intValue();
		if (op == PropertyManager.CUT_OPERATION) {
		    editUndo.setText("Undo Cut");
		    editUndo.setEnabled(true);

		} else if (op == PropertyManager.PASTE_OPERATION) {
		    editUndo.setText("Undo Paste");
		    editUndo.setEnabled(true);
		
		} else if (op == PropertyManager.UNDO_OPERATION) {
		    editUndo.setText("Undo");
		    editUndo.setEnabled(false);
		
		}
	    }
	}
    }
}
