/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: PieMenuBar.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.usrinterface.util.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;


public class PieMenuBar extends PieMenu {

    private static PieMenuBar pieMenuBar;

    // The pie sub menus
    private PieMenu fileMenu, editMenu, layerMenu, storyboardMenu;
    private PieMenu multiViewMenu, operationMenu, windowMenu; 
    private Listener listener;


    static {
	pieMenuBar = null;
    }

    public static PieMenuBar getPieMenuBar() {
	if (pieMenuBar == null) {
	    pieMenuBar = new PieMenuBar();
	}
	return pieMenuBar;
    }


    public PieMenuBar() {
	super();

	listener = new Listener();

	fileMenu = new PieFileMenu();
	add(fileMenu);

	editMenu = new PieEditMenu();
	add(editMenu);

	storyboardMenu = new PieStoryboardMenu();
	add(storyboardMenu);

	layerMenu = new PieLayerMenu();
	add(layerMenu);

	multiViewMenu = new PieMultiViewMenu();
	multiViewMenu.setEnabled(false);
	add(multiViewMenu);

	operationMenu = new PieOperationMenu();
	add(operationMenu);

	windowMenu = new PieWindowMenu();
	add(windowMenu);

	setAllClipping(true);
	setAllTapHoldOpen();
	setAllAutoOpen(true);
	setAllInitialDelay(400);
	setAllSubmenuDelay(450);
	setVisible(false);
    }


    public void hidePieMenu() {
	if (isVisible()) {
	    setVisible(false);
	}
    }


    class Listener implements PropertyListenerInterface {
	public Listener() {
	    // Listen for property changes
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	}


	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		DataModel fromModel = (DataModel) oldValue;
		DataModel toModel = (DataModel) newValue;
		
		int mode = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		if (mode == PropertyManager.APPLICATION_EDIT) {
		    if (toModel != null) {
			if (toModel instanceof StoryboardModel) {
			    storyboardMenu.setEnabled(true);
			    layerMenu.setEnabled(true);
			    multiViewMenu.setEnabled(false);			    
			    
			} else if (toModel instanceof MultiViewModel) {
			    storyboardMenu.setEnabled(false);
			    layerMenu.setEnabled(false);
			    multiViewMenu.setEnabled(true);
			}
		    }
		
		} else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		    int fromMode = ((Integer) oldValue).intValue();
		    int toMode = ((Integer) newValue).intValue();
		    if ((fromMode == PropertyManager.APPLICATION_EDIT) &&
			(toMode == PropertyManager.APPLICATION_PLAY)) {
			storyboardMenu.setEnabled(true);
			layerMenu.setEnabled(true);
			multiViewMenu.setEnabled(false);
		    }
		}
	    }
	}
    }
}


