/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: PieMultiViewMenu.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.awt.event.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.util.*;




public class PieMultiViewMenu extends PieMenu {

    // The listener for all menu items (an inner class below)
    private Listener listener;

    // The permanent menu items
    private JMenuItem mvNew, mvDelete, mvProperties;



    public PieMultiViewMenu() {
	super("MultiView");

	listener = new Listener();

	mvNew = add("New");
	mvNew.setEnabled(true);
	mvNew.addActionListener(listener);
	
	mvDelete = add("Delete");
	mvDelete.setEnabled(true);
	mvDelete.addActionListener(listener);	

	mvProperties = add("Properties");
	mvProperties.setEnabled(true);
	mvProperties.addActionListener(listener);	
    }



    // An inner class to handle all menu item events
    class Listener implements ActionListener, PropertyListenerInterface {

	// Constructor
	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MULTIVIEW_COUNT, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	}


	// Menu item callback
	public void actionPerformed(ActionEvent e) {
	    Object selection = e.getSource();

	    if (selection == mvNew) {
		handleMultiViewNew();
		
	    } else if (selection == mvDelete) {
		handleMultiViewDelete();

	    } else if (selection == mvProperties) {
		handleMultiViewProperties();
	    
	    }
	}
	

	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	   
	    if (property.equals(PropertyManager.APPLICATION_MULTIVIEW_COUNT)) {
		int count = ((Integer) newValue).intValue();
		mvDelete.setEnabled(count > 0);
		mvProperties.setEnabled(count > 0);

	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int mode = ((Integer) newValue).intValue();
		switch(mode) {

		case PropertyManager.APPLICATION_EDIT:
		    mvNew.setEnabled(true);
		    int count = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_MULTIVIEW_COUNT)).intValue();
		    mvDelete.setEnabled(count > 0);
		    mvProperties.setEnabled(count > 0);
		    break;

		case PropertyManager.APPLICATION_PLAY:
		    mvNew.setEnabled(false);
		    mvDelete.setEnabled(false);
		    mvProperties.setEnabled(false);
		    break;

		default:
		    break;
		}
	    } 
	}


	private void handleMultiViewNew() {
	    MultiViewModel model = new MultiViewModel(PropertyManager.APPLICATION_SCOPE, null, null);
	    handleGoToMultiView(model);
	}

	private void handleMultiViewDelete() {
	    DataModel dataModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	    if (dataModel != null) {
		// Will reset the current model
		dataModel.deleteModel();
	    }
	}

	private void handleMultiViewProperties() {
	    DataModel model = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	    if (model != null) {
		ContentPropertyFrame frame = new ContentPropertyFrame("MultiView Properties");
	    }
	}


	private void handleGoToMultiView(MultiViewModel model) {
	    PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, model);
	}
    }
}



