/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: PieOperationMenu.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.lang.*;
import java.awt.event.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.util.*;




public class PieOperationMenu extends PieMenu {

    // The listener for all menu items (an inner class below)
    private Listener listener;

    // The menu items
    private JMenuItem operationPlay, operationStopEdit;
 

    public PieOperationMenu() {
	super("Operation");

	listener = new Listener();

	operationPlay = add("Play");
	operationPlay.addActionListener(listener);

	operationStopEdit = add("Stop & Edit");
	operationStopEdit.setEnabled(false);
	operationStopEdit.addActionListener(listener);

    }

    
    // An inner class to handle all menu item events
    class Listener implements ActionListener, PropertyListenerInterface {

	// Constructor
	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_STORYBOARD_COUNT, this);
	}


	// Menu item callback
	public void actionPerformed(ActionEvent e) {
	    Object selection = e.getSource();
	 
	    if (selection == operationPlay) {
		OperationMenuCommands.getOperationMenuCommands().handleOperationPlay();

	    } else if (selection == operationStopEdit) {
		OperationMenuCommands.getOperationMenuCommands().handleOperationStopAndEdit();
	
	    } else {
		System.out.println("ERROR: Unknown operation menu command");
	    }
	}
	

	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {

	    if (property.equals(PropertyManager.APPLICATION_MODE)) {
		
		int toMode = ((Integer) newValue).intValue();
		switch (toMode) {

		case PropertyManager.APPLICATION_EDIT:
		    // Going to edit mode
		    //operationPlay.setVisible(true);
		    operationPlay.setEnabled(true);

		    //operationStopEdit.setVisible(false);
		    operationStopEdit.setEnabled(false);

		    break;

		case PropertyManager.APPLICATION_PLAY:
		    // Going to play mode
		    //operationPlay.setVisible(false);
		    //operationStopEdit.setVisible(true);
		    operationPlay.setEnabled(false);
		    operationStopEdit.setEnabled(true);
		    break;
		
		default:
		    break;
		}

	    } else if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {

		// If switching models in play mode, do nothing
		int mode = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		if (mode == PropertyManager.APPLICATION_PLAY) {
		    return;
		}

		DataModel fromModel = (DataModel) oldValue;
		DataModel toModel = (DataModel) newValue;
		if (toModel != null) {
		    
		    if (fromModel == null) {
			// switching from null model to non-null model in edit mode
			operationPlay.setEnabled(true);
			//operationPlay.setVisible(true);

			//operationStopEdit.setVisible(false);
			operationStopEdit.setEnabled(false);

		    }

		} else if (toModel == null) {
		    // Realistically, must be in edit mode
		    operationPlay.setEnabled(false);
		}

		
	    } else if (property.equals(PropertyManager.APPLICATION_STORYBOARD_COUNT)) {
		int sCount = ((Integer) newValue).intValue();
		operationPlay.setEnabled(sCount > 0);
		
	    } 
	}
    }
}



