/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ShortcutPanel.java

  Provides shortcut buttons for switching among multiviews and storyboards, and runtime operations
*/


// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
//import edu.uofminnesota.DEMAIS.runtime.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;





public class ShortcutPanel extends JPanel {

    private JPanel editPanel, playPanel;
    private JButton storyboard, multiView, contentMgr, play, stop;
    private ShortcutPanelListener listener;



    public ShortcutPanel() {
	super(new GridBagLayout());
	setMinimumSize(new Dimension(105, 24));
	setPreferredSize(new Dimension(105, 24));

	listener = new ShortcutPanelListener();
	GridBagConstraints c = new GridBagConstraints();
	editPanel = new JPanel(new GridBagLayout());
	playPanel = new JPanel(new GridBagLayout());

	// Create the icons and buttons
	c.gridx = 0;
	c.gridy = 0;
	c.insets = new Insets(0, 3, 0, 3);

	contentMgr = createIconButton("contentmanager", true);
	contentMgr.setToolTipText("Open content manager");
	editPanel.add(contentMgr, c);

	multiView = createIconButton("multiView", true);
	multiView.setToolTipText("Edit multiview");
	++c.gridx;
	editPanel.add(multiView, c);

	storyboard = createIconButton("storyboard", true);
	storyboard.setVisible(false);
	storyboard.setToolTipText("Edit storyboard");
	++c.gridx;
	editPanel.add(storyboard, c);

	play = createIconButton("play", true);
	play.setToolTipText("Operationalize design");
	play.setEnabled(true);
	++c.gridx;
	editPanel.add(play, c);

	stop = createIconButton("stop", false);
	stop.setToolTipText("Stop and edit storyboard");
	c.gridx = 0;
	playPanel.add(stop, c);

	c.gridx = 0;
	c.gridy = 0;
	c.insets = new Insets(0, 0, 0, 0);
	c.anchor = GridBagConstraints.WEST;
	add(editPanel, c);

	c.gridx = 1;
	playPanel.setVisible(false);
	add(playPanel, c);

    }

  
    private JButton createIconButton(String name, boolean needsDisabled) {
	ImageIcon enabled = 
	    new ImageIcon(PropertyManager.APPLICATION_IMAGE_DIRECTORY +
			  PropertyManager.FILE_SEPARATOR +
			  name +
			  "-enabled.gif");

	JButton button = new JButton(enabled);
	
	if (needsDisabled) {
	    ImageIcon disabled = 
		new ImageIcon(PropertyManager.APPLICATION_IMAGE_DIRECTORY +
			      PropertyManager.FILE_SEPARATOR +
			      name +
			      "-disabled.gif");
	    button.setDisabledIcon(disabled); 
	}

	button.addActionListener(listener);

	Dimension d = new Dimension(28, 22);
	button.setPreferredSize(d);
	button.setMinimumSize(d);
	button.setMaximumSize(d);
	return button;
    }

 

    class ShortcutPanelListener implements ActionListener, PropertyListenerInterface {

	public ShortcutPanelListener() {
	    // Listen for property changes
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_STORYBOARD_COUNT, this);
	}


	public void actionPerformed(ActionEvent e) {
	    JButton selection = (JButton) e.getSource();
	    

	    if (selection == contentMgr) {
		ContentManagerFrame.getFrame().makeVisible(true);
	    
	    } else if (selection == storyboard) {
		DataModel toModel = StoryboardModel.getFirstStoryboardModel();
		if (toModel == null) {
		    toModel = new StoryboardModel(PropertyManager.APPLICATION_SCOPE, null, null);
		}
		PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, toModel);

	    } else if (selection == multiView) {
		DataModel toModel = MultiViewModel.getFirstMultiViewModel();
		if (toModel == null) {
		    toModel = new MultiViewModel(PropertyManager.APPLICATION_SCOPE, null, null);
		}
		PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, toModel);
	    
	    } else if (selection == play) {
		OperationMenuCommands.getOperationMenuCommands().handleOperationPlay();
		
	    } else if (selection == stop) {
		OperationMenuCommands.getOperationMenuCommands().handleOperationStopAndEdit();
		
	    } else {
		System.out.println("ERROR: Unknown shortcutpanel command");
	    }
	}
	
	
	
	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int toMode = ((Integer) newValue).intValue();

		switch (toMode) {

		case PropertyManager.APPLICATION_EDIT:
		    // Switch the panels
		    editPanel.setVisible(true);
		    playPanel.setVisible(false);
		    break;

		case PropertyManager.APPLICATION_PLAY:
		    // Switch the panels
		    editPanel.setVisible(false);
		    playPanel.setVisible(true);
		    break;
		
		default:
		    break;
		}		

	    } else if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		int mode = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		if (mode == PropertyManager.APPLICATION_PLAY) {
		    // If switching data models in play mode, do nothing
		    return;
		}

		DataModel toModel = (DataModel) newValue;
		if (toModel != null) {

		    if (toModel instanceof StoryboardModel) {
			// Switching to a storyboard model in edit mode
			storyboard.setVisible(false);
			storyboard.setEnabled(true);

			multiView.setVisible(true);
			multiView.setEnabled(true);

		    } else if (toModel instanceof MultiViewModel) {
			// Switching to a multi view model in edit mode
			storyboard.setVisible(true);
			storyboard.setEnabled(true);

			multiView.setVisible(false);
			multiView.setEnabled(true);
		    }
		    
		} else {
		    storyboard.setVisible(true);
		    storyboard.setEnabled(true);
		    multiView.setVisible(false);
		    multiView.setEnabled(true);
		}
	    
	    } else if (property.equals(PropertyManager.APPLICATION_STORYBOARD_COUNT)) {
		int sCount = ((Integer) newValue).intValue();
		play.setEnabled(sCount > 0);
	    }
	}
    } 
}
