/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: VisualLanguagePanel.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class VisualLanguagePanel extends JPanel {

    // Instance variables
    private VLButton singleMouseClickEvent;
    private VLButton doubleMouseClickEvent;
    private VLButton rolloverEvent;
    private VLButton enterTextEvent;
    private VLButton dragAndDropEvent;
    private VLButton beginMediaEvent;
    private VLButton syncMarkerEvent;
    private VLButton endMediaEvent;
    private VLButton temporalEvent;
    
    private VLButton navigateAction;
    private VLButton transitionAction;
    private VLButton beginAction;
    private VLButton setMediaTimeAction;
    private VLButton pauseAction;
    private VLButton endAction;
    private VLButton delayAction;
    private VLButton displayAction;
    private VLButton unDisplayAction;
    private VLButton highlightAction;
    private VLButton unHighlightAction;
    private VLButton playAudioAction;

    private Listener listener;
    private JLabel eventLabel, actionLabel;

    

    public VisualLanguagePanel() {
	super(new GridBagLayout());
	listener = new Listener();

	GridBagConstraints c = new GridBagConstraints();
	
	// Label the events
	eventLabel = new JLabel("Events: ");
	eventLabel.setFont(PropertyManager.APPLICATION_TITLE_FONT);

	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.NONE;
	add(eventLabel, c);

	// Create the events
	singleMouseClickEvent = new VLSingleClickEvent();
	singleMouseClickEvent.setToolTipText("Single Mouse Click");
	++c.gridx;
	add(singleMouseClickEvent, c);

	doubleMouseClickEvent = new VLDoubleClickEvent();
	doubleMouseClickEvent.setToolTipText("Double Mouse Click");
	c.insets = new Insets(0, 0, 0, 0);
	++c.gridx;
	add(doubleMouseClickEvent, c);
	
	rolloverEvent = new VLRolloverEvent();
	rolloverEvent.setToolTipText("Rollover");
	++c.gridx;
	add(rolloverEvent, c);

	enterTextEvent = new VLEnterTextEvent();
	enterTextEvent.setToolTipText("Enter text");
	++c.gridx;
	add(enterTextEvent, c);
	
	dragAndDropEvent = new VLDragAndDropEvent();
	dragAndDropEvent.setToolTipText("Drag and Drop");
	++c.gridx;
	add(dragAndDropEvent, c);
	
	beginMediaEvent = new VLBeginMediaEvent();
	beginMediaEvent.setToolTipText("Begin Media Event");
	++c.gridx;
	add(beginMediaEvent, c);
	
	syncMarkerEvent = new VLSyncMarkerEvent();
	syncMarkerEvent.setToolTipText("Syncrhonization Marker");
	++c.gridx;
	add(syncMarkerEvent, c);

	endMediaEvent = new VLEndMediaEvent();
	endMediaEvent.setToolTipText("End Media Event");
	++c.gridx;
	add(endMediaEvent, c);
	
	temporalEvent = new VLTemporalEvent();
	temporalEvent.setToolTipText("Timer");
	++c.gridx;
	add(temporalEvent, c);


	// Label the actions
	actionLabel = new JLabel("Actions: ");
	actionLabel.setFont(PropertyManager.APPLICATION_TITLE_FONT);

	c.insets = new Insets(0, 20, 0, 0);

	++c.gridx;
	add(actionLabel, c);

	navigateAction = new VLNavigationAction();
	navigateAction.setToolTipText("Navigation");
	c.insets = new Insets(0, 0, 0, 0);
	++c.gridx;
	add(navigateAction, c);

	//c.insets = new Insets(0, 0, 0, 0);

	transitionAction = new VLTransitionAction();
	transitionAction.setToolTipText("Visual Transition");
	++c.gridx;
	add(transitionAction, c);

	beginAction = new VLBeginMediaAction();
	beginAction.setToolTipText("Begin Media");
	++c.gridx;
	add(beginAction, c);

	setMediaTimeAction = new VLSetMediaTimeAction();
	setMediaTimeAction.setToolTipText("Set Media Time");
	++c.gridx;
	add(setMediaTimeAction, c);

	pauseAction = new VLPauseMediaAction();
	pauseAction.setToolTipText("Pause Media");
	++c.gridx;
	add(pauseAction, c);

	endAction = new VLEndMediaAction();
	endAction.setToolTipText("End Media");
	++c.gridx;
	add(endAction, c);

	delayAction = new VLDelayAction();
	delayAction.setToolTipText("Delay");
	++c.gridx;
	add(delayAction, c);

	displayAction = new VLDisplayAction();
	displayAction.setToolTipText("Display");
	++c.gridx;
	add(displayAction, c);

	unDisplayAction = new VLUnDisplayAction();
	unDisplayAction.setToolTipText("Undisplay");
	++c.gridx;
	add(unDisplayAction, c);

	highlightAction = new VLHighlightAction();
	highlightAction.setToolTipText("Highlight");
	++c.gridx;
	add(highlightAction, c);

	unHighlightAction = new VLUnHighlightAction();
	unHighlightAction.setToolTipText("Unhighlight");
	++c.gridx;
	add(unHighlightAction, c);

	playAudioAction = new VLPlayAudioAction();
	playAudioAction.setToolTipText("Play Audio");
	++c.gridx;
	add(playAudioAction, c);

    }



    class Listener implements PropertyListenerInterface {
	public Listener() {
	    // Listen for property changes
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	}


	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int fromMode = ((Integer) oldValue).intValue();
		int toMode = ((Integer) newValue).intValue();

		if ((fromMode == PropertyManager.APPLICATION_EDIT) &&
		    (toMode == PropertyManager.APPLICATION_PLAY)) {
		    // Set label foreground to grey
		    eventLabel.setEnabled(false);
		    actionLabel.setEnabled(false);
		    
		    // Resets the visual language icon panel
		    PropertyManager.setProperty(PropertyManager.APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED, new Integer(-1));
		    

		} else if ((fromMode == PropertyManager.APPLICATION_PLAY) &&
			   (toMode == PropertyManager.APPLICATION_EDIT)) {
		    // Set label foreground to black
		    eventLabel.setEnabled(true);
		    actionLabel.setEnabled(true);
		}   
	    }
	}
    }
}



