/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: WindowMenu.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.base;

// Necessary imports
import java.lang.*;
import java.awt.event.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.narration.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;
import edu.uofminnesota.DEMAIS.usrinterface.layer.*;



public class WindowMenu extends JMenu {

    // The listener for all menu items (an inner class below)
    private Listener listener;

    // The menu items
    private JMenuItem vScript, cManager, layerControl, objectControl;
  

    public WindowMenu() {
	super("Window");

	listener = new Listener();

	cManager = new JMenuItem("Content Manager");
	cManager.addActionListener(listener);
	add(cManager);

	vScript = new JMenuItem("Voice Script");
	vScript.addActionListener(listener);
	add(vScript);
	
	addSeparator();

	layerControl = new JMenuItem("Hide Layers");
	layerControl.addActionListener(listener);
	add(layerControl);

	objectControl = new JMenuItem("Show Ink Stroke Properties");
	objectControl.addActionListener(listener);
	add(objectControl);

    }


    // An inner class to handle all menu item events
    class Listener implements ActionListener, PropertyListenerInterface {

	// Constructor
	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.LAYER_CONTROL_VISIBLE, this);
	    PropertyManager.addPropertyListener(PropertyManager.OBJECT_PROPERTY_FRAME_VISIBLE, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	}


	// Menu item callback
	public void actionPerformed(ActionEvent e) {
	    Object selection = e.getSource();

	    if (selection == layerControl) {
		handleLayerControl();
		
	    } else if (selection == cManager) {
		handleContentManager();

	    } else if (selection == vScript) {
		handleVoiceScript();

	    } else if (selection == objectControl) {
		handleObjectControl();

	    } else {
		System.out.println("ERROR: Unknown window menu command");
	    }
	}
	

	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.LAYER_CONTROL_VISIBLE)) {
		boolean visible = ((Boolean) newValue).booleanValue();
		if (visible) {
		    layerControl.setText("Hide Layers");
		} else {
		    layerControl.setText("Show Layers");
		}

	    } else if (property.equals(PropertyManager.OBJECT_PROPERTY_FRAME_VISIBLE)) {
		boolean visible = ((Boolean) newValue).booleanValue();
		if (visible) {
		    objectControl.setText("Hide Ink Stroke Properties");
		} else {
		    objectControl.setText("Show Ink Stroke Properties");
		}

	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int mode = ((Integer) newValue).intValue();
		switch (mode) {
		case PropertyManager.APPLICATION_EDIT:
		    layerControl.setEnabled(true);
		    // Visibility will be set elsewhere, causing the text to be set properly
		    objectControl.setEnabled(true);
		    cManager.setEnabled(true);
		    vScript.setEnabled(true);
		    break;

		case PropertyManager.APPLICATION_PLAY:
		    layerControl.setEnabled(false);
		    objectControl.setEnabled(false);
		    cManager.setEnabled(false);
		    vScript.setEnabled(false);
		    break;

		default:
		    break;
		}

	    } else if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		int mode = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		if (mode == PropertyManager.APPLICATION_EDIT) {
		    DataModel toModel = (DataModel) newValue;
		    if (toModel instanceof StoryboardModel) {
			layerControl.setVisible(true);
			objectControl.setVisible(true);

		    } else if (toModel instanceof MultiViewModel) {
			layerControl.setVisible(false);
			objectControl.setVisible(false);
		    }
		}
	    }
	}

	
	private void handleLayerControl() {
	    boolean visible = ((Boolean)PropertyManager.getProperty(PropertyManager.LAYER_CONTROL_VISIBLE)).booleanValue();
	    LayerControl.getLayerControl().makeVisible(!visible, true);
	}


	private void handleObjectControl() {
	    boolean visible = ((Boolean)PropertyManager.getProperty(PropertyManager.OBJECT_PROPERTY_FRAME_VISIBLE)).booleanValue();
	    ObjectPropertyFrame.getObjectPropertyFrame().makeVisible(!visible, true);
	}

	private void handleContentManager() {
	    ContentManagerFrame cmFrame = ContentManagerFrame.getFrame();
	    cmFrame.makeVisible(true);
	}
	

	private void handleVoiceScript() {
	    // If narration object is selected in the manager view
	    // open that model, otherwise create a new one
	    NarrationModel model = new NarrationModel(PropertyManager.APPLICATION_SCOPE, null, null, null);
	    NarrationEditView editView = new NarrationEditView(model);
	    model.addView(editView);
	    editView.show();
	}

    }
}



