/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentManagerCell.java

  Represents one of the squares in the ContentManager.
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.contentmanager;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.meal.*;




public class ContentManagerCell extends JPanel {

    public static final int WIDTH = 100;
    public static final int HEIGHT = 100;
    private ContentManagerCellView cmcView;



    public ContentManagerCell() {
	super(new GridBagLayout());

	setBorder(BorderFactory.createLoweredBevelBorder());
	cmcView = null;

	Dimension d = new Dimension(WIDTH, HEIGHT);
	setMinimumSize(d);
	setPreferredSize(d);
	setMaximumSize(d);
    }


    public void insertView(ContentManagerCellView view) {
	cmcView = view;
	view.setParentCell(this);

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	add(view, c);
    }


    public boolean containsView(ContentManagerCellView view) {
	return (cmcView == view);
    }


    public boolean containsModel(DataModel model) {
	if (cmcView != null) {
	    DataModel dm = cmcView.getModel();
	    return (dm.equals(model));
	}
	return false;
    }


    public void deleteView() {
	cmcView = null;
	removeAll();
	repaint();
    }


    public ContentManagerCellView getView() {
	return cmcView;
    }


    public boolean isEmpty() {
	return (cmcView == null);
    }


    public boolean isCellSelected() {
	return ( (cmcView != null) && (cmcView.isCellSelected()));
    }

    
    public void setCellSelected(boolean value) {
	if (cmcView != null) {
	    cmcView.setCellSelected(value);
	}
    }
    
    public void refreshView() {
	repaint();
    }

}

