/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentManagerEditMenu.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.contentmanager;

// Necessary imports
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.undo.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;




public class ContentManagerEditMenu extends JMenu {

    private static final int DELETE = 0;
    private static final int COPY = 1;
    private static final int PASTE = 2;


    // The listener for all menu items (an inner class below)
    private Listener listener;

    // The menu items
    private JMenuItem editDelete, editCopy, editPaste, editProperties;
      
    private Collection clipboard;
    private int lastOp;
    private ContentManagerFrame cmFrame;



    public ContentManagerEditMenu(ContentManagerFrame cmFrame) {
	super("Edit");
	this.cmFrame = cmFrame;

	PropertyManager.CONTENT_MANAGER_EDIT_MENU_WIDGET = this;
	clipboard = new Vector();
	lastOp = DELETE;

	listener = new Listener();	

	editDelete = new JMenuItem("Delete");
	editDelete.setEnabled(false);
	editDelete.addActionListener(listener);
	add(editDelete);

	editCopy = new JMenuItem("Copy");
	editCopy.setEnabled(false);
	editCopy.addActionListener(listener);
	add(editCopy);

	editPaste = new JMenuItem("Paste");
	editPaste.setEnabled(false);
	editPaste.addActionListener(listener);
	add(editPaste);

	addSeparator();

	editProperties = new JMenuItem("Properties");
	editProperties.setEnabled(false);
	editProperties.addActionListener(listener);
	add(editProperties);

    }
    
    
    protected void handleDelete() {
	PropertyManager.setProperty(PropertyManager.CONTENT_MANAGER_OBJECT_IN_CLIPBOARD, Boolean.FALSE);
	cmFrame.deleteSelectedCells();
	lastOp = DELETE;
    }
    
    
    protected void handleCopy() {
	PropertyManager.setProperty(PropertyManager.CONTENT_MANAGER_OBJECT_IN_CLIPBOARD, Boolean.TRUE);
	clipboard = cmFrame.getSelectedCells();
	lastOp = COPY;
    }
    
    
    protected void handlePaste() {
	Iterator it = clipboard.iterator();
	while (it.hasNext()) {
	    DataModel model = (DataModel) it.next();
	    model.clone();
	}
	lastOp = PASTE;
	cmFrame.getPane().revalidate();
    }
    
    
    protected void handleProperties() {

	// Determine which cell view object is selected
	ContentManagerCellView cellView = cmFrame.getFirstSelectedCellView();	    
	
	// Open a properties dialog box for that cell view's object
	ContentManagerPropertyFrame frame = new ContentManagerPropertyFrame(cellView);
    }
    

    
    // An inner class to handle all menu item events
    class Listener implements ActionListener, PropertyListenerInterface {

	// Constructor
	public Listener() {	
	    PropertyManager.addPropertyListener(PropertyManager.CONTENT_MANAGER_OBJECT_SELECTED_COUNT, this);
	    PropertyManager.addPropertyListener(PropertyManager.CONTENT_MANAGER_OBJECT_IN_CLIPBOARD, this);   

	}

	// Menu item callback
	public void actionPerformed(ActionEvent e) {
	    Object selection = e.getSource();

	    if (selection == editProperties) {
		handleProperties();

	    } else if (selection == editDelete) {
		handleDelete();

	    } else if (selection == editCopy) {
		handleCopy();
 
	    } else if (selection == editPaste) {
		handlePaste();

	    } else {
		System.out.println("ERROR: Unknown edit menu command");
	    }
	}


	// PropertyListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {

	    if (property.equals(PropertyManager.CONTENT_MANAGER_OBJECT_SELECTED_COUNT)) {
		int selected = ((Integer) newValue).intValue();		
		editDelete.setEnabled(selected > 0);
		editCopy.setEnabled(selected > 0);
		editProperties.setEnabled(selected == 1);

	    } else if (property.equals(PropertyManager.CONTENT_MANAGER_OBJECT_IN_CLIPBOARD)) {
		boolean b = ((Boolean) newValue).booleanValue();
		editPaste.setEnabled(b);
	    } 
	}
    }
}
