/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentManagerPane.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.contentmanager;

// Necessary imports
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;




public class ContentManagerPane extends JScrollPane {

    private JPanel container;
    private ContentManagerCell cells[];
    private Listener listener;


 
    public ContentManagerPane() {
	super(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
	      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	JScrollBar bar = getVerticalScrollBar();
	if (bar != null) {
	    bar.setUnitIncrement(ContentManagerCell.HEIGHT);
	}
	bar = getHorizontalScrollBar();
	if (bar != null) {
	    bar.setUnitIncrement(ContentManagerCell.WIDTH);
	}
	
	listener = new Listener();

	int cmRows = PropertyManager.CONTENT_MANAGER_ROWS;
	int cmColumns = PropertyManager.CONTENT_MANAGER_COLUMNS;
	container = new JPanel(new GridLayout(cmRows, cmColumns));

	createCells(container);
	setViewportView(container);
    }



    public void addDataModel(DataModel model) {
	int position = locateNextCell();
	addDataModel(model, position); 
    }
    
    public void addDataModel(DataModel model, int position) {
	ContentManagerCellView view = model.getContentManagerCellView();
	if (view != null) {
	    cells[position].insertView(view);
	}
    }


    public Collection getContentCellViewModels() {
	Vector v = new Vector();
	for (int i=0; i<cells.length; ++i) {
	    if (! cells[i].isEmpty()) {
		ContentManagerCellView cmcv = cells[i].getView();
		DataModel model = cmcv.getModel();
		v.add(model);
	    }
	}
	return v;
    }


    public void unSelectAll() {
	for (int i=0; i<cells.length; ++i) {
	    cells[i].setCellSelected(false);
	}
	PropertyManager.setProperty(PropertyManager.CONTENT_MANAGER_OBJECT_SELECTED_COUNT, new Integer(0));
    }


    public void selectOnlyModel(DataModel model) {
	unSelectAll();
	for (int i=0; i<cells.length; ++i) {
	    if ( (! cells[i].isEmpty()) && (cells[i].containsModel(model)) ) {
		cells[i].setCellSelected(true);
	    }
	}
    }


    public void deleteSelectedCells() {
	Collection c = getSelectedCells();
	Iterator it = c.iterator();
	while (it.hasNext()) {
	    DataModel model = (DataModel) it.next();
	    model.deleteModel();
	}
    }

    // Returns a list of references to selected storyboard models
    public Collection getSelectedCells() {
	Vector v = new Vector();
	for (int i=0; i<cells.length; ++i) {
	    if (cells[i].isCellSelected()) {
		ContentManagerCellView smcv = cells[i].getView();
		DataModel model = smcv.getModel();
		v.add(model);
	    }
	}
	return v;
    }

  
    public ContentManagerCellView getFirstSelectedCellView() {
	for (int i=0; i<cells.length; ++i) {
	    if (cells[i].isCellSelected()) {
		return (cells[i].getView());
	    }
	}
	return null;
    }




    // Private methods
    private void createCells(JPanel container) {
	int cmRows = PropertyManager.CONTENT_MANAGER_ROWS;
	int cmColumns = PropertyManager.CONTENT_MANAGER_COLUMNS;

	cells = new ContentManagerCell[cmRows * cmColumns];
	for (int i=0; i < cells.length; ++i) {
	    cells[i] = new ContentManagerCell();
	    container.add(cells[i]);
	}
    }


    // Locate the next unused cell
    private int locateNextCell() {
	int pos = -1;
	for (int i=0; i<cells.length; ++i) {
	    if (cells[i].isEmpty()) {
		pos = i;
		break;
	    }
	}
	if (pos < 0) {
	    DEMAISError.showErrorMessage("ContentManagerPane: locateNextCell() - All cells are occupied",
					    "ContentManagerPane", true);
	}
	return pos;
    }


    public void refreshView() {
	repaint();
    }


    public void deleteCellView(ContentManagerCellView view) {
	for (int i=0; i<cells.length; ++i) {
	    if ( (!cells[i].isEmpty()) && (cells[i].containsView(view)) ) {
		cells[i].deleteView();
		break;
	    }
	}
    }

    class Listener implements PropertyListenerInterface {
	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	}


	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		DataModel model = (DataModel) newValue;
		if (model != null) {
		    selectOnlyModel(model);
		}
	    }
	}
    }
}
