/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentManagerPopupMenu.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.contentmanager;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;



public class ContentManagerPopupMenu extends JPopupMenu {

    // Local instance of the popup menu
    private static ContentManagerPopupMenu popupMenu;
    // Listener for the menu selection command
    private Listener listener;
    // The ContentManagerEditMenu object
    private ContentManagerEditMenu editMenuObject;
    // Local widgets for this popup
    private JMenuItem deleteMenu, copyMenu, pasteMenu, propertiesMenu;


    // Static initializer
    static {
	popupMenu = null;
    }

    public static ContentManagerPopupMenu getPopupMenu() {
	if (popupMenu == null) {
	    popupMenu = new ContentManagerPopupMenu();
	}
	return popupMenu;
    }


    // Do no allow others to instantiate this object
    private ContentManagerPopupMenu() {
	editMenuObject = (ContentManagerEditMenu) PropertyManager.CONTENT_MANAGER_EDIT_MENU_WIDGET;
	
	listener = new Listener();
	createPopupMenu();
    }

	
    private void createPopupMenu() {
	// Set the cursor
	setCursor(PropertyManager.APPLICATION_CURSOR);

	deleteMenu = new JMenuItem("Delete");
	deleteMenu.addActionListener(listener);
	deleteMenu.setEnabled(true);
	this.add(deleteMenu);
	
	copyMenu = new JMenuItem("Copy");
	copyMenu.addActionListener(listener);
	copyMenu.setEnabled(true);
	this.add(copyMenu);
	
	pasteMenu = new JMenuItem("Paste");
	pasteMenu.addActionListener(listener);
	pasteMenu.setEnabled(false);
	this.add(pasteMenu);
	
	this.addSeparator();

	propertiesMenu = new JMenuItem("Properties");
	propertiesMenu.addActionListener(listener);
	propertiesMenu.setEnabled(true);
	this.add(propertiesMenu);
	    
    }


    class Listener implements ActionListener, PropertyListenerInterface {

	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.CONTENT_MANAGER_OBJECT_SELECTED_COUNT, this);
	    PropertyManager.addPropertyListener(PropertyManager.CONTENT_MANAGER_OBJECT_IN_CLIPBOARD, this);   
	}
	
	public void actionPerformed(ActionEvent e) {
	    JMenuItem item = (JMenuItem) e.getSource();
	    if (item == deleteMenu) {
		editMenuObject.handleDelete();

	    } else if (item == copyMenu) {
		editMenuObject.handleCopy();

	    } else if (item == pasteMenu) {
		editMenuObject.handlePaste();

	    } else if (item == propertiesMenu) {
		editMenuObject.handleProperties();

	    } else {
		System.out.println("Error: unknown popup menu command");
	    }
	}

	// PropertyEventListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.CONTENT_MANAGER_OBJECT_IN_CLIPBOARD)) {
		boolean b = ((Boolean) newValue).booleanValue();
		pasteMenu.setEnabled(b);
	    } 
	}


    }
}


