/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: ContentManagerPropertyFrame.java

  Implemented as a singleton pattern
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.contentmanager;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;



public class ContentManagerPropertyFrame extends JDialog {

    private ContentManagerPropertyPanel cmpPanel;
    private ContentManagerCellView cellView;
    private Listener listener;



    private ContentManagerPropertyFrame() {
	this(null);
    }

    
    public ContentManagerPropertyFrame(ContentManagerCellView cView) {
	// Set the appropriate title
	DataModel model = cView.getModel();
	if (model instanceof StoryboardModel) {
	    setTitle("Storyboard Properties");
	} else if (model instanceof MultiViewModel) {
	    setTitle("MultiView Properties");
	} else if (model instanceof NarrationModel) {
	    setTitle("Narration Properties");
	} else {
	    setTitle("Properties");
	}

	setModal(true);

	// Set the cursor
	setCursor(PropertyManager.APPLICATION_CURSOR);

	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

	cellView = cView;
	listener = new Listener();
	
	// Listen for window events
	addWindowListener(listener);
	setLookAndFeel();
	setupContentPane();    
	
	setResizable(false);
	pack();
	setRootLocation();
	setVisible(true);
    }


    public void closeWindow() {
	cmpPanel.updateProperties();
	setVisible(false);
	dispose();
    }

    
    // Private methods
    private void setLookAndFeel() {
	try {
	    UIManager.setLookAndFeel(DEMAISFrame.getLookAndFeelClassName());
	} catch (Exception xcptn) {
	    DEMAISError.showErrorMessage("Could not set look and feel", "Content Manager Property", false);
	    DEMAISError.logException(xcptn);
	}
    }


    private void setupContentPane() {
	JPanel contentPane = new JPanel(new GridBagLayout());
	contentPane.setBorder(BorderFactory.createLoweredBevelBorder());
	setContentPane(contentPane);
	
	GridBagConstraints c = new GridBagConstraints();

	cmpPanel = new ContentManagerPropertyPanel(this, cellView);
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	contentPane.add(cmpPanel, c);
    }


    private void setRootLocation() {
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension me = this.getSize();

	Point p = new Point( (int)(screen.getWidth() - me.getWidth()) / 2, 
			     (int)(screen.getHeight() - me.getHeight()) / 2);
	setLocation(p);
    }



    class Listener extends WindowAdapter {

	public Listener() {
	}

	// WindowListener Interface
	public void windowClosing(WindowEvent e) {
	    // The window is closing
	    closeWindow();
	}
    }
  
}




