/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File LayerControl.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.layer;
    

// Necessary imports
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.meal.layer.*;
import edu.uofminnesota.DEMAIS.property.*;




public class LayerControl extends JDialog {

    // Singleton reference
    private static LayerControl layerControl;


    // Cached reference to current storyboard
    private StoryboardModel model;

    // Linked List of the layer controls 
    private Vector layerControls;
 
    // Event listener
    private Listener listener;
   
    // Layer panel
    private JPanel layerPanel;

    // Last panel dnd cursor was in
    private int dnd_last_layer;

    // Position at which to insert layer for dnd
    private int dnd_position;





    // Static initializer
    static {
	layerControl = null;
    }

    public static LayerControl getLayerControl() {
	if (layerControl == null) {
	    layerControl = new LayerControl();
	    // Force the popup menu to load as well
	    LayerPopupMenu.getPopupMenu();
	}
	return layerControl;
    }



    private LayerControl() {
	super(PropertyManager.APPLICATION_FRAME_WIDGET, "Layers", false);
	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	
	// Set the cursor
	setCursor(PropertyManager.APPLICATION_CURSOR);

	// Retrieve initial storyboard model
	DataModel dModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	if (dModel instanceof StoryboardModel) {
	    model = (StoryboardModel) dModel;
	} else {
	    model = null;
	}

	// Create an empty list
	layerControls = new Vector();

	Listener listener = new Listener();

	dnd_last_layer = 0;
	dnd_position = 0 ;

	// Listen for window events
	addWindowListener(listener);

	// Add key listener
	addKeyListener(listener);

	// Create the content panel
	JPanel contentPanel = new JPanel(new GridBagLayout());
	setContentPane(contentPanel);
	
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.insets = new Insets(12, 0, 12, 0);
	contentPanel.add(createTaskPanel(), c);

	// Create a container for the layer panel
	JPanel layerPanelContainer = new JPanel(new GridBagLayout());

	layerPanel = new JPanel(new GridBagLayout());
	layerPanel.setBorder(BorderFactory.createMatteBorder(1, 0, 1, 0, Color.black));

	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.NORTH;
	c.insets = new Insets(0, 0, 0, 0);
	layerPanelContainer.add(layerPanel, c);

	// Put the layer panel inside a scroll pane 
	JScrollPane scrollPane = new JScrollPane(layerPanelContainer, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	JScrollBar bar = scrollPane.getVerticalScrollBar();
	if (bar != null) {
	    bar.setUnitIncrement(PropertyManager.LAYER_CONTROL_LAYER_HEIGHT);
	}
	bar = scrollPane.getHorizontalScrollBar();
	if (bar != null) {
	    bar.setUnitIncrement(5);
	}

	// Stick the scroll pane inside the content pane
	++c.gridy;
	c.weightx = 1.0;
	c.weighty = 1.0;
	c.fill = GridBagConstraints.BOTH;
	contentPanel.add(scrollPane, c);

	// Load the layer controls into the layer panel
	refreshLayers();

	pack();

	setSize(new Dimension(PropertyManager.LAYER_CONTROL_WIDTH, PropertyManager.LAYER_CONTROL_HEIGHT));
	int appWidth = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_WIDTH)).intValue();
	int appHeight = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_HEIGHT)).intValue();
	setLocation(appWidth - getWidth() - ((int) Math.round(.05 * appWidth)), 
		    ((int) Math.round(.10 * appHeight)));

	makeVisible(true, false);
    }



    // Public methods
    public void makeVisible(boolean visible, boolean byUser) {
	if (byUser) {
	    PropertyManager.setProperty(PropertyManager.LAYER_CONTROL_HIDDEN_BY_USER, new Boolean(! visible));
	}

	/* 
	   If the request is to make the Layer Control visible, then it must not have been previously closed by the user
	 */
	boolean hiddenByUser = ((Boolean)PropertyManager.getProperty(PropertyManager.LAYER_CONTROL_HIDDEN_BY_USER)).booleanValue();
	if ((visible) && (!hiddenByUser)) {
	    setVisible(true);
	    PropertyManager.setProperty(PropertyManager.LAYER_CONTROL_VISIBLE, Boolean.TRUE);

	} else if (!visible) {
	    setVisible(false);
	    PropertyManager.setProperty(PropertyManager.LAYER_CONTROL_VISIBLE, Boolean.FALSE);

	}
    }



    public void newLayer() {
	// Pop up new layer property panel
	LayerNewPropertyFrame lnpf = new LayerNewPropertyFrame();
	// When the user selects the close button, the lnpf calls doNewLayer
    }


    protected void doNewLayer(String name, int type) {
	// First, deselect current layer
	if (model != null) {
	    // Retrieve and then unselect the current layer
	    int selectedLayer = model.getSelectedLayer();
	    model.setLayerSelected(selectedLayer, false);

	    // New layer is selected and visible
	    int size = model.getNumLayers();
	    LayerModel lm = null;
	    if (type == PropertyManager.ANNOTATION_LAYER) {
		lm = new AnnotationLayer(name, size, true, true);
	    } else {
		lm = new ContentLayer(name, size, true, true);
	    }
	    model.newLayer(lm);

	    // Refresh the controls
	    refreshLayers();
	}
    }


    public void copyLayer() {
	if (model != null) {
	    // Retrieve and then unselect the current layer
	    int selectedLayer = model.getSelectedLayer();
	    model.setLayerSelected(selectedLayer, false);
	    
	    // Copy layer to a new layer
	    int size = model.getNumLayers();
	    // Copied layer is selected and visible
	    model.copyLayer(selectedLayer, 
			    model.getLayerName(selectedLayer) + " copy", 
			    size, true, true);
	    
	    // Refresh the controls
	    refreshLayers();
	}
    }


    public void deleteLayer() {
	if (model != null) {
	    // Deselect current layer
	    int selectedLayer = model.getSelectedLayer();
	    model.setLayerSelected(selectedLayer, false);

	    // Determine next layer
	    int nextSelected = selectedLayer - 1;
	    if (nextSelected < 0) {
		nextSelected = 0;
	    }
	    
	    // Delete the layer
	    model.deleteLayer(selectedLayer, nextSelected);
	    model.setLayerSelected(nextSelected, true);

	    refreshLayers();
	}
    }


    protected void repositionLayer(int sourceLayer, int toPosition) {
	if (model != null) {
	    model.repositionLayer(sourceLayer, toPosition);
	    refreshLayers();
	}
    }

    
    public void flattenLayers() {
	if (model != null) {
	    model.flattenLayers();
	    refreshLayers();
	}
    }


    protected void setLayerName(int layerNum, String name) {
	LayerControlPanel lcp = (LayerControlPanel) getLayerControlPanelByNumber(layerNum);
	if (lcp != null) {
	    lcp.setLabelText(name);
	    if (model != null) {
		model.setLayerName(layerNum, name);
	    }
	}
    }


    protected void setLayerVisible(int layer, boolean visible) {
	if (model != null) {
	    // Updates the model, which then updates the storyboard
	    model.setLayerVisible(layer, visible);
	}
    }


    protected LayerModel getSelectedLayerModel() {
	LayerModel lm = null;
	if (model != null) {
	    lm = model.getSelectedLayerModel();
	}
	return lm;
    }


    protected int getNumLayers() {
	int num = 0;
	if (model != null) {
	    num = model.getNumLayers();
	}
	return num;
    }



    // Next four routines support the rearrangement of layers using drag and drop
    protected int getLastDNDLayer() {
	return dnd_last_layer;
    }

    protected void setLastDNDLayer(int l) {
	dnd_last_layer = l;
    }

    protected int getDNDPosition() {
	return dnd_position;
    }

    protected void setDNDPosition(int pos) {
	dnd_position = pos;
    }


    protected void setLayerBorder(int layer, Border border) {
	LayerControlPanel lcp = (LayerControlPanel) getLayerControlPanelByNumber(layer);
	if (lcp != null) {
	    lcp.setLayerBorder(border);
	}
    }

    protected void setLayerBorders(Border border) {
	Iterator it = layerControls.iterator();
	while (it.hasNext()) {
	    LayerControlPanel lcp = (LayerControlPanel) it.next();
	    lcp.setLayerBorder(border);
	}
    }



    private void refreshLayers() {
	// Remove all layer controls
	layerPanel.removeAll();
	
	if (model == null) {
	    layerControls.clear();
	    ((JPanel)getContentPane()).revalidate();
	    PropertyManager.setProperty(PropertyManager.LAYER_CONTROL_LAYER_COUNT, new Integer(0));
	    PropertyManager.setProperty(PropertyManager.LAYER_CONTROL_LAYER_SELECTED, new Integer(-1));
	    return;
	}

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 1.0;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.WEST;

	LayerList ll = model.getLayers();
	int size = ll.size();
	layerControls = new Vector();

	int selected = 0;
	ListIterator it = ll.listIterator(size);
	while (it.hasPrevious()) {
	    LayerModel lm = (LayerModel) it.previous();
	    
	    LayerControlPanel lcp = new LayerControlPanel(lm, size);
	    layerPanel.add(lcp, c);

	    /*
	      Add the control to the head of the list, thus preserving the fact that the control's layer number represents its position in the list
	    */
	    layerControls.insertElementAt(lcp, 0);
	    ++c.gridy;
	    if (lm.isSelected()) {
		selected = lm.getLayer();
	    }
	}
	// Redisplay the layer controls
	((JPanel)getContentPane()).revalidate();

	// Update properties
	PropertyManager.setProperty(PropertyManager.LAYER_CONTROL_LAYER_COUNT, new Integer(ll.size()));
	PropertyManager.setProperty(PropertyManager.LAYER_CONTROL_LAYER_SELECTED, new Integer(selected));

    }


    private JPanel createTaskPanel() {
	JPanel taskPanel = new JPanel(new GridBagLayout());
	LayerTaskButton button = new LayerTaskButton();

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.EAST;
	c.insets = new Insets(0, 0, 0, 22);
	taskPanel.add(button, c);

	return taskPanel;
    }


    private LayerControlPanel getLayerControlPanelByNumber(int layerNum) {
	LayerControlPanel lcp = null;
	if ((layerNum >= 0) && (layerNum < layerControls.size())) {
	    lcp = (LayerControlPanel)layerControls.elementAt(layerNum);
	}
	return lcp;
    }



    class Listener extends WindowAdapter 
	implements KeyListener, PropertyListenerInterface {

	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, this);
	    PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, this);
	    PropertyManager.addPropertyListener(PropertyManager.LAYER_CONTROL_LAYER_SELECTED, this);

	}

	public void windowClosing(WindowEvent e) {
	    // Hide the window, but do not dispose of it
	    makeVisible(false, true);
	}

	
	public void keyPressed(KeyEvent evt) {
	}
	
	public void keyReleased(KeyEvent evt) {
	    DataModel target = null;
      
	    int keyCode = evt.getKeyCode();
	    switch (keyCode) {
	    case KeyEvent.VK_PAGE_UP:
		target = StoryboardModel.getPreviousStoryboardModel(model, false);
		if (target != null) {
		    PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, target);
		} else {
		    Toolkit.getDefaultToolkit().beep();
		}
		break;
		
	    case KeyEvent.VK_PAGE_DOWN:
		target = StoryboardModel.getNextStoryboardModel(model);
		if (target != null) {
		    PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, target);
		} else {
		    Toolkit.getDefaultToolkit().beep();
		}
		
		break;
		
	    default:
		break;
	    }
	}
	
	public void keyTyped(KeyEvent evt) {
	}


	
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {

	    if (property.equals(PropertyManager.LAYER_CONTROL_LAYER_SELECTED)) {
		int currentSelected = ((Integer) oldValue).intValue();
		int nextSelected = ((Integer) newValue).intValue();
		// Unselect current
		if (model != null) {
		    model.setLayerSelected(currentSelected, false);
		}
		LayerControlPanel lcp = getLayerControlPanelByNumber(currentSelected);
		if (lcp != null) {
		    lcp.setLayerSelected(false);
		}

		// Select the new layer
		if (model != null) {
		    model.setLayerSelected(nextSelected, true);
		}
		lcp = getLayerControlPanelByNumber(nextSelected);
		if (lcp != null) {
		    lcp.setLayerSelected(true);
		}		
		
	    } else if (property.equals(PropertyManager.APPLICATION_CURRENT_DATA_MODEL)) {
		int mode = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
		if (mode == PropertyManager.APPLICATION_EDIT) {
		    DataModel toModel = (DataModel) newValue;
		    if (toModel != null) {
			if (toModel instanceof StoryboardModel) {
			    model = (StoryboardModel) toModel;
			    refreshLayers();
			    makeVisible(true, false);
			    
			} else if (toModel instanceof MultiViewModel) {
			    makeVisible(false, false);
			}

		    } else {
			model = null;
			refreshLayers();
		    }
		}
		
	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int fromMode = ((Integer) oldValue).intValue();
		int toMode = ((Integer) newValue).intValue();
		if ((fromMode == PropertyManager.APPLICATION_EDIT) &&
		    (toMode == PropertyManager.APPLICATION_PLAY)) {
		    makeVisible(false, false);
		}
		// Going from play to edit mode is captured by a model change
	    }
	}
    }
}
