/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File LayerControlPanel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.layer;
    

// Necessary imports
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.layer.*;




public class LayerControlPanel extends JPanel {

    // Class variables
    public static final String DND_CODE = "LAYER_CONTROL_DRAG ";
    private static final Border STANDARD_BORDER;
    private static final Border DND_TOP_BORDER;
    private static final Border DND_BOTTOM_ONLY_BORDER;


    // Instance variables
    private LayerModel layerModel;
    private String layerName;
    private int layer;
    private boolean selected;

    private Listener listener;
    private JCheckBox checkBox;
    private JPanel labelPanel;
    private LayerLabel label;



    static {
	STANDARD_BORDER = BorderFactory.createMatteBorder(0, 0, 2, 0, Color.white);
	DND_TOP_BORDER = BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(3, 0, 0, 0, Color.black), BorderFactory.createMatteBorder(0, 0, 2, 0, Color.white));
	DND_BOTTOM_ONLY_BORDER = BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 0, 3, 0, Color.black), null);
	
    }


    // Constructors
    private LayerControlPanel() {
	this(null, 0);
    }
    
    public LayerControlPanel(LayerModel lm, int maxLayers) {
	super(new GridBagLayout());
	layerModel = lm;
	layerName = layerModel.getName();
	layer = layerModel.getLayer();
	selected = layerModel.isSelected();

	listener = new Listener();

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.NONE;
	add(createCheckBoxPanel(layerModel.isVisible()), c);

	++c.gridx;
	c.weightx = 1.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.insets = new Insets(0, 2, 0, 0);
	add(createLabelPanel(maxLayers), c);
    }


    public void setLabelText(String name) {
	layerName = name;
	label.setText(name);
    }


    public boolean isLayerVisible() {
	return checkBox.isSelected();
    }

    public boolean isLayerSelected() {
	return selected;
    }


    protected void setLayerBorder(Border border) {
	if (layer > 0) {
	    labelPanel.setBorder(border);
	} else {
	    labelPanel.setBorder(null);
	}
    }


    protected void setLayerSelected(boolean s) {
	selected = s;
	if (selected) {
	    labelPanel.setBackground(PropertyManager.LAYER_CONTROL_SELECTED_COLOR);
	    label.setBackground(PropertyManager.LAYER_CONTROL_SELECTED_COLOR);
	    label.setForeground(Color.white);
	} else {
	    labelPanel.setBackground(PropertyManager.LAYER_CONTROL_UNSELECTED_COLOR);
	    label.setBackground(PropertyManager.LAYER_CONTROL_UNSELECTED_COLOR);
	    label.setForeground(Color.black);
	}
    }



    private void setLayerVisible(boolean visible) {
	LayerControl.getLayerControl().setLayerVisible(layer, visible);
    }


    private JPanel createCheckBoxPanel(boolean visible) {
	JPanel panel = new JPanel(new GridBagLayout());

	checkBox = new JCheckBox();
	checkBox.setSelected(visible);
	checkBox.addItemListener(listener);

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.NONE;
	c.insets = new Insets(0, 8, 0, 0);
	panel.add(checkBox, c);

	return panel;
    }


    private JPanel createLabelPanel(int max) {
	labelPanel = new JPanel(new GridBagLayout());
	labelPanel.setPreferredSize(PropertyManager.LAYER_CONTROL_LAYER_DIMENSION);
	setLayerBorder(STANDARD_BORDER);
	labelPanel.addMouseListener(listener);

	if (layerModel.isAnnotationLayer()) {
	    label = new LayerAnnotationLabel(layerName, SwingConstants.LEFT, this);
	    
	} else {
	    label = new LayerContentLabel(layerName, SwingConstants.LEFT, this);
	}
	
	if (layer > 0) {
	    label.setFont(PropertyManager.LAYER_CONTROL_LAYER_TEXT_FONT);
	} else {
	    label.setFont(PropertyManager.LAYER_CONTROL_BACKGROUND_LAYER_TEXT_FONT);
	}
	label.addMouseListener(listener);

	if (PropertyManager.LAYER_CONTROL_UNSELECTED_COLOR == null) {
	    PropertyManager.LAYER_CONTROL_UNSELECTED_COLOR = label.getBackground();
	}

	if (selected) {
	    labelPanel.setBackground(PropertyManager.LAYER_CONTROL_SELECTED_COLOR);
	    label.setBackground(PropertyManager.LAYER_CONTROL_SELECTED_COLOR);
	    label.setForeground(Color.white);
	}

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(0, 5, 0, 0);
	labelPanel.add(label, c);
	
	// Add drag behavior if layer > 0, but still can be a drop target
	if (layer > 0) {
	    DragSource dragSource = DragSource.getDefaultDragSource();
	    dragSource.createDefaultDragGestureRecognizer(label,
							  DnDConstants.ACTION_COPY_OR_MOVE,
							  listener);
	    
	    dragSource.createDefaultDragGestureRecognizer(labelPanel,
							  DnDConstants.ACTION_COPY_OR_MOVE,
							  listener);
	}
	new DropTarget(label, DnDConstants.ACTION_COPY_OR_MOVE, listener);
	new DropTarget(labelPanel, DnDConstants.ACTION_COPY_OR_MOVE, listener);

	return labelPanel;
    }


    public void handleRepositionLayer(int sourceLayer) {
	int toPosition = LayerControl.getLayerControl().getDNDPosition();
	if (sourceLayer == toPosition) {
	    // Dropped on same layer - do nothing
	    return;
	}
	LayerControl.getLayerControl().repositionLayer(sourceLayer, toPosition);
    }



    class Listener extends MouseAdapter 
	implements ItemListener, 
		   DragGestureListener,
		   DragSourceListener,
		   DropTargetListener {

	private JComponent pressComponent;


	public Listener() {
	    pressComponent = null;
	}


	public void itemStateChanged(ItemEvent e) {
	    Object source = e.getItemSelectable();
	    if (source == checkBox) {
		setLayerVisible(e.getStateChange() == ItemEvent.SELECTED);
	    }
	}


	// Work around a WACOM table driver bug (sporadic, no mouseClick events!)
	public void mousePressed(MouseEvent e) {
	    pressComponent = (JComponent) e.getSource();
	}
	
	public void mouseReleased(MouseEvent e) {
	    JComponent releaseComponent = (JComponent) e.getSource();
	    if ((PropertyManager.APPLICATION_INPUT_DEVICE == PropertyManager.STYLUS_INPUT_DEVICE) &&
		(pressComponent == releaseComponent)) {
		handleMouseClicked(e);
	    }
	    pressComponent = null;
	}

	
	public void mouseClicked(MouseEvent e) {
	    if (PropertyManager.APPLICATION_INPUT_DEVICE == PropertyManager.MOUSE_INPUT_DEVICE) {
		handleMouseClicked(e);
	    }
	}

	
	private void handleMouseClicked(MouseEvent e) {
	    if (e.getClickCount() == 1) {
		if (! selected) {
		    // Not already selected
		    PropertyManager.setProperty(PropertyManager.LAYER_CONTROL_LAYER_SELECTED, new Integer(layer));
		    // Flush the Q so that the LayerLabel picks up the right values on the next repaint
		    PropertyManager.flushQ();
		}
		
	    } else if (e.getClickCount() == 2) {
		LayerPropertyFrame lpf = new LayerPropertyFrame();
	    }	 
	}


	// Drag gesture and source interface
        public void dragEnter(DragSourceDragEvent e) {}
        public void dragExit(DragSourceEvent e) {}
        public void dragOver(DragSourceDragEvent e) {}
        public void dropActionChanged(DragSourceDragEvent e) {}
	
	public void dragGestureRecognized(DragGestureEvent e) {
	    e.startDrag(DragSource.DefaultMoveDrop, new StringSelection(DND_CODE + Integer.toString(layer)), this); 
	    LayerControl.getLayerControl().setDNDPosition(layer);
	    LayerControl.getLayerControl().setLastDNDLayer(layer);
	}

        public void dragDropEnd(DragSourceDropEvent e) {
	    LayerControl.getLayerControl().setLayerBorders(STANDARD_BORDER);
	}


	// Drop target interface
	public void dragOver(DropTargetDragEvent e) {}
	public void dropActionChanged(DropTargetDragEvent e) {}
	public void dragExit(DropTargetEvent e) {}

	public void dragEnter(DropTargetDragEvent e) {
	    Component component = e.getDropTargetContext().getComponent();
	    int max = LayerControl.getLayerControl().getNumLayers();
	    int last = LayerControl.getLayerControl().getLastDNDLayer();

	    if ((layer + 1) >= max) {
		// Set all borders back to original state (no repaint, so this is quick)
		LayerControl.getLayerControl().setLayerBorders(STANDARD_BORDER);
		
		// The last layer requires special handling
		handleDragEnterForLastLayer(component, e.getLocation().y, last, max);
	    
	    } else {
		// Discard events while within a specific layer,
		// due to the label widget also being a drop target
		if (layer == last) {
		    return;
		}

		// Set all borders back to original state (no repaint, so this is quick)
		LayerControl.getLayerControl().setLayerBorders(STANDARD_BORDER);
		
		if ((layer - last) < 0) {
		    // Moving towards the bottom of the panel
		    // Set current border
		    LayerControl.getLayerControl().setLayerBorder(layer + 1, DND_BOTTOM_ONLY_BORDER);
		    LayerControl.getLayerControl().setDNDPosition(layer + 1);
			
		} else if ((layer - last) > 0) {
		    // Moving towards the top of the panel
		    // Set current border
		    labelPanel.setBorder(DND_BOTTOM_ONLY_BORDER);
		    LayerControl.getLayerControl().setDNDPosition(layer);
		} 

		LayerControl.getLayerControl().setLastDNDLayer(layer);
	    }
	}


	public void handleDragEnterForLastLayer(Component source, int y, int last, int max) { 
	    if (source == labelPanel) {
		if ((layer - 1) == last) {
		    // Entered the label panel from previous layer, moving toward the top
		    labelPanel.setBorder(DND_BOTTOM_ONLY_BORDER);
		    LayerControl.getLayerControl().setLastDNDLayer(layer);
		    LayerControl.getLayerControl().setDNDPosition(layer);
		    		    
		} else {
		    if (y < (labelPanel.getHeight() / 2)) {
			// Entered the label panel from the label, moving upward
			labelPanel.setBorder(DND_TOP_BORDER);
			LayerControl.getLayerControl().setDNDPosition(max);
		    
		    } else {
			// Entered the label panel from the label, moving towards the bottom
			labelPanel.setBorder(DND_BOTTOM_ONLY_BORDER);
			LayerControl.getLayerControl().setDNDPosition(layer);
		    }
		}
	    } else {
		labelPanel.setBorder(DND_BOTTOM_ONLY_BORDER);
		LayerControl.getLayerControl().setDNDPosition(layer);
	    }
	}


	public void drop(DropTargetDropEvent e) {
	    try {
		// Reset the borders immediately
		LayerControl.getLayerControl().setLayerBorders(STANDARD_BORDER);

		DataFlavor stringFlavor = DataFlavor.stringFlavor;
		Transferable tr = e.getTransferable();
		
		if (e.isDataFlavorSupported(stringFlavor)) {
		    String dnd_code_string = (String)tr.getTransferData(stringFlavor);
		    // Split the code
		    int index = dnd_code_string.lastIndexOf(' ');
		    String sCommand = null;
		    if (index >= 0) {
			sCommand = dnd_code_string.substring(0, index);
			if (sCommand != null) {   
			    String sCode = dnd_code_string.substring(index + 1);
			    int sourceLayer = Integer.parseInt(sCode);		    
			    if (sCommand.equals("LAYER_CONTROL_DRAG")) {
				e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				e.dropComplete(true);
				handleRepositionLayer(sourceLayer);
			    }
			}
		    }
		}

	    } catch(Exception xcptn) {
		System.out.println(xcptn.toString());
	    }
	}
    }
}
