/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: LayerNewPropertyPanel.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.layer;

// Necessary imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import edu.uofminnesota.DEMAIS.meal.layer.*;
import edu.uofminnesota.DEMAIS.property.*;



public class LayerNewPropertyPanel extends JPanel {

    private static final int NAME_FIELD_WIDTH = 18;


    // Containing frame
    private LayerNewPropertyFrame layerFrame;

    // Local widgets
    private JTextField titleField;
    private JButton closeButton;
    private JCheckBox annotationCheckBox;

    // Event listener
    private Listener listener;

    // Current type selection
    private int type;



    // Constructors
    private LayerNewPropertyPanel() {
	this(null);
    }

    public LayerNewPropertyPanel(LayerNewPropertyFrame frame) {
	super(new GridBagLayout());
	layerFrame = frame;
	listener = new Listener();
	type = PropertyManager.INTERACTION_LAYER;

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(10, 10, 0, 10);
	add(createTitlePanel(), c);

	++c.gridy;
	c.insets = new Insets(10, 10, 0, 10);
	add(createTypePanel(), c);
	
	++c.gridy;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.NONE;
	c.insets = new Insets(10, 0, 10, 10);
	c.anchor = GridBagConstraints.EAST;
	add(createClosePanel(), c);	
    }


    public JPanel createTitlePanel() {
	JPanel titlePanel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	JLabel titleLabel = new JLabel("New Layer ");
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	titlePanel.add(titleLabel, c);

	titleField = new JTextField("Layer " + Integer.toString(LayerControl.getLayerControl().getNumLayers()),
				    NAME_FIELD_WIDTH);
	titleField.setFont(PropertyManager.APPLICATION_TEXT_FIELD_FONT);
	titleField.setActionCommand("close");
	titleField.addActionListener(listener);

	++c.gridx;
	c.weightx = 1.0;
	c.fill = GridBagConstraints.NONE;
	c.insets = new Insets(0, 3, 0, 0);
	titlePanel.add(titleField, c);

	return titlePanel;
    }



    public JPanel createTypePanel() {
	JPanel typePanel = new JPanel(new GridBagLayout());
	typePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Type", TitledBorder.LEADING, TitledBorder.TOP));
	
	GridBagConstraints c = new GridBagConstraints();
	annotationCheckBox = new JCheckBox("Annotation", false);
	annotationCheckBox.addItemListener(listener);
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	typePanel.add(annotationCheckBox, c);

	return typePanel;
    }

    
    public JPanel createClosePanel() {
	JPanel closePanel = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	closeButton = new JButton("Close");
	closeButton.setActionCommand("close");
	closeButton.addActionListener(listener);
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 1.0;
	c.fill = GridBagConstraints.NONE;
	closePanel.add(closeButton, c);

	return closePanel;
    }


    public void handleProperties() {
	// Create the new layer
	LayerControl.getLayerControl().doNewLayer(titleField.getText(), type);
    }


    class Listener implements ActionListener, ItemListener {
	
	public Listener() {
	}

	public void itemStateChanged(ItemEvent e) {
            Object source = e.getItemSelectable();
	    if (source == annotationCheckBox) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
		    type = PropertyManager.ANNOTATION_LAYER;
		} else {
		    type = PropertyManager.INTERACTION_LAYER;
		}
	    }
	}
          

	public void actionPerformed(ActionEvent evt) {
	    String cmd = evt.getActionCommand();
	    handleAction(cmd);
	}


	private void handleAction(String cmd) {
	    if (cmd.equals("close")) {
		layerFrame.closeWindow();
	    }
	}
    }

}
