/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: LayerPopupMenu.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.layer;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;



public class LayerPopupMenu extends JPopupMenu {

    // Local instance of the popup menu
    private static LayerPopupMenu popupMenu;
    // Listener for the menu selection command
    private Listener listener;

    // Local widgets for this popup
    private JMenuItem layerNew, layerCopy, layerDelete, layerFlatten, layerProperty;


    // Static initializer
    static {
	popupMenu = null;
    }

    public static LayerPopupMenu getPopupMenu() {
	if (popupMenu == null) {
	    popupMenu = new LayerPopupMenu();
	}
	return popupMenu;
    }


    // Do no allow others to instantiate this object
    private LayerPopupMenu() {	
	listener = new Listener();
	createPopupMenu();
    }

	
    private void createPopupMenu() {
	// Set the cursor
	setCursor(PropertyManager.APPLICATION_CURSOR);
	
	int count = ((Integer) PropertyManager.getProperty(PropertyManager.LAYER_CONTROL_LAYER_COUNT)).intValue(); 
	int selected = ((Integer) PropertyManager.getProperty(PropertyManager.LAYER_CONTROL_LAYER_SELECTED)).intValue(); 
	
	layerNew = new JMenuItem("New Layer");
	layerNew.addActionListener(listener);
	layerNew.setEnabled(count > 0);
	add(layerNew);
	
	layerCopy = new JMenuItem("Copy Layer");
	layerCopy.addActionListener(listener);
	layerCopy.setEnabled(count > 0);
	add(layerCopy);
	
	layerDelete = new JMenuItem("Delete Layer");
	layerDelete.addActionListener(listener);
	layerDelete.setEnabled((count > 1) && (selected > 0));
	add(layerDelete);
	
	layerFlatten = new JMenuItem("Flatten Layers");
	layerFlatten.addActionListener(listener);
	layerFlatten.setEnabled(count > 1);
	add(layerFlatten);

	addSeparator();

	layerProperty = new JMenuItem("Properties");
	layerProperty.addActionListener(listener);
	layerProperty.setEnabled(count > 0);
	add(layerProperty);
	    
    }


    class Listener implements ActionListener, PropertyListenerInterface {

	public Listener() {
	    PropertyManager.addPropertyListener(PropertyManager.LAYER_CONTROL_LAYER_COUNT, this);
	    PropertyManager.addPropertyListener(PropertyManager.LAYER_CONTROL_LAYER_SELECTED, this);

	}
	
	public void actionPerformed(ActionEvent e) {
	    JMenuItem selection = (JMenuItem) e.getSource();
	    if (selection == layerNew) {
		LayerMenuCommands.getLayerMenuCommands().handleLayerNew();

	    } else if (selection == layerCopy) {
		LayerMenuCommands.getLayerMenuCommands().handleLayerCopy();

	    } else if (selection == layerDelete) {
		LayerMenuCommands.getLayerMenuCommands().handleLayerDelete();

	    } else if (selection == layerFlatten) {
		LayerMenuCommands.getLayerMenuCommands().handleLayerFlatten();

	    } else if (selection == layerProperty) {
		LayerMenuCommands.getLayerMenuCommands().handleLayerProperties();

	    } else {
		System.out.println("Error: unknown popup menu command");
	    }
	}

	// PropertyEventListener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.LAYER_CONTROL_LAYER_COUNT)) {
		int count = ((Integer) newValue).intValue();
		int selected = ((Integer) PropertyManager.getProperty(PropertyManager.LAYER_CONTROL_LAYER_SELECTED)).intValue(); 
		
		layerNew.setEnabled(count > 0);
		layerCopy.setEnabled(count > 0);
		layerDelete.setEnabled((count > 1) && (selected > 0));
		layerFlatten.setEnabled(count > 1);
		layerProperty.setEnabled(count > 0);
		
	    } else if (property.equals(PropertyManager.LAYER_CONTROL_LAYER_SELECTED)) {
		int selected = ((Integer) newValue).intValue();
		int count = ((Integer) PropertyManager.getProperty(PropertyManager.LAYER_CONTROL_LAYER_COUNT)).intValue(); 
		layerDelete.setEnabled((count > 1) && (selected > 0));
	    }
	}
    }
}




