/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File MultiViewCompartment.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.multiview;
    

// Necessary imports
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;


	

public abstract class MultiViewCompartment 
    implements ActionListener {

    // Load the delete image icon
    private ImageIcon DELETE_ICON = new ImageIcon(PropertyManager.APPLICATION_IMAGE_DIRECTORY + PropertyManager.FILE_SEPARATOR + "X.gif");
 

    // Location and size of the compartment
    protected Rectangle rectangle;
        
    // The containing panel
    protected MultiViewPanel mvp;

    // 4 points around the rectangle for nav links to be attached
    protected Point attachPoints[];
    
    // The delete button in the upper right corner of the compartment
    private JButton deleteB;



    private MultiViewCompartment() {
	this((Rectangle)null, null);
    }
    
    public MultiViewCompartment(Rectangle r, MultiViewPanel mp) {
	rectangle = r;
	mvp = mp;
	attachPoints = new Point[4];
	attachPoints[0] = new Point(rectangle.x + 25, rectangle.y + (rectangle.height / 2));
	attachPoints[1] = new Point(rectangle.x + (rectangle.width / 2), rectangle.y + 25);
	attachPoints[2] = new Point(rectangle.x + rectangle.width - 25, rectangle.y + (rectangle.height / 2));
	attachPoints[3] = new Point(rectangle.x + (rectangle.width / 2), rectangle.y + rectangle.height - 25);

	deleteB = new JButton(DELETE_ICON);
	deleteB.addActionListener(this);
	deleteB.setBounds(rectangle.x + rectangle.width - 20, rectangle.y + 1, 16, 16);
	mvp.add(deleteB);
    }
 

    public MultiViewCompartment(DataInputStream stream, MultiViewPanel mp) throws IOException {
	int x = stream.readInt();
	int y = stream.readInt();
	int w = stream.readInt();
	int h = stream.readInt();
	rectangle = new Rectangle(x, y, w, h);
	mvp = mp;
	attachPoints = new Point[4];
	attachPoints[0] = new Point(rectangle.x + 25, rectangle.y + (rectangle.height / 2));
	attachPoints[1] = new Point(rectangle.x + (rectangle.width / 2), rectangle.y + 25);
	attachPoints[2] = new Point(rectangle.x + rectangle.width - 25, rectangle.y + (rectangle.height / 2));
	attachPoints[3] = new Point(rectangle.x + (rectangle.width / 2), rectangle.y + rectangle.height - 25);

	deleteB = new JButton(DELETE_ICON);
	deleteB.addActionListener(this);
	deleteB.setBounds(rectangle.x + rectangle.width - 20, rectangle.y + 1, 16, 16);
	mvp.add(deleteB);
    }

    public void completeIO() {
    }


    public Rectangle getBounds() {
	return rectangle;
    }

    protected Point[] getAttachPoints() {
	return attachPoints;
    }
    

    public boolean contains(int x, int y) {
	return (rectangle.contains(x, y));
    }

    public boolean contains(Point p) {
	return (contains(p.x, p.y));
    }
    

    public GraphicalObjectCollection getCollection() {
	return null;
    }


    protected void deleteView() {
	// Delete the button
	if (deleteB != null) {
	    mvp.remove(deleteB);
	}
	deleteB = null;

	// Delete the collection
	deleteCollection();
    }

    protected void deleteCollection() {
    }

	
    protected void refreshView() {
    }


    public void onInstall() {
    }


    public void onUnInstall() {
    }


    // Abstract methods
    public abstract DataModel getModel();
    public abstract void render(Graphics2D g);


    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(rectangle.x);
	stream.writeInt(rectangle.y);
	stream.writeInt(rectangle.width);
	stream.writeInt(rectangle.height);
    }


    // User selected the delete view button
    public void actionPerformed(ActionEvent evt) {
	mvp.deleteView(getModel(), true);
    }
}
