/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: MultiViewContentManagerCellView.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.multiview;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;




public class MultiViewContentManagerCellView 
    extends ContentManagerCellView {


    private MultiViewModel model;
    private MultiViewPanel mvp;
    private Listener listener;



    private MultiViewContentManagerCellView() {
	//empty
    }

    public MultiViewContentManagerCellView(MultiViewModel m) {
	super(m);
	model = m;
	listener = new Listener();
	
	mvp = (MultiViewPanel) model.getMultiViewPanel().clone(model);
	mvp.setEditable(false);
	mvp.setSize(new Dimension(ContentManagerCell.WIDTH, ContentManagerCell.HEIGHT));
	mvp.scaleTo(ContentManagerCell.WIDTH, ContentManagerCell.HEIGHT, 23);
	mvp.addMouseListener(listener);

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;

	// Add a multiview panel to the contentPanel
	contentPanel.add(mvp, c);
    }


    public void paintComponent(Graphics g) {
	super.paintComponent(g);

	g.setColor(Color.black);
	
	// Get the viewable contents of the current model's storyboard
	GraphicalObjectCollection goc = (GraphicalObjectCollection) model.getMultiViewPanel().getGraphicalObjectCollection();
	GraphicalObjectCollection goc_clone = (GraphicalObjectCollection) goc.clone(mvp, false);
	
	// Put those contents into the current multi view
	mvp.setGraphicalObjectCollection(goc_clone);

	// Scale the objects
	mvp.scaleTo(contentPanel.getWidth(), contentPanel.getHeight());

	int width = contentPanel.getWidth() / 3;
	int height = contentPanel.getHeight() / 2;
	int x = 0;
	int y = 0;

	// Obtain the references for the compartment models placed in the multiview
	MultiViewCompartment compartments[] = model.getMultiViewPanel().getCompartments();

	for (int i=0; i<compartments.length; ++i) {
	    if (i == 0) {
		x = 0;
		y = 0;
	    } else if (i == 1) {
		x = width;
		y = 0;
	    } else if (i == 2) {
		x = 2 * width;
		y = 0;
	    } else if (i == 3) {
		x = 0;
		y = height;
	    } else if (i == 4) {
		x = width;
		y = height;
	    } else if (i == 5) {
		x = 2 * width;
		y = height;
	    }

	    MultiViewCompartment c = compartments[i];
	    if (c != null) {
		if (c instanceof MultiViewStoryboardCompartment) {
		    MultiViewStoryboardCompartment mvsc = (MultiViewStoryboardCompartment) c;
		    Storyboard sReference = (Storyboard) mvsc.getModel().getStoryboard();
		    
		    GraphicalObjectCollection sGoc = (GraphicalObjectCollection) sReference.getGraphicalObjectCollection().clone(sReference, false);			
		    sGoc.scaleTo(width - 3, height - 3);
		    sGoc.translate(x, y, 10000, 10000);
		    goc_clone.addAll(0, sGoc);
		    
		} else if (c instanceof MultiViewNarrationCompartment) {
		    MultiViewNarrationCompartment mvnc = (MultiViewNarrationCompartment) c;

		    // Just draw lines representing the text
		    goc_clone.add(0, new InkLine(x + 2, y + 2, x + width - 4, y + 2, mvp));
		    goc_clone.add(0, new InkLine(x + 2, y + 6, x + width - 4, y + 6, mvp));
		    goc_clone.add(0, new InkLine(x + 2, y + 10, x + width - 4, y + 10, mvp));
		    goc_clone.add(0, new InkLine(x + 2, y + 14, x + width - 4, y + 14, mvp));
		    goc_clone.add(0, new InkLine(x + 2, y + 18, x + width - 4, y + 18, mvp));

		}		    
	    }
	}
    }


    public void handleOpenCell() {	
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, model);

	JFrame frame = PropertyManager.APPLICATION_FRAME_WIDGET;
	if (frame.getState() == Frame.ICONIFIED) {
	    frame.setState(Frame.NORMAL);
	    frame.show();
	}
    }


    public boolean equals(Object obj) {
	return (this == obj);
    }


    class Listener extends MouseAdapter {

	public Listener() {
	}

	public void mouseClicked(MouseEvent e) {
	    // Forward to the superclass
	    handleMouseClicked(e);
	}
    }
}
