/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File MultiViewDropTargetListener.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.multiview;
    

// Necessary imports
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.multiview.*;
import edu.uofminnesota.DEMAIS.usrinterface.narration.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior.*;




public class MultiViewDropTargetListener extends MouseAdapter 
    implements DropTargetListener {
    

    // The mvp affected by the events
    private MultiViewPanel mvp;
  


    private MultiViewDropTargetListener() {
	this(null);
    }

    public MultiViewDropTargetListener(MultiViewPanel mvp) {
	this.mvp = mvp;
	new DropTarget(mvp, DnDConstants.ACTION_COPY_OR_MOVE, this);
    }


    public void drop(DropTargetDropEvent e) {
	try {
	    DataFlavor stringFlavor = DataFlavor.stringFlavor;
	    Transferable tr = e.getTransferable();
	    
	    if (e.isDataFlavorSupported(stringFlavor)) {
		String dnd_code_string = (String)tr.getTransferData(stringFlavor);
		// Split the code
		int index = dnd_code_string.lastIndexOf(' ');
		String sCommand = null;
		if (index >= 0) {
		    sCommand = dnd_code_string.substring(0, index);
		    if (sCommand != null) {   
			String sCode = dnd_code_string.substring(index + 1);
			int code = Integer.parseInt(sCode);
			if (sCommand.equals("STORYBOARD_DRAG")) {
			    StoryboardModel sModel = StoryboardModel.getStoryboardModelById(new Integer(code));
			    if (sModel != null) {
				if (! mvp.isModelPresent(sModel)) {
				    e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				    e.dropComplete(true);
				    Point p = e.getLocation();
				    handleStoryboardDrop(sModel, p.x, p.y);
				    return;
				}
			    }

			} else if (sCommand.equals("NARRATION_DRAG")) {
			    // Lets see if its a Narration model
			    NarrationModel nModel = NarrationModel.getNarrationModelById(new Integer(code));
			    if (nModel != null) {
				// Is there still space available and is this model not already present
				if (! mvp.isModelPresent(nModel)) {
				    e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				    e.dropComplete(true);
				    Point p = e.getLocation();
				    handleNarrationDrop(nModel, p.x, p.y);
				    return;
				} 
			    }  
			} 
		    }
		}
	    }
	} catch(Exception xcptn) {
	    DEMAISError.showErrorMessage(xcptn.toString(), "MultiView DropTargetListener", false);
	    DEMAISError.logException(xcptn);
	}

	Toolkit.getDefaultToolkit().beep();
	e.rejectDrop();
    }

    
    public void dragEnter(DropTargetDragEvent e) {
    }
    public void dragExit(DropTargetEvent e) {
    }
    public void dragOver(DropTargetDragEvent e) {
    }
    public void dropActionChanged(DropTargetDragEvent e) {
    }


    public void handleStoryboardDrop(StoryboardModel model, int x, int y) {
	int index = mvp.mapXYToCompartment(x, y);
	if (! mvp.isCompartmentAvailable(index)) {
	    mvp.deleteView(index, true);
	}
	mvp.handleStoryboardModelDrop(model, index);
    }

    public void handleNarrationDrop(NarrationModel model, int x, int y) {
	int index = mvp.mapXYToCompartment(x, y);
	if (! mvp.isCompartmentAvailable(index)) {
	    mvp.deleteView(index, true);
	}
	mvp.handleNarrationModelDrop(model, index);
    }

}

