/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File MultiViewNarrationCompartment.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.multiview;
    

// Necessary imports
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.narration.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;


	

public class MultiViewNarrationCompartment extends MultiViewCompartment {

    // Class id 
    public static final int CLASSID = 2000;

    // Text font to use for this compartment view
    public static final int NWORDS;


    // Underlying narration model
    private NarrationModel model;

    // The text from the model
    private String text;

    // A collection of rectangles representing each
    // sync marker displayed in the compartment
    private Vector syncMarkerLocations;
    
    // The width and height of the narration sync marker icon
    private int iconWidth;
    private int iconHeight;
    
    // For file I/O only
    private int ioNarration;





    // Static initializer
    static {
	NWORDS = 5;
    }


    private MultiViewNarrationCompartment() {
	this(null, null, null);
    }
    
    
    public MultiViewNarrationCompartment(Rectangle r, MultiViewPanel mp, NarrationModel nModel) {
	super(r, mp);
	model = nModel;
	syncMarkerLocations = new Vector();
	iconWidth = NarrationTextPane.SYNC_ICON.getIconWidth();
	iconHeight = NarrationTextPane.SYNC_ICON.getIconHeight();
    }
    

    public MultiViewNarrationCompartment(DataInputStream stream, MultiViewPanel mp) throws IOException {
	super(stream, mp);
	syncMarkerLocations = new Vector();
	ioNarration = stream.readInt();
	iconWidth = NarrationTextPane.SYNC_ICON.getIconWidth();
	iconHeight = NarrationTextPane.SYNC_ICON.getIconHeight();
    }

    public void completeIO() {
	super.completeIO();
	model = NarrationModel.getNarrationModelById(new Integer(ioNarration));
	if (model != null) {
	    buildSyncMarkerList();
	}
    }



    public DataModel getModel() {
	return model;
    }
    

    public int getSyncMarkerIndexAt(Point p) {
	return (getSyncMarkerIndexAt(p.x, p.y));
    }

    public int getSyncMarkerIndexAt(int x, int y) {
	boolean found = false;
	int index = 0;
	Iterator it = syncMarkerLocations.iterator();
	while ((it.hasNext()) && (!found)) {
	    Rectangle marker = (Rectangle) it.next();
	    if (marker.contains(x, y)) {
		found = true;
	
	    } else {
		++index;
	    }
	}
	if (! found) {
	    index = -1;
	}
	return index;
    }

    
    public void refreshView() {
	buildSyncMarkerList();
	integrityCheck();
    }



    public void render(Graphics2D g) {
	// Render the title first
	g.setColor(Color.black);
	g.setFont(PropertyManager.MULTIVIEW_TITLE_FONT);
	FontMetrics fm = g.getFontMetrics();
	g.drawString(model.getName(), rectangle.x + 4, rectangle.y + rectangle.height - fm.getMaxDescent());
	
	syncMarkerLocations.clear();

	DefaultStyledDocument document = (DefaultStyledDocument) model.getDocument();
	// Get the complete text - including sync icon spaces!
	text = null;
	try {
	    text = document.getText(0, document.getLength());
	} catch (Exception xcptn) {
	    System.out.println(xcptn.toString());
	}

	g.setFont(PropertyManager.MULTIVIEW_NARRATION_TEXT_FONT);
	g.setColor(Color.black);
	fm = g.getFontMetrics();    
	int approxMax = (((rectangle.width - 30) / (fm.charWidth('K') + 1)) * 
			 ((rectangle.height - fm.getHeight()) / (fm.getHeight())));

	if (text.length() < approxMax) {
	    renderCompleteText(g);
	} else {
	    renderPartialText(g);
	}

	integrityCheck();
    }


    private void renderCompleteText(Graphics2D g) {
	FontMetrics fm = g.getFontMetrics();    
	DefaultStyledDocument document = (DefaultStyledDocument) model.getDocument();
	int sX = rectangle.x + 10;
	int sY = rectangle.y + fm.getHeight();
	int maxX = rectangle.x + rectangle.width;

	for (int i=0; i<text.length(); ++i) {
	    char ch = text.charAt(i);
	    String s = "";
	    s += ch;
	    Element e = document.getCharacterElement(i);
	    if (e.getName().equals("icon")) {
		g.drawImage(NarrationTextPane.SYNC_ICON.getImage(), sX, sY - 16, PropertyManager.STORYBOARD_BACKGROUND_COLOR, null);
		syncMarkerLocations.add(new Rectangle(sX, sY - 16, iconWidth + 6, iconHeight + 6));

		sX += iconWidth;

	    } else {
		g.drawString(s, sX, sY);
		sX += fm.charWidth(ch);
	    }

	    if ( (sX + 10) >= (maxX)) {
		sY += fm.getHeight();
		sX = rectangle.x + 10;
	    }
	}
    }


    private void renderPartialText(Graphics2D g) {
	FontMetrics fm = g.getFontMetrics();    
	DefaultStyledDocument document = (DefaultStyledDocument) model.getDocument();
	int sX = rectangle.x + 10;
	int sY = rectangle.y + fm.getHeight();
	int maxX = rectangle.x + rectangle.width;
	int yIncr = fm.getHeight();
	int i = 0;
	String s = "";
	char ch = 'a';

	Collection syncMarkers = model.getSyncMarkers();
	Iterator it = syncMarkers.iterator();
	while (it.hasNext()) {
	    int index = ((Integer) it.next()).intValue();
	    String before = getNWordsBefore(NWORDS, index);
	    String after = getNWordsAfter(NWORDS, index);

	    String beginning = "...";
	    for (i=0; i<beginning.length(); ++i) {
		ch = beginning.charAt(i);
		s = "";
		s += ch;	 
		g.drawString(s, sX, sY);
		sX += fm.charWidth(ch);
		
		if ((sX + 10) >= maxX) {
		    sY += yIncr;
		    sX = rectangle.x + 10;
		}
	    }

	    int start = index - before.length() + 1;
	    for (i=0; i<before.length(); ++i) {
		ch = text.charAt(start + i);
		s = "";
		s += ch;	 
		g.drawString(s, sX, sY);
		sX += fm.charWidth(ch);
		
		// Use 20, to ensure space for the upcoming icon
		if ( (sX + 20) >= maxX) {
		    sY += yIncr;
		    sX = rectangle.x + 10;
		}
	    }
	    
	    g.drawImage(NarrationTextPane.SYNC_ICON.getImage(), sX, sY - 16, PropertyManager.STORYBOARD_BACKGROUND_COLOR, null);
	    syncMarkerLocations.add(new Rectangle(sX, sY - 16, iconWidth + 6, iconHeight + 6));

	    sX += iconWidth;
	    if ( (sX + 10) >= maxX) {
		sY += yIncr;
		sX = rectangle.x + 10;
	    }

	    start = index;
	    for (i=0; i<after.length(); ++i) {
		ch = text.charAt(start + i);
		s = "";
		s += ch;	 
		g.drawString(s, sX, sY);
		sX += fm.charWidth(ch);
		
		if ((sX + 10) >= maxX) {
		    sY += yIncr;
		    sX = rectangle.x + 10;
		}
	    }
	}

	String ending = "...";
	for (i=0; i<ending.length(); ++i) {
	    ch = ending.charAt(i);
	    s = "";
	    s += ch;	 
	    g.drawString(s, sX, sY);
	    sX += fm.charWidth(ch);
	    
	    if ((sX + 10) >= maxX) {
		sY += yIncr;
		sX = rectangle.x + 10;
	    }
	}
    }


    
    private void buildSyncMarkerList() {
	BufferedImage image = new BufferedImage(rectangle.width, rectangle.height, BufferedImage.TYPE_INT_RGB);
	Graphics2D graphics = image.createGraphics();
	if (graphics != null) {
	    // Have a graphics object
	    syncMarkerLocations.clear();

	    FontMetrics fm = graphics.getFontMetrics(PropertyManager.MULTIVIEW_NARRATION_TEXT_FONT);    
	    DefaultStyledDocument document = (DefaultStyledDocument) model.getDocument();
	    int sX = rectangle.x + 10;
	    int sY = rectangle.y + fm.getHeight();
	    int maxX = rectangle.x + rectangle.width;

	    text = null;
	    try {
		text = document.getText(0, document.getLength());
	    } catch (Exception xcptn) {
		System.out.println(xcptn.toString());
		return;
	    }
	    
	    for (int i=0; i<text.length(); ++i) {
		char ch = text.charAt(i);
		Element e = document.getCharacterElement(i);
		if (e.getName().equals("icon")) {
		    syncMarkerLocations.add(new Rectangle(sX, sY - 16, iconWidth + 6, iconHeight + 6));
		    sX += iconWidth;
		    
		} else {
		    sX += fm.charWidth(ch);
		}
		
		if ( (sX + 10) >= (maxX)) {
		    sY += fm.getHeight();
		    sX = rectangle.x + 10;
		}
	    }
	    // Release resources
	    graphics.dispose();
	}
    }




    private void integrityCheck() {
	// Do an integrity check
	Vector toRemove = new Vector();
	Iterator it = mvp.getGraphicalObjectCollection().iterator();
	while (it.hasNext()) {
	    BehavioralInkStroke bStroke = (BehavioralInkStroke) it.next();
	    Point sourcePoint = bStroke.getFirstPoint();
	    if ((contains(sourcePoint)) && 
		(getSyncMarkerIndexAt(sourcePoint) == -1)) {
		toRemove.add(bStroke);
	    }
	}
	// do NOT perform a repaint
	mvp.deleteGraphicalObjects(toRemove, false);
    }


    
    private String getNWordsBefore(int n, int index) {
	String s = "";
	int cIndex = index;
	
	int spaceCount = 0;
	while ((cIndex >= 0) &&
	       (spaceCount <= n) ) {
	    
	    if (Character.isWhitespace(text.charAt(cIndex))) {
		++spaceCount;
	    }
	    s += text.charAt(cIndex);
	    --cIndex;
	}
	return s;
    }


    private String getNWordsAfter(int n, int index) {
	String s = "";
	int cIndex = index;

	int spaceCount = 0;
	while ((cIndex < text.length()) &&
	       (spaceCount <= n) ) {

	    if (Character.isWhitespace(text.charAt(cIndex))) {
		++spaceCount;
	    }
	    s += text.charAt(cIndex);
	    ++cIndex;
	}

	return s;
    }
   
    
    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);

	super.write(stream);
	stream.writeInt(model.getIdentifier().intValue());
    }
    
}
