/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   AudioTimerLabel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.narration;

// Import packages
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;




public class AudioTimerLabel extends JLabel {

    // The timer for updating the label
    private javax.swing.Timer timer;
    
    // Action listener for the timer
    private Listener listener;
    
    // When the timer was started
    private long startTime, cTime, prevTime;

    private Calendar calendar;




    public AudioTimerLabel() {
	this("00:00.00");
    }

    public AudioTimerLabel(String label) {
	super(label);
	setEnabled(false);

	listener = new Listener();
	timer = new javax.swing.Timer(10000, listener);
	timer.stop();
	timer.setCoalesce(false);
	timer.setRepeats(true);
	timer.setInitialDelay(100);
	timer.setDelay(100);

	startTime = 0;
	cTime = 0;
	prevTime = 0;
	calendar = Calendar.getInstance();
    }


    public void start() {
	setEnabled(true);
	startTime = System.currentTimeMillis();
	prevTime = startTime;
	cTime = 0;
	timer.start();
    }


    public void stop() {
	setEnabled(false);
	timer.stop();
    }


    public void pause() {
	timer.stop();
    }


    public void resume() {
	prevTime = System.currentTimeMillis();
	timer.start();
    }


    protected void updateTime() {
	long now = System.currentTimeMillis();
	cTime += (now - prevTime);
	prevTime = now;

	Date cDate = new Date(cTime);

	calendar.setTime(cDate);
	int minutes = calendar.get(Calendar.MINUTE);
	int seconds = calendar.get(Calendar.SECOND);
	int hundredths = ((int)Math.round(calendar.get(Calendar.MILLISECOND) / 100)) * 10;

	String minuteS = null;
	if (minutes < 10) {
	    minuteS = "0" + Integer.toString(minutes);
	} else {
	    minuteS = Integer.toString(minutes);
	}

	String secondS = null;
	if (seconds < 10) {
	    secondS = "0" + Integer.toString(seconds);
	} else {
	    secondS = Integer.toString(seconds);
	}

	String hundredthsS = null;
	if (hundredths < 10) {
	    hundredthsS = "0" + Integer.toString(hundredths);
	} else {
	    hundredthsS = Integer.toString(hundredths);
	}

	String timeString = minuteS + ":" + secondS + "." + hundredthsS;	
	setText(timeString);
    }



    class Listener implements ActionListener {
	public Listener() {
	}

	public void actionPerformed(ActionEvent evt) {
	    updateTime();
	}

    }
}
