/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: NarrationContentManagerCellView.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.narration;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;



public class NarrationContentManagerCellView 
    extends ContentManagerCellView {    

    // Font for displaying the naration text
    public static final String DND_CODE = "NARRATION_DRAG ";

    private NarrationModel nModel;
    private Listener listener;
    private JLabel label;




    private NarrationContentManagerCellView() {
	this(null);
    }

    public NarrationContentManagerCellView(NarrationModel m) {
	super(m);
	nModel = m;
	listener = new Listener();

	label = new JLabel("default", JLabel.LEFT);
	label.setVerticalAlignment(JLabel.TOP);
	label.setBackground(Color.white);
	label.addMouseListener(listener);
	label.setFont(PropertyManager.MULTIVIEW_NARRATION_TEXT_FONT);

	label.setText(createLabelString());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;

	// Add the label to the contentPanel
	contentPanel.add(label, c);

	// Add Drag and Drop behavior
	DragSource dragSource = DragSource.getDefaultDragSource();
	dragSource.createDefaultDragGestureRecognizer(label,
						      DnDConstants.ACTION_COPY_OR_MOVE,
						      listener);

	dragSource.createDefaultDragGestureRecognizer(contentPanel,
						      DnDConstants.ACTION_COPY_OR_MOVE,
						      listener);
    }
    

    private String createLabelString() {
	String s1 = null;
	String s2 = null;
	String s3 = null;
	int offset = 0;
	int length = 10;

	String narrationString = nModel.getNarrationText(false);
	if (narrationString.length() < length) {
	    // Retrieve available content
	    s1 = narrationString.substring(0);
	    
	} else {
	    s1 = narrationString.substring(0, length);
	    offset += 10;
	    if (narrationString.length() < (offset + length)) {
		s2 = narrationString.substring(offset);

	    } else {
		s2 = narrationString.substring(offset, (offset + length));
		offset += 10;
		if (narrationString.length() < (offset + length)) {
		    // Get the remainder of the text
		    s3 = narrationString.substring(offset);
		} else {
		    s3 = narrationString.substring(offset, (offset + length));
		}
	    }
	}

	String labelString = "<HTML>";

	if (s1 != null) {
	    labelString += s1;
	    //labelString += "<br>";
	    if (s2 != null) {
		labelString += s2;
		//labelString += "<br>";
		if (s3 != null) {
		    labelString += s3;
		}
	    }
	}

	labelString += "</HTML>";
	return labelString;
    }
    

    public void paintComponent(Graphics g) {
	super.paintComponent(g);

	// Set the text of the label
	label.setText(createLabelString());
    }
    

    public void handleOpenCell() {
	NarrationEditView view = nModel.getNarrationEditView();
	view.setState(Frame.NORMAL);
	view.show();
    }


    public boolean equals(Object obj) {
	return (this == obj);
    }
    

    class Listener extends MouseAdapter implements DragGestureListener, DragSourceListener {

	public Listener() {
	}

	public void mouseClicked(MouseEvent e) {
	    // Forward to the superclass
	    handleMouseClicked(e);
	}

	
	public void dragGestureRecognized(DragGestureEvent e) {
	    int mode = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
	    if (mode == PropertyManager.APPLICATION_EDIT) {
		e.startDrag(DragSource.DefaultCopyDrop, new StringSelection(DND_CODE + nModel.getIdentifier().toString()), this);
	    } else {
		Toolkit.getDefaultToolkit().beep();
	    }
        }
        public void dragDropEnd(DragSourceDropEvent e) {}
        public void dragEnter(DragSourceDragEvent e) {}
        public void dragExit(DragSourceEvent e) {}
        public void dragOver(DragSourceDragEvent e) {}
        public void dropActionChanged(DragSourceDragEvent e) {}

    }

}
