/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: NarrationEditPanel.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.narration;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.text.*;
import javax.swing.text.html.*;



public class NarrationEditPanel extends JPanel {
    // The frame containing this panel
    private NarrationEditView view;

    // The shortcut buttons
    private JButton syncB, closeB;

    // The text pane to perform the play/insert actions on
    private NarrationTextPane textPane;



    public NarrationEditPanel(NarrationEditView v, NarrationTextPane tPane) {
	super(new GridBagLayout());

	view = v;
	textPane = tPane;

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	add(createTextPanel(), c);


	c.gridy = 1;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.weightx = 1.0;
	c.weighty = 0.0;
	add(createControlPanel(), c);

    }


    public void setEditable(boolean b) {
	textPane.setEditable(b);
	syncB.setEnabled(b);
	closeB.setEnabled(b);
    }


    private JPanel createTextPanel() {

	// Create a panel for the text area - allows a border to enhace the look
	JPanel textPanel = new JPanel(new GridBagLayout());
	textPanel.setBorder(BorderFactory.createLoweredBevelBorder());

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weighty = 1.0;
	c.weightx = 1.0;
	textPanel.add(textPane, c);

	return textPanel;
    }


    private JPanel createControlPanel() {
	JPanel controlPanel = new JPanel(new GridBagLayout());
	Listener listener = new Listener();

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.NONE;
	c.weightx = 1.0;
	c.anchor = GridBagConstraints.WEST;

	// Create the play panel
	NarrationEditControlPanel ecp = new NarrationEditControlPanel(view);
	controlPanel.add(ecp, c);

	// Create the sync point button
	syncB = new JButton("Insert sync marker");
	syncB.addActionListener(listener);
	c.gridx = 1;
	c.insets = new Insets(3, 3, 3, 3);
	c.anchor = GridBagConstraints.CENTER;
	controlPanel.add(syncB, c);

	// Create a dismiss button at the bottom right
	closeB = new JButton("Close");
	closeB.addActionListener(listener);
	c.gridx = 2;
	c.anchor = GridBagConstraints.EAST;
	controlPanel.add(closeB, c);

	return controlPanel;
    }



    class Listener implements ActionListener {

	public Listener() {
	}

	public void actionPerformed(ActionEvent e) {
	    JButton selection = (JButton) e.getSource();
	    if (selection == syncB) {
		// Insert a sync point at the current cursor position		
		textPane.insertIcon(NarrationTextPane.SYNC_ICON);
		
	    } else if (selection == closeB) {
		// Close the window
		view.closeWindow();
	    }
	}
    }

}


