/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: NarrationFileMenu.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.narration;

// Necessary imports
import java.lang.*;
import java.awt.event.*;
import java.awt.print.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;




public class NarrationFileMenu extends JMenu {

    // The Frame for this window
    private NarrationEditView view;

    // The listener for narration file menu items
    private Listener listener;

    // The menu items
    private JMenuItem filePrint, fileClose;
    
  

    public NarrationFileMenu(NarrationEditView v) {
	super("File");

	view = v;

	listener = new Listener();

	filePrint = new JMenuItem("Print");
	filePrint.addActionListener(listener);
	add(filePrint);

	fileClose = new JMenuItem("Close");
	fileClose.addActionListener(listener);
	add(fileClose);

    }

    
    // An inner class to handle all menu item events
    class Listener implements ActionListener {

	// Constructor
	public Listener() {	    
	}


	// Action callback
	public void actionPerformed(ActionEvent e) {
	    Object selection = e.getSource();

	    if (selection == filePrint) {
		handleFilePrint();

	    } else if (selection == fileClose) {
		handleFileClose();

	    } else {
		System.out.println("ERROR: Unknown file menu command");
	    }
	}


	private void handleFilePrint() {
	    NarrationModel model = view.getModel();
	    PrinterJob printerJob = PrinterJob.getPrinterJob();
	    if (printerJob.printDialog()) {
		try {

		    Book book = new Book();
		    PageFormat pageFormat = new PageFormat();
		    pageFormat.setOrientation(PageFormat.PORTRAIT);
		    book.append(model, pageFormat);

		    printerJob.setPageable(book);
		    printerJob.print();

		} catch (Exception xcptn) {
		    DEMAISError.showErrorMessage(xcptn.toString(), "Print Error", false);
		    DEMAISError.logException(xcptn);
		}
	    }
	}

	// Close the main view window
	private void handleFileClose() {
	    view.closeWindow();
	}
    }
}



