/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: NarrationTextPane.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.narration;

// Necessary imports
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;



public class NarrationTextPane extends JScrollPane {
    // The little red circle representing a sync point
    public static ImageIcon SYNC_ICON;


    // Handle to the top level window
    private NarrationEditView view;
    private Listener listener;
    private JTextPane textPane;



    static {
	SYNC_ICON = new ImageIcon(PropertyManager.APPLICATION_IMAGE_DIRECTORY +
				  PropertyManager.FILE_SEPARATOR +
				  "sync-enabled.gif");
    }


    // Constructors
    private NarrationTextPane() {
	this(null);
    }

    public NarrationTextPane(NarrationEditView view) {
	super(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	this.view = view;

	textPane = new JTextPane(view.getModelDocument());
	textPane.setFont(PropertyManager.APPLICATION_TEXT_AREA_FONT);

	listener = new Listener();
	textPane.addCaretListener(listener);

	setViewportView(textPane);

	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
	if (screen.getWidth() < 650) {
	    // 640 x 480 screen
	    setPreferredSize(new Dimension(351, 208));

	} else if (screen.getWidth() < 810) {
	    // 800 x 600 screen
	    setPreferredSize(new Dimension(468, 312));

	} else {
	    // 1024 x 768 or higher screen
	    setPreferredSize(new Dimension(600, 400));
	}
    }



    public void setEditable(boolean b) {
	textPane.setEditable(b);
    }

    public void insertIcon(ImageIcon icon) {
	textPane.insertIcon(icon);
    }

    public Action[] getActions() {
	return textPane.getActions();
    }


    protected int getCaretPosition() {
	return textPane.getCaretPosition();
    }


    class Listener implements CaretListener {

	public Listener() {
	}

	public void caretUpdate(CaretEvent e) {
	    //Get the location in the text
              int dot = e.getDot();
              int mark = e.getMark();
              if (dot == mark) {  
		  // No selection
		  PropertyManager.setProperty(PropertyManager.VS_TEXT_SELECTED, Boolean.FALSE);

	      } else {
		  // Selection
		  PropertyManager.setProperty(PropertyManager.VS_TEXT_SELECTED, Boolean.TRUE);
	      }
	}
    }
}
