/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: StoryboardContentManagerCellView.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.contentmanager.*;




public class StoryboardContentManagerCellView 
    extends ContentManagerCellView {

    public static final String DND_CODE = "STORYBOARD_DRAG ";

    private StoryboardModel model;
    private Storyboard storyboard;
    private Listener listener;



    private StoryboardContentManagerCellView() {
	//empty
    }

    public StoryboardContentManagerCellView(StoryboardModel m) {
	super(m);
	model = m;
	listener = new Listener();
	
	storyboard = (Storyboard) model.getStoryboard().clone(model, false);
	storyboard.setEditable(false);
	storyboard.setSize(new Dimension(ContentManagerCell.WIDTH, ContentManagerCell.HEIGHT));
	storyboard.scaleTo(ContentManagerCell.WIDTH, ContentManagerCell.HEIGHT, 23);
	storyboard.addMouseListener(listener);
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;

	// Add the storyboard view to the contentPanel
	// set up in the superclass
	contentPanel.add(storyboard, c);
	
	// Add Drag and Drop behavior
	DragSource dragSource = DragSource.getDefaultDragSource();
	dragSource.createDefaultDragGestureRecognizer(storyboard,
						      DnDConstants.ACTION_COPY_OR_MOVE,
						      listener);
    }


    public void paintComponent(Graphics g) {
	super.paintComponent(g);

	// Get the viewable contents of the current model's storyboard
	GraphicalObjectCollection goc = (GraphicalObjectCollection) model.getStoryboard().getGraphicalObjectCollection();
	GraphicalObjectCollection goc_clone = (GraphicalObjectCollection) goc.clone(storyboard, false);
      	
	// Put those contents into the content manager storyboard view
	storyboard.setGraphicalObjectCollection(goc_clone);
	
	// Scale the objects
	storyboard.scaleTo(storyboard.getWidth(), storyboard.getHeight());

	// Storyboard will be repainted next by Swing (it is
	// a child of this widget)
    }



    public void handleOpenCell() {	
	// Double clicked on the item - open it in the storyboard view
	PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, model);

	JFrame frame = PropertyManager.APPLICATION_FRAME_WIDGET;
	if (frame.getState() == Frame.ICONIFIED) {
	    frame.setState(Frame.NORMAL);
	    frame.show();
	}
    }



    public boolean equals(Object obj) {
	return (this == obj);
    }


    class Listener extends MouseAdapter implements DragGestureListener, DragSourceListener {

	public Listener() {
	}

	public void mouseClicked(MouseEvent e) {
	    // Forward to the superclass
	    handleMouseClicked(e);
	}


	public void dragGestureRecognized(DragGestureEvent e) {
	    int mode = ((Integer)PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
	    if (mode == PropertyManager.APPLICATION_EDIT) {
		e.startDrag(DragSource.DefaultCopyDrop, new StringSelection(DND_CODE + model.getIdentifier().toString()), this);
	    } else {
		Toolkit.getDefaultToolkit().beep();
	    }
        }
        public void dragDropEnd(DragSourceDropEvent e) {}
        public void dragEnter(DragSourceDragEvent e) {}
        public void dragExit(DragSourceEvent e) {}
        public void dragOver(DragSourceDragEvent e) {}
        public void dropActionChanged(DragSourceDragEvent e) {}

    }
}
