/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File StoryboardKeyListener.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard;
    

// Necessary imports
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;




// Storyboard class
public class StoryboardKeyListener implements KeyListener {
    
    // Speed up the translation for each successive keypress
    private static int INCREMENT = 1;

    // The storyboard affected by key events
    private Storyboard storyboard;
    
    // A key release event
    private boolean keyReleased;

    private int translation;




    public StoryboardKeyListener(Storyboard pStoryboard) {
	storyboard = pStoryboard;
	keyReleased = true;
	translation = 2;
    }

    
    public void keyPressed(KeyEvent evt) {
	int keyCode = evt.getKeyCode();
	Collection c = storyboard.getSelectedGraphicalObjects();
	
	switch (keyCode) {
	case KeyEvent.VK_UP:
	    if (evt.isControlDown()) {
		storyboard.changeFontSize(c, 2);
	    } else {
		storyboard.translateAndUpdate(c, 0, -translation);
	    }
	    break;
	    
	case KeyEvent.VK_DOWN:
	    if (evt.isControlDown()) {
		storyboard.changeFontSize(c, -2);
	    } else {
		storyboard.translateAndUpdate(c, 0, translation);
	    }
	    break;

	case KeyEvent.VK_LEFT:
	    storyboard.translateAndUpdate(c, -translation, 0);
	    break;
	
	case KeyEvent.VK_RIGHT:
	    storyboard.translateAndUpdate(c, translation, 0);
	    break;
	    
	case KeyEvent.VK_DELETE:
	    EditMenuCommands.getEditMenuCommands().handleCut(c);
	    break;

	case KeyEvent.VK_ESCAPE:
	    storyboard.cancelStroke();
	    break;

	default:
	    break;
	}

	if (! keyReleased) {
	    if (translation < 20) {
		translation += INCREMENT;
	    }
	}
	keyReleased = false;
    }
      

    public void keyReleased(KeyEvent evt) {
	keyReleased = true;
	translation = 2;
	DataModel cModel = storyboard.getModel();
	DataModel target = null;

	int keyCode = evt.getKeyCode();
	switch (keyCode) {

	case KeyEvent.VK_PAGE_UP:
	    if (cModel instanceof StoryboardModel) {
		target = StoryboardModel.getPreviousStoryboardModel((StoryboardModel) cModel, false);
	    } else if (cModel instanceof MultiViewModel) {
		target = MultiViewModel.getPreviousMultiViewModel((MultiViewModel) cModel, false);
	    }
	    if (target != null) {
		PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, target);
	    } else {
		Toolkit.getDefaultToolkit().beep();
	    }
	    break;
	    
	case KeyEvent.VK_PAGE_DOWN:

	    if (cModel instanceof StoryboardModel) {
		target = StoryboardModel.getNextStoryboardModel((StoryboardModel) cModel);
	    } else if (cModel instanceof MultiViewModel) {
		target = MultiViewModel.getNextMultiViewModel((MultiViewModel) cModel);
	    }
	    if (target != null) {
		PropertyManager.setProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL, target);
	    } else {
		Toolkit.getDefaultToolkit().beep();
	    }

	    break;
	    
	default:
	    break;
	}
    }

    
    public void keyTyped(KeyEvent evt) {
    }
}
