/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File StoryboardMouseListener.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard;
    

// Necessary imports
import java.lang.*;
import java.awt.event.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.property.*;



// Storyboard class
public class StoryboardOpMouseListener extends MouseAdapter 
    implements MouseMotionListener {
    

    // The storyboard affected by the events
    private Storyboard storyboard;
  


    private StoryboardOpMouseListener() {
	this(null);
    }
    
    public StoryboardOpMouseListener(Storyboard s) {
	storyboard = s;
	storyboard.addMouseListener(this);
	storyboard.addMouseMotionListener(this);
    }


    public void removeListener() {
	storyboard.removeMouseListener(this);
	storyboard.removeMouseMotionListener(this);
    }


    public void mousePressed(MouseEvent evt) {
	storyboard.handleMousePressed(evt);
    }
    
    public void mouseReleased(MouseEvent evt) {
	storyboard.handleMouseReleased(evt);
	// Work around yet another bug with the WACOM display driver
	// A mouseClicked event is spuriously not received!!!! 
	if (PropertyManager.APPLICATION_INPUT_DEVICE == PropertyManager.STYLUS_INPUT_DEVICE) {
	    if (evt.getClickCount() == 1) {
		if ((storyboard.getOpPressedObject() == storyboard.getOpCurrentObject()) &&
		    storyboard.getOpCurrentObject() != null) {
		    storyboard.handleMouseSingleClicked(evt);
		}
	    }
	}
    }

    public void mouseClicked(MouseEvent evt) {
	int clickCount = evt.getClickCount();
	if ((PropertyManager.APPLICATION_INPUT_DEVICE == PropertyManager.MOUSE_INPUT_DEVICE) &&
	    (clickCount == 1)) {
	    storyboard.handleMouseSingleClicked(evt);
	} else if (clickCount == 2) {
	    storyboard.handleMouseDoubleClicked(evt);
	}
    }
    
    public void mouseDragged(MouseEvent evt) {
	storyboard.handleMouseDragged(evt);
    }
    
    public void mouseMoved(MouseEvent evt) {
	storyboard.handleMouseMoved(evt);
    }

}
