/*
Copyright (c) 2000 Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:

      This product includes software developed by the Group for User 
      Interface Research at the University of California at Berkeley.

4. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// Modified by Brian P. Bailey for use with DEMAIS

/*
  GestureSet.java
*/


// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.gesture;


// Import needed packages
import java.lang.*;
import java.io.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;



public class GestureSet {
    private String name;
    private String keywords;
    private Vector categories;


    public GestureSet() {
	name = null;
	keywords = null;
	categories = new Vector();
    }



    public void add(GestureCategory gc) {
	categories.add(gc);
    }

    public void add(GestureGroup gg) {
	categories.add(gg);
    }

    public Vector getCategories() {
	return categories;
    }

    public Vector getEnabledCategories() {
	return getCategories();
    }


    public String getName() {
	return name;
    }

    public void setName(String n) {
	name = n;
    }


    public String getKeywords() {
	return keywords;
    }

    public void setKeywords(String k) {
	keywords = k;
    }

    public int size() {
	return categories.size();
    }

 
    public static GestureSet read(Reader reader) {
	GestureSet result = new GestureSet();

	boolean done = false;
	TokenReader r = new TokenReader(reader);
	while (!done) {
	    String token;
	    try {
		token = r.readToken().intern();
		//System.out.println("GestureSet    \ttoken: " + token);
		if (token == "name") {
		    result.setName(r.readLine());
		}
		else if (token == "keywords") {
		    result.setKeywords(r.readLine());
		}
		else if (token == "author") {
		    // Skip it
		    r.readLine();
		}
		else if (token == "categories") {
		    // for old format, where a set only contained categories
		    int numMembers = Integer.parseInt(r.readToken());
		    for (int i=0; i<numMembers; ++i) {
			GestureCategory gc = GestureCategory.read(r);
			result.add(gc);
		    }
		}
		else if (token == "category") {
		    GestureCategory gc = GestureCategory.read(r);
		    result.add(gc);
		}
		else if (token == "group") {
		    GestureGroup gg = GestureGroup.read(r);
		    result.add(gg);
		}
		else if (token == "endset") {
		    done = true;
		}
	    }
	    catch (EOFException xcptn1) {
		done = true;

	    } 
	    catch (IOException xcptn2) {
		DEMAISError.showErrorMessage(xcptn2.toString(), "GestureSet", false);
	    }
	    catch (NumberFormatException xcptn3) {
		DEMAISError.showErrorMessage(xcptn3.toString(), "GestureSet", false);
	    }
	}
	return result;
    }
 
}

