/*
Copyright (c) 2000 Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:

      This product includes software developed by the Group for User 
      Interface Research at the University of California at Berkeley.

4. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// Modified by Brian P. Bailey for use with DEMAIS

/* 
   File RubineInterpreter.java
*/


// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.gesture;



// Import the necessary packages
import java.lang.*;
import java.io.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;



public class RubineInterpreter {

    // Implement a singleton pattern
    private static RubineInterpreter interpreter;

    // Directory where interpreter data is stored
    private static final String RUBINE_INTERPRETER_DATA_DIRECTORY;
    public static final String DEMAIS_GESTURE_FILE;
    public static final String STANDARD_RECTANGLE_FILE;
    
 
    // Recognizer cache
    private final Hashtable recognizerCache;
    
    
    
    // static initializer
    static {
	interpreter = null;
	
	RUBINE_INTERPRETER_DATA_DIRECTORY = 
	    PropertyManager.APPLICATION_DIRECTORY + 
	    PropertyManager.FILE_SEPARATOR + 
	    "data" + 
	    PropertyManager.FILE_SEPARATOR + 
	    "interpreters";
	
	DEMAIS_GESTURE_FILE = RUBINE_INTERPRETER_DATA_DIRECTORY + 
	    PropertyManager.FILE_SEPARATOR + 
	    "demais.gsa";

	STANDARD_RECTANGLE_FILE = RUBINE_INTERPRETER_DATA_DIRECTORY + 
	    PropertyManager.FILE_SEPARATOR + 
	    "rectangles.gsa";

    }

    public static RubineInterpreter getRubineInterpreter() {
	if (interpreter == null) {
	    interpreter = new RubineInterpreter();
	}
	return interpreter;
    }



    private RubineInterpreter() {
	// Create the recognizer cache
	recognizerCache = new Hashtable(5);
	// Pre-load the primary gesture files
	getRubineRecognizer(DEMAIS_GESTURE_FILE);
	//getRubineRecognizer(STANDARD_RECTANGLE_FILE);
    } 
    
  
    
    // fileName - full path to the gesture data file
    public RubineRecognizer getRubineRecognizer(String fileName) {
	// See if the recognizer is already loaded
	RubineRecognizer rr = (RubineRecognizer) recognizerCache.get(fileName);
	if (rr == null) {
	    // Not loaded, so load it now.
	    try {
		String name = fileName;
		int index = fileName.lastIndexOf(PropertyManager.FILE_SEPARATOR);
		if ((index + 1) < fileName.length()) { 
		    name = fileName.substring(index+1);
		}
		System.out.println("Reading " + name + " gesture file...please wait...");
		
		InputStreamReader rdr = new InputStreamReader(new FileInputStream(fileName));	
		rr = new RubineRecognizer(rdr);
		rdr.close();

		recognizerCache.put(fileName, rr);

		System.out.println("Gestures installed successfully");
		System.out.println("");
	    }
	    catch (Exception xcptn) {
		DEMAISError.showErrorMessage(xcptn.toString(), "RubineInterpreter", false);
		DEMAISError.logException(xcptn);
	    }
	}
	return rr;
    }
    
   
    // fileName - full path to the gesture file
    public Recognizer getRecognizer(String fileName) {
	return (getRubineRecognizer(fileName));
    }

}
