/*
Copyright (c) 2000 Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:

      This product includes software developed by the Group for User 
      Interface Research at the University of California at Berkeley.

4. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// Modified by Brian P. Bailey for use with DEMAIS

/*
  TokenReader.java
*/


// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.gesture;

// Import needed packages
import java.lang.*;
import java.io.*;
import java.util.*;



public class TokenReader {

    private String token;
    private boolean isPushedBack;
    private Reader delegate;

  
    public TokenReader(Reader reader) {
	token = null;
	isPushedBack = false;
	delegate = reader;
    }


    public int read() throws IOException {
	int result;
	
	if (isPushedBack) {
	    char c = token.charAt(0);
	    token = token.substring(1);
	    if (token.length() == 0) {
		isPushedBack = false;
	    }
	    result = Character.getNumericValue(c);
	}
	else {
	    result = delegate.read();
	}
	
	return result;
    }


    public String readLine() throws IOException {
	StringBuffer result = new StringBuffer();
	
	if (isPushedBack) {
	    result.append(token);
	    isPushedBack = false;
	}
	int c;
	while (((c = delegate.read()) != '\n') && (c != -1)) {
	    result.append((char) c);
	}
	return result.toString();
    }


    
    public boolean readBoolean() throws IOException {
	return Boolean.valueOf(readToken()).booleanValue();
    }
    
    
    public int readInt() throws IOException, NumberFormatException {
	return Integer.parseInt(readToken());
    }
    
    public long readLong() throws IOException, NumberFormatException {
	return Long.parseLong(readToken());
    }


    public String readToken() throws IOException { 
	StringBuffer buf = new StringBuffer();
	
	final String whitespace = " \t\n\r";
	int c;
	do {
	    c = delegate.read();
	    if (c == -1) {
		throw new EOFException();
	    }
	} while (whitespace.indexOf(c) != -1);
	
	buf.append((char) c);

	// now read the actual token
	boolean done = false;
	while (!done) {
	    c = delegate.read();
	    if (c == -1) {
		throw new EOFException();
	    }
	    if (whitespace.indexOf(c) != -1) {
		done = true;
	    }
	    else {
		buf.append((char) c);
	    }
	}	
	return buf.toString();
    }
    


    public void close() throws IOException {
	delegate.close();
    }
    
 
}







