/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File: GestureCommand.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.gesture.commands;


// Import necessary packages
import java.awt.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.gesture.*;




public class GestureCommand {

    // The gesture representing the command
    protected Gesture gesture;

    // The storyboard on which the gesture was made
    protected Storyboard storyboard;

    // Reference to the actual gesture command
    private GestureCommand cmd;





    protected GestureCommand() {
    }

    public GestureCommand(Gesture g, Storyboard s) {
	gesture = g;
	storyboard = s;
	cmd = null;

	Classification map = storyboard.getGestureRecognizer().classify(gesture);
	if (map != null) {
	    String key = (String) map.getSmallestKey();
	    double value = ((Double) map.getSmallestValue()).doubleValue();
	    // Distance metric must be reasonably close to a category
	    // Empirically, 100 seems lenient enough while also allowing
	    // the user to "escape" the current gesture command
	    if (value < 150.0) {
		// Create the actual gesture command
		cmd = mapNameToCommand(key);
	    }
	}
    }


    // All gesture commands must override this method
    public void performAction() {
	if (cmd != null) {
	    cmd.performAction();
	}
    }


    // To obtain the gesture names, open the demais.gsa file
    // using Quill
    private GestureCommand mapNameToCommand(String gestureName) {
	if (gestureName.startsWith("Cut")) { 
	    return (new CutGestureCommand(this));
	    
	} else if (gestureName.startsWith("Copy")) {
	    return (new CopyGestureCommand(this));
	    
	} else if (gestureName.startsWith("Paste")) {
	    return (new PasteGestureCommand(this));
	    
	} else if (gestureName.startsWith("Select")) {
	    return (new SelectGestureCommand(this));
	    
	} else if (gestureName.startsWith("Undo")) {
	    return (new UndoGestureCommand(this));

	} else if (gestureName.startsWith("Demote")) {
	    return (new DemoteGestureCommand(this));
	
	} else if (gestureName.startsWith("Promote")) {
	    return (new PromoteGestureCommand(this));
	    
	} else {
	    System.out.println("GestureCommand: Unknown gesture name");
	}

	return null;
    }
}

