/*
Copyright (c) 2000 Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:

      This product includes software developed by the Group for User 
      Interface Research at the University of California at Berkeley.

4. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// Modified by Brian P. Bailey for use with DEMAIS

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.gesture.features;

// Import packages
import java.awt.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.gesture.*;



/**
 * The sum of the square of absolute value of the angle at each point.
 */
public class TotalAbsAngle extends Feature {

    public TotalAbsAngle() {
	this(null);
    }

    public TotalAbsAngle(Gesture g) {
	super(g);
    }
    
    /**
     * Non-incremental computation.  Should set @value to new value and
     * @valueOk to true.
     */
    protected void computeValue() {
	value = 0;
	Polygon points = gesture.getPoints();
	for (int i = 2; i < points.npoints; i++) {
	    double dx = points.xpoints[i] - points.xpoints[i-1];
	    double dy = points.ypoints[i] - points.ypoints[i-1];
	    double dx2 = points.xpoints[i-1] - points.xpoints[i-2];
	    double dy2 = points.ypoints[i-1] - points.ypoints[i-2];
	    double theta = Math.atan2(dx*dy2 - dx2*dy, dx*dx2 + dy*dy2);
	    value += Math.abs(theta);
	}
    }
    
}
