/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File AudioRectangleInkStroke.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs;
    

// Necessary imports
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;




public class AudioRectangleInkStroke extends ContinuousMediaRectangleInkStroke { 

    public static final int CLASSID = 180;
    private static final Border ACTIVE_BORDER;
    private static final Border NOT_ACTIVE_BORDER;
    private static final Border HIGHLIGHT_BORDER;


    // The panel holding the audio controller
    private JPanel audioPanel;

    // Constraints for the audioPanel's controller
    private GridBagConstraints panelConstraints;

    // Parallel audio model for Nsync
    private AudioModel audioModel;



    // Static initializer
    static {
	ACTIVE_BORDER = BorderFactory.createMatteBorder(2, 2, 2, 2, ACTIVE_COLOR);
	NOT_ACTIVE_BORDER = BorderFactory.createMatteBorder(2, 2, 2, 2, NOT_ACTIVE_COLOR);
	HIGHLIGHT_BORDER = BorderFactory.createMatteBorder(2, 2, 2, 2, HIGHLIGHT_COLOR);
    }


    // Do not allow default constructor to be called
    private AudioRectangleInkStroke() {
	this((RectangleInkStroke)null, (String)null);
    }

    public AudioRectangleInkStroke(RectangleInkStroke copy, String filePathName) {
	super(true, copy);
	audioPanel = null;
	panelConstraints = null;
	audioModel = new AudioModel(PropertyManager.LOCAL_SCOPE, getScope(), null, filePathName, this);
	realize(true, true);
    }


    public AudioRectangleInkStroke(AudioRectangleInkStroke copy) {
	this(copy, copy.storyboard);
    }

    public AudioRectangleInkStroke(AudioRectangleInkStroke copy, Storyboard toStoryboard) {
	this(copy, toStoryboard, true);
    }

    public AudioRectangleInkStroke(AudioRectangleInkStroke copy, Storyboard toStoryboard, boolean copyAModel) {
	super(copy, toStoryboard, copyAModel);
	audioModel = null;
	audioPanel = null;
	panelConstraints = null;
	if (copy.audioModel != null) {
	    if (copyAModel) {
		audioModel = (AudioModel) copy.audioModel.clone(this);
	    } else {
		audioModel = copy.audioModel;
	    }
	}
    }


    public AudioRectangleInkStroke(DataInputStream stream, Storyboard pStoryboard) throws IOException {
	super(stream, pStoryboard);

	audioPanel = null;
	panelConstraints = null;

	// Read the model information
	audioModel = null;
	boolean hasModel = stream.readBoolean();
	if (hasModel) {
	    audioModel = new AudioModel(stream, this, pStoryboard.getModel());
	    realize(true, true);
	}
    }



    // Public methods
    public void setLayer(int layer) {
	super.setLayer(layer);
	if (audioPanel != null) {
	    storyboard.setLayer(audioPanel, layer, 0);
	    storyboard.revalidate();
	}
    }


    public double getDuration() {
	return audioModel.getDuration();
    }


    public void setMediaTime(double value) {
	audioModel.setMediaTime(value);
    }


    public double getMediaTime() {
	return audioModel.getMediaTime();
    }


    public int getState() {
	return audioModel.getState();
    }

    public void setState(int state, double mediaTime) {
	audioModel.setState(state, mediaTime);
    }


    public void setMouseIsOver(boolean b) {
	super.setMouseIsOver(b);
	if (audioModel != null) {
	    audioModel.setIsOver(b);
	}
    }


    public String getMediaFilePathName() {
	return audioModel.getMediaFilePathName();
    }


    public void scale(double sx, double sy) {
	super.scale(sx, sy);
	if (audioPanel != null) {
	    Rectangle r = getBasicBounds(); // Already scaled
	    audioPanel.setSize(r.width, r.height);
	}
    }


    public void resetToOriginalScale() {
	super.resetToOriginalScale();
	if (audioPanel != null) {
	    Rectangle r = getBasicBounds(); // already scaled back
	    audioPanel.setSize(r.width, r.height);
	}
    }


    public void doTranslate(int deltaX, int deltaY) {
	super.doTranslate(deltaX, deltaY);
	if (audioPanel != null) {
	    Rectangle r = getBasicBounds();
	    audioPanel.setLocation(r.x, r.y);
	}
    }


    public void deleteGraphicalObject(boolean deleteModel) {
	super.deleteGraphicalObject(deleteModel);
	audioModel.reset();
	if (audioPanel != null) {
	    storyboard.remove(audioPanel);
	    storyboard.revalidate();
	    audioPanel = null;
	}
	if (deleteModel) {
	    audioModel.deleteModel();
	}
    }
    
    
    public void display() {
	setVisible(true);
    }

    public void unDisplay() {
	setVisible(false);
    }

    
    public void highlight() {
	opIsHighlighted = true;
	if (audioPanel != null) {
	    audioPanel.setBorder(HIGHLIGHT_BORDER);
	}
    }

    public void unHighlight() {
	opIsHighlighted = false;
	if (audioPanel != null) {
	    audioPanel.setBorder(NOT_ACTIVE_BORDER);
	}
    }


    public void setVisible(boolean v) {
	super.setVisible(v);
	if (v) {
	    if (audioPanel != null) {
		// May already be visible
		if (! containPanel(audioPanel)) {
		    audioPanel.setVisible(true);
		    storyboard.add(audioPanel, new Integer(getLayer()), 0);
		    storyboard.revalidate();
		}
	    }
	} else {
	    if (audioPanel != null) {
		if (containPanel(audioPanel)) {
		    audioPanel.setVisible(false);
		    storyboard.remove(audioPanel);
		    storyboard.revalidate();
		}
	    }
	}
    }
    
    
    public void realize(boolean showControls, boolean recreate) {
	super.realize(showControls, recreate);
	audioModel.realize();

	if (audioPanel == null) {
	    audioPanel = createAudioPanel();
	    setBorder();
	    setVisible(isVisible());
	    
	} else if (recreate) {
	    /* 
	       When a storyboard is dragged to a MVP, then the same controller is added to that panel, causing the one in the storyboard to disappear. The parent of the component is changed (and there is only one controller component to be obtained), which is the cause of the problem. Thus, each time I need to reinstall the control component here
	    */
	    audioPanel.removeAll();
	    Component controlPanel = audioModel.getControlPanelComponent();
	    if (controlPanel != null) {
		audioPanel.add(controlPanel, panelConstraints);
		audioPanel.revalidate();
	    }
	}
    }


    public void unRealize(boolean keepVisuals) {
	super.unRealize(keepVisuals);
	audioModel.unRealize();
    }


    public void reset() {
	audioModel.reset();
    }


    public void begin() {
	storyboard.opResetCMObjects(this);
    }


    public void operationalize() {
	super.operationalize();
	audioModel.operationalize();
	setVisible(false);
    }

    public void unOperationalize() {
	super.unOperationalize();
	audioModel.unOperationalize();
	setVisible(true);
    }



    public Object clone() {
	return (new AudioRectangleInkStroke(this));
    }

    public Object clone(Storyboard toStoryboard) {
	return (new AudioRectangleInkStroke(this, toStoryboard));
    }

    public Object clone(Storyboard toStoryboard, boolean copyAModel) {
	return (new AudioRectangleInkStroke(this, toStoryboard, copyAModel));
    }



    // Mouse movement methods
    protected void mouseEnter() {
    }

    protected void mouseLeave() {
    }

    protected void mouseSingleClicked() {
    }

    protected void mouseDoubleClicked() {
    }

    public String getNamedModelId() {
	return audioModel.getIdentifierId();
    }

    public NamedModel getNamedModel() {
	return audioModel;
    }

    
    protected void doRender(Graphics2D g) {
	if (audioPanel != null) {
	    setBorder();
	} else {
	    super.doRender(g);
	}
    }


    // Private methods
    private void setBorder() {
	if (audioPanel != null) {
	    if (opIsHighlighted) {
		if (audioPanel.getBorder() != HIGHLIGHT_BORDER) {
		    audioPanel.setBorder(HIGHLIGHT_BORDER);
		}
	    } else if (opMouseIsOver) {
		if (audioPanel.getBorder() != ACTIVE_BORDER) {
		    audioPanel.setBorder(ACTIVE_BORDER);
		}
	    } else if (operationalized) {
		if (audioPanel.getBorder() != NOT_ACTIVE_BORDER) {
		    audioPanel.setBorder(NOT_ACTIVE_BORDER);
		}
	    } else if (inSelectedLayer) {
		if (audioPanel.getBorder() != ACTIVE_BORDER) {
		    audioPanel.setBorder(ACTIVE_BORDER);
		}
	    } else {
		if (audioPanel.getBorder() != NOT_ACTIVE_BORDER) {
		    audioPanel.setBorder(NOT_ACTIVE_BORDER);
		}
	    }
	}
    }
    
    private JPanel createAudioPanel() {
	JPanel panel = new JPanel(new GridBagLayout());
	panel.putClientProperty("DEMAIS", this);
	
	Rectangle r = getBasicBounds();
	panel.setBounds(r.x, r.y, r.width, r.height);
	
	panelConstraints = new GridBagConstraints();
	panelConstraints.gridx = 0;
	panelConstraints.gridy = 0;
	panelConstraints.fill = GridBagConstraints.HORIZONTAL;
	panelConstraints.weightx = 1.0;
	panelConstraints.weighty = 0.0;
	Component controlPanel = audioModel.getControlPanelComponent();
	if (controlPanel != null) {
	    panel.add(controlPanel, panelConstraints);
	}

	return panel;
    }



    public void write(DataOutputStream stream) throws IOException {
	// Write the class id
	stream.writeInt(CLASSID);

	// Write the parent information
	super.write(stream);

	// Write the model information
	stream.writeBoolean(audioModel != null);
	if (audioModel != null) {
	    audioModel.write(stream);
	}
    }

}
