/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File BeginAnchor.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs;
    

// Necessary imports
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.multiview.*;




// InkStroke class
public class BeginAnchor extends GraphicalObject {

    // Class variables
    public static final int CLASSID = 100;
    private static final int WIDTH = 30;
    private static final int HEIGHT = 30;


    // Instance variables
    private int x, y, width, height;
    private Rectangle bounds;



    private BeginAnchor() {
	this(-1, null);
    }

    public BeginAnchor(int layer, Storyboard storyboard) {
	super(layer, true, false, storyboard); 
	x = 0;
	y = 0;
	width = WIDTH;
	height = HEIGHT;
	bounds = null;
    }

    public BeginAnchor(BeginAnchor copy) {
	this(copy, copy.storyboard);
    }

    public BeginAnchor(BeginAnchor copy, Storyboard toStoryboard) {
	super(copy, toStoryboard);
	x = copy.x;
	y = copy.y;
	width = copy.width;
	height = copy.height;
	bounds = null;
    }


    public BeginAnchor(DataInputStream stream, Storyboard pStoryboard) throws IOException {
	super(stream, pStoryboard);
	this.x = stream.readInt();
	this.y = stream.readInt();
	this.width = stream.readInt();
	this.height = stream.readInt();
	bounds = null;
    }


    public void scale(double sx, double sy) {
	super.scale(sx, sy);
	x = (int) Math.round(x * sx);
	y = (int) Math.round(y * sy);
	width = (int) Math.round(width * sx);
	height = (int) Math.round(height * sy);
	bounds = null;
    }


    public void resetToOriginalScale() {
	super.resetToOriginalScale();
	x = 0;
	y = 0;
	width = WIDTH;
	height = HEIGHT;
	bounds = null;
    }


    public void doTranslate(int deltaX, int deltaY) {
	x += deltaX;
	y += deltaY;
	bounds = null;
    }

    public void unOperationalize() {
	super.unOperationalize();
	setVisible(true);
    }


    public Object clone() {
	return (new BeginAnchor(this));
    }

    public Object clone(Storyboard toStoryboard) {
	return (new BeginAnchor(this, toStoryboard));
    }

    public Object clone(Storyboard toStoryboard, boolean copyOfModel) {
	return (new BeginAnchor(this, toStoryboard));
    }


    protected void mouseEnter() {
    }
    protected void mouseLeave() {
    }
    protected void mouseSingleClicked() {
    }
    protected void mouseDoubleClicked() {
    }

    public String getNamedModelId() {
	return "0";
    }
    public NamedModel getNamedModel() {
	return null;
    }


    // Protected methods
    protected void setStroke(Graphics2D g) {
	if ((scaleFactorX == 1.0) && (scaleFactorY == 1.0)) {
	    g.setStroke(MEDIUM_STROKE);
	    
	} else {
	    g.setStroke(LIGHT_STROKE);
	}
    }

    protected void setColor(Graphics2D g) {
	g.setColor(ACTIVE_COLOR);
    }

    protected void doRender(Graphics2D g) {
	g.setColor(PropertyManager.DARK_GREEN);
	g.fillRect(x, y, width, height);

	// Draw outer border
	setColor(g);
	if ((scaleFactorX == 1.0) && (scaleFactorY == 1.0)) {
	    g.drawRect(x - 1, y - 1, width, height);
	} else {
	    g.drawRect(x, y, width, height);
	}
    }

    
    public Rectangle getBasicBounds() {
	if (bounds == null) {
	    bounds = new Rectangle(x, y, width, height);
	}
	return bounds;
    }
    
    protected Rectangle doGetBounds() {
	return getBasicBounds();
    }



    public void write(DataOutputStream stream) throws IOException {
	// Write the class id
	stream.writeInt(CLASSID);

	// Write the parent information
	super.write(stream);

	// Write the anchor information
	stream.writeInt(x);
	stream.writeInt(y);
	stream.writeInt(width);
	stream.writeInt(height);
    }


}

