/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File ImageRectangleInkStroke.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs;
    

// Necessary imports
import java.io.*;
import java.awt.*;
import java.awt.color.*;
import java.awt.image.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;




// InkStroke class
public class ImageRectangleInkStroke extends RectangleInkStroke { 

    // Class id for file I/O
    public static final int CLASSID = 150;


    // The original image before scaling
    private BufferedImage originalImage;
    private BufferedImage bwOriginalImage;

    // The current image
    private BufferedImage image;
    private BufferedImage bwImage;

    // MEAL model
    private ImageModel imageModel;




    // Do not allow default constructor to be called
    private ImageRectangleInkStroke() {
	this((RectangleInkStroke)null, (String)null);
    }

    public ImageRectangleInkStroke(RectangleInkStroke copy, String filePathName) {
	super(true, copy);
	originalImage = null;
	bwOriginalImage = null;

	image = null;
	bwImage = null;

	imageModel = new ImageModel(PropertyManager.LOCAL_SCOPE, getScope(), null, filePathName, this);
	loadImageFromFile(filePathName);
    }


    public ImageRectangleInkStroke(ImageRectangleInkStroke copy) {
	this(copy, copy.storyboard);	
    }

    public ImageRectangleInkStroke(ImageRectangleInkStroke copy, Storyboard toStoryboard) {
	this(copy, toStoryboard, true);
    }

    public ImageRectangleInkStroke(ImageRectangleInkStroke copy, Storyboard toStoryboard, boolean copyIModel) {
	super(copy, toStoryboard, copyIModel);
	imageModel = null;
	if (copy.imageModel != null) {
	    if (copyIModel) {
		imageModel = (ImageModel) copy.imageModel.clone(this);
	    } else {
		imageModel = copy.imageModel;
	    }
	}	

	originalImage = null;
	if (copy.originalImage != null) {
	    originalImage = createBufferedImage(copy.originalImage);
	}

	image = null;
	if (copy.image != null) {
	    image = createBufferedImage(copy.image);
	}
	

	bwOriginalImage = null;
	if (copy.bwOriginalImage != null) {
	    bwOriginalImage = createBufferedImage(copy.bwOriginalImage, BufferedImage.TYPE_INT_RGB);
	}


	bwImage = null;
	if (copy.bwImage != null) {
	    bwImage = createBufferedImage(copy.bwImage, BufferedImage.TYPE_INT_RGB);
	}
    }


    public ImageRectangleInkStroke(DataInputStream stream, Storyboard pStoryboard) throws IOException {
	super(stream, pStoryboard);
	originalImage = null;
	bwOriginalImage = null;
	image = null;
	bwImage = null;

	// Read model
	imageModel = null;
	boolean hasModel = stream.readBoolean();
	if (hasModel) {
	    imageModel = new ImageModel(stream, this, pStoryboard.getModel());
	    loadImageFromFile(imageModel.getMediaFilePathName());
	}
    }


 
    public void scale(double sx, double sy) {
	super.scale(sx, sy);
	Rectangle r = getBasicBounds(); // Already scaled value

	Image img = image.getScaledInstance(r.width, r.height, Image.SCALE_DEFAULT);
	waitForImage(img);
	image = createBufferedImage(img);

	img = bwImage.getScaledInstance(r.width, r.height, Image.SCALE_DEFAULT);
	waitForImage(img);
	bwImage = createBufferedImage(img);
    }


    public void resetToOriginalScale() {
	super.resetToOriginalScale();
	image = originalImage;
	bwImage = bwOriginalImage;
    }
      
    public void deleteGraphicalObject(boolean deleteModel) {
	super.deleteGraphicalObject(deleteModel);
	if (deleteModel) {
	    if (imageModel != null) {
		imageModel.deleteModel();
	    }
	}
    }
    

    public void setMouseIsOver(boolean b) {
	super.setMouseIsOver(b);
	if (imageModel != null) {
	    imageModel.setIsOver(b);
	}
    }


    public Object clone() {
	return (new ImageRectangleInkStroke(this));
    }

    public Object clone(Storyboard toStoryboard) {
	return (new ImageRectangleInkStroke(this, toStoryboard));
    }

    public Object clone(Storyboard toStoryboard, boolean copyIModel) {
	return (new ImageRectangleInkStroke(this, toStoryboard, copyIModel));
    }

    
    public void display() {
	setVisible(true);
	storyboard.paintComponent();
    }

    public void unDisplay() {
	setVisible(false);
	storyboard.paintComponent();
    }


 
    // Protected methods

    // Mouse movement methods
    protected void mouseEnter() {
	setMouseIsOver(true);
	storyboard.paintComponent();
	imageModel.mouseEnter();
    }


    protected void mouseLeave() {
	setMouseIsOver(false);
	storyboard.paintComponent();
	imageModel.mouseLeave();
    }


    protected void mouseSingleClicked() {
	imageModel.mouseSingleClicked();
    }

    protected void mouseDoubleClicked() {
	imageModel.mouseDoubleClicked();
    }

    public String getNamedModelId() {
	return imageModel.getIdentifierId();
    }

    public NamedModel getNamedModel() {
	return imageModel;
    }

 
    protected void doRender(Graphics2D g) {
	// Do not draw parent (underlying ink stroke)
	// Get the basic image bounds
	Rectangle r = getBasicBounds();

	// Draw the appropriate image
	if (operationalized || inSelectedLayer) {
	    g.drawImage(image, r.x, r.y, PropertyManager.STORYBOARD_BACKGROUND_COLOR, null);
	    
	} else {
	    // Not operationalized, not a selected layer
	    g.drawImage(bwImage, r.x, r.y, PropertyManager.STORYBOARD_BACKGROUND_COLOR, null);
	}
	
	// Draw an appropriate border
	if (opMouseIsOver || opIsHighlighted) {
	    // Draw border - leave room for the wider stroke
	    g.drawRect(r.x - 1, r.y - 1, r.width + 2, r.height + 2);
	    
	} else if ((operationalized) || 
		   (scaleFactorX < 1.0) ||
		   (scaleFactorY < 1.0)) {
	    // 1 pixel border
	    g.drawRect(r.x - 1, r.y - 1, r.width + 1, r.height + 1);
	
	} else if (inSelectedLayer) {
	    g.setStroke(MEDIUM_STROKE);
	    g.drawRect(r.x - 1, r.y - 1, r.width + 2, r.height + 2);
	    
	} else {
	    // Draw grey border
	    g.setStroke(MEDIUM_STROKE);
	    g.drawRect(r.x - 1, r.y - 1, r.width + 2, r.height + 2);
	}
    }

    public void mediateRender(Graphics2D g, int offsetX, int offsetY) {
	Rectangle r = getBasicBounds();
	g.drawImage(image, r.x + offsetX, r.y + offsetY, PropertyManager.STORYBOARD_BACKGROUND_COLOR, null);
	
    }


    protected Rectangle doGetBounds() {
	Rectangle r = getBasicBounds();

	if (opIsHighlighted || opMouseIsOver) {
	    r = new Rectangle(r.x - 2, r.y - 2, r.width + 4, r.height + 4);
	    
	} else if ((operationalized) || 
		   (scaleFactorX < 1.0) ||
		   (scaleFactorY < 1.0)) {
	    r = new Rectangle(r.x - 1, r.y - 1, r.width + 2, r.height + 2);
	    
	} else {
	    r = new Rectangle(r.x - 2, r.y - 2, r.width + 4, r.height + 4);
	}

	return r;
    }


    // Private methods
    // Created a BufferedImage from an Image
    private BufferedImage createBufferedImage(Image img) {
	return (createBufferedImage(img, BufferedImage.TYPE_4BYTE_ABGR));
    }

    private BufferedImage createBufferedImage(Image img, int type) {
	BufferedImage bufferedImage = null;
	if (img != null) {
	    int width = img.getWidth(null);
	    int height = img.getHeight(null);
	    bufferedImage = new BufferedImage(width, height, type); 
	    Graphics2D g = bufferedImage.createGraphics();
	    g.drawImage(img, 0, 0, null);
	}
	return bufferedImage;
    }
    
    
    private BufferedImage createBlackAndWhiteImage(BufferedImage img) {
	BufferedImage newImage = null;
	try {
	    ColorConvertOp op = new ColorConvertOp(ColorSpace.getInstance(ColorSpace.CS_GRAY), null);
	    newImage = op.filter(img, null);
	} catch (Exception xcptn) {
	    newImage = createBufferedImage(img);
	}
	return newImage;
    }
    

    private void loadImageFromFile(String file) {
	ImageIcon icon = new ImageIcon(file);
	Image img = icon.getImage();
	if (img != null) {
	    Rectangle r = getBasicBounds();
	    Image scaledImage = img.getScaledInstance(r.width, r.height, Image.SCALE_DEFAULT);	
	    waitForImage(scaledImage);

	    originalImage = createBufferedImage(scaledImage);
	    image = createBufferedImage(scaledImage);

	    bwOriginalImage = createBlackAndWhiteImage(createBufferedImage(scaledImage, BufferedImage.TYPE_INT_RGB));
	    bwImage = createBufferedImage(bwOriginalImage, BufferedImage.TYPE_INT_RGB);
	}
    }


    private void waitForImage(Image img) {
	// Be sure the image is loaded before creating the buffered image
	MediaTracker mediaTracker = new MediaTracker(storyboard);
	mediaTracker.addImage(img, 0);
	try {
	    mediaTracker.waitForID(0);
	} catch(InterruptedException xcptn) {
	    System.out.println(xcptn.toString());
	}
    }


    public void write(DataOutputStream stream) throws IOException {
	// Write the class id
	stream.writeInt(CLASSID);

	// write the parent information
	super.write(stream);

	// Write the model information
	stream.writeBoolean(imageModel != null);
	if (imageModel != null) {
	    imageModel.write(stream);
	}
    }

}

