/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File InkLine.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs;
    

// Necessary imports
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;




// InkStroke class
public class InkLine extends GraphicalObject {

    public static final int CLASSID = 110;

    private int x1, y1;
    private int x2, y2;
    private int tx1, ty1;
    private int tx2, ty2;



    private InkLine() {
	this(0, 0, 0, 0, null);
    }

    public InkLine(int x1, int y1, int x2, int y2, Storyboard s) {
	super(0, true, false, s);
	this.x1 = x1;
	this.y1 = y1;
	this.x2 = x2;
	this.y2 = y2;
    }


    public InkLine(InkLine copy) {
	this(copy, copy.storyboard);
    }


    public InkLine(InkLine copy, Storyboard toStoryboard) {
	super(copy, toStoryboard);
	this.x1 = copy.x1;
	this.y1 = copy.y1;
	this.x2 = copy.x2;
	this.y2 = copy.y2;
    }


    public InkLine(DataInputStream stream, Storyboard pStoryboard) throws IOException {
	super(stream, pStoryboard);

	// Read the line information
	x1 = stream.readInt();
	y1 = stream.readInt();
	x2 = stream.readInt();
	y2 = stream.readInt();
    }



    public void scale(double sx, double sy) {
	super.scale(sx, sy);
	tx1 = x1;
	ty1 = y1;
	tx2 = x2;
	ty2 = y2;

	x1 = (int) Math.round(x1 * sx);
	y1 = (int) Math.round(y1 * sy);

	x2 = (int) Math.round(x2 * sx);
	y2 = (int) Math.round(y2 * sy);
    }


    public void resetToOriginalScale() {
	super.resetToOriginalScale();
	x1 = tx1;
	y1 = ty1;
	x2 = tx2;
	y2 = ty2;
    }


    public void doTranslate(int deltaX, int deltaY) {
	x1 += deltaX;
	x2 += deltaX;

	y1 += deltaY;
	y2 += deltaY;
    }


    public Object clone() {
	return (new InkLine(this));
    }

    public Object clone(Storyboard toStoryboard) {
	return (new InkLine(this, toStoryboard));
    }

    public Object clone(Storyboard toStoryboard, boolean copyOfModel) {
	return (new InkLine(this, toStoryboard));
    }


    protected void mouseEnter() {
    }
    protected void mouseLeave() {
    }
    protected void mouseSingleClicked() {
    }
    protected void mouseDoubleClicked() {
    }
    public String getNamedModelId() {
	return "0";
    }
    public NamedModel getNamedModel() {
	return null;
    }


    // Protected methods
    protected void setColor(Graphics2D g) {
	g.setColor(ACTIVE_COLOR);
    }

    protected void setStroke(Graphics2D g) {
	g.setStroke(LIGHT_STROKE);
    }

    protected void doRender(Graphics2D g) {
	g.drawLine(x1, y1, x2, y2);
    }


    public Rectangle getBasicBounds() {
	Rectangle r = null;
	if (x1 <= x2) {
	    r = new Rectangle(x1, y1, Math.abs(x2 - x1), Math.abs(y2 - y1));
	} else {
	    r = new Rectangle(x2, y2, Math.abs(x2 - x1), Math.abs(y2 - y1));
	}
	return r;
    }

    protected Rectangle doGetBounds() {
	return getBasicBounds();
    }


    public void write(DataOutputStream stream) throws IOException {
	// Write the class id
	stream.writeInt(CLASSID);

	// Write the parent information
	super.write(stream);

	// Write the line information
	stream.writeInt(x1);
	stream.writeInt(y1);
	stream.writeInt(x2);
	stream.writeInt(y2);
    }


}

