/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File InkStroke.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs;
    

// Necessary imports
import java.awt.*;
import java.io.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.stroke.*;




// InkStroke class
public class InkStroke extends GraphicalObject {
    
    public static final int CLASSID = 120;

    // The timed stroke made by the user
    protected TimedStroke timedStroke;




    // Do not allow default constructor to be called
    private InkStroke() {
	this((TimedStroke) null, 0, false, false, (Storyboard)null);
    }

    public InkStroke(TimedStroke ts, int layer, boolean visible, boolean selected, Storyboard pStoryboard) {
	super(layer, visible, selected, pStoryboard);
	timedStroke = ts;
    }
    
    public InkStroke(InkStroke copy) {
	this(copy, copy.storyboard);
    }

    public InkStroke(InkStroke copy, Storyboard toStoryboard) {
	super(copy, toStoryboard);
	timedStroke = null;
	if (copy.timedStroke != null) {
	    timedStroke = (TimedStroke) copy.timedStroke.clone();
	}
    }


    public InkStroke(DataInputStream stream, Storyboard pStoryboard) throws IOException {
	super(stream, pStoryboard);
	timedStroke = new TimedStroke(stream);
    }



    // Public methods
    public Point getFirstPoint() {
	return (new Point(timedStroke.xpoints[0], 
			  timedStroke.ypoints[0]));
    }


    public Point getLastPoint() {
	return (new Point(timedStroke.xpoints[timedStroke.npoints - 1],
			  timedStroke.ypoints[timedStroke.npoints - 1]));
    }


    public TimedStroke getTimedStroke() {
	return timedStroke;
    }


    // Override the default intersection routine - Check that a
    // point on the line is actually contained by the rectangle
    public boolean strokeIntersects(Rectangle r) {
	TimedStroke stroke = getTimedStroke();
	for (int i=0; i<stroke.npoints; ++i) {
	    int x = stroke.xpoints[i];
	    int y = stroke.ypoints[i];
	    if (r.contains(x, y)) {
		return true;
	    }
	}
	return false;
    }


    public void scale(double sx, double sy) {
	super.scale(sx, sy);
	timedStroke.scale(sx, sy);
    }


    public void resetToOriginalScale() {
	super.resetToOriginalScale();
	timedStroke.resetToOriginalScale();
    }

    public void doTranslate(int deltaX, int deltaY) {
	timedStroke.translate(deltaX, deltaY);
    }


    public Object clone() {
	return (new InkStroke(this));
    }

    public Object clone(Storyboard toStoryboard) {
	return (new InkStroke(this, toStoryboard));
    }

    public Object clone(Storyboard toStoryboard, boolean copyOfModel) {
	return (new InkStroke(this, toStoryboard));
    }


    protected void mouseEnter() {
    }
    protected void mouseLeave() {
    }
    protected void mouseSingleClicked() {
    }
    protected void mouseDoubleClicked() {
    }
    public String getNamedModelId() {
	return "0";
    }
    public NamedModel getNamedModel() {
	return null;
    }


    // Protected methods

    protected void doRender(Graphics2D g) {
	g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
			   RenderingHints.VALUE_ANTIALIAS_ON);
	drawInkStroke(g);
	g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
			   RenderingHints.VALUE_ANTIALIAS_OFF);
    }


    protected void drawInkStroke(Graphics2D g) {
	for (int i=1; i<timedStroke.npoints; ++i) {
	    g.drawLine(timedStroke.xpoints[i-1], timedStroke.ypoints[i-1],
		       timedStroke.xpoints[i], timedStroke.ypoints[i]);
	}
    }


    public Rectangle getBasicBounds() {
	return timedStroke.getBounds();
    }

    protected Rectangle doGetBounds() {
	return getBasicBounds();
    }


    public void write(DataOutputStream stream) throws IOException {
	// Write the class id
	stream.writeInt(CLASSID);

	// Write the base information
	super.write(stream);

	int mode = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_MODE)).intValue();
	if (mode == PropertyManager.APPLICATION_PLAY) {
	    resetToOriginalLocation();
	}
	// Write the TimedStroke information
	timedStroke.write(stream);
    }

}

