/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File RectangleInkStroke.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.JTextField;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.stroke.*;





public class RectangleInkStroke extends InkStroke { 

    // Class ID for file I/O
    public static final int CLASSID = 140;
    

    // The associated MEAL object
    protected ButtonModel bModel;

    // Used only for enter_text_events
    private TextEntryModel tModel;
    private JTextField textField;



    // Do not allow default constructor to be called
    private RectangleInkStroke() {
	this((TimedStroke) null, 0, false, false, (Storyboard)null);
    }

    // Provided only for subclasses
    protected RectangleInkStroke(boolean subclass, RectangleInkStroke copy) {
	super(copy);
	bModel = null;
	tModel = null;
    }


    public RectangleInkStroke(TimedStroke ts, int layer, boolean visible, boolean selected, Storyboard pStoryboard) {
	super(ts, layer, visible, selected, pStoryboard);
	bModel = new ButtonModel(PropertyManager.LOCAL_SCOPE, getScope(), null, this);
	tModel = null;
    }


    public RectangleInkStroke(RectangleInkStroke copy) {
	this(copy, copy.storyboard);
    }

    public RectangleInkStroke(RectangleInkStroke copy, Storyboard toStoryboard) {
	this(copy, toStoryboard, true);
    }

    public RectangleInkStroke(RectangleInkStroke copy, Storyboard toStoryboard, boolean copyBModel) {
	super(copy, toStoryboard);
	bModel = null;
	if (copy.bModel != null) {
	    if (copyBModel) {
		bModel = (ButtonModel) copy.bModel.clone(this);
	    } else {
		bModel = copy.bModel;
	    }
	}
	tModel = null;
    }


    public RectangleInkStroke(DataInputStream stream, Storyboard pStoryboard) throws IOException {
	super(stream, pStoryboard);

	bModel = null;
	tModel = null;
	// Read the model information, if any
	boolean hasModel = stream.readBoolean();
	if (hasModel) {
	    bModel = new ButtonModel(stream, this, pStoryboard.getModel());
	}
    }


    public String getTextModelId() {
	String id = null;
	if (tModel != null) {
	    id = tModel.getIdentifierId();
	}
	return id;
    }

    public TextEntryModel getTextModel() {
	return tModel;
    }


    public Object clone() {
	return (new RectangleInkStroke(this));
    }

    public Object clone(Storyboard toStoryboard) {
	return (new RectangleInkStroke(this, toStoryboard));
    }

    public Object clone(Storyboard toStoryboard, boolean copyBModel) {
	return (new RectangleInkStroke(this, toStoryboard, copyBModel));
    }


    public void deleteGraphicalObject(boolean deleteModel) {
	super.deleteGraphicalObject(deleteModel);
	if (deleteModel) {
	    if (bModel != null) {
		bModel.deleteModel();
	    }
	}
    }

    public void setMouseIsOver(boolean b) {
	super.setMouseIsOver(b);
	if (bModel != null) {
	    bModel.setIsOver(b);
	}
    }


    public void display() {
	setVisible(true);
	storyboard.setVisibleOnObjectsContainedBy(getBasicBounds(), true, getLayer(), false, this);
	storyboard.paintComponent();
    }


    public void unDisplay() {
	setVisible(false);
	storyboard.setVisibleOnObjectsContainedBy(doGetBounds(), false, getLayer(), false, this);
	storyboard.paintComponent();
    }


    public void highlight() {
	opIsHighlighted = true;
	storyboard.paintComponent();
    }

    public void unHighlight() {
	opIsHighlighted = false;
	storyboard.paintComponent();
    }


    // Mouse movement methods
    protected void mouseEnter() {
	setMouseIsOver(true);
	storyboard.setMouseIsOverOnObjectsContainedBy(doGetBounds(), true, getLayer(), false, this);
	storyboard.paintComponent();
	if (bModel != null) {
	    bModel.mouseEnter();
	}
    }


    protected void mouseLeave() {
	setMouseIsOver(false);
	storyboard.setMouseIsOverOnObjectsContainedBy(doGetBounds(), false, getLayer(), false, this);
	storyboard.paintComponent();
	if (bModel != null) {
	    bModel.mouseLeave();
	}
    }


    protected void mouseSingleClicked() {
	if (bModel != null) {
	    bModel.mouseSingleClicked();
	}
    }

    protected void mouseDoubleClicked() {
	if (bModel != null) {
	    bModel.mouseDoubleClicked();
	}
    }
    
    
    public String getNamedModelId() {
	return bModel.getIdentifierId();
    }

    public NamedModel getNamedModel() {
	return bModel;
    }


    protected void doRender(Graphics2D g) {
	Rectangle r = getBasicBounds();
	g.drawRect(r.x, r.y, r.width, r.height);
    }

    public void mediateRender(Graphics2D g, int offsetX, int offsetY) {
	Rectangle r = getBasicBounds();
	g.drawRect(r.x + offsetX, r.y + offsetY, r.width, r.height);
    }


    public Rectangle getBasicBounds() {
	return timedStroke.getBounds();
    }

    protected Rectangle doGetBounds() {
	Rectangle r = getBasicBounds();
	if (opIsHighlighted || opMouseIsOver) {
	    r = new Rectangle(r.x - 2, r.y - 2, r.width + 4, r.height + 4);
	} else {
	    r = new Rectangle(r.x - 1, r.y - 1, r.width + 2, r.height + 2);
	}
	return r;
    }



    public void createTextModel() {
	if (tModel == null) {
	    tModel = new TextEntryModel(PropertyManager.LOCAL_SCOPE, getScope(), (String)null, this);
	    textField = new JTextField();
	    textField.setFont(PropertyManager.APPLICATION_TEXT_FIELD_FONT);
	    textField.addActionListener(new TextEntryListener());

	    Rectangle r = getBasicBounds();
	    textField.setBounds(r.x, r.y, r.width, r.height);	    
	    storyboard.add(textField);
	    storyboard.revalidate();
	}
    }


    public void deleteTextModel() {
	if (tModel != null) {
	    tModel.deleteModel();
	    tModel = null;
	}
    }


    public void unOperationalize() {
	super.unOperationalize();
	if (tModel != null) {
	    tModel.deleteModel();
	    tModel = null;

	    storyboard.remove(textField);
	    storyboard.revalidate();
	    textField = null;
	}
    }


    // Override ink stroke method
    public boolean strokeIntersects(Rectangle r) {
	return ((boundsIntersects(r)) && 
		(! contains(r)));
    }




    public void write(DataOutputStream stream) throws IOException {
	// Write the class id
	stream.writeInt(CLASSID);

	// write the parent information
	super.write(stream);

	// Write the model information
	stream.writeBoolean(bModel != null);
	if (bModel != null) {
	    bModel.write(stream);
	} 
    }



    class TextEntryListener implements ActionListener {
	
	public TextEntryListener() {
	}

	public void actionPerformed(ActionEvent evt) {
	    tModel.textEntered();
	}

    }

}
