/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File VideoRectangleInkStroke.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs;
    

// Necessary imports
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.EventListener;
import javax.swing.event.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;




public class VideoRectangleInkStroke extends ContinuousMediaRectangleInkStroke { 

    public static final int CLASSID = 160;
    private static final Border ACTIVE_BORDER;
    private static final Border NOT_ACTIVE_BORDER;
    private static final Border HIGHLIGHT_BORDER;


    // The panel holding the video and controllers
    private JPanel videoPanel;

    // Constraints for the audioPanel's controller
    private GridBagConstraints panelConstraints;

    // component in which video is playing
    private Component visualComponent;

    // controls gain, position, start, stop
    private Component controlPanelComponent;

    // Parallel video model for Nsync
    private VideoModel videoModel;




    // Static initializer
    static {
	ACTIVE_BORDER = BorderFactory.createMatteBorder(2, 2, 2, 2, ACTIVE_COLOR);
	NOT_ACTIVE_BORDER = BorderFactory.createMatteBorder(2, 2, 2, 2, NOT_ACTIVE_COLOR);
	HIGHLIGHT_BORDER = BorderFactory.createMatteBorder(2, 2, 2, 2, HIGHLIGHT_COLOR);

    }


    // Do not allow default constructor to be called
    private VideoRectangleInkStroke() {
	this((RectangleInkStroke)null, (String)null);
    }

    public VideoRectangleInkStroke(RectangleInkStroke copy, String filePathName) {
	super(true, copy);
	videoPanel = null;
	panelConstraints = null;
	visualComponent = null;
	controlPanelComponent = null;

	videoModel = new VideoModel(PropertyManager.LOCAL_SCOPE, getScope(), null, filePathName, this);
	realize(true, true);
    }


    public VideoRectangleInkStroke(VideoRectangleInkStroke copy) {
	this(copy, copy.storyboard);
    }

    public VideoRectangleInkStroke(VideoRectangleInkStroke copy, Storyboard toStoryboard) {
	this(copy, toStoryboard, true);
    }

    public VideoRectangleInkStroke(VideoRectangleInkStroke copy, Storyboard toStoryboard, boolean copyVModel) {
	super(copy, toStoryboard, copyVModel);
	videoModel = null;
	videoPanel = null;
	panelConstraints = null;
	visualComponent = null;
	controlPanelComponent = null;
	if (copy.videoModel != null) {
	    if (copyVModel) {
		videoModel = (VideoModel) copy.videoModel.clone(this);
	    } else {
		videoModel = copy.videoModel;
	    }
	}
    }


    public VideoRectangleInkStroke(DataInputStream stream, Storyboard pStoryboard) throws IOException {
	super(stream, pStoryboard);
	videoPanel = null;
	panelConstraints = null;
	visualComponent = null;
	controlPanelComponent = null;

	// Read the model information
	videoModel = null;
	boolean hasModel = stream.readBoolean();
	if (hasModel) {
	    videoModel = new VideoModel(stream, this, pStoryboard.getModel());
	    realize(true, true);
	}
    }



    // Public methods
    public void setControlComponent(boolean visible) {
	// The obvious method of using the controlPanelComponent's
	// setVisible() method is not working - use this as a workaround
	if ((videoPanel != null) && (controlPanelComponent != null)) {
	    if (visible) {
		// Want the control panel to be visible
		// But is it already visible?
		Component[] components = videoPanel.getComponents();
		for (int i=0; i<components.length; ++i) {
		    if (components[i] == controlPanelComponent) {
			// Already visible
			return;
		    }
		}
		
		// Must match the creation constrains below
		panelConstraints.gridx = 0;
		panelConstraints.gridy = 1;
		panelConstraints.weightx = 1.0;
		panelConstraints.weighty = 0.2;
		panelConstraints.fill = GridBagConstraints.BOTH;
		controlPanelComponent.setVisible(true);
		videoPanel.add(controlPanelComponent, panelConstraints);
		
	    } else {
		// Remove the control panel
		controlPanelComponent.setVisible(false);
		videoPanel.remove(controlPanelComponent);
		videoPanel.repaint();
	    }
	}
    }


    public void setVisualComponent(boolean visible) {
	if (visualComponent != null) {
	    visualComponent.setVisible(visible);
	}
    }


    public void setLayer(int layer) {
	super.setLayer(layer);
	if (videoPanel != null) {
	    storyboard.setLayer(videoPanel, layer, 0);
	    storyboard.revalidate();
	}
    }


    public double getDuration() {
	return videoModel.getDuration();
    }


    public void setMediaTime(double value) {
	videoModel.setMediaTime(value);
    }


    public double getMediaTime() {
	return videoModel.getMediaTime();
    }

    public int getState() {
	return videoModel.getState();
    }

    public void setState(int state, double mediaTime) {
	videoModel.setState(state, mediaTime);
    }


    public String getMediaFilePathName() {
	return videoModel.getMediaFilePathName();
    }


    public void setMouseIsOver(boolean b) {
	super.setMouseIsOver(b);
	if (videoModel != null) {
	    videoModel.setIsOver(b);
	}
    }


    public void scale(double sx, double sy) {
	super.scale(sx, sy);
	if (videoPanel != null) {
	    Rectangle r = getBasicBounds(); // Already scaled
	    videoPanel.setSize(r.width, r.height);
	}
    }


    public void resetToOriginalScale() {
	super.resetToOriginalScale();
	if (videoPanel != null) {
	    Rectangle r = getBasicBounds();
	    videoPanel.setSize(r.width, r.height);
	}
    }


    public void doTranslate(int deltaX, int deltaY) {
	super.doTranslate(deltaX, deltaY);
	if (videoPanel != null) {
	    Rectangle r = getBasicBounds();
	    videoPanel.setLocation(r.x, r.y);
	}
    }


    public void deleteGraphicalObject(boolean deleteModel) {
	super.deleteGraphicalObject(deleteModel);
	videoModel.reset();
	if (videoPanel != null) {
	    storyboard.remove(videoPanel);
	    storyboard.revalidate();
	    videoPanel = null;
	}

	if (deleteModel) {
	    videoModel.deleteModel();
	}
    }
    
    
    public void display() {
	setVisible(true);
    }

    public void unDisplay() {
	videoModel.reset();
	setVisible(false);
    }


    public void highlight() {
	opIsHighlighted = true;
	if (videoPanel != null) {
	    videoPanel.setBorder(HIGHLIGHT_BORDER);
	}
    }

    public void unHighlight() {
	opIsHighlighted = false;
	if (videoPanel != null) {
	    videoPanel.setBorder(NOT_ACTIVE_BORDER);
	}
    }


    public void setVisible(boolean v) {
	super.setVisible(v);
	if (v) {
	    if (videoPanel != null) {
		if (! containPanel(videoPanel)) {
		    videoPanel.setVisible(true);
		    storyboard.add(videoPanel, new Integer(getLayer()), 0);
		    storyboard.revalidate();
		}
	    }
	} else {
	    if (videoPanel != null) {
		if (containPanel(videoPanel)) {
		    videoPanel.setVisible(false);
		    storyboard.remove(videoPanel);
		    storyboard.revalidate();
		}
	    }
	}
    }
    

    public void realize(boolean showControls, boolean recreate) {
	super.realize(showControls, recreate);
	videoModel.realize();  

	if (videoPanel == null) {
	    // Create the panel
	    videoPanel = createVideoPanel(showControls);
	    setBorder();
	    setVisible(isVisible());

	} else if (recreate) {
	    /*
	      I tried code similar to what I wrote for the AudioInkStroke, but that did not work for video. This is somewhat wasteful, but it seems to work as desired.
	    */
	    
	    // Remove it
	    storyboard.remove(videoPanel);
	    
	    // Re-create it
	    videoPanel = createVideoPanel(showControls);
	    setBorder();
	    setVisible(isVisible());
	} 
    }


    
    public void unRealize(boolean keepVisuals) {
	super.unRealize(keepVisuals);
	videoModel.unRealize();
    }


    public void operationalize() {
	super.operationalize();
	realize(false, true);
	visualComponent.setSize(videoPanel.getWidth(), videoPanel.getHeight());
	videoModel.operationalize();
    }


    public void unOperationalize() {
	super.unOperationalize();
	videoModel.unOperationalize();
	if (videoPanel != null) {
	    // Re-insert the panel into its specified layer
	    storyboard.remove(videoPanel);
	    storyboard.add(videoPanel, getLayer());
	    setControlComponent(true);
	}
    }


    public void reset() {
	videoModel.reset();
    }


    public void begin() {
	if (videoPanel != null) {
	    // Bring to the front
	    storyboard.setLayer(videoPanel, storyboard.highestLayer(), 0);

	    // Shut off other CM objects
	    storyboard.opResetCMObjects(this);
	}
    }


    // Mouse movement methods
    protected void mouseEnter() {
	if (videoPanel != null) {
	    videoPanel.setBorder(ACTIVE_BORDER);
	}
	videoModel.mouseEnter();
    }


    protected void mouseLeave() {
	if (videoPanel != null) {
	    videoPanel.setBorder(NOT_ACTIVE_BORDER);
	}
	videoModel.mouseLeave();
    }

    protected void mouseSingleClicked() {
	videoModel.mouseSingleClicked();
    }

    protected void mouseDoubleClicked() {
	videoModel.mouseDoubleClicked();
    }


    public String getNamedModelId() {
	return videoModel.getIdentifierId();
    }

    public NamedModel getNamedModel() {
	return videoModel;
    }

    public Object clone() {
	return (new VideoRectangleInkStroke(this));
    }

    public Object clone(Storyboard toStoryboard) {
	return (new VideoRectangleInkStroke(this, toStoryboard));
    }

    public Object clone(Storyboard toStoryboard, boolean copyVModel) {
	return (new VideoRectangleInkStroke(this, toStoryboard, copyVModel));
    }
    
    protected void doRender(Graphics2D g) {
	if (videoPanel != null) {
	    setBorder();
	} else {
	    super.doRender(g);
	}
    }


    // Private methods
    private void setBorder() {
	if (videoPanel != null) {
	    if (opIsHighlighted) {
		if (videoPanel.getBorder() != HIGHLIGHT_BORDER) {
		    videoPanel.setBorder(HIGHLIGHT_BORDER);
		}
	   
	    } else if (opMouseIsOver) {
		if (videoPanel.getBorder() != ACTIVE_BORDER) {
		    videoPanel.setBorder(ACTIVE_BORDER);
		}
	    
	    } else if (operationalized) {
		if (videoPanel.getBorder() != NOT_ACTIVE_BORDER) {
		    videoPanel.setBorder(NOT_ACTIVE_BORDER);
		}
		
	    } else if (inSelectedLayer) {
		if (videoPanel.getBorder() != ACTIVE_BORDER) {
		    videoPanel.setBorder(ACTIVE_BORDER);
		}
		
	    } else {
		if (videoPanel.getBorder() != NOT_ACTIVE_BORDER) {
		    videoPanel.setBorder(NOT_ACTIVE_BORDER);
		}
	    }
	}
    }

    private JPanel createVideoPanel(boolean wantControlPanel) {
	JPanel panel = new JPanel(new GridBagLayout());
	panel.putClientProperty("DEMAIS", this);

	panelConstraints = new GridBagConstraints();
	panelConstraints.gridx = 0;
	panelConstraints.gridy = 0;
	panelConstraints.fill = GridBagConstraints.BOTH;
	panelConstraints.weightx = 1.0;
	panelConstraints.weighty = 1.0;
	visualComponent = videoModel.getVisualComponent();
	if (visualComponent != null) {
	    panel.add(visualComponent, panelConstraints);
	}
	
	if (wantControlPanel) {
	    panelConstraints.gridy = 1;
	    panelConstraints.fill = GridBagConstraints.BOTH;
	    panelConstraints.weightx = 1.0;
	    panelConstraints.weighty = 0.2;
	    controlPanelComponent = videoModel.getControlPanelComponent();
	    if (controlPanelComponent != null) {
		panel.add(controlPanelComponent, panelConstraints);
	    }
	}

	Rectangle r = getBasicBounds();
	panel.setBounds(r.x, r.y, r.width, r.height);

	return panel;
    }



    public void write(DataOutputStream stream) throws IOException {
	// Write the class id
	stream.writeInt(CLASSID);

	// Write the parent information
	super.write(stream);

	// Write the model information
	stream.writeBoolean(videoModel != null);
	if (videoModel != null) {
	    videoModel.write(stream);
	}
    }

}
