/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File BehaviorEvent.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.util.*;
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;





public abstract class BehaviorEvent extends GraphicalObject {

    // Class variables used for painting the icons
    public static final int VLI_WIDTH;
    public static final int VLI_HEIGHT;
    protected static final Font TRANSITION_LETTER_FONT;
    protected static final Font LETTER_FONT;
    protected static final Font NUMBER_FONT_LARGE;
    protected static final Font NUMBER_FONT_MEDIUM;
    protected static final Font NUMBER_FONT_SMALL;
    protected static final Font LETTER_FONT_TINY;


    // The event object
    protected GraphicalObject eventObject;

    // Location of the icon on the screen
    protected int x, y;

    // Width and height of the icon on the screen
    protected int width, height;

    // If this behavior event is in the selected layer
    protected boolean selected;

    // Used only for I/O purposes
    protected int ioEventObjectIdentifier;

    // Cached bounds objects
    private Rectangle bounds;
    private Rectangle targetBounds;




    // Static initializer
    static {
	VLI_WIDTH = 16;
	VLI_HEIGHT = 16;

	TRANSITION_LETTER_FONT = new Font("Arial", Font.BOLD, 14);
	LETTER_FONT = new Font("Arial", Font.BOLD, 14);
	NUMBER_FONT_LARGE = new Font("Arial", Font.BOLD, 14);
	NUMBER_FONT_MEDIUM = new Font("Arial", Font.BOLD, 11);
	NUMBER_FONT_SMALL = new Font("Arial", Font.BOLD, 8);
	LETTER_FONT_TINY = new Font("Arial", Font.PLAIN, 8);
    }




    // Constructors
    private BehaviorEvent() {
	this(null, 0, 0);
    }

    public BehaviorEvent(GraphicalObject eObject, int x, int y) {
	this(eObject, x, y, VLI_WIDTH, VLI_HEIGHT);
    }

    public BehaviorEvent(GraphicalObject eObject, int x, int y, int width, int height) {
	super(0, true, false, null);
	eventObject = eObject;
	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
	selected = false;
	ioEventObjectIdentifier = -1;
	bounds = null;
	targetBounds = null;
    }


    public BehaviorEvent(BehaviorEvent copy) {
	this(copy, copy.eventObject);
    }

    public BehaviorEvent(BehaviorEvent copy, GraphicalObject eObject) {
	super(copy);
	eventObject = eObject;
	x = copy.x;
	y = copy.y;
	width = copy.width;
	height = copy.height;
	selected = copy.selected;
	ioEventObjectIdentifier = -1;
	bounds = null;
	targetBounds = null;
    }


    public BehaviorEvent(DataInputStream stream) throws IOException {
	super(0, true, false, null);
	x = stream.readInt();
	y = stream.readInt();
	width = stream.readInt();
	height = stream.readInt();
	selected = false;
	ioEventObjectIdentifier = -1;
	eventObject = null;

	boolean isID = stream.readBoolean();
	if (isID) {
	    ioEventObjectIdentifier = stream.readInt();
	}
	bounds = null;
	targetBounds = null;
    }

    public void completeIO() {
	if (ioEventObjectIdentifier != -1) {
	    Iterator it = StoryboardModel.getStoryboardModels().iterator();
	    while (it.hasNext()) {
		StoryboardModel sModel = (StoryboardModel) it.next();
		Storyboard storyboard = sModel.getStoryboard();
		eventObject = storyboard.getGraphicalObjectById(ioEventObjectIdentifier);
		if (eventObject != null) {
		    break;
		}
	    }
	}
    }
    


    public GraphicalObject getEventObject() {
	return eventObject;
    }

    public DataModel getEventObjectModel() {
	DataModel ret = null;
	if (eventObject != null) {
	    ret = eventObject.getStoryboard().getModel();
	}
	return ret;
    }
    


    public int getX() {
	return x;
    }

    public int getY() {
	return y;
    }

    public void setLocation(int x, int y) {
	this.x = x;
	this.y = y;
	bounds = null;
	targetBounds = null;
    }

    public void scale(double sx, double sy) {
	bounds = null;
	targetBounds = null;
    }

    public void resetToOriginalScale() {
	bounds = null;
	targetBounds = null;
    }

    public void doTranslate(int deltaX, int deltaY) {
	x += deltaX;
	y += deltaY;
	bounds = null;
	targetBounds = null;
    }


    protected void mouseEnter() {
    }
    protected void mouseLeave() {
    }
    protected void mouseSingleClicked() {
    }
    protected void mouseDoubleClicked() {
    }
    protected void draggedAndDropped(GraphicalObject target) {
    }
    public String getNamedModelId() {
	return "0";
    }
    public NamedModel getNamedModel() {
	return null;
    }


    public Rectangle getTargetBounds() {
	if (targetBounds == null) {
	    targetBounds = new Rectangle(x - 4, y - 4, width + 8, height + 8);
	}
	return targetBounds;
    }

    public Rectangle getBasicBounds() {
	if (bounds == null) {
	    bounds = new Rectangle(x, y, width, height);
	}
	return bounds;
    }

    protected Rectangle doGetBounds() {
	return getBasicBounds();
    }

    public Object clone() {
	return null;
    }
    public Object clone(Storyboard pStoryboard) {
	return null;
    }
    public Object clone(Storyboard pStoryboard, boolean copy) {
	return null;
    }


    protected void setColor(Graphics2D g) {
	if (selected) {
	    g.setColor(ACTIVE_COLOR);
	} else {
	    g.setColor(NOT_ACTIVE_COLOR);
	}
    }
    
    protected void setStroke(Graphics2D g) {
	if (selected) {
	    g.setStroke(MEDIUM_STROKE);	    
	} else {
	    g.setStroke(LIGHT_STROKE);
	}
    }

    protected void setFont(Graphics2D g) {
	g.setFont(LETTER_FONT);
    }


    public void render(Graphics2D g, boolean sel) {
	selected = sel;
	setColor(g);
	setStroke(g);
	setFont(g);

	doRender(g);
    }


    // Subclasses must define!
    public abstract Object clone(GraphicalObject eventObject);
    public abstract int getClassId();


    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(x);
	stream.writeInt(y);
	stream.writeInt(width);
	stream.writeInt(height);

	stream.writeBoolean(eventObject != null);
	if (eventObject != null) {
	    stream.writeInt(eventObject.getIdentifier());
	}
    }
}


