/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File DoubleMouseClickEvent.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class DoubleMouseClickEvent extends SourceEvent {
    
    public static final int CLASSID = 1040;



    private DoubleMouseClickEvent() {
	this((RectangleInkStroke)null);
    }

    public DoubleMouseClickEvent(RectangleInkStroke eObject) {
	this(eObject, 0, 0);
    }

    public DoubleMouseClickEvent(RectangleInkStroke eObject, int x, int y) {
	super(eObject, x, y);
    }

    public DoubleMouseClickEvent(TextGraphicalObject eObject) {
	this(eObject, 0, 0);
    }

    public DoubleMouseClickEvent(TextGraphicalObject eObject, int x, int y) {
	super(eObject, x, y);
    }

    public DoubleMouseClickEvent(DoubleMouseClickEvent copy) {
	super(copy);
    }

    public DoubleMouseClickEvent(DoubleMouseClickEvent copy, GraphicalObject eObject) {
	super(copy, eObject);
    }


    public DoubleMouseClickEvent(DataInputStream stream) throws IOException {
	super(stream);
    }



    public void doRender(Graphics2D g) {
	// Draw outer circle
	g.drawArc(x, y, width, height, 0, 360);
	// Draw inner circle
	g.drawArc(x + 4, y + 4, 8, 8, 0, 360);
    }


    public String toString() {
	String ret = null;
	if (eventObject != null) {
	    String scopeName = eventObject.getScope().getIdentifierId();
	    ret = "When the user doubleclicks " + scopeName + "::" + eventObject.getNamedModelId();
	}
	return ret;
    }



    public boolean requiresRuntimeMouseEvents() {
	return true;
    }


    public Object clone(GraphicalObject eObject) {
	return (new DoubleMouseClickEvent(this, eObject));
    }


    public int getClassId() {
	return CLASSID;
    }

              
    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	super.write(stream);
    }

}
