/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File DragAndDropEvent.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class DragAndDropEvent extends SourceEvent {
    
    public static final int CLASSID = 1050;

    private GraphicalObject target;
    private int ioTargetId;



    // Constructors
    private DragAndDropEvent() {
	this((RectangleInkStroke)null, (RectangleInkStroke) null);
    }


    public DragAndDropEvent(RectangleInkStroke eObject, GraphicalObject target) {
	this(eObject, 0, 0, target);
    }

    public DragAndDropEvent(RectangleInkStroke eObject, int x, int y, GraphicalObject target) {
	super(eObject, x, y);
	this.target = target;
    }

    public DragAndDropEvent(TextGraphicalObject eObject, GraphicalObject target) {
	this(eObject, 0, 0, target);
    }

    public DragAndDropEvent(TextGraphicalObject eObject, int x, int y, GraphicalObject target) {
	super(eObject, x, y);
	this.target = target;
    }

    public DragAndDropEvent(DragAndDropEvent copy) {
	super(copy);
	target = copy.target;
    }

    public DragAndDropEvent(DragAndDropEvent copy, GraphicalObject eObject) {
	super(copy, eObject);
	target = copy.target;
    }
   
    public DragAndDropEvent(DataInputStream stream) throws IOException {
	super(stream);
	ioTargetId = -1;
	target = null;
	boolean hasTarget = stream.readBoolean();
	if (hasTarget) {
	    ioTargetId = stream.readInt();
	}
    }


    // Public methods
    public void completeIO() {
	super.completeIO();

	// Target must be on the same storyboard as the source event object
	target = null;
	if (ioTargetId >= 0) {
	    Storyboard storyboard = eventObject.getStoryboard();
	    if (storyboard != null) {
		target = (GraphicalObject) storyboard.getGraphicalObjectById(ioTargetId);
	    }
	}
    }


    public GraphicalObject getTarget() {
	return target;
    }

    public void setTarget(GraphicalObject ris) {
	target = ris;
    }


    public void doRender(Graphics2D g) {
	// Draw outer circle
	g.drawArc(x, y, width, height, 0, 360);
	// Draw filled inner circle
	g.fillArc(x + 4, y + 4, 8, 8, 0, 360);

	// Draw outer inner circle
	g.setStroke(LIGHT_STROKE);
	g.drawArc(x + 4, y + 4, 8, 8, 0, 360);
    }


    public String toString() {
	String ret = null;
	if ((eventObject != null) && (target != null)) {
	    if ((target instanceof RectangleInkStroke) || (target instanceof TextGraphicalObject) ) {
		String scopeName = eventObject.getScope().getIdentifierId();
		ret = "When the user drags " + 
		    scopeName + "::" + eventObject.getNamedModelId() + 
		    " to " + 
		    scopeName + "::" + target.getNamedModelId();
	    }
	}
	return ret;
    }


    public boolean requiresRuntimeMouseEvents() {
	return true;
    }


    public Object clone(GraphicalObject eObject) {
	return (new DragAndDropEvent(this, eObject));
    }
    

    public int getClassId() {
	return CLASSID;
    }


    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	super.write(stream);
	
	stream.writeBoolean(target != null);
	if (target != null) {
	    stream.writeInt(target.getIdentifier());
	}
    }

}
