/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File EnterTextEvent.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class EnterTextEvent extends SourceEvent {
    
    public static final int CLASSID = 1080;



    private EnterTextEvent() {
	this((RectangleInkStroke)null);
    }

    public EnterTextEvent(RectangleInkStroke eObject) {
	this(eObject, 0, 0);
    }

    public EnterTextEvent(RectangleInkStroke eObject, int x, int y) {
	super(eObject, x, y);
    }

    public EnterTextEvent(EnterTextEvent copy) {
	super(copy);
    }
    
    public EnterTextEvent(EnterTextEvent copy, GraphicalObject eObject) {
	super(copy, eObject);
    }

    public EnterTextEvent(DataInputStream stream) throws IOException {
	super(stream);
    }



    public void doRender(Graphics2D g) {
	g.drawArc(x, y, width, height, 0, 360);

	// Draw the A
	g.drawString("A", x + 3, y + 13);
    }


    public String toString() {
	String ret = null;

	// Special case behavior in that the eventObject needs
	// to be notified to create the text model. A rectangle ink
	// stroke does not create a text model by default
	if ((eventObject != null) && (eventObject instanceof RectangleInkStroke)) {
	    RectangleInkStroke ris = (RectangleInkStroke) eventObject;
	    ris.createTextModel();
	    String scopeName = eventObject.getScope().getIdentifierId();
	    ret = "When the user enters text into " + scopeName + "::" + ris.getTextModelId();

	} else {
	    DEMAISError.showErrorMessage("Event object is not a rectangle", "EnterTextEvent", false);
	}
	return ret;
    }


    public Object clone(GraphicalObject eObject) {
	return (new EnterTextEvent(this, eObject));
    }


    public int getClassId() {
	return CLASSID;
    }

              
    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	super.write(stream);
    }

}
