/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File NarrationSyncMarkerEvent.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class NarrationSyncMarkerEvent extends SourceEvent {
    
    public static final int CLASSID = 1210;


    private NarrationModel narrationModel;
    private int syncIndex;
    private int ioModelId;


    private NarrationSyncMarkerEvent() {
	this(null, 0);
    }

    public NarrationSyncMarkerEvent(NarrationModel nModel, int index) {
	super(null, 0, 0);
	narrationModel = nModel;
	syncIndex = index;
    }

    public NarrationSyncMarkerEvent(NarrationSyncMarkerEvent copy) {
	this(copy, null);
    }

    public NarrationSyncMarkerEvent(NarrationSyncMarkerEvent copy, GraphicalObject eObject) {
	super(copy, eObject);
	narrationModel = copy.narrationModel;
	syncIndex = copy.syncIndex;
    }


    public NarrationSyncMarkerEvent(DataInputStream stream) throws IOException {
	super(stream);
	narrationModel = null;
	syncIndex = -1;

	ioModelId = -1;
	if (stream.readBoolean()) {
	    ioModelId = stream.readInt();
	    syncIndex = stream.readInt();
	}
    }

    public void completeIO() {
	super.completeIO();
	if (ioModelId != -1) {
	    narrationModel = NarrationModel.getNarrationModelById(new Integer(ioModelId));
	}
    }


    public void doRender(Graphics2D g) {
	// Suppress rendering of the event? - yes for now!
    }


    public String toString() {
	String ret = null;
	if (narrationModel != null) {
	    Variable variable = narrationModel.getVariableAtIndex(syncIndex);
	    if (variable != null) {
		String scopeName = narrationModel.getIdentifierId();
		ret = "When (" + scopeName + "::" + variable.getIdentifierId() + " == 1.0)";
	    }
	}
	return ret;
    }


    public Object clone(GraphicalObject eObject) {
	return (new NarrationSyncMarkerEvent(this));
    }


    public int getClassId() {
	return CLASSID;
    }

    
    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);

	super.write(stream);

	stream.writeBoolean(narrationModel != null);
	if (narrationModel != null) {
	    stream.writeInt(narrationModel.getIdentifier().intValue());
	    stream.writeInt(syncIndex);
	} 
    }
}
