/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File NavigationAction.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.core.parser.Terminal;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class NavigationAction extends DestinationAction {

    public static final int CLASSID = 1100;


    private StoryboardModel destModel;
    private int ioDestModel;


    // Constructs
    private NavigationAction() {
	this((StoryboardModel) null);
    }

    public NavigationAction(StoryboardModel model) {
	this(model, (GraphicalObject) null);
    }

    public NavigationAction(StoryboardModel model, GraphicalObject eObject) {
	this(model, eObject, 0, 0);
    }

    public NavigationAction(StoryboardModel model, int x, int y) {
	this(model, null, x, y);
    }

    public NavigationAction(StoryboardModel model, GraphicalObject eObject, int x, int y) {
	super(eObject, x, y);
	destModel = model;
	if (destModel == null) {
	    destModel = retrieveStoryboardFromUser();
	}
    }



    public NavigationAction(TextGraphicalObject eObject) {
	this(eObject, 0, 0);
    }

    public NavigationAction(TextGraphicalObject eObject, int x, int y) {
	super(eObject, x, y);
	destModel = null;
    }

 
    public NavigationAction(NavigationAction copy) {
	this(copy, copy.eventObject);
    }


    public NavigationAction(NavigationAction copy, GraphicalObject eObject) {
	super(copy, eObject);
	destModel = copy.destModel;
    }


    public NavigationAction(DataInputStream stream) throws IOException {
	super(stream);

	ioDestModel = -1;
	boolean hasDestModel = stream.readBoolean();
	if (hasDestModel) {
	    ioDestModel = stream.readInt();
	}
    }

    public void completeIO() {
	super.completeIO();
	if (ioDestModel != -1) {
	    destModel = StoryboardModel.getStoryboardModelById(new Integer(ioDestModel));
	}
    }



    public void doRender(Graphics2D g) {
	// Draw the outer rect
	g.drawRoundRect(x, y, width, height, 3, 3);

	// Draw the N
	g.drawString("N", x + 3, y + 13);
    }


    public String toString() {
	String ret = null;
	if (destModel != null) {
	    ret = "navigate to " + destModel.getIdentifierId();
	
	} else if ((eventObject != null) && (eventObject instanceof TextGraphicalObject)) {
	    TextGraphicalObject tgo = (TextGraphicalObject) eventObject;
	    ret = tgo.getText();
	}
	return ret;
    }


    public boolean hasDestModel() {
	return (destModel != null);
    }

    public StoryboardModel getDestModel() {
	return destModel;
    }


    public Object clone(GraphicalObject eObject) {
	return (new NavigationAction(this, eObject));
    }


    public boolean requiresRuntimeInitialRemoval() {
	if ((eventObject != null) && (eventObject instanceof TextGraphicalObject)) {
	    TextGraphicalObject tgo = (TextGraphicalObject) eventObject;
	    Terminal terminal;
	    if (((terminal = tgo.isNavigationAction()) != null) ||
		 ((terminal = tgo.isCMAction()) != null)) {
		return true;
	    }
	}
	return false;
    }


    private StoryboardModel retrieveStoryboardFromUser() {
        String directions = "Select destination storyboard ";
	Vector sNames = StoryboardModel.getStoryboardModelNames(true);
	JComboBox comboBox = new JComboBox(sNames);
	if (sNames.size() > 0) {
	    comboBox.setSelectedIndex(0);
	}

        Object[] array = {directions, comboBox};

        String btnString1 = "Ok";
        Object[] options = {btnString1};
	
	DataModel dModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
        int val = JOptionPane.showOptionDialog(dModel.getStoryboard(),
					       array,
					       "Select Options",
					       JOptionPane.YES_NO_OPTION,
					       JOptionPane.PLAIN_MESSAGE,
					       null,
					       options,
					       options[0]);
	
	StoryboardModel ret = null;
	if (val == 0) {
	    ret = StoryboardModel.getStoryboardModelByName((String)comboBox.getSelectedItem());
	}
	return ret;
    }


    public int getClassId() {
	return CLASSID;
    }


          
    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	super.write(stream);

	stream.writeBoolean(destModel != null);
	if (destModel != null) {
	    stream.writeInt(destModel.getIdentifier().intValue());
	}
    }
    
}
