/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File PauseMediaAction.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class PauseMediaAction extends DestinationAction {

    public static final int CLASSID = 1110;

    private NarrationModel narrationModel;
    private int ioModel;


    // Constructors
    private PauseMediaAction() {
	this((VideoRectangleInkStroke)null);
    }

 
    public PauseMediaAction(ContinuousMediaRectangleInkStroke eObject) {
	this(eObject, 0, 0);
    }
    
    public PauseMediaAction(ContinuousMediaRectangleInkStroke eObject, int x, int y) {
	super(eObject, x, y);
	narrationModel = null;

    }

    public PauseMediaAction(TextGraphicalObject eObject) {
	this(eObject, 0, 0);
    }

    public PauseMediaAction(TextGraphicalObject eObject, int x, int y) {
	super(eObject, x, y);
	narrationModel = null;

    }

    public PauseMediaAction(NarrationModel model) {
	super(null, 0, 0);
	narrationModel = model;
    }

    public PauseMediaAction(PauseMediaAction copy) {
	super(copy);
	narrationModel = copy.narrationModel;

    }

    public PauseMediaAction(PauseMediaAction copy, GraphicalObject eObject) {
	super(copy, eObject);
	narrationModel = copy.narrationModel;

    }
    
    public PauseMediaAction(DataInputStream stream) throws IOException {
	super(stream);
	narrationModel = null;
	ioModel = -1;
	if (stream.readBoolean()) {
	    ioModel = stream.readInt();
	}
    }

    
    public void completeIO() {
	super.completeIO();
	if (ioModel != -1) {
	    narrationModel = NarrationModel.getNarrationModelById(new Integer(ioModel));
	}
    }



    public void doRender(Graphics2D g) {
	// Draw filled interior
	if (selected) {
	    g.setColor(ACTIVE_COLOR);
	}
	g.fillRoundRect(x, y, width, height, 3, 3);

	// Draw outer edge
	setColor(g);
	g.drawRoundRect(x, y, width, height, 3, 3);
    }


    public String toString() {
	String ret = null;
	if (narrationModel == null) {

	    if (eventObject != null) {
		String scopeName = eventObject.getScope().getIdentifierId();
		if (eventObject instanceof ContinuousMediaRectangleInkStroke) {
		    ret = "pause " + scopeName + "::" + eventObject.getNamedModelId();

		} else if (eventObject instanceof TextGraphicalObject) {
		    TextGraphicalObject tgo = (TextGraphicalObject) eventObject;
		    if (tgo.isCMAction() != null) {
			ret = tgo.getText();
			
		    } else {
			ret = "pause " + scopeName + "::" + tgo.getNamedModelId();
			
		    }
		}
	    }
	    
	} else {
	    ret = "pause " + narrationModel.getIdentifierId();
	}
	return ret;
    }


    public Object clone(GraphicalObject eObject) {
	return (new PauseMediaAction(this, eObject));
    }


    public int getClassId() {
	return CLASSID;
    }

          
    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	
	// Write the parent information
	super.write(stream);

	stream.writeBoolean(narrationModel != null);
	if (narrationModel != null) {
	    stream.writeInt(narrationModel.getIdentifier().intValue());
	} 
    }

}

