/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File PlayAudioAction.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class PlayAudioAction extends DestinationAction {

    public static final int CLASSID = 1120;


    // Default exclamation sound
    public static AudioModel DEFAULT_EXCLAIM_SOUND;

 
    // The audio model to play for the action
    private AudioModel audioModel;



    // Static initializer
    static {
	DEFAULT_EXCLAIM_SOUND = AudioModel.getAudioModel("Exclaim");
	if (DEFAULT_EXCLAIM_SOUND != null) {
	    DEFAULT_EXCLAIM_SOUND.realizePlayer();
	} else {
	    System.out.println("ERROR: Could not load exclaim sound");
	}
    }
 

    // Constructs
    private PlayAudioAction() {
	this(null, (String)null);
    }

    public PlayAudioAction(GraphicalObject eObject) {
	this(eObject, (String)null);
    }

    public PlayAudioAction(GraphicalObject eObject, String audioName) {
	this(eObject, 0, 0, audioName);
    }
    
    public PlayAudioAction(GraphicalObject eObject, int x, int y, String audioName) {
	super(eObject, x, y);
	if (audioName == null) {
	    audioName = retrieveAudioFileName();
	}
	audioModel = AudioModel.getAudioModel(audioName);
	if (audioModel != null) {
	    audioModel.realizePlayer();
	}
    }
 

    public PlayAudioAction(GraphicalObject eObject, AudioModel audioModel) {
	this(eObject, 0, 0, audioModel);
    }

    public PlayAudioAction(GraphicalObject eObject, int x, int y, AudioModel audioModel) {
	super(eObject, x, y);
	this.audioModel = audioModel;
    }


    public PlayAudioAction(PlayAudioAction copy) {
	super(copy);
	audioModel = copy.audioModel;
    }


    public PlayAudioAction(PlayAudioAction copy, GraphicalObject eObject) {
	super(copy, eObject);
	audioModel = copy.audioModel;
    }


   public PlayAudioAction(DataInputStream stream) throws IOException {
	super(stream);

	// Read the model
	audioModel = null;
	boolean hasModel = stream.readBoolean();
	if (hasModel) {
	    audioModel = new AudioModel(stream, null, null);
	}
   }



    public void doRender(Graphics2D g) {
	// Draw the outer circle
	g.drawRoundRect(x, y, width, height, 3, 3);

	// Draw the lower left circle
	g.fillArc(x + 2, y + 10, 4, 4, 0, 360);

	// Draw the left line
	g.drawLine(x + 5, y + 11, x + 5, y + 4);

	// Draw the upper line
	g.drawLine(x + 5, y + 4, x + 12, y + 4);

	// Draw the right line
	g.drawLine(x + 12, y + 4, x + 12, y + 10);

	// Draw the lower right circle
	g.fillArc(x + 10, y + 10, 4, 4, 0, 360);
    }


    public String toString() {
	String ret = null;
	if (audioModel != null) {
	    ret = "play " + audioModel.getIdentifierId();
	}
	return ret;
    }


    public Object clone(GraphicalObject eObject) {
	return (new PlayAudioAction(this, eObject));
    }
  


    private String retrieveAudioFileName() {
        String directions = "Select audio object ";
	JComboBox comboBox = new JComboBox(AudioModel.getAudioModelNames());
	comboBox.setSelectedIndex(0);

        Object[] array = {directions, comboBox};

        String btnString1 = "Ok";
        Object[] options = {btnString1};
	
	DataModel dModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
        int val = JOptionPane.showOptionDialog(dModel.getStoryboard(),
					       array,
					       "Select Audio",
					       JOptionPane.YES_NO_OPTION,
					       JOptionPane.PLAIN_MESSAGE,
					       null,
					       options,
					       options[0]);
	
	String ret = "Exclaim";
	if (val == 0) {
	    ret = (String) comboBox.getSelectedItem();
	}
	return ret;
    }


    public int getClassId() {
	return CLASSID;
    }

    
    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	super.write(stream);
	
	// Write the audioModel
	stream.writeBoolean(audioModel != null);
	if (audioModel != null) {
	    audioModel.write(stream);
	}
    }
}

