/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File SetMediaTimeAction.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.io.*;
import java.awt.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class SetMediaTimeAction extends DestinationAction {

    public static final int CLASSID = 1140;

    // Saved media time
    protected double mediaTime;
    


    private SetMediaTimeAction() {
	this((VideoRectangleInkStroke)null, 0.0);
    }


    public SetMediaTimeAction(ContinuousMediaRectangleInkStroke eObject) {
	this(eObject, 0, 0, eObject.getMediaTime());
    }

    public SetMediaTimeAction(ContinuousMediaRectangleInkStroke eObject, double time) {
	this(eObject, 0, 0, time);
    }

    public SetMediaTimeAction(ContinuousMediaRectangleInkStroke eObject, int x, int y, double time) {
	super(eObject, x, y, VLI_WIDTH + 2, VLI_HEIGHT + 2);
	mediaTime = time;
    }
 
    public SetMediaTimeAction(SetMediaTimeAction copy) {
	super(copy);
	mediaTime = copy.mediaTime;
    }
    
    public SetMediaTimeAction(SetMediaTimeAction copy, GraphicalObject eObject) {
	super(copy, eObject);
	mediaTime = copy.mediaTime;
    }

    public SetMediaTimeAction(DataInputStream stream) throws IOException {
	super(stream);
	mediaTime = stream.readDouble();
    }



    public void setMediaTime(double time) {
	mediaTime = time;
    }

    public double getMediaTime() {
	return mediaTime;
    }

    public void doRender(Graphics2D g) {
	// Draw filled interior
	if (selected) {
	    g.setColor(PropertyManager.DARK_BLUE);
	}
	g.fillRoundRect(x, y, width, height, 3, 3);

	// Draw outer edge
	setColor(g);
	g.drawRoundRect(x, y, width, height, 3, 3);

	Font font = NUMBER_FONT_SMALL;
	if (mediaTime < 9.5) {
	    font = NUMBER_FONT_LARGE;
	} else if (mediaTime < 99.5) {
	    font = NUMBER_FONT_MEDIUM;
	}
	g.setFont(font);
	
	// Draw the time
	int t = (int) Math.round(mediaTime);
	int sX = x;
	int sY = y;
	if (t < 9.5) {
	    sX += 6;
	    sY += 14;
	} else if (t < 99.5) {
	    sX += 3;
	    sY += 13;
	}  else {
	    sX += 2;
	    sY += 12;
	}
	
	g.drawString(Integer.toString(t), sX, sY);
    }


    public String toString() {
	String ret = null;
	if (eventObject != null) {
	    String scopeName = eventObject.getScope().getIdentifierId();
	    ret = "set " + scopeName + "::" + eventObject.getNamedModelId() + " to " + Double.toString(mediaTime);
	}
	return ret;
    }

    public Object clone(GraphicalObject eObject) {
	return (new SetMediaTimeAction(this, eObject));
    }

   
    public int getClassId() {
	return CLASSID;
    }


    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	super.write(stream);

	stream.writeDouble(mediaTime);
    }

}
