/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File TemporalEvent.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.awt.*;
import java.io.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class TemporalEvent extends SourceEvent {
    
    public static final int CLASSID = 1170;


    // Time to wait before event fires
    private double time;


    public TemporalEvent(GraphicalObject eObject) {
	this(eObject, 0, 0, -1.0);
    }

    public TemporalEvent(GraphicalObject eObject, int t) {
	this(eObject, 0, 0, (double) t);
    }
    
    public TemporalEvent(GraphicalObject eObject, double t) {
	this(eObject, 0, 0, t);
    }
    
    public TemporalEvent(GraphicalObject eObject, int x, int y, int t) {
	this(eObject, x, y, (double) t);
    }
    
    public TemporalEvent(GraphicalObject eObject, int x, int y, double t) {
	super(eObject, x, y, VLI_WIDTH + 2, VLI_HEIGHT + 2);
	time = t;
	if (time == -1) {
	    time = retrieveTimeFromUser();
	}
    }

    public TemporalEvent(TemporalEvent copy) {
	super(copy);
	time = copy.time;
    }

    public TemporalEvent(TemporalEvent copy, GraphicalObject eObject) {
	super(copy, eObject);
	time = copy.time;
    }


    public TemporalEvent(DataInputStream stream) throws IOException {
	super(stream);
	time = stream.readDouble();
    }



    public void doRender(Graphics2D g) {
	g.drawArc(x, y, width, height, 0, 360);

	Font font = NUMBER_FONT_SMALL;
	if (time < 9.5) {
	    font = NUMBER_FONT_LARGE;
	} else if (time < 99.5) {
	    font = NUMBER_FONT_MEDIUM;
	} 
	g.setFont(font);

	int t = (int) Math.round(time);
	int sX = x;
	int sY = y;
	if (t < 9.5) {
	    sX += 6;
	    sY += 14;
	} else if (t < 99.5) {
	    sX += 3;
	    sY += 13;
	} else {
	    sX += 2;
	    sY += 12;
	}
	g.drawString(Integer.toString(t), sX, sY);
    }


    public String toString() {
	return ("After " + Double.toString(time) + " seconds");
    }


    public Object clone(GraphicalObject eObject) {
	return (new TemporalEvent(this, eObject));
    }


    public int getClassId() {
	return CLASSID;
    }


    private double retrieveTimeFromUser() {
        String directions = "Enter Amount of Time ";
        JTextField textField = new JTextField("0", 10);
        Object[] array = {directions, textField};

        String btnString1 = "Ok";
        Object[] options = {btnString1};
	
        int val = JOptionPane.showOptionDialog(eventObject.getStoryboard(),
						   array,
						   "Enter Amount of Time ",
						   JOptionPane.YES_NO_OPTION,
						   JOptionPane.QUESTION_MESSAGE,
						   null,
						   options,
						   options[0]);
	
	double value = 0.0;
	if (val == 0) {
	    try {
		value = Double.parseDouble(textField.getText());
	    } catch (NumberFormatException xcptn) {
	    }
	}
	    
	return value;
    }


    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	super.write(stream);

	stream.writeDouble(time);
    }
    
}
