/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File TransitionAction.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.behavior;
    

// Necessary imports
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.core.parser.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;
import edu.uofminnesota.DEMAIS.usrinterface.multiview.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;




public class TransitionAction extends DestinationAction {
    
    public static final int CLASSID = 1180;
    private static int LAST_SPEED = PropertyManager.TRANSITION_MEDIUM;
    private static int LAST_TYPE = PropertyManager.WIPE_TRANSITION;

    private StoryboardModel destModel;
    private int ioDestModel;

    // Type of transition
    private int type;

    // Speed of transition
    private int speed;

    // Dialog used to obtain the information
    private JDialog dialog;

    private JComboBox destComboBox;



    private TransitionAction() {
	this((StoryboardModel)null);
    }    

    public TransitionAction(StoryboardModel model) {
	this(model, (GraphicalObject)null);
    }

    public TransitionAction(StoryboardModel model, GraphicalObject eObject) {
	this(model, eObject, 0, 0);
    }
    public TransitionAction(TextGraphicalObject eObject) {
	this(eObject, 0, 0);
    }

    public TransitionAction(StoryboardModel model, int x, int y) {
	this(model, null, x, y);
    }

    public TransitionAction(StoryboardModel model, GraphicalObject eObject, int x, int y) {
	this(model, eObject, x, y, -1, -1);
    }
    public TransitionAction(TextGraphicalObject eObject, int x, int y) {
	this(eObject, x, y, -1, -1);
    }

    public TransitionAction(StoryboardModel model, GraphicalObject eObject, int x, int y, int type, int speed) {
	super(eObject, x, y, BehaviorEvent.VLI_WIDTH + 6, BehaviorEvent.VLI_HEIGHT + 2);
	destModel = model;
	this.type = type;
	this.speed = speed;
	if ((type == -1) || (speed == -1)) {
	    retrieveModelTypeSpeedFromUser((destModel == null));
	}
    }

    public TransitionAction(TextGraphicalObject eObject, int x, int y, int type, int speed) {
	super(eObject, x, y, BehaviorEvent.VLI_WIDTH + 6, BehaviorEvent.VLI_HEIGHT + 2);
	destModel = null;
	this.type = type;
	this.speed = speed;
	if ((type == -1) || (speed == -1)) {
	    retrieveModelTypeSpeedFromUser((eObject == null));
	}
    }



    public TransitionAction(TransitionAction copy) {
	this(copy, copy.eventObject);
    }
    
    public TransitionAction(TransitionAction copy, GraphicalObject eObject) {
	super(copy, eObject);
	destModel = copy.destModel;
	type = copy.type;
	speed = copy.speed;
    }


    public TransitionAction(DataInputStream stream) throws IOException {
	super(stream);

	ioDestModel = -1;
	boolean hasDestModel = stream.readBoolean();
	if (hasDestModel) {
	    ioDestModel = stream.readInt();
	}
	
	type = stream.readInt();
	speed = stream.readInt();
    }

    public void completeIO() {
	super.completeIO();
	if (ioDestModel != -1) {
	    destModel = StoryboardModel.getStoryboardModelById(new Integer(ioDestModel));
	}
    }


    public void doRender(Graphics2D g) {
	// Draw the outer circle
	g.drawRoundRect(x, y, width, height, 3, 3);

	// Draw a smaller 'T'
	g.setFont(TRANSITION_LETTER_FONT);
	g.drawString("T", x + 7, y + 14);
	
	g.setFont(LETTER_FONT_TINY);
	if (type == PropertyManager.DISSOLVE_TRANSITION) {
	    g.drawString("d", x + 3, y + 14);
	    
	} else if (type == PropertyManager.WIPE_TRANSITION) {
	    g.drawString("w", x + 3, y + 14);
	}
	
	if (speed == PropertyManager.TRANSITION_SLOW) {
	    g.drawString("s", x + 15, y + 14);
	    
	} else if (speed == PropertyManager.TRANSITION_MEDIUM) {
	    g.drawString("m", x + 15, y + 14);
	    
	} else if (speed == PropertyManager.TRANSITION_FAST) {
	    g.drawString("f", x + 15, y + 14);
	    
	}
    }


    public String toString() {
	String typeString = null;
	if (type == PropertyManager.DISSOLVE_TRANSITION) {
	    typeString = "dissolve";
	} else if (type == PropertyManager.WIPE_TRANSITION) {
	    typeString = "wipe";
	} else {
	    typeString = "unknown";
	}

	String speedString = null;
	if (speed == PropertyManager.TRANSITION_SLOW) {
	    speedString = "slow";
	} else if (speed == PropertyManager.TRANSITION_MEDIUM) {
	    speedString = "medium";
	} else if (speed == PropertyManager.TRANSITION_FAST) {
	    speedString = "fast";
	} else {
	    speedString = "unknown";
	}

	String ret = "transition [" + typeString + ", " + speedString + "] to ";
	if (destModel != null) {
	    ret += destModel.getIdentifierId();
	
	} else {
	    DEMAISError.showErrorMessage("Uknown transition destination", "Transition", false);
	}

	return ret;
    }


    public Object clone(GraphicalObject eObject) {
	return (new TransitionAction(this, eObject));
    }


    public int getClassId() {
	return CLASSID;
    }


    public boolean hasDestModel() {
	return (destModel != null);
    }

    public StoryboardModel getDestModel() {
	return destModel;
    }


    private void retrieveModelTypeSpeedFromUser(boolean retrieveDest) {
	JFrame frame = PropertyManager.APPLICATION_FRAME_WIDGET;
	dialog = new JDialog(frame, "Select Options", true);
	dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	dialog.setResizable(false);

	Listener listener = new Listener();
	dialog.addWindowListener(listener);

	JPanel contentPanel = new JPanel(new GridBagLayout());
	contentPanel.setBorder(BorderFactory.createEtchedBorder());

	dialog.setContentPane(contentPanel);

	GridBagConstraints c = new GridBagConstraints();

	JPanel enterDestPanel = new JPanel(new GridBagLayout());
	destComboBox = null;

	if (retrieveDest) {
	    JLabel enterDestLabel = new JLabel("Select destination storyboard ");
	    Vector sNames = StoryboardModel.getStoryboardModelNames(true);
	    destComboBox = new JComboBox(sNames);
	    if (sNames.size() > 0) {
		destComboBox.setSelectedIndex(0);
	    }

	    c.gridx = 0;
	    c.gridy = 0;
	    c.fill = GridBagConstraints.BOTH;
	    c.weightx = 1.0;
	    c.weighty = 1.0;
	    enterDestPanel.add(enterDestLabel, c);
	    
	    c.gridy = 1;
	    enterDestPanel.add(destComboBox, c);
	}

	JPanel typePanel = new JPanel(new GridBagLayout());
	typePanel.setBorder(BorderFactory.createEtchedBorder());
	JPanel speedPanel = new JPanel(new GridBagLayout());
	speedPanel.setBorder(BorderFactory.createEtchedBorder());

	JLabel typeLabel = new JLabel("Select transition type:");
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.weightx = 1.0;
	c.weighty = 0.0;
	typePanel.add(typeLabel, c);

	JPanel typeTogglePanel = new JPanel(new GridBagLayout());

	JRadioButton dissolveB = new JRadioButton("Dissolve");
	dissolveB.setActionCommand("Dissolve");
	dissolveB.addActionListener(listener);
	if (LAST_TYPE == PropertyManager.DISSOLVE_TRANSITION) {
	    dissolveB.setSelected(true);
	}
	type = PropertyManager.DISSOLVE_TRANSITION;
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 0.5;
	typeTogglePanel.add(dissolveB, c);

	JRadioButton wipeB = new JRadioButton("Wipe");
	wipeB.addActionListener(listener);
	wipeB.setActionCommand("Wipe");
	if (LAST_TYPE == PropertyManager.WIPE_TRANSITION) {
	    wipeB.setSelected(true);
	}
	c.gridx = 2;
	typeTogglePanel.add(wipeB, c);

	ButtonGroup typeGroup = new ButtonGroup();
	typeGroup.add(dissolveB);
	typeGroup.add(wipeB);
	
	c.gridx = 0;
	c.gridy = 1;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	typePanel.add(typeTogglePanel, c);
	

	JLabel speedLabel = new JLabel("Select the transition speed:");
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.weightx = 1.0;
	c.weighty = 0.0;
	speedPanel.add(speedLabel, c);

	JPanel speedTogglePanel = new JPanel(new GridBagLayout());

	JRadioButton slowB = new JRadioButton("Slow");
	slowB.setActionCommand("Slow");
	slowB.addActionListener(listener);
	if (LAST_SPEED == PropertyManager.TRANSITION_SLOW) {
	    slowB.setSelected(true);
	}
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.weightx = 0.3;
	c.weighty = 0.0;
	speedTogglePanel.add(slowB, c);

	JRadioButton mediumB = new JRadioButton("Medium");
	mediumB.setActionCommand("Medium");
	mediumB.addActionListener(listener);
	if (LAST_SPEED == PropertyManager.TRANSITION_MEDIUM) {
	    mediumB.setSelected(true);
	}
	speed = PropertyManager.TRANSITION_MEDIUM;

	c.gridx = 1;
	speedTogglePanel.add(mediumB, c);

	JRadioButton fastB = new JRadioButton("Fast");
	fastB.setActionCommand("Fast");
	fastB.addActionListener(listener);
	if (LAST_SPEED == PropertyManager.TRANSITION_FAST) {
	    fastB.setSelected(true);
	}
	c.gridx = 2;
	speedTogglePanel.add(fastB, c);

	ButtonGroup speedGroup = new ButtonGroup();
	speedGroup.add(slowB);
	speedGroup.add(mediumB);
	speedGroup.add(fastB);

	c.gridx = 0;
	c.gridy = 1;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	speedPanel.add(speedTogglePanel, c);


	c.gridy = 0;
	c.insets = new Insets(6, 20, 6, 20);
	if (destModel == null) {
	    contentPanel.add(enterDestPanel, c);
	    ++c.gridy;
	}

	contentPanel.add(typePanel, c);
	
	++c.gridy;
	contentPanel.add(speedPanel, c);


	JButton closeB = new JButton("Close");
	closeB.setActionCommand("Close");
	closeB.addActionListener(listener);

	++c.gridy;
	c.fill = GridBagConstraints.NONE;
	contentPanel.add(closeB, c);
	
	dialog.pack();

	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	int dialogWidth = dialog.getWidth();
	int dialogHeight = dialog.getHeight();
	dialog.setLocation((screenSize.width - dialogWidth) / 2,
			   (screenSize.height - dialogHeight) / 2);
	dialog.show();
   
    }


    class Listener extends WindowAdapter implements ActionListener {
	public Listener() {
	}
	
	public void actionPerformed(ActionEvent evt) {
	    String cmd = evt.getActionCommand();
	    handleAction(cmd);
	}


	public void windowClosing(WindowEvent e) {
	    handleAction("Close");
	}


	private void handleAction(String cmd) {
	    if (cmd.equals("Dissolve")) {
		type = PropertyManager.DISSOLVE_TRANSITION;
		LAST_TYPE = type;
		
	    } else if (cmd.equals("Wipe")) {
		type = PropertyManager.WIPE_TRANSITION;
		LAST_TYPE = type;
		
	    } else if (cmd.equals("Slow")) {
		speed = PropertyManager.TRANSITION_SLOW;
		LAST_SPEED = speed;

	    } else if (cmd.equals("Medium")) {
		speed = PropertyManager.TRANSITION_MEDIUM;
		LAST_SPEED = speed;
		
	    } else if (cmd.equals("Fast")) {
		speed = PropertyManager.TRANSITION_FAST;
		LAST_SPEED = speed;

	    } else if (cmd.equals("Close")) {
		if (destComboBox != null) {
		    destModel = StoryboardModel.getStoryboardModelByName((String)destComboBox.getSelectedItem());
		}   
		dialog.dispose();
	    }
	}
    }


    public void write(DataOutputStream stream) throws IOException {
	stream.writeInt(CLASSID);
	super.write(stream);

	stream.writeBoolean(destModel != null);
	if (destModel != null) {
	    stream.writeInt(destModel.getIdentifier().intValue());
	}

	stream.writeInt(type);
	stream.writeInt(speed);
    }
    
}
