/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   File StoryboardMediatorFrame.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.mediation;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;




public class StoryboardMediatorFrame extends JDialog {

    private Listener listener;
    private ButtonListener buttonListener;
    private GraphicalObject selectedObject;
    private Vector objects;


    public StoryboardMediatorFrame(Vector graphicalObjs) {
	super(PropertyManager.APPLICATION_FRAME_WIDGET, "Which Object Did You Mean ?", true);
	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

	listener = new Listener();
	buttonListener = new ButtonListener();
	addWindowListener(listener);
	selectedObject = null;
	objects = graphicalObjs;

	setLookAndFeel();
	setupContentPane(graphicalObjs);
	
	pack();
	//setResizable(false);
	setRootLocation();
	setVisible(true);
    }

    

    // Private methods
    private void setLookAndFeel() {
	try {
	    UIManager.setLookAndFeel(DEMAISFrame.getLookAndFeelClassName());
	
	} catch (Exception xcptn) {
	    DEMAISError.showErrorMessage(xcptn.toString(), "Content Manager", false);
	    DEMAISError.logException(xcptn);
	}
    }


    public GraphicalObject getSelectedObject() {
	return selectedObject;
    }


    private void setRootLocation() {
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension me = this.getSize();
	
	Point p = new Point( (int)(screen.getWidth() - me.getWidth()) / 2, 
			     (int)(screen.getHeight() - me.getHeight()) / 2);
	setLocation(p);
    }


    private void setupContentPane(Vector graphicalObjs) {
	JPanel contentPane = new JPanel(new GridBagLayout());
	contentPane.setBorder(BorderFactory.createLoweredBevelBorder());
	this.setContentPane(contentPane);

	//JScrollPane jScrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
	//JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	JPanel container = new JPanel(new GridBagLayout());
	//jScrollPane.setViewportView(container);
	GridBagConstraints c = new GridBagConstraints();	
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	contentPane.add(container, c);

	Iterator it = graphicalObjs.iterator();
	int index = 0;
	while (it.hasNext()) {
	    GraphicalObject go = (GraphicalObject) it.next();
	    Rectangle r = go.getBasicBounds();
	    BufferedImage bImage = new BufferedImage(r.width + 16, r.height + 16, BufferedImage.TYPE_INT_RGB);
	    Graphics2D g2d = bImage.createGraphics();
	    g2d.setColor(PropertyManager.STORYBOARD_BACKGROUND_COLOR);
	    g2d.fillRect(0, 0, r.width + 16, r.height + 16);
	    g2d.setStroke(new BasicStroke(1.0f));
	    g2d.setColor(Color.black);	    
	    if (go instanceof TextGraphicalObject) {
		TextGraphicalObject tgo = (TextGraphicalObject) go;
		g2d.setFont(tgo.getFont());
		tgo.mediateRender(g2d, 8, (r.height + 16) / 2);

	    } else if (go instanceof ContinuousMediaRectangleInkStroke) {
		g2d.setFont(new Font("Arial", Font.BOLD, 14));
		g2d.drawString("Not supported", 8, 8);

	    } else if (go instanceof RectangleInkStroke) {
		RectangleInkStroke ris = (RectangleInkStroke) go;
		ris.mediateRender(g2d, -r.x + 8, -r.y + 8);
	    }

	    // Scale the image
	    JButton button = new JButton(new ImageIcon(bImage.getScaledInstance(100, 75, Image.SCALE_DEFAULT)));
	    button.setActionCommand(Integer.toString(index));

	    button.addActionListener(buttonListener);
	    container.add(button, c);
	    ++c.gridy;
	    ++index;
	    // Add this image to the container
	}
    }


    private void setSelectedObject(String index) {
	int i = 0;
	try {
	    i = Integer.parseInt(index);
	} catch (Exception xcptn) {
	}

	selectedObject = (GraphicalObject) objects.elementAt(i);
    }


    
    class Listener extends WindowAdapter {
	public Listener() {
	}
		
	public void windowClosing(WindowEvent e) {
	    setVisible(false);
	    dispose();
	}
    }


    class ButtonListener implements ActionListener {
	public ButtonListener() {
	}

	public void actionPerformed(ActionEvent evt) {
	    String index = evt.getActionCommand();
	    setSelectedObject(index);
	    setVisible(false);
	    dispose();
	}

    }
}
