/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  AbstractStrokeAssembler.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.stroke;

// Necessary imports
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.visuallanguage.*;
import edu.uofminnesota.DEMAIS.usrinterface.base.*;




public abstract class AbstractStrokeAssembler 
    extends MouseInputAdapter {
    
    protected Storyboard storyboard;
    protected StrokeAssemblerTimer assemblerTimer;
    protected MouseEvent lastPressEvent;
    protected TimedStroke currentStroke;
    private boolean vliDrop;



    // Prevent default constructor
    private AbstractStrokeAssembler() {
	this(null, true);
    }


    public AbstractStrokeAssembler(Storyboard pStoryboard, boolean holdAndPressTimer) {
	storyboard = pStoryboard;
	storyboard.addMouseListener(this);
	storyboard.addMouseMotionListener(this);
	assemblerTimer = null;
	if (holdAndPressTimer) {
	    assemblerTimer = new StrokeAssemblerTimer(this);
	    assemblerTimer.stop();
	}
	lastPressEvent = null;
	currentStroke = null;
	vliDrop = false;
    }
    


    // Public methods
    public void mousePressed(MouseEvent evt) {
	// First, check for a visual language icon drop
	vliDrop = false;
	int code = ((Integer) PropertyManager.getProperty(PropertyManager.APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED)).intValue();
	if (code >= 0) {
	    // A possible drop, but need to check if over a VLI on the storyboard
	    BehavioralInkStroke bStroke = (BehavioralInkStroke) 
		storyboard.getBehavioralInkStrokeContaining(evt.getX(), evt.getY(), VLButton.isVLIconASourceEvent(code), storyboard.getCurrentLayer(), false);
	    if (bStroke != null) {	    
		storyboard.onVisualLanguageIconDropped(evt, code, bStroke);
		currentStroke = null;
		vliDrop = true;
	    }
	}
	
	if (! vliDrop) {
	    handleMousePressed(evt);
	    lastPressEvent = evt;
	}
    }


    public void mouseDragged(MouseEvent evt) { 
	if (PieMenuBar.getPieMenuBar().isVisible()) {
	    return;
	}

	if (! vliDrop) {
	    // Not part of a VLI sequence
	    handleMouseDragged(evt);
	}
    }


    public void mouseReleased(MouseEvent evt) {
	// Shut off the timer immediately
	if (! vliDrop) {
	    if (assemblerTimer != null) {
		assemblerTimer.stop();
	    }
	    
	    if (currentStroke != null) {	    
		storyboard.onMouseReleased(evt, currentStroke);
	    }
	    currentStroke = null;
	}
    }


    public void cancelStroke() {
	if (assemblerTimer != null) {
	    assemblerTimer.stop();
	}
	currentStroke = null;
    }
    

    public void removeStrokeAssembler() {
	if (assemblerTimer != null) {
	    assemblerTimer.stop();
	}
	storyboard.removeMouseListener(this);
	storyboard.removeMouseMotionListener(this);
    }


    protected void timerPopped() {
	currentStroke = null;
	storyboard.onHoldAndPress(lastPressEvent);
    }


    // Abstract methods for the specific input devices to handle
    protected abstract int getHoldAndPressTime(MouseEvent evt);
    protected abstract void handleMousePressed(MouseEvent evt);
    protected abstract void handleMouseDragged(MouseEvent evt);
    protected abstract void handleMouseReleased(MouseEvent evt);

}
