/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  MouseStrokeAssembler.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.stroke;

// Necessary imports
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;



public class MouseStrokeAssembler extends AbstractStrokeAssembler {  

    // Prevent default constructor
    private MouseStrokeAssembler() {
	this(null, true);
    }

    public MouseStrokeAssembler(Storyboard pStoryboard, boolean holdAndPressTimer) {
	super(pStoryboard, holdAndPressTimer);
    }



    protected void handleMousePressed(MouseEvent evt) {
	if (currentStroke == null) {
	    if (SwingUtilities.isLeftMouseButton(evt)) {
		currentStroke = new InkTimedStroke();
	    } else {
		currentStroke = new GestureTimedStroke();
	    }
	    currentStroke.addPoint(evt.getX(), evt.getY());    
	    storyboard.onMousePressed(evt, currentStroke);
	    
	    if (assemblerTimer != null) {
		// Re-set the timer
		assemblerTimer.start(getHoldAndPressTime(evt));
	    }
	    
	} else {
	    Toolkit.getDefaultToolkit().beep();
	}
    }


    protected void handleMouseDragged(MouseEvent evt) {
	if (assemblerTimer != null) {
	    // Stop the timer immediately
	    assemblerTimer.stop();
	}

	if ((SwingUtilities.isLeftMouseButton(evt)) &&
	    (storyboard.isGraphicalObjectSelected())) {
	    storyboard.onMouseDragged(evt);
	    
	} else if (currentStroke != null) {
	    currentStroke.addPoint(evt.getX(), evt.getY());
	    storyboard.onMouseDragged(currentStroke);
	}
    }
    

    protected void handleMouseReleased(MouseEvent evt) {
    }
    

    protected int getHoldAndPressTime(MouseEvent evt) {
	if (SwingUtilities.isLeftMouseButton(evt)) {
	    return 450;
	} else {
	    return 400;
	}
    }

}




