/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  StylusStrokeAssembler.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.storyboard.stroke;

// Necessary imports
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;
import edu.uofminnesota.DEMAIS.usrinterface.util.*;



public class StylusStrokeAssembler extends AbstractStrokeAssembler {

    // Prevent default constructor
    private StylusStrokeAssembler() {
	this(null, true);
    }

    public StylusStrokeAssembler(Storyboard pStoryboard, boolean holdAndPressTimer) {
	super(pStoryboard, holdAndPressTimer);
    }
    


    protected void handleMousePressed(MouseEvent evt) {
	/* 
	   If the user holds the modifier button on the stlyus, and then touches the screen, the WACOM driver sends two consecutive events: a right mouse press event and then a left mouse press event. However, I doubt the order is guaranteed. Need to wait until the left mouse press event is generated, signifying that the user has touched the stylus to the screen.
	*/
	if (SwingUtilities.isLeftMouseButton(evt)) {
	    // Wait for the left mouse press event	    
	    if (! isRightMouseButtonDown(evt)) {
		currentStroke = new InkTimedStroke();
	    } else {
		currentStroke = new GestureTimedStroke();
	    }
	    currentStroke.addPoint(evt.getX(), evt.getY());    
	    storyboard.onMousePressed(evt, currentStroke);
	    
	    if (assemblerTimer != null) {
		// Re-start the timer
		assemblerTimer.start(getHoldAndPressTime(evt));
	    }
	}
    }
    

    protected void handleMouseDragged(MouseEvent evt) {
	/*
	  Using a stylus, a user often moves it slightly when s/he wants a text extry cursor or taps the pen, thus we need to check the length of the current stroke before stopping it
	*/
	if ((currentStroke != null) &&
	    (currentStroke.lengthOfStroke() > PropertyManager.APPLICATION_MAX_TAP_POINTS)) {
	    if (assemblerTimer != null) {
		assemblerTimer.stop();
	    }

	    if ((!isRightMouseButtonDown(evt)) &&
		(storyboard.isGraphicalObjectSelected())) {
		currentStroke = null;
	    }
	}
 	
	if ((!isRightMouseButtonDown(evt)) &&
	    (storyboard.isGraphicalObjectSelected())) {
	    if (currentStroke == null) {
		storyboard.onMouseDragged(evt);
	    } else {
		currentStroke.addPoint(evt.getX(), evt.getY());
	    }

	} else if (currentStroke != null) {		
	    currentStroke.addPoint(evt.getX(), evt.getY());
	    storyboard.onMouseDragged(currentStroke);
	}
   }


    protected void handleMouseReleased(MouseEvent evt) {
    }


    protected int getHoldAndPressTime(MouseEvent evt) {
	if (SwingUtilities.isLeftMouseButton(evt)) {
	    return 450;
	} else {
	    return 400;
	}
    }
    

    private boolean isRightMouseButtonDown(MouseEvent evt) {
	return ((evt.getModifiers() & InputEvent.BUTTON3_MASK) != 0) ||
	    ((evt.getModifiers() & InputEvent.BUTTON2_MASK) != 0) ||
	    ((evt.getModifiers() & InputEvent.ALT_MASK) != 0);
    }

}


