/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   DissolveTransition.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.transition;

// Import packages
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.property.*;
import edu.uofminnesota.DEMAIS.meal.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.*;





public class DissolveTransition extends AbstractTransition {

    public DissolveTransition(StoryboardModel toModel, int speed) {
	super(toModel, speed);
	
	DataModel fromModel = (DataModel) PropertyManager.getProperty(PropertyManager.APPLICATION_CURRENT_DATA_MODEL);
	Storyboard fromStoryboard = fromModel.getStoryboard();
	Graphics2D fromGraphics = (Graphics2D) fromStoryboard.getGraphics();

	Storyboard toStoryboard = toModel.getStoryboard();
	Image imageBuffer = new BufferedImage(fromStoryboard.getWidth(),
					      fromStoryboard.getHeight(),
					      BufferedImage.TYPE_INT_RGB);
	if (imageBuffer != null) {
	    Graphics2D toGraphics = (Graphics2D) imageBuffer.getGraphics();
	    if ((fromGraphics != null) && (toGraphics != null)) {
		int width = imageBuffer.getWidth(null);
		int height = imageBuffer.getHeight(null);
		
		toGraphics.setColor(PropertyManager.STORYBOARD_BACKGROUND_COLOR);
		toGraphics.fillRect(0, 0, width, height);
		toStoryboard.paintStoryboardComponent(toGraphics);
		
		int blockSize = 12;
		if (speed == PropertyManager.TRANSITION_SLOW) {
		    blockSize = 8;
		} else if (speed == PropertyManager.TRANSITION_MEDIUM) {
		    blockSize = 12;
		} else if (speed == PropertyManager.TRANSITION_FAST) {
		    blockSize = 16;
		}
		
		Vector blockList = new Vector(8000);
		for (int ypos = 0; ypos < (height + blockSize); ypos = ypos + blockSize) {
		    for (int xpos = 0; xpos < (width + blockSize); xpos = xpos + blockSize) {
			blockList.add(new Point(xpos, ypos));
		    }
		}
		
		int remaining = blockList.size();
		int block = 0;
		Point p = null;
		while (remaining > 0) {
		    block = (int) Math.round(Math.random() * (remaining - 1));
		    p = (Point) blockList.elementAt(block);
		    fromGraphics.drawImage(imageBuffer,
					   p.x, p.y, p.x + blockSize, p.y + blockSize,
					   p.x, p.y, p.x + blockSize, p.y + blockSize, null);
		    blockList.removeElementAt(block);
		    --remaining;
		}
	    }
	}
    }
}
