/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/* 
   DMath.java
*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.util;

// Import packages
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.graphicalobjs.*;
import edu.uofminnesota.DEMAIS.usrinterface.storyboard.stroke.*;





public class DMath {

    private DMath() {
    }

    // Compute the distance between two points, but leave it squared
    public static double distanceSquared(int x1, int y1, int x2, int y2) {
	int dx = x2 - x1;
	int dy = y2 - y1;
	return (double) (dx*dx + dy*dy);
    }
    
    public static double distanceSquared(double x1, double y1,
					 double x2, double y2) {
	double dx = x2 - x1;
	double dy = y2 - y1;
	return (dx*dx + dy*dy);
    }


    // Compute the distance between two points
    public static double distance(int x1, int y1, int x2, int y2) {
	return (Math.sqrt(distanceSquared(x1, y1, x2, y2)));
    }

    public static double distance(double x1, double y1, double x2, double y2) {
	return (Math.sqrt(distanceSquared(x1, y1, x2, y2)));
    }

    public static double distance(Point p1, Point p2) {
	return distance(p1.x, p1.y, p2.x, p2.y);
    }
    
    public static double distance(int x, int y, Point p) {
	return distance(x, y, p.x, p.y);
    }



    // Locate the GraphicalObject in the collection that is closest to r
    public static GraphicalObject findClosest(Collection c, Rectangle r) {
	return findClosest(c, r.x + (r.width / 2), r.y + (r.height / 2));
    }

    // Locate the GraphicalObject in the collection that is closest to (x, y);
    public static GraphicalObject findClosest(Collection c, int x, int y) {
	GraphicalObject best = null;
	double bestDistance = Double.MAX_VALUE;

	if (c != null) {
	    Iterator it = c.iterator();
	    while(it.hasNext()) {
		GraphicalObject go = (GraphicalObject) it.next();
		Rectangle r = go.getBounds();
		double d = distanceFromPointToRectangle(x, y, r);
		if (d < bestDistance) {
		    bestDistance = d;
		    best = go;
		}
	    }
	}
	return best;
    }


    public static Vector findClosestAndPeripherals(Collection c, int x, int y, int tolerance) {
	GraphicalObject best = null;
	Vector close = new Vector();
	double bestDistance = Double.MAX_VALUE;
	
	if (c != null) {
	    Iterator it = c.iterator();
	    while(it.hasNext()) {
		GraphicalObject go = (GraphicalObject) it.next();
		Rectangle r = go.getBounds();
		double d = distanceFromPointToRectangle(x, y, r);
		if (d < bestDistance) {
		    bestDistance = d;
		    best = go;
		}
		if (d < tolerance) {
		    close.add(go);
		}
	    }
	    if (! close.contains(best)) {
		close.insertElementAt(best, 0);
	    }
	}
	return close;
    }


    public static double distanceFromPointToRectangle(Point p, Rectangle r) {
	return distanceFromPointToRectangle(p.x, p.y, r);
    }

    public static double distanceFromPointToRectangle(int x, int y, Rectangle r) {
	// computer distance from point to each of the four sides
	double left = distance(r.x, y, x, y);
	double top = distance(x, r.y, x, y);
	double right = distance(r.x + r.width, y, x, y);
	double bottom = distance(x, r.y + r.height, x, y);
	return Math.min(Math.min(Math.min(left,top),right),bottom);
    }

} 
