/*
Copyright (c) 2001 Regents of the University of Minnesota.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the University may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
  File: VLButton.java

*/

// Current package
package edu.uofminnesota.DEMAIS.usrinterface.visuallanguage;

// Necessary imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import edu.uofminnesota.DEMAIS.property.*;




public abstract class VLButton extends JPanel {

    // Static member variables
    public static final int SINGLE_MOUSE_CLICK_EVENT = 0;
    public static final int DOUBLE_MOUSE_CLICK_EVENT = 1;
    public static final int ROLLOVER_EVENT = 2;
    public static final int ENTER_TEXT_EVENT = 3;
    public static final int DRAG_AND_DROP_EVENT = 4;
    public static final int BEGIN_MEDIA_EVENT = 5;
    public static final int SYNC_MARKER_EVENT = 6;
    public static final int END_MEDIA_EVENT = 7;
    public static final int TEMPORAL_EVENT = 8;

    public static final int NAVIGATE_ACTION = 10;
    public static final int TRANSITION_ACTION = 11;
    public static final int BEGIN_MEDIA_ACTION = 12;
    public static final int SET_MEDIA_TIME_ACTION = 13;
    public static final int PAUSE_MEDIA_ACTION = 14;
    public static final int END_MEDIA_ACTION = 15;
    public static final int DELAY_ACTION = 16;
    public static final int DISPLAY_ACTION = 17;
    public static final int UNDISPLAY_ACTION = 18;
    public static final int HIGHLIGHT_ACTION = 19;
    public static final int UNHIGHLIGHT_ACTION = 20;
    public static final int PLAY_AUDIO_ACTION = 21;

    protected static final int WIDTH = 20;
    protected static final int HEIGHT = 20;
    protected static final int OFFSET = 4;

    private static final Color SELECTED_FOREGROUND_COLOR = Color.white;
    private static final Color SELECTED_BACKGROUND_COLOR = new Color(10, 36, 106);
    
    protected static final Color ENABLED_FOREGROUND_COLOR = Color.black;
    private static Color ENABLED_BACKGROUND_COLOR = null;

    private static final Color DISABLED_FOREGROUND_COLOR = new Color(150, 150, 150);
    private static Color DISABLED_BACKGROUND_COLOR = null;

    private static final BasicStroke STROKE = new BasicStroke(2.0f);
    private static final Font LETTER_FONT = new Font("Arial", Font.BOLD, 14);

    private static final Border MOUSE_OVER_BORDER = BorderFactory.createMatteBorder(2, 2, 2, 2, Color.black);
    private static Border NULL_BORDER = null;


    // Instance variables

    // Whether this button is selected
    protected boolean selected;

    // Whether this button is enabled
    protected boolean enabled;

    // button listener
    protected Listener listener;



    public static boolean isVLIconASourceEvent(int vl_icon) {
	return ( (vl_icon >= 0) && (vl_icon < 10) );
    }

    public static boolean isVLIconAnAction(int vl_icon) {
	return (vl_icon >= 10);
    }




    // Constructors
    public VLButton() {
        this(false, true);
    }

    public VLButton(boolean selected, boolean enabled) {
	// Set instance variables
	this.selected = selected;
	this.enabled = enabled;

	if (NULL_BORDER == null) {
	    ENABLED_BACKGROUND_COLOR = getBackground();
	    DISABLED_BACKGROUND_COLOR = ENABLED_BACKGROUND_COLOR;

	    NULL_BORDER = BorderFactory.createMatteBorder(2, 2, 2, 2, ENABLED_BACKGROUND_COLOR);
	}
	setBorder(NULL_BORDER);

	// Set width and height
	Dimension dim = new Dimension(WIDTH + (OFFSET * 2), HEIGHT + (OFFSET * 2));
	setMinimumSize(dim);
	setPreferredSize(dim);
	setMaximumSize(dim);

	// Create and attach the listener object
	listener = new Listener();
	addMouseListener(listener);
	PropertyManager.addPropertyListener(PropertyManager.APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED, listener);
	PropertyManager.addPropertyListener(PropertyManager.APPLICATION_MODE, listener);

    }



    // Public member functions
    public void setSelected(boolean b) {
	selected = b;
	
	setBorder(NULL_BORDER);
	if (selected) {
	    setBackground(SELECTED_BACKGROUND_COLOR);
	} else {
	    setBackground(ENABLED_BACKGROUND_COLOR);
	}
	paintImmediately(0, 0, getWidth(), getHeight());
    }


    public void setEnabled(boolean b) {
	enabled = b;
	if (enabled) {
	    setBackground(ENABLED_BACKGROUND_COLOR);
	    
	} else {
	    setBackground(DISABLED_BACKGROUND_COLOR);
	}
	paintImmediately(0, 0, getWidth(), getHeight());
    }


    public void paint(Graphics graphics) {
	Graphics2D g = (Graphics2D) graphics;
	super.paint(g);
	
	g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
			   RenderingHints.VALUE_ANTIALIAS_ON);
	setColor(g);
	setStroke(g);
	setFont(g);
    }


    protected void setColor(Graphics2D g) {
	if (selected) {
	    g.setColor(SELECTED_FOREGROUND_COLOR);

	} else if (enabled) {
	    g.setColor(ENABLED_FOREGROUND_COLOR);

	} else {
	    g.setColor(DISABLED_FOREGROUND_COLOR);
	}
    }


    protected void setStroke(Graphics2D g) {
	g.setStroke(STROKE);
    }


    protected void setFont(Graphics2D g) {
	g.setFont(LETTER_FONT);
    }


    // Subclasses must override
    public abstract int getVLClass();


    
    class Listener extends MouseAdapter
	implements PropertyListenerInterface {
	
	// Constructors
	public Listener() {
	}

	// Mouse interface
	public void mouseEntered(MouseEvent e) {
	    if ((enabled) && (! selected)) {
		setBorder(MOUSE_OVER_BORDER);
	    }
	}

	public void mouseExited(MouseEvent e) {
	    if ((enabled) && (! selected)) {
		setBorder(NULL_BORDER);
	    }
	}
	
	public void mousePressed(MouseEvent e) {
	    if (enabled) {
		setSelected(!selected);
		if (selected) {
		    PropertyManager.setProperty(PropertyManager.APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED, new Integer(getVLClass()));
		} else {
		    PropertyManager.setProperty(PropertyManager.APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED, new Integer(-1));
		}
	    }
	}
	
	
	// Property Listener interface
	public void propertyChanged(Integer property, Object oldValue, Object newValue) {
	    if (property.equals(PropertyManager.APPLICATION_LAST_VISUAL_LANGUAGE_ICON_SELECTED)) {
		int value = ((Integer) newValue).intValue();
		// User may have disselected this VLI by selecting another
		setSelected(value == getVLClass());
	    
	    } else if (property.equals(PropertyManager.APPLICATION_MODE)) {
		int toMode = ((Integer)newValue).intValue();
		switch (toMode) {
		case PropertyManager.APPLICATION_PLAY:
		    setEnabled(false);
		    break;

		case PropertyManager.APPLICATION_EDIT:
		    setEnabled(true);
		    break;   

		default:
		    break;
		}	
	    }
	}    
    }
}
